@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2004 Hiroyuki Yamamoto & The Sylpheed Claws Team
+ * Copyright (C) 1999-2005 Hiroyuki Yamamoto & The Sylpheed Claws Team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,9 +24,12 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
-#include <gdk/gdkx.h>
+#ifdef GDK_WINDOWING_X11
+#  include <gdk/gdkx.h>
+#endif /* GDK_WINDOWING_X11 */
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -36,7 +39,6 @@
 #include <signal.h>
 #include <unistd.h>
 
-#include "intl.h"
 #include "utils.h"
 #include "gtkutils.h"
 #include "manage_window.h"
@@ -47,7 +49,6 @@
 #include "action.h"
 #include "compose.h"
 #include "procmsg.h"
-#include "gtkstext.h"
 #include "textview.h"
 #include "matcher_parser.h" /* CLAWS */
 #include "filtering.h"
@@ -76,7 +77,8 @@ struct _Children
 	gboolean	 output;
 
 	GtkWidget	*msg_text;
-	GdkFont		*msgfont;
+
+ 	gboolean	 is_selection;
 };
 
 struct _ChildInfo
@@ -120,7 +122,6 @@ static gboolean execute_filtering_actions(gchar		*action,
 static gboolean execute_actions		(gchar		*action, 
 					 GSList		*msg_list, 
 					 GtkWidget	*text,
-					 GdkFont	*msgfont,
 					 gint		 body_pos,
 					 MimeInfo	*partinfo);
 
@@ -508,7 +509,7 @@ static void compose_actions_execute_cb(Compose *compose, guint action_nb,
 		return;
 	}
 
-	execute_actions(action, NULL, compose->text, NULL, 0, NULL);
+	execute_actions(action, NULL, compose->text, 0, NULL);
 }
 
 static void mainwin_actions_execute_cb(MainWindow *mainwin, guint action_nb,
@@ -540,7 +541,6 @@ static void message_actions_execute(MessageView *msgview, guint action_nb,
 	gchar *buf;
 	gchar *action;
 	GtkWidget *text = NULL;
-	GdkFont *msgfont = NULL;
 	guint body_pos = 0;
 	ActionType action_type;
 	
@@ -549,7 +549,7 @@ static void message_actions_execute(MessageView *msgview, guint action_nb,
 	buf = (gchar *)g_slist_nth_data(prefs_common.actions_list, action_nb);
 
 	g_return_if_fail(buf);
-	g_return_if_fail(action = strstr(buf, ": "));
+	g_return_if_fail((action = strstr(buf, ": ")));
 
 	/* Point to the beginning of the command-line */
 	action += 2;
@@ -557,7 +557,6 @@ static void message_actions_execute(MessageView *msgview, guint action_nb,
 	textview = messageview_get_current_textview(msgview);
 	if (textview) {
 		text     = textview->text;
-		msgfont  = textview->msgfont;
 		body_pos = textview->body_pos;
 	}
 	partinfo = messageview_get_selected_mime_part(msgview);
@@ -572,7 +571,7 @@ static void message_actions_execute(MessageView *msgview, guint action_nb,
 		 * filtering */
 		execute_filtering_actions(action, msg_list);
 	else
-		execute_actions(action, msg_list, text, msgfont, body_pos, partinfo);
+		execute_actions(action, msg_list, text, body_pos, partinfo);
 }
 
 static gboolean execute_filtering_actions(gchar *action, GSList *msglist)
@@ -603,7 +602,7 @@ static gboolean execute_filtering_actions(gchar *action, GSList *msglist)
 }
 
 static gboolean execute_actions(gchar *action, GSList *msg_list,
-				GtkWidget *text, GdkFont *msgfont,
+				GtkWidget *text,
 				gint body_pos, MimeInfo *partinfo)
 {
 	GSList *children_list = NULL;
@@ -614,11 +613,12 @@ static gboolean execute_actions(gchar *action, GSList *msg_list,
 	ActionType action_type;
 	MsgInfo *msginfo;
 	gchar *cmd;
-	guint start = 0, end = 0;
 	gchar *sel_str = NULL;
 	gchar *msg_str = NULL;
 	gchar *user_str = NULL;
 	gchar *user_hidden_str = NULL;
+	GtkTextIter start_iter, end_iter;
+	gboolean is_selection = FALSE;
 
 	g_return_val_if_fail(action && *action, FALSE);
 
@@ -645,32 +645,20 @@ static gboolean execute_actions(gchar *action, GSList *msg_list,
 	}
 
 	if (text) {
-		if (GTK_EDITABLE(text)->has_selection) {
-			start = GTK_EDITABLE(text)->selection_start_pos;
-			end   = GTK_EDITABLE(text)->selection_end_pos;
-			if (start > end) {
-				guint tmp;
-				tmp = start;
-				start = end;
-				end = tmp;
-			}
+		GtkTextBuffer *textbuf;
 
-			if (start == end) {
-				start = body_pos;
-				end = gtk_stext_get_length(GTK_STEXT(text));
-				msg_str = gtk_editable_get_chars
-					(GTK_EDITABLE(text), start, end);
-			} else {
-				sel_str = gtk_editable_get_chars
-					(GTK_EDITABLE(text), start, end);
-				msg_str = g_strdup(sel_str);
-			}
-		} else {
-			start = body_pos;
-			end = gtk_stext_get_length(GTK_STEXT(text));
-			msg_str = gtk_editable_get_chars(GTK_EDITABLE(text),
-							 start, end);
+		textbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text));
+		is_selection = gtk_text_buffer_get_selection_bounds
+			(textbuf, &start_iter, &end_iter);
+		if (!is_selection) {
+			gtk_text_buffer_get_iter_at_offset
+				(textbuf, &start_iter, body_pos);
+			gtk_text_buffer_get_end_iter(textbuf, &end_iter);
 		}
+		msg_str = gtk_text_buffer_get_text
+			(textbuf, &start_iter, &end_iter, FALSE);
+		if (is_selection)
+			sel_str = g_strdup(msg_str);
 	}
 
 	if (action_type & ACTION_USER_STR) {
@@ -691,11 +679,10 @@ static gboolean execute_actions(gchar *action, GSList *msg_list,
 		}
 	}
 
-	if (action_type & ACTION_PIPE_OUT) {
-		gtk_stext_freeze(GTK_STEXT(text));
-		gtk_stext_set_point(GTK_STEXT(text), start);
-		gtk_stext_forward_delete(GTK_STEXT(text), end - start);
-		gtk_stext_thaw(GTK_STEXT(text));
+ 	if (text && (action_type & ACTION_PIPE_OUT)) {
+ 		GtkTextBuffer *textbuf;
+ 		textbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text));
+ 		gtk_text_buffer_delete(textbuf, &start_iter, &end_iter);
 	}
 
 	children = g_new0(Children, 1);
@@ -703,7 +690,7 @@ static gboolean execute_actions(gchar *action, GSList *msg_list,
 	children->action      = g_strdup(action);
 	children->action_type = action_type;
 	children->msg_text    = text;
-	children->msgfont     = msgfont;
+ 	children->is_selection = is_selection;
 
 	if ((action_type & (ACTION_USER_IN | ACTION_USER_HIDDEN_IN)) &&
 	    ((action_type & ACTION_SINGLE) == 0 || msg_list_len == 1))
@@ -778,10 +765,11 @@ static ChildInfo *fork_child(gchar *cmd, const gchar *msg_str,
 			     Children *children)
 {
 	gint chld_in[2], chld_out[2], chld_err[2], chld_status[2];
-	gchar *cmdline[4];
+	gchar *cmdline[4], *ret_str;
 	pid_t pid, gch_pid;
 	ChildInfo *child_info;
 	gint sync;
+	gssize by_written = 0, by_read = 0;
 
 	sync = !(children->action_type & ACTION_ASYNC);
 
@@ -815,7 +803,9 @@ static ChildInfo *fork_child(gchar *cmd, const gchar *msg_str,
 		if (setpgid(0, 0))
 			perror("setpgid");
 
+#ifdef GDK_WINDOWING_X11
 		close(ConnectionNumber(gdk_display));
+#endif /* GDK_WINDOWING_X11 */
 
 		gch_pid = fork();
 
@@ -847,11 +837,18 @@ static ChildInfo *fork_child(gchar *cmd, const gchar *msg_str,
 
 			cmdline[0] = "sh";
 			cmdline[1] = "-c";
-			cmdline[2] = cmd;
+			ret_str = g_locale_from_utf8(cmd, strlen(cmd),
+						     &by_read, &by_written,
+						     NULL);
+			if (ret_str && by_written)
+				cmdline[2] = ret_str;
+			else
+				cmdline[2] = cmd;
 			cmdline[3] = NULL;
 			execvp("/bin/sh", cmdline);
 
 			perror("execvp");
+			g_free(ret_str);
 			_exit(1);
 		} else if (gch_pid < (pid_t) 0) { /* Fork error */
 			if (sync)
@@ -924,7 +921,13 @@ static ChildInfo *fork_child(gchar *cmd, const gchar *msg_str,
 		return child_info;
 
 	if ((children->action_type & ACTION_PIPE_IN) && msg_str) {
-		write(chld_in[1], msg_str, strlen(msg_str));
+		ret_str = g_locale_from_utf8(msg_str, strlen(msg_str),
+					     &by_read, &by_written, NULL);
+		if (ret_str && by_written) {
+			write(chld_in[1], ret_str, strlen(ret_str));
+			g_free(ret_str);
+		} else
+			write(chld_in[1], msg_str, strlen(msg_str));
 		if (!(children->action_type &
 		      (ACTION_USER_IN | ACTION_USER_HIDDEN_IN)))
 			close(chld_in[1]);
@@ -1007,8 +1010,9 @@ static void hide_io_dialog_cb(GtkWidget *w, gpointer data)
 	Children *children = (Children *)data;
 
 	if (!children->nb) {
-		gtk_signal_disconnect_by_data(GTK_OBJECT(children->dialog),
-					      children);
+		g_signal_handlers_disconnect_matched
+			(G_OBJECT(children->dialog), G_SIGNAL_MATCH_DATA,
+			 0, 0, NULL, NULL, children);
 		gtk_widget_destroy(children->dialog);
 		free_children(children);
 	}
@@ -1070,10 +1074,17 @@ static void update_io_dialog(Children *children)
 	debug_print("Updating actions input/output dialog.\n");
 
 	if (children->progress_bar) {
-		gtk_progress_configure(GTK_PROGRESS(children->progress_bar),
-				children->initial_nb -children->nb,
-				0.0, children->initial_nb);
-	}
+		gchar *text;
+		
+		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(children->progress_bar),
+					      (gdouble) (children->initial_nb - children->nb) /
+					      (gdouble) children->initial_nb);
+		text = g_strdup_printf("%s %d/%d", _("Completed"), 
+				       children->initial_nb - children->nb,
+				       children->initial_nb);
+		gtk_progress_bar_set_text(GTK_PROGRESS_BAR(children->progress_bar), text);
+		g_free(text);
+	}					      
 
 	if (!children->nb) {
 		gtk_widget_set_sensitive(children->abort_btn, FALSE);
@@ -1081,25 +1092,25 @@ static void update_io_dialog(Children *children)
 		if (children->input_hbox)
 			gtk_widget_set_sensitive(children->input_hbox, FALSE);
 		gtk_widget_grab_focus(children->close_btn);
-		gtk_signal_connect(GTK_OBJECT(children->dialog),
-				   "key_press_event",
-				   GTK_SIGNAL_FUNC(io_dialog_key_pressed_cb),
-				   children);
+		g_signal_connect(G_OBJECT(children->dialog),
+				 "key_press_event",
+				 G_CALLBACK(io_dialog_key_pressed_cb),
+				 children);
 	}
 
 	if (children->output) {
 		GtkWidget *text = children->text;
+		GtkTextBuffer *textbuf;
+		GtkTextIter iter, start_iter, end_iter;
 		gchar *caption;
 		ChildInfo *child_info;
-		GdkFont *font;
 
-		font = gtk_object_get_data(GTK_OBJECT(children->dialog), 
-					   "s_txtfont");
 		gtk_widget_show(children->scrolledwin);
-		gtk_text_freeze(GTK_TEXT(text));
-		gtk_text_set_point(GTK_TEXT(text), 0);
-		gtk_text_forward_delete(GTK_TEXT(text), 
-					gtk_text_get_length(GTK_TEXT(text)));
+		textbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text));
+		gtk_text_buffer_get_bounds(textbuf, &start_iter, &end_iter);
+		gtk_text_buffer_delete(textbuf, &start_iter, &end_iter);
+		gtk_text_buffer_get_start_iter(textbuf, &iter);
+
 		for (cur = children->list; cur; cur = cur->next) {
 			child_info = (ChildInfo *)cur->data;
 			if (child_info->pid)
@@ -1111,14 +1122,12 @@ static void update_io_dialog(Children *children)
 					(_("--- Ended: %s\n"),
 					 child_info->cmd);
 
-			gtk_text_insert(GTK_TEXT(text), font, NULL, NULL,
-					caption, -1);
-			gtk_text_insert(GTK_TEXT(text), font, NULL, NULL,
-					child_info->output->str, -1);
+			gtk_text_buffer_insert(textbuf, &iter, caption, -1);
+			gtk_text_buffer_insert(textbuf, &iter,
+					       child_info->output->str, -1);
 			g_free(caption);
 			child_info->new_out = FALSE;
 		}
-		gtk_text_thaw(GTK_TEXT(text));
 	}
 }
 
@@ -1136,7 +1145,7 @@ static void create_io_dialog(Children *children)
 	GtkWidget *progress_bar = NULL;
 	GtkWidget *abort_button;
 	GtkWidget *close_button;
-	GdkFont   *output_font;
+	PangoFontDescription *text_font;
 
 	debug_print("Creating action IO dialog\n");
 
@@ -1147,11 +1156,11 @@ static void create_io_dialog(Children *children)
 	gtk_window_set_title(GTK_WINDOW(dialog), _("Action's input/output"));
 	gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
 	manage_window_set_transient(GTK_WINDOW(dialog));
-	gtk_signal_connect(GTK_OBJECT(dialog), "delete_event",
-			GTK_SIGNAL_FUNC(delete_io_dialog_cb), children);
-	gtk_signal_connect(GTK_OBJECT(dialog), "destroy",
-			GTK_SIGNAL_FUNC(hide_io_dialog_cb),
-			children);
+	g_signal_connect(G_OBJECT(dialog), "delete_event",
+			 G_CALLBACK(delete_io_dialog_cb), children);
+	g_signal_connect(G_OBJECT(dialog), "destroy",
+			 G_CALLBACK(hide_io_dialog_cb),
+			 children);
 
 	vbox = gtk_vbox_new(FALSE, 8);
 	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), vbox);
@@ -1164,16 +1173,21 @@ static void create_io_dialog(Children *children)
 
 	scrolledwin = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwin),
-				       GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
+				       GTK_POLICY_AUTOMATIC, 
+				       GTK_POLICY_AUTOMATIC);
 	gtk_box_pack_start(GTK_BOX(vbox), scrolledwin, TRUE, TRUE, 0);
-	gtk_widget_set_usize(scrolledwin, 480, 200);
+	gtk_widget_set_size_request(scrolledwin, 480, 200);
 	gtk_widget_hide(scrolledwin);
 
-	text = gtk_text_new(gtk_scrolled_window_get_hadjustment
-			    (GTK_SCROLLED_WINDOW(scrolledwin)),
-			    gtk_scrolled_window_get_vadjustment
-			    (GTK_SCROLLED_WINDOW(scrolledwin)));
-	gtk_text_set_editable(GTK_TEXT(text), FALSE);
+	text = gtk_text_view_new();
+	if (prefs_common.textfont) {
+		if (NULL != (text_font = pango_font_description_from_string
+				(prefs_common.textfont))) {
+			gtk_widget_modify_font(text, text_font);
+			pango_font_description_free(text_font);
+		}			
+	}			
+	gtk_text_view_set_editable(GTK_TEXT_VIEW(text), FALSE);
 	gtk_container_add(GTK_CONTAINER(scrolledwin), text);
 	gtk_widget_show(text);
 
@@ -1182,17 +1196,17 @@ static void create_io_dialog(Children *children)
 		gtk_widget_show(input_hbox);
 
 		entry = gtk_entry_new();
-		gtk_widget_set_usize(entry, 320, -1);
-		gtk_signal_connect(GTK_OBJECT(entry), "activate",
-				   GTK_SIGNAL_FUNC(send_input), children);
+		gtk_widget_set_size_request(entry, 320, -1);
+		g_signal_connect(G_OBJECT(entry), "activate",
+				 G_CALLBACK(send_input), children);
 		gtk_box_pack_start(GTK_BOX(input_hbox), entry, TRUE, TRUE, 0);
 		if (children->action_type & ACTION_USER_HIDDEN_IN)
 			gtk_entry_set_visibility(GTK_ENTRY(entry), FALSE);
 		gtk_widget_show(entry);
 
-		send_button = gtk_button_new_with_label(_(" Send "));
-		gtk_signal_connect(GTK_OBJECT(send_button), "clicked",
-				   GTK_SIGNAL_FUNC(send_input), children);
+		send_button = gtk_button_new_from_stock(GTK_STOCK_EXECUTE);
+		g_signal_connect(G_OBJECT(send_button), "clicked",
+				 G_CALLBACK(send_input), children);
 		gtk_box_pack_start(GTK_BOX(input_hbox), send_button, FALSE,
 				   FALSE, 0);
 		gtk_widget_show(send_button);
@@ -1202,30 +1216,26 @@ static void create_io_dialog(Children *children)
 	}
 
 	if (children->initial_nb > 1) {
+		gchar * text;
+		
 		progress_bar = gtk_progress_bar_new();
-		gtk_progress_bar_set_bar_style(GTK_PROGRESS_BAR(progress_bar),
-				GTK_PROGRESS_CONTINUOUS);
 		gtk_progress_bar_set_orientation(GTK_PROGRESS_BAR(progress_bar),
 				GTK_PROGRESS_LEFT_TO_RIGHT);
-		gtk_progress_set_activity_mode(GTK_PROGRESS(progress_bar), 
-				FALSE);
-		gtk_progress_set_format_string(GTK_PROGRESS(progress_bar),
-				_("Completed %v/%u"));
-		gtk_progress_set_show_text(GTK_PROGRESS(progress_bar), TRUE);
-		gtk_progress_configure(GTK_PROGRESS(progress_bar),
-				children->initial_nb -children->nb,
-				0.0, children->initial_nb);
-
+		text = g_strdup_printf("%s 0/%d\n", _("Completed"), 
+		                       children->initial_nb);
+		gtk_progress_bar_set_text(GTK_PROGRESS_BAR(progress_bar),
+					  text);
+		g_free(text);
 		gtk_box_pack_start(GTK_BOX(vbox), progress_bar, FALSE, FALSE, 0);
 		gtk_widget_show(progress_bar);
 	}
 
-	gtkut_button_set_create(&hbox, &abort_button, _("Abort"),
-				&close_button, _("Close"), NULL, NULL);
-	gtk_signal_connect(GTK_OBJECT(abort_button), "clicked",
-			GTK_SIGNAL_FUNC(kill_children_cb), children);
-	gtk_signal_connect(GTK_OBJECT(close_button), "clicked",
-			GTK_SIGNAL_FUNC(hide_io_dialog_cb), children);
+	gtkut_stock_button_set_create(&hbox, &abort_button, GTK_STOCK_STOP,
+				      &close_button, GTK_STOCK_CLOSE, NULL, NULL);
+	g_signal_connect(G_OBJECT(abort_button), "clicked",
+			 G_CALLBACK(kill_children_cb), children);
+	g_signal_connect(G_OBJECT(close_button), "clicked",
+			 G_CALLBACK(hide_io_dialog_cb), children);
 	gtk_widget_show(hbox);
 
 	if (children->nb)
@@ -1242,10 +1252,6 @@ static void create_io_dialog(Children *children)
 	children->abort_btn    = abort_button;
 	children->close_btn    = close_button;
 
-	output_font = gtkut_font_load_from_fontset(prefs_common.textfont);
-	gtk_object_set_data_full(GTK_OBJECT(dialog), "s_txtfont",
-				 output_font, 
-				 (GtkDestroyNotify)gdk_font_unref); 
 	gtk_widget_show(dialog);
 }
 
@@ -1271,8 +1277,9 @@ static void catch_input(gpointer data, gint source, GdkInputCondition cond)
 {
 	Children *children = (Children *)data;
 	ChildInfo *child_info = (ChildInfo *)children->list->data;
-	gchar *input;
+	gchar *input, *ret_str;
 	gint c, count, len;
+	gssize by_read = 0, by_written = 0;
 
 	debug_print("Sending input to grand child.\n");
 	if (!(cond && GDK_INPUT_WRITE))
@@ -1283,6 +1290,13 @@ static void catch_input(gpointer data, gint source, GdkInputCondition cond)
 
 	input = gtk_editable_get_chars(GTK_EDITABLE(children->input_entry),
 				       0, -1);
+	ret_str = g_locale_from_utf8(input, strlen(input), &by_read,
+				     &by_written, NULL);
+	if (ret_str && by_written) {
+		g_free(input);
+		input = ret_str;
+	}
+
 	len = strlen(input);
 	count = 0;
 
@@ -1307,42 +1321,66 @@ static void catch_input(gpointer data, gint source, GdkInputCondition cond)
 static void catch_output(gpointer data, gint source, GdkInputCondition cond)
 {
 	ChildInfo *child_info = (ChildInfo *)data;
-	gint c, i;
+	gint c;
 	gchar buf[BUFFSIZE];
 
 	debug_print("Catching grand child's output.\n");
 	if (child_info->children->action_type &
 	    (ACTION_PIPE_OUT | ACTION_INSERT)
 	    && source == child_info->chld_out) {
-		gboolean is_selection = FALSE;
-		GtkWidget *text = child_info->children->msg_text;
+		GtkTextView *text =
+			GTK_TEXT_VIEW(child_info->children->msg_text);
+		GtkTextBuffer *textbuf = gtk_text_view_get_buffer(text);
+		GtkTextIter iter1, iter2;
+		GtkTextMark *mark;
+
+		mark = gtk_text_buffer_get_insert(textbuf);
+		gtk_text_buffer_get_iter_at_mark(textbuf, &iter1, mark);
+		gtk_text_buffer_get_iter_at_mark(textbuf, &iter2, mark);
 
-		if (GTK_EDITABLE(text)->has_selection)
-			is_selection = TRUE;
-		gtk_stext_freeze(GTK_STEXT(text));
 		while (TRUE) {
+			gsize bytes_read = 0, bytes_written = 0;
+			gchar *ret_str;
+
 			c = read(source, buf, sizeof(buf) - 1);
 			if (c == 0)
 				break;
-			gtk_stext_insert(GTK_STEXT(text),
-					 child_info->children->msgfont,
-					 NULL, NULL, buf, c);
+
+			ret_str = g_locale_to_utf8
+				(buf, c, &bytes_read, &bytes_written, NULL);
+			if (ret_str && bytes_written > 0) {
+				gtk_text_buffer_insert
+					(textbuf, &iter2, ret_str,
+					 bytes_written);
+				g_free(ret_str);
+			} else
+				gtk_text_buffer_insert(textbuf, &iter2, buf, c);
 		}
-		if (is_selection) {
-			/* Using the select_region draws things. Should not.
-			 * so we just change selection position and 
-			 * defere drawing when thawing. Hack?
-			 */
-			GTK_EDITABLE(text)->selection_end_pos =
-					gtk_stext_get_point(GTK_STEXT(text));
+
+		if (child_info->children->is_selection) {
+			gtk_text_buffer_place_cursor(textbuf, &iter1);
+			gtk_text_buffer_move_mark_by_name
+				(textbuf, "selection_bound", &iter2);
 		}
-		gtk_stext_thaw(GTK_STEXT(text));
 	} else {
 		c = read(source, buf, sizeof(buf) - 1);
-		for (i = 0; i < c; i++)
-			g_string_append_c(child_info->output, buf[i]);
-		if (c > 0)
+		
+		if (c > 0) {
+			gsize bytes_read = 0, bytes_written = 0;
+			gchar *ret_str;
+
+			ret_str = g_locale_to_utf8
+				(buf, c, &bytes_read, &bytes_written, NULL);
+			if (ret_str && bytes_written > 0) {
+				g_string_append_len
+					(child_info->output, ret_str,
+					 bytes_written);
+				g_free(ret_str);
+			} else
+				g_string_append_len(child_info->output, buf, c);
+
 			child_info->new_out = TRUE;
+		}
 	}
 	if (c == 0) {
 		if (source == child_info->chld_out) {
