@@ -201,7 +201,8 @@ static HeaderEntry *addrharvest_find(
 		HeaderEntry *entry;
 
 		entry = node->data;
-		if( g_strcasecmp( entry->header, name ) == 0 ) {
+		if (g_ascii_strncasecmp(entry->header, name,
+					strlen(entry->header)) == 0 ) {
 			retVal = entry;
 			break;
 		}
@@ -513,7 +514,6 @@ static void addrharvest_parse_address(
 		AddressCache *cache, const gchar *hdrBuf )
 {
 	gchar buffer[ ADDR_BUFFSIZE + 2 ];
-	gchar buf[ADDR_BUFFSIZE];
 	const gchar *bp;
 	const gchar *ep;
 	gchar *atCh, *email, *name;
@@ -551,13 +551,14 @@ static void addrharvest_parse_address(
 			mgu_str_ltc2space( buffer, '(', ')' );
 			g_strstrip( buffer );
 
-			if( g_strcasecmp( buffer, email ) == 0 ) {
+			if( g_ascii_strcasecmp( buffer, email ) == 0 ) {
 				name = "";
 			}
 			else {
-				name = buffer;
-				conv_unmime_header(buf, sizeof(buf), name,
-				NULL);
+                                gchar *tmp = conv_unmime_header(buffer, NULL);
+                                strncpy2(buffer, tmp, sizeof buffer);
+                                name = buffer;
+                                g_free(tmp);
 			}
 
 			/* Insert into address book */
@@ -589,7 +590,7 @@ static gboolean addrharvest_check_hdr( GList *listHdr, gchar *buf ) {
 		node = listHdr;
 		while( node ) {
 			nhdr = node->data;
-			if( g_strcasecmp( nhdr, hdr ) == 0 ) {
+			if (g_ascii_strncasecmp(nhdr, hdr, strlen(nhdr)) == 0 ) {
 				retVal = TRUE;
 				break;
 			}
