@@ -28,8 +28,8 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 
-#include "intl.h"
 #include "mgutils.h"
 #include "addritem.h"
 #include "addrcache.h"
@@ -1722,8 +1722,7 @@ gint addrindex_write_to( AddressIndex *addrIndex, const gchar *newFile ) {
 	g_free( fileSpec );
 	if( pfile ) {
 		fp = pfile->fp;
-		fprintf( fp, "<?xml version=\"1.0\" encoding=\"%s\" ?>\n",
-				conv_get_current_charset_str() );
+		fprintf( fp, "<?xml version=\"1.0\" encoding=\"%s\" ?>\n", CS_INTERNAL );
 #endif
 		addrindex_write_elem_s( fp, 0, TAG_ADDRESS_INDEX );
 		fputs( ">\n", fp );
