@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2001 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2005 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,10 +22,12 @@
 #endif
 
 #include <stddef.h>
+
+#include <glib.h>
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
-#include "intl.h"
 #include "mainwindow.h"
 #include "alertpanel.h"
 #include "manage_window.h"
@@ -33,15 +35,8 @@
 #include "gtkutils.h"
 #include "inc.h"
 #include "logwindow.h"
-#include "pixmaps/stock_dialog-info.xpm"
-#include "pixmaps/stock_dialog-question.xpm"
-#include "pixmaps/stock_dialog-error.xpm"
-#include "pixmaps/stock_dialog-warning.xpm"
+#include "prefs_common.h"
 
-#define TITLE_FONT	"-*-helvetica-medium-r-normal--17-*-*-*-*-*-*-*," \
-			"-*-*-medium-r-normal--16-*-*-*-*-*-*-*,*"
-#define MESSAGE_FONT	"-*-helvetica-medium-r-normal--14-*-*-*-*-*-*-*," \
-			"-*-*-medium-r-normal--14-*-*-*-*-*-*-*,*"
 #define ALERT_PANEL_WIDTH	380
 #define TITLE_HEIGHT		72
 #define MESSAGE_HEIGHT		62
@@ -68,7 +63,7 @@ static void alertpanel_button_clicked	(GtkWidget		*widget,
 static gint alertpanel_deleted		(GtkWidget		*widget,
 					 GdkEventAny		*event,
 					 gpointer		 data);
-static void alertpanel_close		(GtkWidget		*widget,
+static gboolean alertpanel_close	(GtkWidget		*widget,
 					 GdkEventAny		*event,
 					 gpointer		 data);
 
@@ -78,8 +73,8 @@ AlertValue alertpanel(const gchar *title,
 		      const gchar *button2_label,
 		      const gchar *button3_label)
 {
-	return alertpanel_with_type(title, message, button1_label, 
-				    button2_label, button3_label, 
+	return alertpanel_with_type(title, message, button1_label,
+				    button2_label, button3_label,
 				    NULL, ALERT_QUESTION);
 }
 
@@ -90,8 +85,8 @@ AlertValue alertpanel_with_widget(const gchar *title,
 				  const gchar *button3_label,
 				  GtkWidget *widget)
 {
-	return alertpanel_with_type(title, message, button1_label, 
-				    button2_label, button3_label, 
+	return alertpanel_with_type(title, message, button1_label,
+				    button2_label, button3_label,
 				    widget, ALERT_QUESTION);
 }
 
@@ -101,18 +96,18 @@ AlertValue alertpanel_with_type(const gchar *title,
 				const gchar *button2_label,
 				const gchar *button3_label,
 				GtkWidget   *widget,
-				gint	     alert_type)
+				gint         alert_type)
 {
- 	if (alertpanel_is_open)
- 		return -1;
- 	else
- 		alertpanel_is_open = TRUE;
+	if (alertpanel_is_open)
+		return -1;
+	else
+		alertpanel_is_open = TRUE;
 	
 	alertpanel_create(title, message, button1_label, button2_label,
 			  button3_label, FALSE, widget, alert_type);
- 	alertpanel_show();
- 
- 	debug_print("return value = %d\n", value);
+	alertpanel_show();
+
+	debug_print("return value = %d\n", value);
 	return value;
 }
 
@@ -129,9 +124,6 @@ static void alertpanel_message(const gchar *title, const gchar *message, gint ty
 
 AlertValue alertpanel_message_with_disable(const gchar *title,
 					   const gchar *message,
-					   const gchar	*button1_label,
-					   const gchar	*button2_label,
-					   const gchar	*button3_label,
 					   gint alert_type)
 {
 	if (alertpanel_is_open)
@@ -139,8 +131,8 @@ AlertValue alertpanel_message_with_disable(const gchar *title,
 	else
 		alertpanel_is_open = TRUE;
 
-	alertpanel_create(title, message, button1_label, button2_label, 
-			  button3_label, TRUE, NULL, alert_type);
+	alertpanel_create(title, message, NULL, NULL, NULL, TRUE, NULL,
+			  alert_type);
 	alertpanel_show();
 
 	return value;
@@ -239,8 +231,7 @@ static void alertpanel_create(const gchar *title,
 			      GtkWidget   *custom_widget,
 			      gint	   alert_type)
 {
-	static GdkFont *titlefont;
-	GtkStyle *style;
+	static PangoFontDescription *font_desc;
 	GtkWidget *label;
 	GtkWidget *w_hbox;
 	GtkWidget *hbox;
@@ -251,33 +242,32 @@ static void alertpanel_create(const gchar *title,
 	GtkWidget *button2;
 	GtkWidget *button3;
 	GtkWidget *icon;
-	GdkPixmap *iconpix;
-	GdkBitmap *mask;
 	const gchar *label2;
 	const gchar *label3;
-	void *icon_desc[] = {	stock_dialog_info_xpm,
-				stock_dialog_question_xpm,
-				stock_dialog_warning_xpm,
-				stock_dialog_error_xpm };
+	gchar *title_full = g_strdup_printf("<span weight=\"bold\" "
+				"size=\"larger\">%s</span>",
+				title?title:"");
+
+	gchar *icon_desc[] = {	GTK_STOCK_DIALOG_INFO,
+				GTK_STOCK_DIALOG_QUESTION,
+				GTK_STOCK_DIALOG_WARNING,
+				GTK_STOCK_DIALOG_ERROR };
 
 	debug_print("Creating alert panel dialog...\n");
 
 	dialog = gtk_dialog_new();
 	gtk_window_set_title(GTK_WINDOW(dialog), title);
-	gtk_window_set_policy(GTK_WINDOW(dialog), FALSE, FALSE, FALSE);
-/*	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE); */
-	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
-
-	gtk_container_set_border_width
-		(GTK_CONTAINER(GTK_DIALOG(dialog)->action_area), 5);
-	gtk_window_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
-	gtk_signal_connect(GTK_OBJECT(dialog), "delete_event",
-			   GTK_SIGNAL_FUNC(alertpanel_deleted),
-			   (gpointer)G_ALERTOTHER);
-	gtk_signal_connect(GTK_OBJECT(dialog), "key_press_event",
-			   GTK_SIGNAL_FUNC(alertpanel_close),
-			   (gpointer)G_ALERTOTHER);
-	gtk_widget_realize(dialog);
+	gtk_window_set_resizable(GTK_WINDOW(dialog), FALSE);
+	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+	
+	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
+	gtk_dialog_set_has_separator(GTK_DIALOG(dialog), FALSE);
+	g_signal_connect(G_OBJECT(dialog), "delete_event",
+			 G_CALLBACK(alertpanel_deleted),
+			 (gpointer)G_ALERTOTHER);
+	g_signal_connect(G_OBJECT(dialog), "key_press_event",
+			 G_CALLBACK(alertpanel_close),
+			 (gpointer)G_ALERTOTHER);
 
 	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 14);
 	hbox = gtk_hbox_new (FALSE, 12);
@@ -291,9 +281,9 @@ static void alertpanel_create(const gchar *title,
 	
 	if (alert_type < 0 || alert_type > 3)
 		alert_type = 0;
-	iconpix = gdk_pixmap_create_from_xpm_d(dialog->window, &mask,
-			NULL, icon_desc[alert_type]);
-	icon = gtk_pixmap_new(iconpix, mask);
+	
+	icon = gtk_image_new_from_stock(icon_desc[alert_type],
+        				GTK_ICON_SIZE_DIALOG); 
 	gtk_misc_set_alignment (GTK_MISC (icon), 0.5, 0.0);
 	gtk_box_pack_start (GTK_BOX (hbox), icon, FALSE, FALSE, 0);
 	
@@ -301,18 +291,26 @@ static void alertpanel_create(const gchar *title,
 	gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
 	gtk_widget_show (vbox);
 	
-	label = gtk_label_new(title);
+	label = gtk_label_new(title_full);
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
 	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
+	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
 	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
-	gtk_widget_show(label);
-	
-	style = gtk_style_copy(gtk_widget_get_style(label));
-	if (!titlefont)
-		titlefont = gtkut_font_load(TITLE_FONT);
-	if (titlefont)
-		style->font = titlefont;
-	gtk_widget_set_style(label, style);
+	gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
+	if (!font_desc) {
+		gint size;
+
+		size = pango_font_description_get_size
+			(label->style->font_desc);
+		font_desc = pango_font_description_new();
+		pango_font_description_set_weight
+			(font_desc, PANGO_WEIGHT_BOLD);
+		pango_font_description_set_size
+			(font_desc, size * PANGO_SCALE_LARGE);
+	}
+	if (font_desc)
+		gtk_widget_modify_font(label, font_desc);
+	g_free(title_full);
 	
 	label = gtk_label_new(message);
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
@@ -327,31 +325,37 @@ static void alertpanel_create(const gchar *title,
 	}
 	
 	if (can_disable) {
+		hbox = gtk_hbox_new(FALSE, 0);
+		gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox,
+				   FALSE, FALSE, 0);
+
 		disable_chkbtn = gtk_check_button_new_with_label
 			(_("Show this message next time"));
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(disable_chkbtn),
 					     TRUE);
-		gtk_box_pack_start(GTK_BOX(vbox), disable_chkbtn,
-				   FALSE, FALSE, 0);
-		gtk_signal_connect(GTK_OBJECT(disable_chkbtn), "toggled",
-				 GTK_SIGNAL_FUNC(alertpanel_button_toggled),
+		gtk_box_pack_start(GTK_BOX(hbox), disable_chkbtn,
+				   FALSE, FALSE, 12);
+		g_signal_connect(G_OBJECT(disable_chkbtn), "toggled",
+				 G_CALLBACK(alertpanel_button_toggled),
 				 GUINT_TO_POINTER(G_ALERTDISABLE));
-	} 
+	}
 
 	/* for button(s) */
 	if (!button1_label)
-		button1_label = _("OK");
+		button1_label = GTK_STOCK_OK;
 	label2 = button2_label;
 	label3 = button3_label;
 	if (label2 && *label2 == '+') label2++;
 	if (label3 && *label3 == '+') label3++;
 
-	gtkut_button_set_create(&confirm_area,
-				&button1, button1_label,
-				button2_label ? &button2 : NULL, label2,
-				button3_label ? &button3 : NULL, label3);
+	gtkut_stock_button_set_create(&confirm_area,
+				      &button1, button1_label,
+				      button2_label ? &button2 : NULL, label2,
+				      button3_label ? &button3 : NULL, label3);
 
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), confirm_area, FALSE, FALSE, 12);
+	gtk_box_pack_end(GTK_BOX(GTK_DIALOG(dialog)->action_area),
+			 confirm_area, FALSE, FALSE, 0);
+	gtk_container_set_border_width(GTK_CONTAINER(confirm_area), 5);
 	gtk_widget_grab_default(button1);
 	gtk_widget_grab_focus(button1);
 	if (button2_label && *button2_label == '+') {
@@ -363,16 +367,16 @@ static void alertpanel_create(const gchar *title,
 		gtk_widget_grab_focus(button3);
 	}
 
-	gtk_signal_connect(GTK_OBJECT(button1), "clicked",
-			 GTK_SIGNAL_FUNC(alertpanel_button_clicked),
+	g_signal_connect(G_OBJECT(button1), "clicked",
+			 G_CALLBACK(alertpanel_button_clicked),
 			 GUINT_TO_POINTER(G_ALERTDEFAULT));
 	if (button2_label)
-		gtk_signal_connect(GTK_OBJECT(button2), "clicked",
-				 GTK_SIGNAL_FUNC(alertpanel_button_clicked),
+		g_signal_connect(G_OBJECT(button2), "clicked",
+				 G_CALLBACK(alertpanel_button_clicked),
 				 GUINT_TO_POINTER(G_ALERTALTERNATE));
 	if (button3_label)
-		gtk_signal_connect(GTK_OBJECT(button3), "clicked",
-				 GTK_SIGNAL_FUNC(alertpanel_button_clicked),
+		g_signal_connect(G_OBJECT(button3), "clicked",
+				 G_CALLBACK(alertpanel_button_clicked),
 				 GUINT_TO_POINTER(G_ALERTOTHER));
 
 	gtk_widget_show_all(dialog);
@@ -399,12 +403,13 @@ static gint alertpanel_deleted(GtkWidget *widget, GdkEventAny *event,
 	return TRUE;
 }
 
-static void alertpanel_close(GtkWidget *widget, GdkEventAny *event,
-			     gpointer data)
+static gboolean alertpanel_close(GtkWidget *widget, GdkEventAny *event,
+				 gpointer data)
 {
 	if (event->type == GDK_KEY_PRESS)
 		if (((GdkEventKey *)event)->keyval != GDK_Escape)
-			return;
+			return FALSE;
 
 	value = (value & ~G_ALERT_VALUE_MASK) | (AlertValue)data;
+	return FALSE;
 }
