@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2001 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2005 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,7 +21,6 @@
 #define __ALERTPANEL_H__
 
 #include <glib.h>
-#include <gtk/gtkwindow.h>
 
 typedef enum
 {
@@ -65,9 +64,6 @@ AlertValue alertpanel_with_type	(const gchar *title,
 
 AlertValue alertpanel_message_with_disable	(const gchar	*title,
 						 const gchar	*message,
-						 const gchar	*button1_label,
-						 const gchar	*button2_label,
-						 const gchar	*button3_label,
 						 gint		 alert_type);
 
 void alertpanel_notice	(const gchar	*format,
