@@ -21,7 +21,10 @@
 #  include "config.h"
 #endif
 
+#include "defs.h"
+
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <string.h>
 #include <ctype.h>
 #include <stdlib.h>
@@ -31,11 +34,8 @@
 #  include <locale.h>
 #endif
 
-#if HAVE_ICONV
-#  include <iconv.h>
-#endif
+#include <iconv.h>
 
-#include "intl.h"
 #include "codeconv.h"
 #include "unmime.h"
 #include "base64.h"
@@ -51,7 +51,7 @@ typedef enum
 	JIS_AUXKANJI
 } JISState;
 
-#define SUBST_CHAR	'_'
+#define SUBST_CHAR	0x5f;
 #define ESC		'\033'
 
 #define iseuckanji(c) \
@@ -104,7 +104,31 @@ typedef enum
 		state = JIS_AUXKANJI;	\
 	}
 
-void conv_jistoeuc(gchar *outbuf, gint outlen, const gchar *inbuf)
+static void conv_jistoeuc(gchar *outbuf, gint outlen, const gchar *inbuf);
+static void conv_euctojis(gchar *outbuf, gint outlen, const gchar *inbuf);
+static void conv_sjistoeuc(gchar *outbuf, gint outlen, const gchar *inbuf);
+
+static void conv_jistoutf8(gchar *outbuf, gint outlen, const gchar *inbuf);
+static void conv_sjistoutf8(gchar *outbuf, gint outlen, const gchar *inbuf);
+static void conv_euctoutf8(gchar *outbuf, gint outlen, const gchar *inbuf);
+static void conv_anytoutf8(gchar *outbuf, gint outlen, const gchar *inbuf);
+
+static void conv_utf8toeuc(gchar *outbuf, gint outlen, const gchar *inbuf);
+static void conv_utf8tojis(gchar *outbuf, gint outlen, const gchar *inbuf);
+
+static void conv_unreadable_eucjp(gchar *str);
+static void conv_unreadable_8bit(gchar *str);
+static void conv_unreadable_latin(gchar *str);
+
+static void conv_jistodisp(gchar *outbuf, gint outlen, const gchar *inbuf);
+static void conv_sjistodisp(gchar *outbuf, gint outlen, const gchar *inbuf);
+static void conv_euctodisp(gchar *outbuf, gint outlen, const gchar *inbuf);
+
+static void conv_anytodisp(gchar *outbuf, gint outlen, const gchar *inbuf);
+static void conv_ustodisp(gchar *outbuf, gint outlen, const gchar *inbuf);
+static void conv_noconv(gchar *outbuf, gint outlen, const gchar *inbuf);
+
+static void conv_jistoeuc(gchar *outbuf, gint outlen, const gchar *inbuf)
 {
 	const guchar *in = inbuf;
 	guchar *out = outbuf;
@@ -239,7 +263,7 @@ static gint conv_jis_hantozen(guchar *outbuf, guchar jis_code, guchar sound_sym)
 	return 1;
 }
 
-void conv_euctojis(gchar *outbuf, gint outlen, const gchar *inbuf)
+static void conv_euctojis(gchar *outbuf, gint outlen, const gchar *inbuf)
 {
 	const guchar *in = inbuf;
 	guchar *out = outbuf;
@@ -327,7 +351,7 @@ void conv_euctojis(gchar *outbuf, gint outlen, const gchar *inbuf)
 	*out = '\0';
 }
 
-void conv_sjistoeuc(gchar *outbuf, gint outlen, const gchar *inbuf)
+static void conv_sjistoeuc(gchar *outbuf, gint outlen, const gchar *inbuf)
 {
 	const guchar *in = inbuf;
 	guchar *out = outbuf;
@@ -373,14 +397,71 @@ void conv_sjistoeuc(gchar *outbuf, gint outlen, const gchar *inbuf)
 	*out = '\0';
 }
 
-void conv_anytoeuc(gchar *outbuf, gint outlen, const gchar *inbuf)
+static void conv_jistoutf8(gchar *outbuf, gint outlen, const gchar *inbuf)
+{
+	gchar *eucstr;
+
+	Xalloca(eucstr, outlen, return);
+
+	conv_jistoeuc(eucstr, outlen, inbuf);
+	conv_euctoutf8(outbuf, outlen, eucstr);
+}
+
+static void conv_sjistoutf8(gchar *outbuf, gint outlen, const gchar *inbuf)
+{
+	gchar *tmpstr;
+
+	tmpstr = conv_iconv_strdup(inbuf, CS_SHIFT_JIS, CS_UTF_8);
+	if (tmpstr) {
+		strncpy2(outbuf, tmpstr, outlen);
+		g_free(tmpstr);
+	} else
+		strncpy2(outbuf, inbuf, outlen);
+}
+
+static void conv_euctoutf8(gchar *outbuf, gint outlen, const gchar *inbuf)
+{
+	static iconv_t cd = (iconv_t)-1;
+	static gboolean iconv_ok = TRUE;
+	gchar *tmpstr;
+
+	if (cd == (iconv_t)-1) {
+		if (!iconv_ok) {
+			strncpy2(outbuf, inbuf, outlen);
+			return;
+		}
+		cd = iconv_open(CS_UTF_8, CS_EUC_JP_MS);
+		if (cd == (iconv_t)-1) {
+			cd = iconv_open(CS_UTF_8, CS_EUC_JP);
+			if (cd == (iconv_t)-1) {
+				g_warning("conv_euctoutf8(): %s\n",
+					  g_strerror(errno));
+				iconv_ok = FALSE;
+				strncpy2(outbuf, inbuf, outlen);
+				return;
+			}
+		}
+	}
+
+	tmpstr = conv_iconv_strdup_with_cd(inbuf, cd);
+	if (tmpstr) {
+		strncpy2(outbuf, tmpstr, outlen);
+		g_free(tmpstr);
+	} else
+		strncpy2(outbuf, inbuf, outlen);
+}
+
+static void conv_anytoutf8(gchar *outbuf, gint outlen, const gchar *inbuf)
 {
 	switch (conv_guess_ja_encoding(inbuf)) {
 	case C_ISO_2022_JP:
-		conv_jistoeuc(outbuf, outlen, inbuf);
+		conv_jistoutf8(outbuf, outlen, inbuf);
 		break;
 	case C_SHIFT_JIS:
-		conv_sjistoeuc(outbuf, outlen, inbuf);
+		conv_sjistoutf8(outbuf, outlen, inbuf);
+		break;
+	case C_EUC_JP:
+		conv_euctoutf8(outbuf, outlen, inbuf);
 		break;
 	default:
 		strncpy2(outbuf, inbuf, outlen);
@@ -388,16 +469,46 @@ void conv_anytoeuc(gchar *outbuf, gint outlen, const gchar *inbuf)
 	}
 }
 
-void conv_anytojis(gchar *outbuf, gint outlen, const gchar *inbuf)
+static void conv_utf8toeuc(gchar *outbuf, gint outlen, const gchar *inbuf)
 {
-	switch (conv_guess_ja_encoding(inbuf)) {
-	case C_EUC_JP:
-		conv_euctojis(outbuf, outlen, inbuf);
-		break;
-	default:
-		strncpy2(outbuf, inbuf, outlen);
-		break;
+	static iconv_t cd = (iconv_t)-1;
+	static gboolean iconv_ok = TRUE;
+	gchar *tmpstr;
+
+	if (cd == (iconv_t)-1) {
+		if (!iconv_ok) {
+			strncpy2(outbuf, inbuf, outlen);
+			return;
+		}
+		cd = iconv_open(CS_EUC_JP_MS, CS_UTF_8);
+		if (cd == (iconv_t)-1) {
+			cd = iconv_open(CS_EUC_JP, CS_UTF_8);
+			if (cd == (iconv_t)-1) {
+				g_warning("conv_utf8toeuc(): %s\n",
+					  g_strerror(errno));
+				iconv_ok = FALSE;
+				strncpy2(outbuf, inbuf, outlen);
+				return;
+			}
+		}
 	}
+
+	tmpstr = conv_iconv_strdup_with_cd(inbuf, cd);
+	if (tmpstr) {
+		strncpy2(outbuf, tmpstr, outlen);
+		g_free(tmpstr);
+	} else
+		strncpy2(outbuf, inbuf, outlen);
+}
+
+static void conv_utf8tojis(gchar *outbuf, gint outlen, const gchar *inbuf)
+{
+	gchar *eucstr;
+
+	Xalloca(eucstr, outlen, return);
+
+	conv_utf8toeuc(eucstr, outlen, inbuf);
+	conv_euctojis(outbuf, outlen, eucstr);
 }
 
 static gchar valid_eucjp_tbl[][96] = {
@@ -482,7 +593,7 @@ static gboolean isprintableeuckanji(guchar c1, guchar c2)
 	return TRUE;
 }
 
-void conv_unreadable_eucjp(gchar *str)
+static void conv_unreadable_eucjp(gchar *str)
 {
 	register guchar *p = str;
 
@@ -525,7 +636,7 @@ void conv_unreadable_eucjp(gchar *str)
 	}
 }
 
-void conv_unreadable_8bit(gchar *str)
+static void conv_unreadable_8bit(gchar *str)
 {
 	register guchar *p = str;
 
@@ -538,7 +649,7 @@ void conv_unreadable_8bit(gchar *str)
 	}
 }
 
-void conv_unreadable_latin(gchar *str)
+static void conv_unreadable_latin(gchar *str)
 {
 	register guchar *p = str;
 
@@ -552,34 +663,6 @@ void conv_unreadable_latin(gchar *str)
 	}
 }
 
-void conv_unreadable_locale(gchar *str)
-{
-	switch (conv_get_current_charset()) {
-	case C_US_ASCII:
-	case C_ISO_8859_1:
-	case C_ISO_8859_2:
-	case C_ISO_8859_3:
-	case C_ISO_8859_4:
-	case C_ISO_8859_5:
-	case C_ISO_8859_6:
-	case C_ISO_8859_7:
-	case C_ISO_8859_8:
-	case C_ISO_8859_9:
-	case C_ISO_8859_10:
-	case C_ISO_8859_11:
-	case C_ISO_8859_13:
-	case C_ISO_8859_14:
-	case C_ISO_8859_15:
-		conv_unreadable_latin(str);
-		break;
-	case C_EUC_JP:
-		conv_unreadable_eucjp(str);
-		break;
-	default:
-		break;
-	}
-}
-
 #define NCV	'\0'
 
 void conv_mb_alnum(gchar *str)
@@ -688,61 +771,68 @@ CharSet conv_guess_ja_encoding(const gchar *str)
 	return guessed;
 }
 
-void conv_jistodisp(gchar *outbuf, gint outlen, const gchar *inbuf)
+static void conv_jistodisp(gchar *outbuf, gint outlen, const gchar *inbuf)
 {
-	conv_jistoeuc(outbuf, outlen, inbuf);
-	conv_unreadable_eucjp(outbuf);
+	conv_jistoutf8(outbuf, outlen, inbuf);
 }
 
-void conv_sjistodisp(gchar *outbuf, gint outlen, const gchar *inbuf)
+static void conv_sjistodisp(gchar *outbuf, gint outlen, const gchar *inbuf)
 {
-	conv_sjistoeuc(outbuf, outlen, inbuf);
-	conv_unreadable_eucjp(outbuf);
+	conv_sjistoutf8(outbuf, outlen, inbuf);
 }
 
-void conv_euctodisp(gchar *outbuf, gint outlen, const gchar *inbuf)
+static void conv_euctodisp(gchar *outbuf, gint outlen, const gchar *inbuf)
 {
-	strncpy2(outbuf, inbuf, outlen);
-	conv_unreadable_eucjp(outbuf);
+	conv_euctoutf8(outbuf, outlen, inbuf);
 }
 
-void conv_anytodisp(gchar *outbuf, gint outlen, const gchar *inbuf)
+void conv_utf8todisp(gchar *outbuf, gint outlen, const gchar *inbuf)
 {
-	conv_anytoeuc(outbuf, outlen, inbuf);
-	conv_unreadable_eucjp(outbuf);
+	if (g_utf8_validate(inbuf, -1, NULL) == TRUE)
+		strncpy2(outbuf, inbuf, outlen);
+	else
+		conv_ustodisp(outbuf, outlen, inbuf);
 }
 
-void conv_ustodisp(gchar *outbuf, gint outlen, const gchar *inbuf)
+static void conv_anytodisp(gchar *outbuf, gint outlen, const gchar *inbuf)
+{
+	conv_anytoutf8(outbuf, outlen, inbuf);
+	if (g_utf8_validate(outbuf, -1, NULL) != TRUE)
+		conv_unreadable_8bit(outbuf);
+}
+
+static void conv_ustodisp(gchar *outbuf, gint outlen, const gchar *inbuf)
 {
 	strncpy2(outbuf, inbuf, outlen);
 	conv_unreadable_8bit(outbuf);
 }
 
-void conv_latintodisp(gchar *outbuf, gint outlen, const gchar *inbuf)
-{
-	strncpy2(outbuf, inbuf, outlen);
-	conv_unreadable_latin(outbuf);
-}
-
 void conv_localetodisp(gchar *outbuf, gint outlen, const gchar *inbuf)
 {
-	strncpy2(outbuf, inbuf, outlen);
-	conv_unreadable_locale(outbuf);
+	gchar *tmpstr;
+
+	tmpstr = conv_iconv_strdup(inbuf, conv_get_locale_charset_str(),
+				   CS_INTERNAL);
+	if (tmpstr) {
+		strncpy2(outbuf, tmpstr, outlen);
+		g_free(tmpstr);
+	} else
+		conv_utf8todisp(outbuf, outlen, inbuf);
 }
 
-void conv_noconv(gchar *outbuf, gint outlen, const gchar *inbuf)
+static void conv_noconv(gchar *outbuf, gint outlen, const gchar *inbuf)
 {
 	strncpy2(outbuf, inbuf, outlen);
 }
 
-CodeConverter *conv_code_converter_new(const gchar *charset)
+CodeConverter *conv_code_converter_new(const gchar *src_charset)
 {
 	CodeConverter *conv;
 
 	conv = g_new0(CodeConverter, 1);
-	conv->code_conv_func = conv_get_code_conv_func(charset, NULL);
-	conv->charset_str = g_strdup(charset);
-	conv->charset = conv_get_charset_from_str(charset);
+	conv->code_conv_func = conv_get_code_conv_func(src_charset, NULL);
+	conv->charset_str = g_strdup(src_charset);
+	conv->charset = conv_get_charset_from_str(src_charset);
 
 	return conv;
 }
@@ -756,7 +846,6 @@ void conv_code_converter_destroy(CodeConverter *conv)
 gint conv_convert(CodeConverter *conv, gchar *outbuf, gint outlen,
 		  const gchar *inbuf)
 {
-#if HAVE_ICONV
 	if (conv->code_conv_func != conv_noconv)
 		conv->code_conv_func(outbuf, outlen, inbuf);
 	else {
@@ -770,9 +859,6 @@ gint conv_convert(CodeConverter *conv, gchar *outbuf, gint outlen,
 			g_free(str);
 		}
 	}
-#else /* !HAVE_ICONV */
-	conv->code_conv_func(outbuf, outlen, inbuf);
-#endif
 
 	return 0;
 }
@@ -794,11 +880,7 @@ gchar *conv_codeset_strdup(const gchar *inbuf,
 		return g_realloc(buf, strlen(buf) + 1);
 	}
 
-#if HAVE_ICONV
 	return conv_iconv_strdup(inbuf, src_code, dest_code);
-#else
-	return g_strdup(inbuf);
-#endif /* HAVE_ICONV */
 }
 
 CodeConvFunc conv_get_code_conv_func(const gchar *src_charset_str,
@@ -809,7 +891,7 @@ CodeConvFunc conv_get_code_conv_func(const gchar *src_charset_str,
 	CharSet dest_charset;
 
 	if (!src_charset_str)
-		src_charset = conv_get_current_charset();
+		src_charset = conv_get_locale_charset();
 	else
 		src_charset = conv_get_charset_from_str(src_charset_str);
 
@@ -825,25 +907,9 @@ CodeConvFunc conv_get_code_conv_func(const gchar *src_charset_str,
 
 	if (dest_charset == C_US_ASCII)
 		return conv_ustodisp;
-	else if (dest_charset == C_UTF_8 ||
-		 (dest_charset == C_AUTO &&
-		  conv_get_current_charset() == C_UTF_8))
-		return conv_noconv;
 
 	switch (src_charset) {
-	case C_ISO_2022_JP:
-	case C_ISO_2022_JP_2:
-	case C_ISO_2022_JP_3:
-		if (dest_charset == C_AUTO &&
-		    conv_get_current_charset() == C_EUC_JP)
-			code_conv = conv_jistodisp;
-		else if (dest_charset == C_EUC_JP)
-			code_conv = conv_jistoeuc;
-		break;
 	case C_US_ASCII:
-		if (dest_charset == C_AUTO)
-			code_conv = conv_ustodisp;
-		break;
 	case C_ISO_8859_1:
 	case C_ISO_8859_2:
 	case C_ISO_8859_3:
@@ -858,26 +924,42 @@ CodeConvFunc conv_get_code_conv_func(const gchar *src_charset_str,
 	case C_ISO_8859_13:
 	case C_ISO_8859_14:
 	case C_ISO_8859_15:
-		if (dest_charset == C_AUTO &&
-		    (conv_get_current_charset() == src_charset ||
-		     MB_CUR_MAX > 1))
-			code_conv = conv_latintodisp;
+		break;
+	case C_ISO_2022_JP:
+	case C_ISO_2022_JP_2:
+	case C_ISO_2022_JP_3:
+		if (dest_charset == C_AUTO)
+			code_conv = conv_jistodisp;
+		else if (dest_charset == C_EUC_JP)
+			code_conv = conv_jistoeuc;
+		else if (dest_charset == C_UTF_8)
+			code_conv = conv_jistoutf8;
 		break;
 	case C_SHIFT_JIS:
-		if (dest_charset == C_AUTO &&
-		    conv_get_current_charset() == C_EUC_JP)
+		if (dest_charset == C_AUTO)
 			code_conv = conv_sjistodisp;
 		else if (dest_charset == C_EUC_JP)
 			code_conv = conv_sjistoeuc;
+		else if (dest_charset == C_UTF_8)
+			code_conv = conv_sjistoutf8;
 		break;
 	case C_EUC_JP:
-		if (dest_charset == C_AUTO &&
-		    conv_get_current_charset() == C_EUC_JP)
+		if (dest_charset == C_AUTO)
 			code_conv = conv_euctodisp;
 		else if (dest_charset == C_ISO_2022_JP   ||
 			 dest_charset == C_ISO_2022_JP_2 ||
 			 dest_charset == C_ISO_2022_JP_3)
 			code_conv = conv_euctojis;
+		else if (dest_charset == C_UTF_8)
+			code_conv = conv_euctoutf8;
+		break;
+	case C_UTF_8:
+		if (dest_charset == C_EUC_JP)
+			code_conv = conv_utf8toeuc;
+		else if (dest_charset == C_ISO_2022_JP   ||
+			 dest_charset == C_ISO_2022_JP_2 ||
+			 dest_charset == C_ISO_2022_JP_3)
+			code_conv = conv_utf8tojis;
 		break;
 	default:
 		break;
@@ -886,11 +968,38 @@ CodeConvFunc conv_get_code_conv_func(const gchar *src_charset_str,
 	return code_conv;
 }
 
-#if HAVE_ICONV
 gchar *conv_iconv_strdup(const gchar *inbuf,
 			 const gchar *src_code, const gchar *dest_code)
 {
 	iconv_t cd;
+	gchar *outbuf;
+
+	if (!src_code)
+		src_code = conv_get_outgoing_charset_str();
+	if (!dest_code)
+		dest_code = CS_INTERNAL;
+
+	/* don't convert if src and dest codeset are identical */
+	if (!strcasecmp(src_code, dest_code))
+		return g_strdup(inbuf);
+
+	/* don't convert if current codeset is US-ASCII */
+	if (!strcasecmp(dest_code, CS_US_ASCII))
+		return g_strdup(inbuf);
+
+	cd = iconv_open(dest_code, src_code);
+	if (cd == (iconv_t)-1)
+		return NULL;
+
+	outbuf = conv_iconv_strdup_with_cd(inbuf, cd);
+
+	iconv_close(cd);
+
+	return outbuf;
+}
+
+gchar *conv_iconv_strdup_with_cd(const gchar *inbuf, iconv_t cd)
+{
 	const gchar *inbuf_p;
 	gchar *outbuf;
 	gchar *outbuf_p;
@@ -901,23 +1010,6 @@ gchar *conv_iconv_strdup(const gchar *inbuf,
 	size_t n_conv;
 	size_t len;
 
-	if (!src_code)
-		src_code = conv_get_outgoing_charset_str();
-	if (!dest_code)
-		dest_code = conv_get_current_charset_str();
-
-	/* don't convert if current codeset is US-ASCII */
-	if (!g_strcasecmp(dest_code, CS_US_ASCII))
-		return g_strdup(inbuf);
-
-	/* don't convert if src and dest codeset are identical */
-	if (!g_strcasecmp(src_code, dest_code))
-		return g_strdup(inbuf);
-
-	cd = iconv_open(dest_code, src_code);
-	if (cd == (iconv_t)-1)
-		return NULL;
-
 	inbuf_p = inbuf;
 	in_size = strlen(inbuf);
 	in_left = in_size;
@@ -938,6 +1030,7 @@ gchar *conv_iconv_strdup(const gchar *inbuf,
 	while ((n_conv = iconv(cd, (ICONV_CONST gchar **)&inbuf_p, &in_left,
 			       &outbuf_p, &out_left)) == (size_t)-1) {
 		if (EILSEQ == errno) {
+			//g_print("iconv(): at %d: %s\n", in_size - in_left, g_strerror(errno));
 			inbuf_p++;
 			in_left--;
 			if (out_left == 0) {
@@ -973,11 +1066,8 @@ gchar *conv_iconv_strdup(const gchar *inbuf,
 	outbuf = g_realloc(outbuf, len + 1);
 	outbuf[len] = '\0';
 
-	iconv_close(cd);
-
 	return outbuf;
 }
-#endif /* HAVE_ICONV */
 
 static const struct {
 	CharSet charset;
@@ -1028,6 +1118,7 @@ static const struct {
 	{C_ISO_2022_JP_3,	CS_ISO_2022_JP_3},
 	{C_EUC_JP,		CS_EUC_JP},
 	{C_EUC_JP,		CS_EUCJP},
+	{C_EUC_JP_MS,		CS_EUC_JP_MS},
 	{C_SHIFT_JIS,		CS_SHIFT_JIS},
 	{C_SHIFT_JIS,		CS_SHIFT__JIS},
 	{C_SHIFT_JIS,		CS_SJIS},
@@ -1284,7 +1375,7 @@ CharSet conv_get_charset_from_str(const gchar *charset)
 	return GPOINTER_TO_UINT(g_hash_table_lookup(table, charset));
 }
 
-CharSet conv_get_current_charset(void)
+CharSet conv_get_locale_charset(void)
 {
 	static CharSet cur_charset = -1;
 	const gchar *cur_locale;
@@ -1315,14 +1406,14 @@ CharSet conv_get_current_charset(void)
 
 		/* "ja_JP.EUC" matches with "ja_JP.eucJP", "ja_JP.EUC" and
 		   "ja_JP". "ja_JP" matches with "ja_JP.xxxx" and "ja" */
-		if (!g_strncasecmp(cur_locale, locale_table[i].locale,
+		if (!g_ascii_strncasecmp(cur_locale, locale_table[i].locale,
 				 strlen(locale_table[i].locale))) {
 			cur_charset = locale_table[i].charset;
 			return cur_charset;
 		} else if ((p = strchr(locale_table[i].locale, '_')) &&
 			 !strchr(p + 1, '.')) {
 			if (strlen(cur_locale) == 2 &&
-			    !g_strncasecmp(cur_locale, locale_table[i].locale, 2)) {
+			    !g_ascii_strncasecmp(cur_locale, locale_table[i].locale, 2)) {
 				cur_charset = locale_table[i].charset;
 				return cur_charset;
 			}
@@ -1333,14 +1424,24 @@ CharSet conv_get_current_charset(void)
 	return cur_charset;
 }
 
-const gchar *conv_get_current_charset_str(void)
+const gchar *conv_get_locale_charset_str(void)
 {
 	static const gchar *codeset = NULL;
 
 	if (!codeset)
-		codeset = conv_get_charset_str(conv_get_current_charset());
+		codeset = conv_get_charset_str(conv_get_locale_charset());
 
-	return codeset ? codeset : CS_US_ASCII;
+	return codeset ? codeset : CS_INTERNAL;
+}
+
+CharSet conv_get_internal_charset(void)
+{
+	return C_INTERNAL;
+}
+
+const gchar *conv_get_internal_charset_str(void)
+{
+	return CS_INTERNAL;
 }
 
 CharSet conv_get_outgoing_charset(void)
@@ -1367,29 +1468,20 @@ CharSet conv_get_outgoing_charset(void)
 	for (i = 0; i < sizeof(locale_table) / sizeof(locale_table[0]); i++) {
 		const gchar *p;
 
-		if (!g_strncasecmp(cur_locale, locale_table[i].locale,
+		if (!g_ascii_strncasecmp(cur_locale, locale_table[i].locale,
 				 strlen(locale_table[i].locale))) {
 			out_charset = locale_table[i].out_charset;
 			break;
 		} else if ((p = strchr(locale_table[i].locale, '_')) &&
 			 !strchr(p + 1, '.')) {
 			if (strlen(cur_locale) == 2 &&
-			    !g_strncasecmp(cur_locale, locale_table[i].locale, 2)) {
+			    !g_ascii_strncasecmp(cur_locale, locale_table[i].locale, 2)) {
 				out_charset = locale_table[i].out_charset;
 				break;
 			}
 		}
 	}
 
-#if !HAVE_ICONV
-	/* encoding conversion without iconv() is only supported
-	   on Japanese locale for now */
-	if (out_charset == C_ISO_2022_JP)
-		return out_charset;
-	else
-		return conv_get_current_charset();
-#endif
-
 	return out_charset;
 }
 
@@ -1409,13 +1501,14 @@ const gchar *conv_get_outgoing_charset_str(void)
 	out_charset = conv_get_outgoing_charset();
 	str = conv_get_charset_str(out_charset);
 
-	return str ? str : CS_US_ASCII;
+	return str ? str : CS_UTF_8;
 }
 
 gboolean conv_is_multibyte_encoding(CharSet encoding)
 {
 	switch (encoding) {
 	case C_EUC_JP:
+	case C_EUC_JP_MS:
 	case C_EUC_KR:
 	case C_EUC_TW:
 	case C_EUC_CN:
@@ -1437,15 +1530,12 @@ gboolean conv_is_multibyte_encoding(CharSet encoding)
 
 const gchar *conv_get_current_locale(void)
 {
-	gchar *cur_locale;
+	const gchar *cur_locale;
 
 	cur_locale = g_getenv("LC_ALL");
-	if (!cur_locale || !strlen(cur_locale)) 
-		cur_locale = g_getenv("LC_CTYPE");
-	if (!cur_locale || !strlen(cur_locale)) 
-		cur_locale = g_getenv("LANG");
-	if (!cur_locale || !strlen(cur_locale)) 
-		cur_locale = setlocale(LC_CTYPE, NULL);
+	if (!cur_locale) cur_locale = g_getenv("LC_CTYPE");
+	if (!cur_locale) cur_locale = g_getenv("LANG");
+	if (!cur_locale) cur_locale = setlocale(LC_CTYPE, NULL);
 
 	debug_print("current locale: %s\n",
 		    cur_locale ? cur_locale : "(none)");
@@ -1453,23 +1543,33 @@ const gchar *conv_get_current_locale(void)
 	return cur_locale;
 }
 
-void conv_unmime_header(gchar *outbuf, gint outlen, const gchar *str,
-			const gchar *charset)
+gchar *conv_unmime_header(const gchar *str, const gchar *default_encoding)
 {
-	CharSet cur_charset;
+	gchar buf[BUFFSIZE];
 
-	cur_charset = conv_get_current_charset();
+	if (is_ascii_str(str))
+	        return unmime_header(str);
 
-	if (cur_charset == C_EUC_JP) {
-		gchar *buf;
-		gint buflen;
+	if (default_encoding) {
+		gchar *utf8_buf;
 
-		buflen = strlen(str) * 2 + 1;
-		Xalloca(buf, buflen, return);
-		conv_anytodisp(buf, buflen, str);
-		unmime_header(outbuf, outlen, buf);
-	} else
-		unmime_header(outbuf, outlen, str);
+		utf8_buf = conv_codeset_strdup
+			(str, default_encoding, CS_INTERNAL);
+		if (utf8_buf) {
+			gchar *decoded_str;
+
+			decoded_str = unmime_header(utf8_buf);
+			g_free(utf8_buf);
+			return decoded_str;
+		}
+	}
+
+	if (conv_get_locale_charset() == C_EUC_JP)
+		conv_anytodisp(buf, sizeof(buf), str);
+	else
+		conv_localetodisp(buf, sizeof(buf), str);
+
+	return unmime_header(buf);
 }
 
 #define MAX_LINELEN		76
@@ -1511,6 +1611,8 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 	guchar *destp = dest;
 	gboolean use_base64;
 
+	g_return_if_fail(g_utf8_validate(src, -1, NULL) == TRUE);
+
 	if (MB_CUR_MAX > 1) {
 		use_base64 = TRUE;
 		mimesep_enc = "?B?";
@@ -1519,9 +1621,7 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 		mimesep_enc = "?Q?";
 	}
 
-	cur_encoding = conv_get_current_charset_str();
-	if (!strcmp(cur_encoding, CS_US_ASCII))
-		cur_encoding = CS_ISO_8859_1;
+	cur_encoding = CS_INTERNAL;
 	out_encoding = conv_get_outgoing_charset_str();
 	if (!strcmp(out_encoding, CS_US_ASCII))
 		out_encoding = CS_ISO_8859_1;
@@ -1583,14 +1683,7 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 				if (addr_field && (*p == '(' || *p == ')'))
 					break;
 
-				if (MB_CUR_MAX > 1) {
-					mb_len = mblen(p, MB_CUR_MAX);
-					if (mb_len < 0) {
-						g_warning("conv_encode_header(): invalid multibyte character encountered\n");
-						mb_len = 1;
-					}
-				} else
-					mb_len = 1;
+				mb_len = g_utf8_skip[*p];
 
 				Xstrndup_a(part_str, srcp, cur_len + mb_len, );
 				out_str = conv_codeset_strdup
@@ -1669,3 +1762,40 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 }
 
 #undef LBREAK_IF_REQUIRED
+gchar *conv_filename_from_utf8(const gchar *utf8_file)
+{
+	gchar *fs_file;
+	GError *error = NULL;
+
+	fs_file = g_filename_from_utf8(utf8_file, -1, NULL, NULL, &error);
+	if (error) {
+		g_warning("failed to convert encoding of file name: %s\n",
+			  error->message);
+		g_error_free(error);
+	}
+	if (!fs_file)
+		fs_file = g_strdup(utf8_file);
+
+	return fs_file;
+}
+
+gchar *conv_filename_to_utf8(const gchar *fs_file)
+{
+	gchar *utf8_file = NULL;
+	GError *error = NULL;
+
+	utf8_file = g_filename_to_utf8(fs_file, -1, NULL, NULL, &error);
+	if (error) {
+		g_warning("failed to convert encoding of file name: %s\n",
+			  error->message);
+		g_error_free(error);
+	}
+
+	if (!utf8_file || !g_utf8_validate(utf8_file, -1, NULL)) {
+		g_free(utf8_file);
+		utf8_file = g_strdup(fs_file);
+		conv_unreadable_8bit(utf8_file);
+	}
+
+	return utf8_file;
+}
