@@ -22,10 +22,10 @@
 #endif
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <stdio.h>
 #include <string.h>
 
-#include "intl.h"
 #include "nntp.h"
 #include "socket.h"
 #include "utils.h"
@@ -95,9 +95,7 @@ Session *nntp_session_new(const gchar *server, gushort port, gchar *buf,
 	SESSION(session)->destroy		= nntp_session_destroy;
 
 	session->group = NULL;
-	
-	nntp_mode(session, FALSE);
-	
+
 	if (userid && passwd) {
 		gint ok;
 
@@ -364,7 +362,7 @@ static gint nntp_gen_send(SockInfo *sock, const gchar *format, ...)
 	va_end(args);
 
 	if (verbose) {
-		if (!g_strncasecmp(buf, "AUTHINFO PASS", 13))
+		if (!g_ascii_strncasecmp(buf, "AUTHINFO PASS", 13))
 			log_print("NNTP> AUTHINFO PASS ********\n");
 		else
 			log_print("NNTP> %s\n", buf);
