@@ -21,9 +21,9 @@
 
 #include "defs.h"
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gmodule.h>
 
-#include "intl.h"
 #include "utils.h"
 #include "plugin.h"
 #include "prefs.h"
@@ -32,9 +32,9 @@ struct _Plugin
 {
 	gchar	*filename;
 	GModule	*module;
-	const gchar *(*name) (void);
-	const gchar *(*desc) (void);
-	const gchar *(*type) (void);
+	gchar	*(*name) (void);
+	gchar	*(*desc) (void);
+	gchar	*(*type) (void);
 };
 
 /**
@@ -48,15 +48,6 @@ static gint list_find_by_string(gconstpointer data, gconstpointer str)
 	return strcmp((gchar *)data, (gchar *)str) ? TRUE : FALSE;
 }
 
-static gint list_find_by_plugin_filename(const Plugin *plugin, const gchar *filename)
-{
-        g_return_val_if_fail(plugin, 1);
-        g_return_val_if_fail(plugin->filename, 1);
-        g_return_val_if_fail(filename, 1);
-        
-        return strcmp(filename, plugin->filename);
-}
-
 void plugin_save_list(void)
 {
 	gchar *rcpath, *block;
@@ -107,13 +98,6 @@ gint plugin_load(const gchar *filename, gchar **error)
 
 	g_return_val_if_fail(filename != NULL, -1);
 	g_return_val_if_fail(error != NULL, -1);
-
-        /* check duplicate plugin path name */
-        if (g_slist_find_custom(plugins, filename, 
-                                (GCompareFunc)list_find_by_plugin_filename)) {
-                *error = g_strdup(_("Plugin already loaded"));
-                return -1;                
-        }                               
 	
 	plugin = g_new0(Plugin, 1);
 	if (plugin == NULL) {
@@ -138,8 +122,8 @@ gint plugin_load(const gchar *filename, gchar **error)
 		return -1;
 	}
 	
-	if (!strcmp(plugin_type(), "GTK2")) {
-		*error = g_strdup(_("This module is for Sylpheed-Claws GTK2."));
+	if (!strcmp(plugin_type(), "GTK")) {
+		*error = g_strdup(_("This module is for Sylpheed-Claws GTK1."));
 		g_module_close(plugin->module);
 		g_free(plugin);
 		return -1;
