@@ -482,6 +482,15 @@ static gboolean session_read_msg_cb(SockInfo *source, GIOCondition condition,
 		read_len = sock_read(session->sock, session->read_buf,
 				     SESSION_BUFFSIZE - 1);
 
+		if (read_len == -1 && session->state == SESSION_DISCONNECTED) {
+			g_warning ("sock_read: session disconnected\n");
+			if (session->io_tag > 0) {
+				g_source_remove(session->io_tag);
+				session->io_tag = 0;
+			}
+			return FALSE;
+		}
+		
 		if (read_len == 0) {
 			g_warning("sock_read: received EOF\n");
 			session->state = SESSION_EOF;
