@@ -22,10 +22,10 @@
 #endif
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <stdio.h>
 #include <string.h>
 
-#include "intl.h"
 #include "smtp.h"
 #include "md5.h"
 #include "base64.h"
@@ -286,7 +286,7 @@ static gint smtp_ehlo_recv(SMTPSession *session, const gchar *msg)
 		const gchar *p = msg;
 		p += 3;
 		if (*p == '-' || *p == ' ') p++;
-		if (g_strncasecmp(p, "AUTH", 4) == 0 && p[4] != '\0') {
+		if (g_ascii_strncasecmp(p, "AUTH", 4) == 0) {
 			p += 5;
 			if (strcasestr(p, "PLAIN"))
 				session->avail_auth_type |= SMTPAUTH_PLAIN;
@@ -297,7 +297,7 @@ static gint smtp_ehlo_recv(SMTPSession *session, const gchar *msg)
 			if (strcasestr(p, "DIGEST-MD5"))
 				session->avail_auth_type |= SMTPAUTH_DIGEST_MD5;
 		}
-		if (g_strncasecmp(p, "SIZE", 4) == 0) {
+		if (g_ascii_strncasecmp(p, "SIZE", 4) == 0) {
 			p += 5;
 			session->max_message_size = atoi(p);
 			session->esmtp_flags |= ESMTP_SIZE;
@@ -575,12 +575,12 @@ static gint smtp_session_recv_msg(Session *session, const gchar *msg)
 		if (cont == TRUE)
 			break;
 		if (smtp_session->max_message_size > 0
-		&& smtp_session->max_message_size < smtp_session->send_data_len) {
+		&& smtp_session->max_message_size < 
+		   smtp_session->send_data_len) {
 			log_warning(_("Message is too big "
-			      "(Maximum size is %s)\n"), 
+			      "(Maximum size is %s)\n"),
 			      to_human_readable(
 			       (off_t)(smtp_session->max_message_size)));
-
 			smtp_session->state = SMTP_ERROR;
 			smtp_session->error_val = SM_ERROR;
 			return -1;
