@@ -26,11 +26,9 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <sys/socket.h>
-#include <sys/stat.h>
 #include <sys/un.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
-#include <resolv.h>
 #include <netdb.h>
 #include <unistd.h>
 #include <stdio.h>
@@ -63,6 +61,7 @@ typedef gint (*SockAddrFunc)	(GList		*addr_list,
 typedef struct _SockConnectData	SockConnectData;
 typedef struct _SockLookupData	SockLookupData;
 typedef struct _SockAddrData	SockAddrData;
+typedef struct _SockSource	SockSource;
 
 struct _SockConnectData {
 	gint id;
@@ -94,19 +93,20 @@ struct _SockAddrData {
 	struct sockaddr *addr;
 };
 
+struct _SockSource {
+	GSource parent;
+	SockInfo *sock;
+};
+
 static guint io_timeout = 60;
 
 static GList *sock_connect_data_list = NULL;
 
-static gboolean sock_prepare		(gpointer	 source_data,
-					 GTimeVal	*current_time,
-					 gint		*timeout,
-					 gpointer	 data);
-static gboolean sock_check		(gpointer	 source_data,
-					 GTimeVal	*current_time,
-					 gpointer	 user_data);
-static gboolean sock_dispatch		(gpointer	 source_data,
-					 GTimeVal	*current_time,
+static gboolean sock_prepare		(GSource	*source,
+					 gint		*timeout);
+static gboolean sock_check		(GSource	*source);
+static gboolean sock_dispatch		(GSource	*source,
+					 GSourceFunc	 callback,
 					 gpointer	 user_data);
 
 GSourceFuncs sock_watch_funcs = {
@@ -161,25 +161,6 @@ gint sock_set_io_timeout(guint sec)
 	return 0;
 }
 
-void refresh_resolvers(void)
-{
-	static time_t resolv_conf_changed = (time_t)NULL;
-	struct stat s;
-
-	/* This makes the glibc re-read resolv.conf, if it changed
-	 * since our startup. Maybe that should be #ifdef'ed, I don't
-	 * know if it'd work on BSDs.
-	 * Why doesn't the glibc do it by itself?
-	 */
-	if (stat("/etc/resolv.conf", &s) == 0) {
-		if (s.st_mtime > resolv_conf_changed) {
-			resolv_conf_changed = s.st_mtime;
-			res_init();
-		}
-	} /* else
-		we'll have bigger problems. */
-}
-
 gint fd_connect_unix(const gchar *path)
 {
 	gint sock;
@@ -290,21 +271,22 @@ gboolean sock_is_nonblocking_mode(SockInfo *sock)
 }
 
 
-static gboolean sock_prepare(gpointer source_data, GTimeVal *current_time,
-			     gint *timeout, gpointer data)
+static gboolean sock_prepare(GSource *source, gint *timeout)
 {
 	*timeout = 1;
 	return FALSE;
 }
 
-static gboolean sock_check(gpointer source_data, GTimeVal *current_time,
-			   gpointer user_data)
+static gboolean sock_check(GSource *source)
 {
-	SockInfo *sock = (SockInfo *)source_data;
+	SockInfo *sock = ((SockSource *)source)->sock;
 	struct timeval timeout = {0, 0};
 	fd_set fds;
 	GIOCondition condition = sock->condition;
 
+	if (!sock || !sock->sock)
+		return FALSE;
+
 #if USE_OPENSSL
 	if (sock->ssl) {
 		if (condition & G_IO_IN) {
@@ -332,12 +314,15 @@ static gboolean sock_check(gpointer source_data, GTimeVal *current_time,
 	return FD_ISSET(sock->sock, &fds) != 0;
 }
 
-static gboolean sock_dispatch(gpointer source_data, GTimeVal *current_time,
+static gboolean sock_dispatch(GSource *source, GSourceFunc callback,
 			      gpointer user_data)
 {
-	SockInfo *sock = (SockInfo *)source_data;
+	SockInfo *sock = ((SockSource *)source)->sock;
 
-	return sock->callback(sock, sock->condition, user_data);
+	if (!sock || !sock->callback || !sock->data)
+		return FALSE;
+
+	return sock->callback(sock, sock->condition, sock->data);
 }
 
 static gboolean sock_watch_cb(GIOChannel *source, GIOCondition condition,
@@ -345,6 +330,9 @@ static gboolean sock_watch_cb(GIOChannel *source, GIOCondition condition,
 {
 	SockInfo *sock = (SockInfo *)data;
 
+	if (!sock || !sock->callback || !sock->data) {
+		return FALSE;
+	}
 	return sock->callback(sock, condition, sock->data);
 }
 
@@ -357,8 +345,15 @@ guint sock_add_watch(SockInfo *sock, GIOCondition condition, SockFunc func,
 
 #if USE_OPENSSL
 	if (sock->ssl)
-		return g_source_add(G_PRIORITY_DEFAULT, FALSE,
-				    &sock_watch_funcs, sock, data, NULL);
+	{
+		GSource *source = g_source_new(&sock_watch_funcs,
+					       sizeof(SockSource));
+		((SockSource *) source)->sock = sock;
+		g_source_set_priority(source, G_PRIORITY_DEFAULT);
+		g_source_set_can_recurse(source, FALSE);
+		sock->g_source = g_source_attach(source, NULL);
+		return sock->g_source;
+	}
 #endif
 
 	return g_io_add_watch(sock->sock_ch, condition, sock_watch_cb, sock);
@@ -408,7 +403,7 @@ static gint sock_connect_with_timeout(gint sock,
 {
 	gint ret;
 	void (*prev_handler)(gint);
-	
+
 	alarm(0);
 	prev_handler = signal(SIGALRM, timeout_handler);
 	if (sigsetjmp(jmpenv, 1)) {
@@ -431,7 +426,7 @@ struct hostent *my_gethostbyname(const gchar *hostname)
 {
 	struct hostent *hp;
 	void (*prev_handler)(gint);
-	
+
 	alarm(0);
 	prev_handler = signal(SIGALRM, timeout_handler);
 	if (sigsetjmp(jmpenv, 1)) {
@@ -488,8 +483,6 @@ static gint sock_connect_by_hostname(gint sock, const gchar *hostname,
 	ad.sin_family = AF_INET;
 	ad.sin_port = htons(port);
 
-	refresh_resolvers();
-
 	if (!my_inet_aton(hostname, &ad.sin_addr)) {
 		if ((hp = my_gethostbyname(hostname)) == NULL) {
 			fprintf(stderr, "%s: unknown host.\n", hostname);
@@ -517,8 +510,6 @@ static gint sock_connect_by_getaddrinfo(const gchar *hostname, gushort	port)
 	struct addrinfo hints, *res, *ai;
 	gchar port_str[6];
 
-	refresh_resolvers();
-
 	memset(&hints, 0, sizeof(hints));
 	/* hints.ai_flags = AI_CANONNAME; */
 	hints.ai_family = AF_UNSPEC;
@@ -634,7 +625,7 @@ static gboolean sock_connect_async_cb(GIOChannel *source,
 
 	if (conn_data->io_tag == 0 && conn_data->channel == NULL)
 		return FALSE;
-	
+
 	fd = g_io_channel_unix_get_fd(source);
 
 	conn_data->io_tag = 0;
@@ -806,7 +797,7 @@ static gboolean sock_get_address_info_async_cb(GIOChannel *source,
 	SockLookupData *lookup_data = (SockLookupData *)data;
 	GList *addr_list = NULL;
 	SockAddrData *addr_data;
-	guint bytes_read;
+	gsize bytes_read;
 	gint ai_member[4];
 	struct sockaddr *addr;
 
@@ -876,8 +867,6 @@ static SockLookupData *sock_get_address_info_async(const gchar *hostname,
 	SockLookupData *lookup_data = NULL;
 	gint pipe_fds[2];
 	pid_t pid;
-	
-	refresh_resolvers();
 
 	if (pipe(pipe_fds) < 0) {
 		perror("pipe");
@@ -1047,7 +1036,7 @@ gint fd_read(gint fd, gchar *buf, gint len)
 #if USE_OPENSSL
 gint ssl_read(SSL *ssl, gchar *buf, gint len)
 {
-	gint err, ret;
+	gint ret;
 
 	if (SSL_pending(ssl) == 0) {
 		if (fd_check_io(SSL_get_rfd(ssl), G_IO_IN) < 0)
@@ -1056,19 +1045,14 @@ gint ssl_read(SSL *ssl, gchar *buf, gint len)
 
 	ret = SSL_read(ssl, buf, len);
 
-	switch ((err = SSL_get_error(ssl, ret))) {
+	switch (SSL_get_error(ssl, ret)) {
 	case SSL_ERROR_NONE:
 		return ret;
 	case SSL_ERROR_WANT_READ:
 	case SSL_ERROR_WANT_WRITE:
 		errno = EAGAIN;
 		return -1;
-	case SSL_ERROR_ZERO_RETURN:
-		return 0;
 	default:
-		g_warning("SSL_read() returned error %d, ret = %d\n", err, ret);
-		if (ret == 0)
-			return 0;
 		return -1;
 	}
 }
@@ -1347,7 +1331,7 @@ gint sock_puts(SockInfo *sock, const gchar *buf)
 #if USE_OPENSSL
 gint ssl_peek(SSL *ssl, gchar *buf, gint len)
 {
-	gint err, ret;
+	gint ret;
 
 	if (SSL_pending(ssl) == 0) {
 		if (fd_check_io(SSL_get_rfd(ssl), G_IO_IN) < 0)
@@ -1356,22 +1340,14 @@ gint ssl_peek(SSL *ssl, gchar *buf, gint len)
 
 	ret = SSL_peek(ssl, buf, len);
 
-	switch ((err = SSL_get_error(ssl, ret))) {
+	switch (SSL_get_error(ssl, ret)) {
 	case SSL_ERROR_NONE:
 		return ret;
 	case SSL_ERROR_WANT_READ:
 	case SSL_ERROR_WANT_WRITE:
 		errno = EAGAIN;
 		return -1;
-	case SSL_ERROR_ZERO_RETURN:
-		return 0;
-	case SSL_ERROR_SYSCALL:
-		g_warning("SSL_peek() returned syscall error. errno=%d\n", errno);
-		return -1;
 	default:
-		g_warning("SSL_peek() returned error %d, ret = %d\n", err, ret);
-		if (ret == 0)
-			return 0;
 		return -1;
 	}
 }
@@ -1401,6 +1377,9 @@ gint sock_close(SockInfo *sock)
 #if USE_OPENSSL
 	if (sock->ssl)
 		ssl_done_socket(sock);
+	if (sock->g_source != 0)
+		g_source_remove(sock->g_source);
+	sock->g_source = 0;
 #endif
 	ret = fd_close(sock->sock); 
 	g_free(sock->hostname);
