@@ -1,7 +1,7 @@
 /*
  * Sylpheed templates subsystem 
  * Copyright (C) 2001 Alexander Barinov
- * Copyright (C) 2001-2004 Hiroyuki Yamamoto
+ * Copyright (C) 2001 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,14 +21,15 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <stdio.h>
 #include <dirent.h>
 #include <sys/stat.h>
 #include <ctype.h>
 
-#include "intl.h"
 #include "utils.h"
 #include "template.h"
+#include "../codeconv.h"
 
 static GSList *template_list;
 
@@ -53,18 +54,18 @@ static Template *template_load(gchar *filename)
 	tmpl->value = NULL;
 
 	while (fgets(buf, sizeof(buf), fp) != NULL) {
+		gchar *tmp = NULL;
+
 		if (buf[0] == '\n')
 			break;
-		else if (!g_strncasecmp(buf, "Name:", 5))
+		else if (!g_ascii_strncasecmp(buf, "Name:", 5))
 			tmpl->name = g_strdup(g_strstrip(buf + 5));
-		else if (!g_strncasecmp(buf, "Subject:", 8))
-			tmpl->subject = g_strdup(g_strstrip(buf + 8));
-		else if (!g_strncasecmp(buf, "To:", 3))
+		else if (!g_ascii_strncasecmp(buf, "To:", 3))
 			tmpl->to = g_strdup(g_strstrip(buf + 3));
-		else if (!g_strncasecmp(buf, "Cc:", 3))
+		else if (!g_ascii_strncasecmp(buf, "Cc:", 3))
 			tmpl->cc = g_strdup(g_strstrip(buf + 3));
-		else if (!g_strncasecmp(buf, "Bcc:", 4))
-			tmpl->bcc = g_strdup(g_strstrip(buf + 4));						
+		else if (!g_ascii_strncasecmp(buf, "Subject:", 8))
+			tmpl->subject = g_strdup(g_strstrip(buf + 8));
 	}
 
 	if (!tmpl->name) {
@@ -111,7 +112,7 @@ void template_clear_config(GSList *tmpl_list)
 
 GSList *template_read_config(void)
 {
-	gchar *path;
+	const gchar *path;
 	gchar *filename;
 	DIR *dp;
 	struct dirent *de;
@@ -136,7 +137,8 @@ GSList *template_read_config(void)
 	while ((de = readdir(dp)) != NULL) {
 		if (*de->d_name == '.') continue;
 
-		filename = g_strconcat(path, G_DIR_SEPARATOR_S, de->d_name, NULL);
+		filename = g_strconcat(path, G_DIR_SEPARATOR_S,
+				       de->d_name, NULL);
 
 		if (stat(filename, &s) != 0 || !S_ISREG(s.st_mode) ) {
 			debug_print("%s:%d %s is not an ordinary file\n",
@@ -147,6 +149,7 @@ GSList *template_read_config(void)
 		tmpl = template_load(filename);
 		if (tmpl)
 			tmpl_list = g_slist_append(tmpl_list, tmpl);
+
 		g_free(filename);
 	}
 
@@ -157,7 +160,7 @@ GSList *template_read_config(void)
 
 void template_write_config(GSList *tmpl_list)
 {
-	gchar *path;
+	const gchar *path;
 	GSList *cur;
 	Template *tmpl;
 	FILE *fp;
@@ -200,13 +203,10 @@ void template_write_config(GSList *tmpl_list)
 			fprintf(fp, "To: %s\n", tmpl->to);
 		if (tmpl->cc && *tmpl->cc != '\0')
 			fprintf(fp, "Cc: %s\n", tmpl->cc);
-		if (tmpl->bcc && *tmpl->bcc != '\0')
-			fprintf(fp, "Bcc: %s\n", tmpl->bcc);						
+		if (tmpl->cc && *tmpl->bcc != '\0')
+			fprintf(fp, "Cc: %s\n", tmpl->bcc);
 		fputs("\n", fp);
-		if (tmpl->value && *tmpl->value != '\0')
-			fwrite(tmpl->value, sizeof(gchar) * strlen(tmpl->value), 1, fp);
-		else
-			fwrite("", sizeof(gchar), 1, fp);
+		fwrite(tmpl->value, sizeof(gchar) * strlen(tmpl->value), 1, fp);
 
 		fclose(fp);
 		g_free(filename);
