@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2004 Hiroyuki Yamamoto & The Sylpheed-Claws Team
+ * Copyright (C) 1999-2005 Hiroyuki Yamamoto & The Sylpheed-Claws Team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
@@ -44,9 +45,9 @@
 #include <regex.h>
 #include <sys/utsname.h>
 
-#include "intl.h"
 #include "utils.h"
 #include "socket.h"
+#include "../codeconv.h"
 
 #define BUFFSIZE	8192
 
@@ -113,7 +114,7 @@ void hash_free_value_mem(GHashTable *table)
 
 gint str_case_equal(gconstpointer v, gconstpointer v2)
 {
-	return g_strcasecmp((const gchar *)v, (const gchar *)v2) == 0;
+	return g_ascii_strcasecmp((const gchar *)v, (const gchar *)v2) == 0;
 }
 
 guint str_case_hash(gconstpointer key)
@@ -297,7 +298,7 @@ gchar *strcasestr(const gchar *haystack, const gchar *needle)
 		return NULL;
 
 	while (haystack_len >= needle_len) {
-		if (!g_strncasecmp(haystack, needle, needle_len))
+		if (!g_ascii_strncasecmp(haystack, needle, needle_len))
 			return (gchar *)haystack;
 		else {
 			haystack++;
@@ -543,11 +544,9 @@ gint get_mbs_len(const gchar *s)
 		return -1;
 
 	while (*p != '\0') {
-		mb_len = mblen(p, MB_LEN_MAX);
+		mb_len = g_utf8_skip[*(guchar *)p];
 		if (mb_len == 0)
 			break;
-		else if (mb_len < 0)
-			return -1;
 		else
 			len++;
 
@@ -614,7 +613,7 @@ gint subject_compare_for_sort(const gchar *s1, const gchar *s2)
 	trim_subject_for_sort(str1);
 	trim_subject_for_sort(str2);
 
-	return g_strcasecmp(str1, str2);
+	return g_utf8_collate(str1, str2);
 }
 
 void trim_subject_for_compare(gchar *str)
@@ -1080,15 +1079,11 @@ void subst_chars(gchar *str, gchar *orig, gchar subst)
 
 void subst_for_filename(gchar *str)
 {
-	if (!str)
-		return;
 	subst_chars(str, "\t\r\n\\/*", '_');
 }
 
 void subst_for_shellsafe_filename(gchar *str)
 {
-	if (!str)
-		return;
 	subst_for_filename(str);
 	subst_chars(str, " \"'|&;()<>'!{}[]",'_');
 }
@@ -1425,17 +1420,17 @@ gchar *trim_string(const gchar *str, gint len)
 	if (!str) return NULL;
 	if (strlen(str) <= len)
 		return g_strdup(str);
+	if (g_utf8_validate(str, -1, NULL) == FALSE)
+		return g_strdup(str);
 
 	while (*p != '\0') {
-		mb_len = mblen(p, MB_LEN_MAX);
+		mb_len = g_utf8_skip[*(guchar *)p];
 		if (mb_len == 0)
 			break;
-		else if (mb_len < 0)
-			return g_strdup(str);
 		else if (new_len + mb_len > len)
 			break;
-		else
-			new_len += mb_len;
+
+		new_len += mb_len;
 		p += mb_len;
 	}
 
@@ -1446,7 +1441,8 @@ gchar *trim_string(const gchar *str, gint len)
 GList *uri_list_extract_filenames(const gchar *uri_list)
 {
 	GList *result = NULL;
-	const guchar *p, *q;
+	const gchar *p, *q;
+	gchar *escaped_utf8uri;
 	gchar *file;
 
 	p = uri_list;
@@ -1455,17 +1451,36 @@ GList *uri_list_extract_filenames(const gchar *uri_list)
 		if (*p != '#') {
 			while (isspace(*p)) p++;
 			if (!strncmp(p, "file:", 5)) {
-				p += 5;
 				q = p;
+				q += 5;
 				while (*q && *q != '\n' && *q != '\r') q++;
 
 				if (q > p) {
+					gchar *file, *locale_file = NULL;
 					q--;
 					while (q > p && isspace(*q)) q--;
-					file = g_malloc(q - p + 2);
-					strncpy(file, p, q - p + 1);
-					file[q - p + 1] = '\0';
-					result = g_list_append(result,file);
+					Xalloca(escaped_utf8uri, q - p + 2,
+						return result);
+					Xalloca(file, q - p + 2,
+						return result);
+					*file = '\0';
+					strncpy(escaped_utf8uri, p, q - p + 1);
+					escaped_utf8uri[q - p + 1] = '\0';
+					decode_uri(file, escaped_utf8uri);
+#warning FIXME_GTK2 /* should we use g_filename_from_utf8()? */
+                    /*
+		     * g_filename_from_uri() rejects escaped/locale encoded uri
+		     * string which come from Nautilus.
+		     */
+					if (g_utf8_validate(file, -1, NULL))
+						locale_file
+							= conv_codeset_strdup(
+								file + 5,
+								CS_UTF_8,
+								conv_get_locale_charset_str());
+					if (!locale_file)
+						locale_file = g_strdup(file + 5);
+					result = g_list_append(result, locale_file);
 				}
 			}
 		}
@@ -1510,24 +1525,39 @@ static gint axtoi(const gchar *hexstr)
 
 gboolean is_uri_string(const gchar *str)
 {
-	return (g_strncasecmp(str, "http://", 7) == 0 ||
-		g_strncasecmp(str, "https://", 8) == 0 ||
-		g_strncasecmp(str, "ftp://", 6) == 0 ||
-		g_strncasecmp(str, "www.", 4) == 0);
+	return (g_ascii_strncasecmp(str, "http://", 7) == 0 ||
+		g_ascii_strncasecmp(str, "https://", 8) == 0 ||
+		g_ascii_strncasecmp(str, "ftp://", 6) == 0 ||
+		g_ascii_strncasecmp(str, "www.", 4) == 0);
 }
 
 gchar *get_uri_path(const gchar *uri)
 {
-	if (g_strncasecmp(uri, "http://", 7) == 0)
+	if (g_ascii_strncasecmp(uri, "http://", 7) == 0)
 		return (gchar *)(uri + 7);
-	else if (g_strncasecmp(uri, "https://", 8) == 0)
+	else if (g_ascii_strncasecmp(uri, "https://", 8) == 0)
 		return (gchar *)(uri + 8);
-	else if (g_strncasecmp(uri, "ftp://", 6) == 0)
+	else if (g_ascii_strncasecmp(uri, "ftp://", 6) == 0)
 		return (gchar *)(uri + 6);
 	else
 		return (gchar *)uri;
 }
 
+gint get_uri_len(const gchar *str)
+{
+	const gchar *p;
+
+	if (is_uri_string(str)) {
+		for (p = str; *p != '\0'; p++) {
+			if (!g_ascii_isgraph(*p) || strchr("()<>\"", *p))
+				break;
+		}
+		return p - str;
+	}
+
+	return 0;
+}
+
 /* Decodes URL-Encoded strings (i.e. strings in which spaces are replaced by
  * plusses, and escape characters are used)
  */
@@ -1598,15 +1628,15 @@ gint scan_mailto_url(const gchar *mailto, gchar **to, gchar **cc, gchar **bcc,
 
 		if (*value == '\0') continue;
 
-		if (cc && !*cc && !g_strcasecmp(field, "cc")) {
+		if (cc && !*cc && !g_ascii_strcasecmp(field, "cc")) {
 			*cc = g_strdup(value);
-		} else if (bcc && !*bcc && !g_strcasecmp(field, "bcc")) {
+		} else if (bcc && !*bcc && !g_ascii_strcasecmp(field, "bcc")) {
 			*bcc = g_strdup(value);
 		} else if (subject && !*subject &&
-			   !g_strcasecmp(field, "subject")) {
+			   !g_ascii_strcasecmp(field, "subject")) {
 			*subject = g_malloc(strlen(value) + 1);
 			decode_uri(*subject, value);
-		} else if (body && !*body && !g_strcasecmp(field, "body")) {
+		} else if (body && !*body && !g_ascii_strcasecmp(field, "body")) {
 			*body = g_malloc(strlen(value) + 1);
 			decode_uri(*body, value);
 		}
@@ -1621,7 +1651,7 @@ gint scan_mailto_url(const gchar *mailto, gchar **to, gchar **cc, gchar **bcc,
  * but as long as we are not able to do our own extensions to glibc, we do
  * it here.
  */
-gchar *get_home_dir(void)
+const gchar *get_home_dir(void)
 {
 #if HAVE_DOSISH_SYSTEM
     static gchar *home_dir;
@@ -1646,7 +1676,7 @@ gchar *get_home_dir(void)
 #endif
 }
 
-gchar *get_rc_dir(void)
+const gchar *get_rc_dir(void)
 {
 	static gchar *rc_dir = NULL;
 
@@ -1657,7 +1687,7 @@ gchar *get_rc_dir(void)
 	return rc_dir;
 }
 
-gchar *get_news_cache_dir(void)
+const gchar *get_news_cache_dir(void)
 {
 	static gchar *news_cache_dir = NULL;
 
@@ -1668,7 +1698,7 @@ gchar *get_news_cache_dir(void)
 	return news_cache_dir;
 }
 
-gchar *get_imap_cache_dir(void)
+const gchar *get_imap_cache_dir(void)
 {
 	static gchar *imap_cache_dir = NULL;
 
@@ -1679,7 +1709,7 @@ gchar *get_imap_cache_dir(void)
 	return imap_cache_dir;
 }
 
-gchar *get_mbox_cache_dir(void)
+const gchar *get_mbox_cache_dir(void)
 {
 	static gchar *mbox_cache_dir = NULL;
 
@@ -1690,7 +1720,7 @@ gchar *get_mbox_cache_dir(void)
 	return mbox_cache_dir;
 }
 
-gchar *get_mime_tmp_dir(void)
+const gchar *get_mime_tmp_dir(void)
 {
 	static gchar *mime_tmp_dir = NULL;
 
@@ -1701,7 +1731,7 @@ gchar *get_mime_tmp_dir(void)
 	return mime_tmp_dir;
 }
 
-gchar *get_template_dir(void)
+const gchar *get_template_dir(void)
 {
 	static gchar *template_dir = NULL;
 
@@ -1712,7 +1742,7 @@ gchar *get_template_dir(void)
 	return template_dir;
 }
 
-gchar *get_header_cache_dir(void)
+const gchar *get_header_cache_dir(void)
 {
 	static gchar *header_dir = NULL;
 
@@ -1723,7 +1753,7 @@ gchar *get_header_cache_dir(void)
 	return header_dir;
 }
 
-gchar *get_tmp_dir(void)
+const gchar *get_tmp_dir(void)
 {
 	static gchar *tmp_dir = NULL;
 
@@ -1745,7 +1775,7 @@ gchar *get_tmp_file(void)
 	return tmp_file;
 }
 
-gchar *get_domain_name(void)
+const gchar *get_domain_name(void)
 {
 	static gchar *domain_name = NULL;
 
@@ -2801,7 +2831,7 @@ gchar *get_outgoing_rfc2822_str(FILE *fp)
 	/* output header part */
 	while (fgets(buf, sizeof(buf), fp) != NULL) {
 		strretchomp(buf);
-		if (!g_strncasecmp(buf, "Bcc:", 4)) {
+		if (!g_ascii_strncasecmp(buf, "Bcc:", 4)) {
 			gint next;
 
 			for (;;) {
@@ -3056,6 +3086,16 @@ gchar *file_read_stream_to_str(FILE *fp)
 	str = (gchar *)array->data;
 	g_byte_array_free(array, FALSE);
 
+	if (!g_utf8_validate(str, -1, NULL)) {
+		const gchar *src_codeset, *dest_codeset;
+		gchar *tmp = NULL;
+		src_codeset = conv_get_locale_charset_str();
+		dest_codeset = CS_UTF_8;
+		tmp = conv_codeset_strdup(str, src_codeset, dest_codeset);
+		g_free(str);
+		str = tmp;
+	}
+
 	return str;
 }
 
@@ -3163,6 +3203,16 @@ gchar *get_command_output(const gchar *cmdline)
 	ret = str->str;
 	g_string_free(str, FALSE);
 
+	if (!g_utf8_validate(ret, -1, NULL)) {
+		const gchar *src_codeset, *dest_codeset;
+		gchar *tmp = NULL;
+		src_codeset = conv_get_locale_charset_str();
+		dest_codeset = CS_UTF_8;
+		tmp = conv_codeset_strdup(ret, src_codeset, dest_codeset);
+		g_free(ret);
+		ret = tmp;
+	}
+	
 	return ret;
 }
 
@@ -3255,7 +3305,7 @@ time_t remote_tzoffset_sec(const gchar *zone)
 		remoteoffset = 0;
 	} else if (strlen(zone3) == 3) {
 		for (p = ustzstr; *p != '\0'; p += 3) {
-			if (!g_strncasecmp(p, zone3, 3)) {
+			if (!g_ascii_strncasecmp(p, zone3, 3)) {
 				iustz = ((gint)(p - ustzstr) / 3 + 1) / 2 - 8;
 				remoteoffset = iustz * 3600;
 				break;
@@ -3471,7 +3521,7 @@ int subject_get_prefix_length(const gchar *subject)
 		for (n = 0; n < PREFIXES; n++)
 			/* Terminate each prefix regexpression by a
 			 * "\ ?" (zero or ONE space), and OR them */
-			g_string_sprintfa(s, "(%s\\ ?)%s",
+			g_string_append_printf(s, "(%s\\ ?)%s",
 					  prefixes[n],
 					  n < PREFIXES - 1 ? 
 					  "|" : "");
@@ -3517,7 +3567,7 @@ gint g_stricase_equal(gconstpointer gptr1, gconstpointer gptr2)
 	const char *str1 = gptr1;
 	const char *str2 = gptr2;
 
-	return !g_strcasecmp(str1, str2);
+	return !g_utf8_collate(str1, str2);
 }
 
 gint g_int_compare(gconstpointer a, gconstpointer b)
@@ -3546,7 +3596,6 @@ gchar *generate_msgid(gchar *buf, gint len)
 	return buf;
 }
 
-
 /*
    quote_cmd_argument()
    
@@ -3695,3 +3744,193 @@ void get_hex_str(gchar *out, guchar ch)
 	INT_TO_HEX(hex, ch & 0x0f);
 	*out++ = hex;
 }
+
+#undef REF_DEBUG
+#ifndef REF_DEBUG
+#define G_PRINT_REF 1 == 1 ? (void) 0 : (void)
+#else
+#define G_PRINT_REF g_print
+#endif
+
+/*!
+ *\brief	Register ref counted pointer. It is based on GBoxed, so should
+ *		work with anything that uses the GType system. The semantics
+ *		are similar to a C++ auto pointer, with the exception that
+ *		C doesn't have automatic closure (calling destructors) when 
+ *		exiting a block scope.
+ *		Use the \ref G_TYPE_AUTO_POINTER macro instead of calling this
+ *		function directly.
+ *
+ *\return	GType A GType type.
+ */
+GType g_auto_pointer_register(void)
+{
+	static GType auto_pointer_type;
+	if (!auto_pointer_type)
+		auto_pointer_type =
+			g_boxed_type_register_static
+				("G_TYPE_AUTO_POINTER",
+				 (GBoxedCopyFunc) g_auto_pointer_copy,
+				 (GBoxedFreeFunc) g_auto_pointer_free);
+	return auto_pointer_type;						     
+}
+
+/*!
+ *\brief	Structure with g_new() allocated pointer guarded by the
+ *		auto pointer
+ */
+typedef struct AutoPointerRef {
+	void	      (*free) (gpointer);
+	gpointer	pointer;
+	glong		cnt;
+} AutoPointerRef;
+
+/*!
+ *\brief	The auto pointer opaque structure that references the
+ *		pointer guard block.
+ */
+typedef struct AutoPointer {
+	AutoPointerRef *ref;
+	gpointer	ptr; /*!< access to protected pointer */
+} AutoPointer;
+
+/*!
+ *\brief	Creates an auto pointer for a g_new()ed pointer. Example:
+ *
+ *\code	
+ *
+ *		... tell gtk_list_store it should use a G_TYPE_AUTO_POINTER
+ *		... when assigning, copying and freeing storage elements
+ *
+ *		gtk_list_store_new(N_S_COLUMNS, 
+ *				   G_TYPE_AUTO_POINTER,
+ *				   -1);
+ *
+ *
+ *		Template *precious_data = g_new0(Template, 1);
+ *		g_pointer protect = g_auto_pointer_new(precious_data);
+ *
+ *		gtk_list_store_set(container, &iter,
+ *				   S_DATA, protect,
+ *				   -1);
+ *
+ *		... the gtk_list_store has copied the pointer and 
+ *		... incremented its reference count, we should free
+ *		... the auto pointer (in C++ a destructor would do
+ *		... this for us when leaving block scope)
+ * 
+ *		g_auto_pointer_free(protect);
+ *
+ *		... gtk_list_store_set() now manages the data. When
+ *		... *explicitly* requesting a pointer from the list 
+ *		... store, don't forget you get a copy that should be 
+ *		... freed with g_auto_pointer_free() eventually.
+ *
+ *\endcode
+ *
+ *\param	pointer Pointer to be guarded.
+ *
+ *\return	GAuto * Pointer that should be used in containers with
+ *		GType support.
+ */
+GAuto *g_auto_pointer_new(gpointer p)
+{
+	AutoPointerRef *ref;
+	AutoPointer    *ptr;
+	
+	if (p == NULL) 
+		return NULL;
+
+	ref = g_new0(AutoPointerRef, 1);
+	ptr = g_new0(AutoPointer, 1);
+
+	ref->pointer = p;
+	ref->free = g_free;
+	ref->cnt = 1;
+
+	ptr->ref = ref;
+	ptr->ptr = p;
+
+	G_PRINT_REF ("XXXX ALLOC(%lx)\n", p);
+
+	return ptr;
+}
+
+/*!
+ *\brief	Allocate an autopointer using the passed \a free function to
+ *		free the guarded pointer
+ */
+GAuto *g_auto_pointer_new_with_free(gpointer p, GFreeFunc free_)
+{
+	AutoPointer *aptr;
+	
+	if (p == NULL)
+		return NULL;
+
+	aptr = g_auto_pointer_new(p);
+	aptr->ref->free = free_;
+	return aptr; 
+}
+
+gpointer g_auto_pointer_get_ptr(GAuto *auto_ptr)
+{
+	if (auto_ptr == NULL) 
+		return NULL;
+	return ((AutoPointer *) auto_ptr)->ptr; 
+}
+
+/*!
+ *\brief	Copies an auto pointer by. It's mostly not necessary
+ *		to call this function directly, unless you copy/assign
+ *		the guarded pointer.
+ *
+ *\param	auto_ptr Auto pointer returned by previous call to 
+ *		g_auto_pointer_new_XXX()
+ *
+ *\return	gpointer An auto pointer
+ */
+GAuto *g_auto_pointer_copy(GAuto *auto_ptr)
+{
+	AutoPointer	*ptr;
+	AutoPointerRef	*ref;
+	AutoPointer	*newp;
+
+	if (auto_ptr == NULL) 
+		return NULL;
+
+	ptr = auto_ptr;
+	ref = ptr->ref;
+	newp = g_new0(AutoPointer, 1);
+
+	newp->ref = ref;
+	newp->ptr = ref->pointer;
+	++(ref->cnt);
+	
+	G_PRINT_REF ("XXXX COPY(%lx) -- REF (%d)\n", ref->pointer, ref->cnt);
+
+	return newp;
+}
+
+/*!
+ *\brief	Free an auto pointer
+ */
+void g_auto_pointer_free(GAuto *auto_ptr)
+{
+	AutoPointer	*ptr;
+	AutoPointerRef	*ref;
+	
+	if (auto_ptr == NULL)
+		return;
+
+	ptr = auto_ptr;
+	ref = ptr->ref;
+
+	if (--(ref->cnt) == 0) {
+		G_PRINT_REF ("XXXX FREE(%lx) -- REF (%d)\n", ref->pointer, ref->cnt);
+		ref->free(ref->pointer);
+		g_free(ref);
+	} else
+		G_PRINT_REF ("XXXX DEREF(%lx) -- REF (%d)\n", ref->pointer, ref->cnt);
+	g_free(ptr);		
+}
+
