@@ -26,6 +26,7 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -49,7 +50,6 @@
 #include "main.h"
 #endif
 #include "sylpheed.h"
-#include "intl.h"
 #include "crash.h"
 #include "utils.h"
 #include "filesel.h"
@@ -214,10 +214,10 @@ static GtkWidget *crash_dialog_show(const gchar *text, const gchar *debug_output
 	gtk_container_add(GTK_CONTAINER(frame1), scrolledwindow1);
 	gtk_container_set_border_width(GTK_CONTAINER(scrolledwindow1), 3);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow1),
-				       GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
+				       GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
 
 	text1 = gtk_text_new(NULL, NULL);
-	gtk_text_set_editable(GTK_TEXT(text1), FALSE);
+	gtk_text_set_editable((text1), FALSE);
 	gtk_widget_show(text1);
 	gtk_container_add(GTK_CONTAINER(scrolledwindow1), text1);
 	
@@ -230,7 +230,7 @@ static GtkWidget *crash_dialog_show(const gchar *text, const gchar *debug_output
 		get_lib_version(),
 		debug_output);
 
-	gtk_text_insert(GTK_TEXT(text1), NULL, NULL, NULL, crash_report, -1);
+	gtk_text_insert((text1), NULL, NULL, NULL, crash_report, -1);
 
 	hbuttonbox3 = gtk_hbutton_box_new();
 	gtk_widget_show(hbuttonbox3);
@@ -255,16 +255,14 @@ static GtkWidget *crash_dialog_show(const gchar *text, const gchar *debug_output
 	gtk_container_add(GTK_CONTAINER(hbuttonbox4), button5);
 	GTK_WIDGET_SET_FLAGS(button5, GTK_CAN_DEFAULT);
 	
-	gtk_signal_connect(GTK_OBJECT(window1), "delete_event",
-			   GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
-	gtk_signal_connect(GTK_OBJECT(button3),   "clicked",
-			   GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
-	gtk_signal_connect(GTK_OBJECT(button4), "clicked",
-			   GTK_SIGNAL_FUNC(crash_save_crash_log),
-			   crash_report);
-	gtk_signal_connect(GTK_OBJECT(button5), "clicked",
-			   GTK_SIGNAL_FUNC(crash_create_bug_report),
-			   NULL);
+	g_signal_connect(G_OBJECT(window1), "delete_event",
+			 G_CALLBACK(gtk_main_quit), NULL);
+	g_signal_connect(G_OBJECT(button3),   "clicked",
+			 G_CALLBACK(gtk_main_quit), NULL);
+	g_signal_connect(G_OBJECT(button4), "clicked",
+			 G_CALLBACK(crash_save_crash_log), crash_report);
+	g_signal_connect(G_OBJECT(button5), "clicked",
+			 G_CALLBACK(crash_create_bug_report), NULL);
 
 	gtk_widget_show(window1);
 
@@ -299,9 +297,10 @@ static void crash_save_crash_log(GtkButton *button, const gchar *text)
 	timer = time(NULL);
 	lt = localtime(&timer);
 	strftime(buf, sizeof buf, "sylpheed-crash-log-%Y-%m-%d-%H-%M-%S.txt", lt);
-	if (NULL != (filename = filesel_select_file(_("Save crash information"), buf))
+	if (NULL != (filename = filesel_select_file_save(_("Save crash information"), buf))
 	&&  *filename)
 		str_write_to_file(text, filename);
+	g_free(filename);	
 }
 
 /*!
