@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 2001-2004 Match Grun
+ * Copyright (C) 2001-2003 Match Grun
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,6 +30,7 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtkwindow.h>
 #include <gtk/gtksignal.h>
@@ -38,7 +39,6 @@
 #include <gtk/gtktable.h>
 #include <gtk/gtkbutton.h>
 
-#include "intl.h"
 #include "addressbook.h"
 #include "prefs_common.h"
 #include "addressitem.h"
@@ -79,7 +79,6 @@ static struct _LDAPEdit {
 	GtkWidget *spinbtn_queryage;
 	GtkWidget *check_dynsearch;
 	GtkWidget *check_matchoption;
-	GtkWidget *check_tls;
 } ldapedit;
 
 /**
@@ -179,7 +178,7 @@ static gboolean edit_ldap_validate( void ) {
 
 	/* Switch to page with error */
 	if( errorFlag ) {
-		gtk_notebook_set_page( GTK_NOTEBOOK(ldapedit.notebook), page );
+		gtk_notebook_set_current_page( GTK_NOTEBOOK(ldapedit.notebook), page );
 	}
 
 	return errorFlag;
@@ -203,11 +202,12 @@ static gint edit_ldap_delete_event( GtkWidget *widget, GdkEventAny *event, gbool
 	return TRUE;
 }
 
-static void edit_ldap_key_pressed( GtkWidget *widget, GdkEventKey *event, gboolean *cancelled ) {
+static gboolean edit_ldap_key_pressed( GtkWidget *widget, GdkEventKey *event, gboolean *cancelled ) {
 	if (event && event->keyval == GDK_Escape) {
 		*cancelled = TRUE;
 		gtk_main_quit();
 	}
+	return FALSE;
 }
 
 static void edit_ldap_server_check( void ) {
@@ -309,18 +309,18 @@ static void addressbook_edit_ldap_dialog_create( gboolean *cancelled ) {
 	GtkWidget *hsbox;
 	GtkWidget *statusbar;
 
-	window = gtk_window_new(GTK_WINDOW_DIALOG);
-	gtk_widget_set_usize(window, 450, -1);
+	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+	gtk_widget_set_size_request(window, 450, -1);
 	gtk_container_set_border_width(GTK_CONTAINER(window), 0);
 	gtk_window_set_title(GTK_WINDOW(window), _("Edit LDAP Server"));
 	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
 	gtk_window_set_modal(GTK_WINDOW(window), TRUE);	
-	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
-			   GTK_SIGNAL_FUNC(edit_ldap_delete_event),
-			   cancelled);
-	gtk_signal_connect(GTK_OBJECT(window), "key_press_event",
-			   GTK_SIGNAL_FUNC(edit_ldap_key_pressed),
-			   cancelled);
+	g_signal_connect(G_OBJECT(window), "delete_event",
+			 G_CALLBACK(edit_ldap_delete_event),
+			 cancelled);
+	g_signal_connect(G_OBJECT(window), "key_press_event",
+			 G_CALLBACK(edit_ldap_key_pressed),
+			 cancelled);
 
 	vbox = gtk_vbox_new( FALSE, 6 );
 	gtk_widget_show( vbox );
@@ -339,15 +339,16 @@ static void addressbook_edit_ldap_dialog_create( gboolean *cancelled ) {
 	gtk_box_pack_start(GTK_BOX(hsbox), statusbar, TRUE, TRUE, BORDER_WIDTH);
 
 	/* Button panel */
-	gtkut_button_set_create(&hbbox, &ok_btn, _("OK"),
-				&cancel_btn, _("Cancel"), NULL, NULL);
+	gtkut_stock_button_set_create(&hbbox, &ok_btn, GTK_STOCK_OK,
+				      &cancel_btn, GTK_STOCK_CANCEL,
+				      NULL, NULL);
 	gtk_box_pack_end(GTK_BOX(vbox), hbbox, FALSE, FALSE, 0);
 	gtk_widget_grab_default(ok_btn);
 
-	gtk_signal_connect(GTK_OBJECT(ok_btn), "clicked",
-			   GTK_SIGNAL_FUNC(edit_ldap_ok), cancelled);
-	gtk_signal_connect(GTK_OBJECT(cancel_btn), "clicked",
-			   GTK_SIGNAL_FUNC(edit_ldap_cancel), cancelled);
+	g_signal_connect(G_OBJECT(ok_btn), "clicked",
+			 G_CALLBACK(edit_ldap_ok), cancelled);
+	g_signal_connect(G_OBJECT(cancel_btn), "clicked",
+			 G_CALLBACK(edit_ldap_cancel), cancelled);
 
 	gtk_widget_show_all(vbox);
 
@@ -373,12 +374,9 @@ static void addressbook_edit_ldap_page_basic( gint pageNum, gchar *pageLbl ) {
 	GtkWidget *entry_baseDN;
 	GtkWidget *check_btn;
 	GtkWidget *lookdn_btn;
-	GtkWidget *check_tls;
 	GtkTooltips *toolTip;
 	gint top;
 
-	check_tls = NULL;	/* Force NULL */
-
 	vbox = gtk_vbox_new( FALSE, 8 );
 	gtk_widget_show( vbox );
 	gtk_container_add( GTK_CONTAINER( ldapedit.notebook ), vbox );
@@ -439,7 +437,7 @@ static void addressbook_edit_ldap_page_basic( gint pageNum, gchar *pageLbl ) {
 	spinbtn_port_adj = gtk_adjustment_new (389, 1, 65535, 100, 1000, 1000);
 	spinbtn_port = gtk_spin_button_new(GTK_ADJUSTMENT (spinbtn_port_adj), 1, 0);
 	gtk_box_pack_start (GTK_BOX (hbox_spin), spinbtn_port, FALSE, FALSE, 0);
-	gtk_widget_set_usize (spinbtn_port, 64, -1);
+	gtk_widget_set_size_request (spinbtn_port, 64, -1);
 	gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbtn_port), TRUE);
 	gtk_table_attach(GTK_TABLE(table), hbox_spin, 1, 2, top, (top + 1),
 		GTK_EXPAND|GTK_SHRINK|GTK_FILL, 0, 0, 0);
@@ -487,24 +485,11 @@ static void addressbook_edit_ldap_page_basic( gint pageNum, gchar *pageLbl ) {
 		"directory names on the server." ),
 		NULL );
 
-#ifdef USE_LDAP_TLS
-	/* Next row */
-	++top;
-	check_tls = gtk_check_button_new_with_label( _("Enable TLS") );
-	gtk_table_attach(GTK_TABLE(table), check_tls, 1, 3, top, (top + 1),
-		GTK_EXPAND|GTK_SHRINK|GTK_FILL, 0, 0, 0);
-
-	toolTip = gtk_tooltips_new();
-	gtk_tooltips_set_tip( toolTip, check_tls, _( 
-		"Connect to the server using TLS encrypted connection." ),
-		NULL );
-#endif
-
 	/* Signal handlers */
-	gtk_signal_connect(GTK_OBJECT(check_btn), "clicked",
-			   GTK_SIGNAL_FUNC(edit_ldap_server_check), NULL);
-	gtk_signal_connect(GTK_OBJECT(lookdn_btn), "clicked",
-			   GTK_SIGNAL_FUNC(edit_ldap_basedn_select), NULL);
+	g_signal_connect(G_OBJECT(check_btn), "clicked",
+			 G_CALLBACK(edit_ldap_server_check), NULL);
+	g_signal_connect(G_OBJECT(lookdn_btn), "clicked",
+			 G_CALLBACK(edit_ldap_basedn_select), NULL);
 
 	/* Done */
 	gtk_widget_show_all(vbox);
@@ -513,7 +498,6 @@ static void addressbook_edit_ldap_page_basic( gint pageNum, gchar *pageLbl ) {
 	ldapedit.entry_server = entry_server;
 	ldapedit.spinbtn_port = spinbtn_port;
 	ldapedit.entry_baseDN = entry_baseDN;
-	ldapedit.check_tls    = check_tls;
 }
 
 static void addressbook_edit_ldap_page_search( gint pageNum, gchar *pageLbl ) {
@@ -583,7 +567,7 @@ static void addressbook_edit_ldap_page_search( gint pageNum, gchar *pageLbl ) {
 		LDAPCTL_DFL_QUERY_AGE, 1, LDAPCTL_MAX_QUERY_AGE, 10, 1000, 1000 );
 	spinbtn_queryage = gtk_spin_button_new(GTK_ADJUSTMENT (spinbtn_queryage_adj), 1, 0);
 	gtk_box_pack_start (GTK_BOX (hbox_spin), spinbtn_queryage, FALSE, FALSE, 0);
-	gtk_widget_set_usize (spinbtn_queryage, 64, -1);
+	gtk_widget_set_size_request (spinbtn_queryage, 64, -1);
 	gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbtn_queryage), TRUE);
 	gtk_table_attach(GTK_TABLE(table), hbox_spin, 1, 2, top, (top + 1),
 		GTK_EXPAND|GTK_SHRINK|GTK_FILL, 0, 0, 0);
@@ -633,13 +617,13 @@ static void addressbook_edit_ldap_page_search( gint pageNum, gchar *pageLbl ) {
 		"search usually takes longer to complete. Note that for " \
 		"performance reasons, address completion uses " \
 		"\"begins-with\" for all searches against other address " \
-		"interfaces."
+		"interfaces." \
 		),
 		NULL );
 
 	/* Signal handlers */
-	gtk_signal_connect(GTK_OBJECT(reset_btn), "clicked",
-			   GTK_SIGNAL_FUNC(edit_ldap_search_reset), NULL);
+	g_signal_connect(G_OBJECT(reset_btn), "clicked",
+			 G_CALLBACK(edit_ldap_search_reset), NULL);
 
 	/* Done */
 	gtk_widget_show_all(vbox);
@@ -707,7 +691,6 @@ static void addressbook_edit_ldap_page_extended( gint pageNum, gchar *pageLbl )
 	entry_bindPW = gtk_entry_new();
 	gtk_table_attach(GTK_TABLE(table), entry_bindPW, 1, 2, top, (top + 1),
 		GTK_EXPAND|GTK_SHRINK|GTK_FILL, 0, 0, 0);
-	gtk_entry_set_visibility(GTK_ENTRY(entry_bindPW), FALSE);
 
 	toolTip = gtk_tooltips_new();
 	gtk_tooltips_set_tip( toolTip, entry_bindPW, _( 
@@ -725,7 +708,7 @@ static void addressbook_edit_ldap_page_extended( gint pageNum, gchar *pageLbl )
 	spinbtn_timeout_adj = gtk_adjustment_new (0, 0, 300, 1, 10, 10);
 	spinbtn_timeout = gtk_spin_button_new(GTK_ADJUSTMENT (spinbtn_timeout_adj), 1, 0);
 	gtk_box_pack_start (GTK_BOX (hbox_spin), spinbtn_timeout, FALSE, FALSE, 0);
-	gtk_widget_set_usize (spinbtn_timeout, 64, -1);
+	gtk_widget_set_size_request (spinbtn_timeout, 64, -1);
 	gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbtn_timeout), TRUE);
 	gtk_table_attach(GTK_TABLE(table), hbox_spin, 1, 2, top, (top + 1),
 		GTK_EXPAND|GTK_SHRINK|GTK_FILL, 0, 0, 0);
@@ -744,7 +727,7 @@ static void addressbook_edit_ldap_page_extended( gint pageNum, gchar *pageLbl )
 	spinbtn_maxentry_adj = gtk_adjustment_new (0, 0, 500, 1, 10, 10);
 	spinbtn_maxentry = gtk_spin_button_new(GTK_ADJUSTMENT (spinbtn_maxentry_adj), 1, 0);
 	gtk_box_pack_start (GTK_BOX (hbox_spin), spinbtn_maxentry, FALSE, FALSE, 0);
-	gtk_widget_set_usize (spinbtn_maxentry, 64, -1);
+	gtk_widget_set_size_request (spinbtn_maxentry, 64, -1);
 	gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbtn_maxentry), TRUE);
 	gtk_table_attach(GTK_TABLE(table), hbox_spin, 1, 2, top, (top + 1),
 		GTK_EXPAND|GTK_SHRINK|GTK_FILL, 0, 0, 0);
@@ -785,7 +768,7 @@ void edit_ldap_set_optmenu( GtkOptionMenu *optmenu, const gint value ) {
 	menu = gtk_option_menu_get_menu(optmenu);
 	for( cur = GTK_MENU_SHELL(menu)->children; cur != NULL; cur = cur->next ) {
 		menuitem = GTK_WIDGET(cur->data);
-		menuVal = GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(menuitem)));
+		menuVal = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(menuitem), "user_data"));
 		if( menuVal == value ) {
 			gtk_option_menu_set_history(optmenu, n);
 			return;
@@ -803,7 +786,7 @@ gint edit_ldap_get_optmenu( GtkOptionMenu *optmenu ) {
 
 	menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(optmenu));
 	menuitem = gtk_menu_get_active(GTK_MENU(menu));
-	return GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(menuitem)));
+	return GPOINTER_TO_INT(g_object_get_data(G_OBJECT(menuitem), "user_data"));
 }
 
 /**
@@ -895,10 +878,6 @@ static void edit_ldap_clear_fields( void ) {
 		GTK_TOGGLE_BUTTON( ldapedit.check_dynsearch), TRUE );
 	gtk_toggle_button_set_active(
 		GTK_TOGGLE_BUTTON( ldapedit.check_matchoption), FALSE );
-#ifdef USE_LDAP_TLS
-	gtk_toggle_button_set_active(
-		GTK_TOGGLE_BUTTON( ldapedit.check_tls), FALSE );
-#endif
 }
 
 /**
@@ -943,16 +922,12 @@ static void edit_ldap_set_fields( LdapServer *server ) {
 		gtk_entry_set_text(GTK_ENTRY(ldapedit.entry_criteria), "" );
 	}
 	gtk_spin_button_set_value(
-		GTK_SPIN_BUTTON(ldapedit.spinbtn_queryage ), ctl->maxQueryAge );
+		GTK_SPIN_BUTTON(ldapedit.spinbtn_queryage), ctl->maxQueryAge );
 	gtk_toggle_button_set_active(
-		GTK_TOGGLE_BUTTON( ldapedit.check_dynsearch ), server->searchFlag );
+		GTK_TOGGLE_BUTTON( ldapedit.check_dynsearch), server->searchFlag );
 	gtk_toggle_button_set_active(
-		GTK_TOGGLE_BUTTON( ldapedit.check_matchoption ),
+		GTK_TOGGLE_BUTTON( ldapedit.check_matchoption),
 		( ctl->matchingOption == LDAPCTL_MATCH_CONTAINS ) );
-#ifdef USE_LDAP_TLS
-	gtk_toggle_button_set_active(
-		GTK_TOGGLE_BUTTON( ldapedit.check_tls ), ctl->enableTLS );
-#endif
 }
 
 /**
@@ -968,7 +943,7 @@ AdapterDSource *addressbook_edit_ldap(
 	static gboolean cancelled;
 	gchar *sName, *sHost, *sBase, *sBind, *sPass, *sCrit;
 	gint iPort, iMaxE, iTime, iAge;
-	gboolean bSrch, bMatch, bTLS;
+	gboolean bSrch, bMatch;
 	AddressDataSource *ds = NULL;
 	LdapServer *server = NULL;
 	LdapControl *ctl = NULL;
@@ -976,7 +951,7 @@ AdapterDSource *addressbook_edit_ldap(
 
 	if (!ldapedit.window)
 		addressbook_edit_ldap_create(&cancelled);
-	gtk_notebook_set_page( GTK_NOTEBOOK(ldapedit.notebook), PAGE_BASIC );
+	gtk_notebook_set_current_page( GTK_NOTEBOOK(ldapedit.notebook), PAGE_BASIC );
 	gtk_widget_grab_focus(ldapedit.ok_btn);
 	gtk_widget_grab_focus(ldapedit.entry_name);
 	gtk_widget_show(ldapedit.window);
@@ -1024,12 +999,6 @@ AdapterDSource *addressbook_edit_ldap(
 			GTK_TOGGLE_BUTTON( ldapedit.check_dynsearch ) );
 	bMatch = gtk_toggle_button_get_active(
 			GTK_TOGGLE_BUTTON( ldapedit.check_matchoption ) );
-#ifdef USE_LDAP_TLS
-	bTLS = gtk_toggle_button_get_active(
-			GTK_TOGGLE_BUTTON( ldapedit.check_tls ) );
-#else
-	bTLS = FALSE;
-#endif
 
 	fin = FALSE;
 	if( *sName == '\0' ) fin = TRUE;
@@ -1060,7 +1029,6 @@ AdapterDSource *addressbook_edit_ldap(
 		ldapctl_set_matching_option(
 			ctl, bMatch ?
 			LDAPCTL_MATCH_CONTAINS : LDAPCTL_MATCH_BEGINWITH );
-		ldapctl_set_tls( ctl, bTLS );
 
 		/* Save attributes */
 		editldap_parse_criteria( sCrit, ctl );
