@@ -28,6 +28,7 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtkwindow.h>
 #include <gtk/gtksignal.h>
@@ -36,7 +37,6 @@
 #include <gtk/gtktable.h>
 #include <gtk/gtkbutton.h>
 
-#include "intl.h"
 #include "gtkutils.h"
 #include "prefs_common.h"
 #include "alertpanel.h"
@@ -106,7 +106,7 @@ static void export_html_message( void ) {
 	gchar *sMsg = NULL;
 	gint pageNum;
 
-	pageNum = gtk_notebook_current_page( GTK_NOTEBOOK(exphtml_dlg.notebook) );
+	pageNum = gtk_notebook_get_current_page( GTK_NOTEBOOK(exphtml_dlg.notebook) );
 	if( pageNum == PAGE_FILE_INFO ) {
 		sMsg = _( "Please specify output directory and file to create." );
 	}
@@ -127,7 +127,7 @@ static void export_html_message( void ) {
 static void export_html_cancel( GtkWidget *widget, gpointer data ) {
 	gint pageNum;
 
-	pageNum = gtk_notebook_current_page( GTK_NOTEBOOK(exphtml_dlg.notebook) );
+	pageNum = gtk_notebook_get_current_page( GTK_NOTEBOOK(exphtml_dlg.notebook) );
 	if( pageNum != PAGE_FINISH ) {
 		exphtml_dlg.cancelled = TRUE;
 	}
@@ -151,10 +151,11 @@ static gint export_html_delete_event( GtkWidget *widget, GdkEventAny *event, gpo
  * \param event  Event object.
  * \param data   User data.
  */
-static void export_html_key_pressed( GtkWidget *widget, GdkEventKey *event, gpointer data ) {
+static gboolean export_html_key_pressed( GtkWidget *widget, GdkEventKey *event, gpointer data ) {
 	if (event && event->keyval == GDK_Escape) {
 		export_html_cancel( widget, data );
 	}
+	return FALSE;
 }
 
 /**
@@ -259,16 +260,16 @@ static void exp_html_finish_show( void ) {
 static void export_html_prev( GtkWidget *widget ) {
 	gint pageNum;
 
-	pageNum = gtk_notebook_current_page( GTK_NOTEBOOK(exphtml_dlg.notebook) );
+	pageNum = gtk_notebook_get_current_page( GTK_NOTEBOOK(exphtml_dlg.notebook) );
 	if( pageNum == PAGE_FORMAT ) {
 		/* Goto file page stuff */
-		gtk_notebook_set_page(
+		gtk_notebook_set_current_page(
 			GTK_NOTEBOOK(exphtml_dlg.notebook), PAGE_FILE_INFO );
 		gtk_widget_set_sensitive( exphtml_dlg.btnPrev, FALSE );
 	}
 	else if( pageNum == PAGE_FINISH ) {
 		/* Goto format page */
-		gtk_notebook_set_page(
+		gtk_notebook_set_current_page(
 			GTK_NOTEBOOK(exphtml_dlg.notebook), PAGE_FORMAT );
 		gtk_widget_set_sensitive( exphtml_dlg.btnNext, TRUE );
 	}
@@ -282,11 +283,11 @@ static void export_html_prev( GtkWidget *widget ) {
 static void export_html_next( GtkWidget *widget ) {
 	gint pageNum;
 
-	pageNum = gtk_notebook_current_page( GTK_NOTEBOOK(exphtml_dlg.notebook) );
+	pageNum = gtk_notebook_get_current_page( GTK_NOTEBOOK(exphtml_dlg.notebook) );
 	if( pageNum == PAGE_FILE_INFO ) {
 		/* Goto format page */
 		if( exp_html_move_file() ) {
-			gtk_notebook_set_page(
+			gtk_notebook_set_current_page(
 				GTK_NOTEBOOK(exphtml_dlg.notebook), PAGE_FORMAT );
 			gtk_widget_set_sensitive( exphtml_dlg.btnPrev, TRUE );
 		}
@@ -295,7 +296,7 @@ static void export_html_next( GtkWidget *widget ) {
 	else if( pageNum == PAGE_FORMAT ) {
 		/* Goto finish page */
 		if( exp_html_move_format() ) {
-			gtk_notebook_set_page(
+			gtk_notebook_set_current_page(
 				GTK_NOTEBOOK(exphtml_dlg.notebook), PAGE_FINISH );
 			exp_html_finish_show();
 			exporthtml_save_settings( _exportCtl_ );
@@ -323,7 +324,7 @@ static void export_html_browse( GtkWidget *widget, gpointer data ) {
  * \param data   User data.
  */
 static void exp_html_file_ok( GtkWidget *widget, gpointer data ) {
-	gchar *sFile;
+	const gchar *sFile;
 	AddressFileSelection *afs;
 	GtkWidget *fileSel;
 
@@ -361,10 +362,12 @@ static void exp_html_file_select_create( AddressFileSelection *afs ) {
 	fileSelector = gtk_file_selection_new( _("Select HTML Output File") );
 	gtk_file_selection_hide_fileop_buttons( GTK_FILE_SELECTION(fileSelector) );
 	gtk_file_selection_complete( GTK_FILE_SELECTION(fileSelector), "*.html" );
-	gtk_signal_connect( GTK_OBJECT (GTK_FILE_SELECTION(fileSelector)->ok_button),
-		"clicked", GTK_SIGNAL_FUNC (exp_html_file_ok), ( gpointer ) afs );
-	gtk_signal_connect( GTK_OBJECT (GTK_FILE_SELECTION(fileSelector)->cancel_button),
-		"clicked", GTK_SIGNAL_FUNC (exp_html_file_cancel), ( gpointer ) afs );
+	g_signal_connect(G_OBJECT(GTK_FILE_SELECTION(fileSelector)->ok_button),
+			 "clicked", 
+			 G_CALLBACK(exp_html_file_ok), (gpointer)afs);
+	g_signal_connect(G_OBJECT(GTK_FILE_SELECTION(fileSelector)->cancel_button),
+			 "clicked", 
+			 G_CALLBACK(exp_html_file_cancel), (gpointer)afs);
 	afs->fileSelector = fileSelector;
 	afs->cancelled = TRUE;
 }
@@ -448,8 +451,8 @@ static void export_html_page_file( gint pageNum, gchar *pageLbl ) {
 	gtk_widget_show_all(vbox);
 
 	/* Button handler */
-	gtk_signal_connect(GTK_OBJECT(btnFile), "clicked",
-			   GTK_SIGNAL_FUNC(exp_html_file_select), NULL);
+	g_signal_connect(G_OBJECT(btnFile), "clicked",
+			 G_CALLBACK(exp_html_file_select), NULL);
 
 	exphtml_dlg.labelBook = labelBook;
 	exphtml_dlg.entryHtml = entryHtml;
@@ -660,8 +663,8 @@ static void export_html_page_finish( gint pageNum, gchar *pageLbl ) {
 	gtk_widget_show_all(vbox);
 
 	/* Button handlers */
-	gtk_signal_connect( GTK_OBJECT(btnBrowse), "clicked",
-		GTK_SIGNAL_FUNC(export_html_browse), NULL );
+	g_signal_connect(G_OBJECT(btnBrowse), "clicked",
+			 G_CALLBACK(export_html_browse), NULL);
 
 	exphtml_dlg.labelOutBook = labelBook;
 	exphtml_dlg.labelOutFile = labelFile;
@@ -682,19 +685,19 @@ static void export_html_dialog_create( void ) {
 	GtkWidget *hsbox;
 	GtkWidget *statusbar;
 
-	window = gtk_window_new(GTK_WINDOW_DIALOG);
+	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_widget_set_usize(window, EXPORTHTML_WIDTH, EXPORTHTML_HEIGHT );
 	gtk_container_set_border_width( GTK_CONTAINER(window), 0 );
 	gtk_window_set_title( GTK_WINDOW(window),
 		_("Export Address Book to HTML File") );
 	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
 	gtk_window_set_modal(GTK_WINDOW(window), TRUE);	
-	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
-			   GTK_SIGNAL_FUNC(export_html_delete_event),
-			   NULL );
-	gtk_signal_connect(GTK_OBJECT(window), "key_press_event",
-			   GTK_SIGNAL_FUNC(export_html_key_pressed),
-			   NULL );
+	g_signal_connect(G_OBJECT(window), "delete_event",
+			 G_CALLBACK(export_html_delete_event),
+			 NULL );
+	g_signal_connect(G_OBJECT(window), "key_press_event",
+			 G_CALLBACK(export_html_key_pressed),
+			 NULL );
 
 	vbox = gtk_vbox_new(FALSE, 4);
 	gtk_widget_show(vbox);
@@ -728,12 +731,12 @@ static void export_html_dialog_create( void ) {
 	gtk_widget_grab_default(btnNext);
 
 	/* Button handlers */
-	gtk_signal_connect(GTK_OBJECT(btnPrev), "clicked",
-			   GTK_SIGNAL_FUNC(export_html_prev), NULL);
-	gtk_signal_connect(GTK_OBJECT(btnNext), "clicked",
-			   GTK_SIGNAL_FUNC(export_html_next), NULL);
-	gtk_signal_connect(GTK_OBJECT(btnCancel), "clicked",
-			   GTK_SIGNAL_FUNC(export_html_cancel), NULL);
+	g_signal_connect(G_OBJECT(btnPrev), "clicked",
+			 G_CALLBACK(export_html_prev), NULL);
+	g_signal_connect(G_OBJECT(btnNext), "clicked",
+			 G_CALLBACK(export_html_next), NULL);
+	g_signal_connect(G_OBJECT(btnCancel), "clicked",
+			 G_CALLBACK(export_html_cancel), NULL);
 
 	gtk_widget_show_all(vbox);
 
@@ -804,7 +807,7 @@ void addressbook_exp_html( AddressCache *cache ) {
 	export_html_fill_fields( _exportCtl_ );
 
 	gtk_widget_grab_default(exphtml_dlg.btnNext);
-	gtk_notebook_set_page( GTK_NOTEBOOK(exphtml_dlg.notebook), PAGE_FILE_INFO );
+	gtk_notebook_set_current_page( GTK_NOTEBOOK(exphtml_dlg.notebook), PAGE_FILE_INFO );
 	gtk_widget_set_sensitive( exphtml_dlg.btnPrev, FALSE );
 	gtk_widget_set_sensitive( exphtml_dlg.btnNext, TRUE );
 
