@@ -27,8 +27,8 @@
 #include <time.h>
 #include <string.h>
 #include <glib.h>
+#include <glib/gi18n.h>
 
-#include "intl.h"
 #include "mgutils.h"
 #include "utils.h"
 #include "exporthtml.h"
@@ -438,7 +438,7 @@ static gint exporthtml_compare_name(
 	if( item2 ) name2 = ADDRITEM_NAME( item2 );
 	if( ! name1 ) return ( name2 != NULL );
 	if( ! name2 ) return -1;
-	return g_strcasecmp( name1, name2 );
+	return g_utf8_collate( name1, name2 );
 }
 
 /*
@@ -454,7 +454,7 @@ static gint exporthtml_compare_email(
 	if( email2 ) name2 = email2->address;
 	if( ! name1 ) return ( name2 != NULL );
 	if( ! name2 ) return -1;
-	return g_strcasecmp( name1, name2 );
+	return g_utf8_collate( name1, name2 );
 }
 
 /*
@@ -470,7 +470,7 @@ static gint exporthtml_compare_attrib(
 	if( attr2 ) name2 = attr2->name;
 	if( ! name1 ) return ( name2 != NULL );
 	if( ! name2 ) return -1;
-	return g_strcasecmp( name1, name2 );
+	return g_utf8_collate( name1, name2 );
 }
 
 /*
@@ -1051,10 +1051,12 @@ static void exporthtml_build_filespec( ExportHtmlCtl *ctl ) {
  */
 void exporthtml_parse_filespec( ExportHtmlCtl *ctl, gchar *fileSpec ) {
 	gchar *t;
+	gchar *base = g_path_get_basename(fileSpec);
 
 	ctl->fileHtml =
-		mgu_replace_string( ctl->fileHtml, g_basename( fileSpec ) );
-	t = g_dirname( fileSpec );
+		mgu_replace_string( ctl->fileHtml, base );
+	g_free(base);
+	t = g_path_get_dirname( fileSpec );
 	ctl->dirOutput = mgu_replace_string( ctl->dirOutput, t );
 	g_free( t );
 	ctl->path = mgu_replace_string( ctl->path, fileSpec );
