@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2003 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2005 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <stdio.h>
 #include <string.h>
 #include <sys/types.h>
@@ -31,7 +32,6 @@
 #include <unistd.h>
 #include <stdlib.h>
 
-#include "intl.h"
 #include "folder.h"
 #include "session.h"
 #include "imap.h"
@@ -413,17 +413,17 @@ void folder_item_set_xml(Folder *folder, FolderItem *item, XMLTag *tag)
 
 		if (!attr || !attr->name || !attr->value) continue;
 		if (!strcmp(attr->name, "type")) {
-			if (!g_strcasecmp(attr->value, "normal"))
+			if (!g_ascii_strcasecmp(attr->value, "normal"))
 				item->stype = F_NORMAL;
-			else if (!g_strcasecmp(attr->value, "inbox"))
+			else if (!g_ascii_strcasecmp(attr->value, "inbox"))
 				item->stype = F_INBOX;
-			else if (!g_strcasecmp(attr->value, "outbox"))
+			else if (!g_ascii_strcasecmp(attr->value, "outbox"))
 				item->stype = F_OUTBOX;
-			else if (!g_strcasecmp(attr->value, "draft"))
+			else if (!g_ascii_strcasecmp(attr->value, "draft"))
 				item->stype = F_DRAFT;
-			else if (!g_strcasecmp(attr->value, "queue"))
+			else if (!g_ascii_strcasecmp(attr->value, "queue"))
 				item->stype = F_QUEUE;
-			else if (!g_strcasecmp(attr->value, "trash"))
+			else if (!g_ascii_strcasecmp(attr->value, "trash"))
 				item->stype = F_TRASH;
 		} else if (!strcmp(attr->name, "name")) {
 			if (item->name != NULL)
@@ -899,7 +899,7 @@ static gboolean folder_get_status_full_all_func(GNode *node, gpointer data)
 
 	if (status->str) {
 		id = folder_item_get_identifier(item);
-		g_string_sprintfa(status->str, "%5d %5d %5d %s\n",
+		g_string_append_printf(status->str, "%5d %5d %5d %s\n",
 				  item->new_msgs, item->unread_msgs,
 				  item->total_msgs, id);
 		g_free(id);
@@ -958,7 +958,7 @@ gchar *folder_get_status(GPtrArray *folders, gboolean full)
 				gchar *id;
 
 				id = folder_item_get_identifier(item);
-				g_string_sprintfa(str, "%5d %5d %5d %s\n",
+				g_string_append_printf(str, "%5d %5d %5d %s\n",
 						  item->new_msgs, item->unread_msgs,
 						  item->total_msgs, id);
 				g_free(id);
@@ -970,9 +970,9 @@ gchar *folder_get_status(GPtrArray *folders, gboolean full)
 	}
 
 	if (full)
-		g_string_sprintfa(str, "%5d %5d %5d\n", new, unread, total);
+		g_string_append_printf(str, "%5d %5d %5d\n", new, unread, total);
 	else
-		g_string_sprintfa(str, "%d %d %d\n", new, unread, total);
+		g_string_append_printf(str, "%d %d %d\n", new, unread, total);
 
 	ret = str->str;
 	g_string_free(str, FALSE);
@@ -1062,9 +1062,14 @@ FolderItem *folder_find_child_item_by_name(FolderItem *item, const gchar *name)
 	FolderItem *child;
 
 	for (node = item->node->children; node != NULL; node = node->next) {
+		gchar *base;
 		child = FOLDER_ITEM(node->data);
-		if (strcmp2(g_basename(child->path), name) == 0)
+		base = g_path_get_basename(child->path);
+		if (strcmp2(base, name) == 0) {
+			g_free(base);
 			return child;
+		}
+		g_free(base);
 	}
 
 	return NULL;
@@ -1077,7 +1082,7 @@ FolderClass *folder_get_class_from_string(const gchar *str)
 	classlist = folder_get_class_list();
 	for (; classlist != NULL; classlist = g_slist_next(classlist)) {
 		FolderClass *class = (FolderClass *) classlist->data;
-		if (g_strcasecmp(class->idstr, str) == 0)
+		if (g_ascii_strcasecmp(class->idstr, str) == 0)
 			return class;
 	}
 
@@ -1096,8 +1101,8 @@ gchar *folder_get_identifier(Folder *folder)
 
 gchar *folder_item_get_identifier(FolderItem *item)
 {
-	gchar *id;
-	gchar *folder_id;
+	gchar *id = NULL;
+	gchar *folder_id = NULL;
 
 	g_return_val_if_fail(item != NULL, NULL);
 	g_return_val_if_fail(item->path != NULL, NULL);
@@ -1170,23 +1175,23 @@ gchar *folder_item_get_name(FolderItem *item)
 
 	switch (item->stype) {
 	case F_INBOX:
-		name = g_strdup(!g_strcasecmp(item->name, INBOX_DIR) ? _("Inbox") :
+		name = g_strdup(!strcmp2(item->name, INBOX_DIR) ? _("Inbox") :
 				item->name);
 		break;
 	case F_OUTBOX:
-		name = g_strdup(!g_strcasecmp(item->name, OUTBOX_DIR) ? _("Sent") :
+		name = g_strdup(!strcmp2(item->name, OUTBOX_DIR) ? _("Sent") :
 				item->name);
 		break;
 	case F_QUEUE:
-		name = g_strdup(!g_strcasecmp(item->name, QUEUE_DIR) ? _("Queue") :
+		name = g_strdup(!strcmp2(item->name, QUEUE_DIR) ? _("Queue") :
 				item->name);
 		break;
 	case F_TRASH:
-		name = g_strdup(!g_strcasecmp(item->name, TRASH_DIR) ? _("Trash") :
+		name = g_strdup(!strcmp2(item->name, TRASH_DIR) ? _("Trash") :
 				item->name);
 		break;
 	case F_DRAFT:
-		name = g_strdup(!g_strcasecmp(item->name, DRAFT_DIR) ? _("Drafts") :
+		name = g_strdup(!strcmp2(item->name, DRAFT_DIR) ? _("Drafts") :
 				item->name);
 		break;
 	default:
@@ -1937,7 +1942,7 @@ void folder_clean_cache_memory(void)
 		while((listitem != NULL) && (memusage > (prefs_common.cache_max_mem_usage * 1024))) {
 			FolderItem *item = (FolderItem *)(listitem->data);
 
-			debug_print("Freeing cache memory for %s\n", item->path ? item->path : item->name);
+			debug_print("Freeing cache memory for %s\n", item->path);
 			memusage -= msgcache_get_memory_usage(item->cache);
 		        folder_item_free_cache(item);
 			listitem = listitem->next;
@@ -2087,9 +2092,7 @@ static void msginfo_set_mime_flags(GNode *node, gpointer data)
 	} else if (mimeinfo->disposition == DISPOSITIONTYPE_UNKNOWN && 
 		 mimeinfo->type != MIMETYPE_TEXT &&
 		 mimeinfo->type != MIMETYPE_MULTIPART) {
-		if (!mimeinfo->subtype 
-		||  strcmp(mimeinfo->subtype, "pgp-signature"))
-			procmsg_msginfo_set_flags(msginfo, 0, MSG_HAS_ATTACHMENT);
+		procmsg_msginfo_set_flags(msginfo, 0, MSG_HAS_ATTACHMENT);
 	}
 
 	/* don't descend below top level message for signed and encrypted info */
@@ -2527,7 +2530,10 @@ gint folder_item_move_to(FolderItem *src, FolderItem *dest, FolderItem **new_ite
 	}
 
 	phys_srcpath = folder_item_get_path(src);
-	phys_dstpath = g_strconcat(folder_item_get_path(dest),G_DIR_SEPARATOR_S,g_basename(phys_srcpath),NULL);
+	phys_dstpath = g_strconcat(folder_item_get_path(dest),
+		       G_DIR_SEPARATOR_S,
+		       g_path_get_basename(phys_srcpath),
+		       NULL);
 
 	if (folder_item_parent(src) == dest || src == dest) {
 		g_free(src_identifier);
@@ -2788,10 +2794,7 @@ gint folder_item_remove_msgs(FolderItem *item, GSList *msglist)
 	folder_item_update_freeze();
 	while (msglist != NULL) {
 		MsgInfo *msginfo = (MsgInfo *)msglist->data;
-		if (MSG_IS_LOCKED(msginfo->flags)) {
-			msglist = msglist->next;
-			continue;
-		}
+
 		ret = folder_item_remove_msg(item, msginfo->msgnum);
 		if (ret != 0) break;
 		msgcache_remove_msg(item->cache, msginfo->msgnum);
