@@ -20,6 +20,7 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtkwidget.h>
 #include <gtk/gtkscrolledwindow.h>
@@ -37,7 +38,6 @@
 #include <string.h>
 #include <stdlib.h>
 
-#include "intl.h"
 #include "main.h"
 #include "mainwindow.h"
 #include "folderview.h"
@@ -151,13 +151,13 @@ static gint folderview_clist_compare	(GtkCList	*clist,
 					 gconstpointer	 ptr2);
 
 /* callback functions */
-static void folderview_button_pressed	(GtkWidget	*ctree,
-					 GdkEventButton	*event,
-					 FolderView	*folderview);
-static void folderview_button_released	(GtkWidget	*ctree,
-					 GdkEventButton	*event,
-					 FolderView	*folderview);
-static void folderview_key_pressed	(GtkWidget	*widget,
+static gboolean folderview_button_pressed	(GtkWidget	*ctree,
+						 GdkEventButton	*event,
+						 FolderView	*folderview);
+static gboolean folderview_button_released	(GtkWidget	*ctree,
+						 GdkEventButton	*event,
+						 FolderView	*folderview);
+static gboolean folderview_key_pressed	(GtkWidget	*widget,
 					 GdkEventKey	*event,
 					 FolderView	*folderview);
 static void folderview_selected		(GtkCTree	*ctree,
@@ -170,8 +170,8 @@ static void folderview_tree_expanded	(GtkCTree	*ctree,
 static void folderview_tree_collapsed	(GtkCTree	*ctree,
 					 GtkCTreeNode	*node,
 					 FolderView	*folderview);
-static void folderview_popup_close      (GtkMenuShell   *menu_shell,
-                                         FolderView     *folderview);
+static void folderview_popup_close	(GtkMenuShell	*menu_shell,
+					 FolderView	*folderview);
 static void folderview_col_resized	(GtkCList	*clist,
 					 gint		 column,
 					 gint		 width,
@@ -236,15 +236,15 @@ GHashTable *folderview_popups;
 
 static GtkItemFactoryEntry folderview_common_popup_entries[] =
 {
-	{N_("/Mark all re_ad"),		NULL, mark_all_read_cb, 0, NULL},
+	{N_("/Mark all _read"),		NULL, mark_all_read_cb, 0, NULL},
 	{N_("/_Search folder..."),	NULL, folderview_search_cb, 0, NULL},
 	{N_("/_Properties..."),		NULL, folderview_property_cb, 0, NULL},
-	{N_("/Process_ing..."),		NULL, folderview_processing_cb, 0, NULL},
+	{N_("/Pr_ocessing..."),		NULL, folderview_processing_cb, 0, NULL},
 };
 
 static GtkItemFactoryEntry folder_view_trash_popup_entries[] = {
 	{N_("/------"),			NULL, NULL, 0, "<Separator>"},
-	{N_("/Empty _trash..."),	NULL, folderview_empty_trash_cb, 0, NULL},
+	{N_("/Empty trash..."),		NULL, folderview_empty_trash_cb, 0, NULL},
 };
 
 
@@ -295,9 +295,9 @@ static GtkItemFactory *create_ifactory(FolderView *folderview, FolderViewPopup *
 			gtk_item_factory_create_item(factory, entries->data, folderview, 1);
 
 	popup = gtk_item_factory_get_widget(factory, fpopup->path);
-        gtk_signal_connect(GTK_OBJECT(popup), "selection_done",
-                           GTK_SIGNAL_FUNC(folderview_popup_close),
-                           folderview);
+        g_signal_connect(G_OBJECT(popup), "selection_done",
+                         G_CALLBACK(folderview_popup_close),
+                         folderview);
 
 	return factory;
 }
@@ -333,7 +333,7 @@ FolderView *folderview_create(void)
 		(GTK_SCROLLED_WINDOW(scrolledwin),
 		 GTK_POLICY_AUTOMATIC,
 		 prefs_common.folderview_vscrollbar_policy);
-	gtk_widget_set_usize(scrolledwin,
+	gtk_widget_set_size_request(scrolledwin,
 			     prefs_common.folderview_width,
 			     prefs_common.folderview_height);
 
@@ -370,50 +370,51 @@ FolderView *folderview_create(void)
 		GTK_WIDGET_UNSET_FLAGS(GTK_CLIST(ctree)->column[i].button,
 				       GTK_CAN_FOCUS);
 
-	gtk_signal_connect(GTK_OBJECT(ctree), "key_press_event",
-			   GTK_SIGNAL_FUNC(folderview_key_pressed),
-			   folderview);
-	gtk_signal_connect(GTK_OBJECT(ctree), "button_press_event",
-			   GTK_SIGNAL_FUNC(folderview_button_pressed),
-			   folderview);
-	gtk_signal_connect(GTK_OBJECT(ctree), "button_release_event",
-			   GTK_SIGNAL_FUNC(folderview_button_released),
-			   folderview);
-	gtk_signal_connect(GTK_OBJECT(ctree), "tree_select_row",
-			   GTK_SIGNAL_FUNC(folderview_selected), folderview);
-	gtk_signal_connect(GTK_OBJECT(ctree), "start_drag",
-			   GTK_SIGNAL_FUNC(folderview_start_drag), folderview);
-	gtk_signal_connect(GTK_OBJECT(ctree), "drag_data_get",
-			   GTK_SIGNAL_FUNC(folderview_drag_data_get),
-			   folderview);
 
-	gtk_signal_connect_after(GTK_OBJECT(ctree), "tree_expand",
-				 GTK_SIGNAL_FUNC(folderview_tree_expanded),
-				 folderview);
-	gtk_signal_connect_after(GTK_OBJECT(ctree), "tree_collapse",
-				 GTK_SIGNAL_FUNC(folderview_tree_collapsed),
-				 folderview);
+	g_signal_connect(G_OBJECT(ctree), "key_press_event",
+			 G_CALLBACK(folderview_key_pressed),
+			 folderview);
+	g_signal_connect(G_OBJECT(ctree), "button_press_event",
+			 G_CALLBACK(folderview_button_pressed),
+			 folderview);
+	g_signal_connect(G_OBJECT(ctree), "button_release_event",
+			 G_CALLBACK(folderview_button_released),
+			 folderview);
+	g_signal_connect(G_OBJECT(ctree), "tree_select_row",
+			 G_CALLBACK(folderview_selected), folderview);
+	g_signal_connect(G_OBJECT(ctree), "start_drag",
+			 G_CALLBACK(folderview_start_drag), folderview);
+	g_signal_connect(G_OBJECT(ctree), "drag_data_get",
+			 G_CALLBACK(folderview_drag_data_get),
+			 folderview);
 
-	gtk_signal_connect(GTK_OBJECT(ctree), "resize_column",
-			   GTK_SIGNAL_FUNC(folderview_col_resized),
-			   folderview);
+	g_signal_connect_after(G_OBJECT(ctree), "tree_expand",
+			       G_CALLBACK(folderview_tree_expanded),
+			       folderview);
+	g_signal_connect_after(G_OBJECT(ctree), "tree_collapse",
+			       G_CALLBACK(folderview_tree_collapsed),
+			       folderview);
+
+	g_signal_connect(G_OBJECT(ctree), "resize_column",
+			 G_CALLBACK(folderview_col_resized),
+			 folderview);
 
         /* drop callback */
 	gtk_drag_dest_set(ctree, GTK_DEST_DEFAULT_ALL & ~GTK_DEST_DEFAULT_HIGHLIGHT,
 			  summary_drag_types, 1,
 			  GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_DEFAULT);
-	gtk_signal_connect(GTK_OBJECT(ctree), "drag_motion",
-			   GTK_SIGNAL_FUNC(folderview_drag_motion_cb),
-			   folderview);
-	gtk_signal_connect(GTK_OBJECT(ctree), "drag_leave",
-			   GTK_SIGNAL_FUNC(folderview_drag_leave_cb),
-			   folderview);
-	gtk_signal_connect(GTK_OBJECT(ctree), "drag_data_received",
-			   GTK_SIGNAL_FUNC(folderview_drag_received_cb),
-			   folderview);
-	gtk_signal_connect(GTK_OBJECT(ctree), "drag_end",
-			   GTK_SIGNAL_FUNC(folderview_drag_end_cb),
-			   folderview);
+	g_signal_connect(G_OBJECT(ctree), "drag_motion",
+			 G_CALLBACK(folderview_drag_motion_cb),
+			 folderview);
+	g_signal_connect(G_OBJECT(ctree), "drag_leave",
+			 G_CALLBACK(folderview_drag_leave_cb),
+			 folderview);
+	g_signal_connect(G_OBJECT(ctree), "drag_data_received",
+			 G_CALLBACK(folderview_drag_received_cb),
+			 folderview);
+	g_signal_connect(G_OBJECT(ctree), "drag_end",
+			 G_CALLBACK(folderview_drag_end_cb),
+			 folderview);
 
 	folderview->scrolledwin  = scrolledwin;
 	folderview->ctree        = ctree;
@@ -433,8 +434,6 @@ FolderView *folderview_create(void)
 
 void folderview_init(FolderView *folderview)
 {
-	static GdkFont *boldfont = NULL;
-	static GdkFont *normalfont = NULL;
 	GtkWidget *ctree = folderview->ctree;
 	GtkWidget *label_new;
 	GtkWidget *label_unread;
@@ -501,11 +500,15 @@ void folderview_init(FolderView *folderview)
 	gtk_clist_set_column_widget(GTK_CLIST(ctree),COL_TOTAL,hbox_total);
 			
 	if (!normal_style) {
+		PangoFontDescription *font_desc;
 		normal_style = gtk_style_copy(gtk_widget_get_style(ctree));
-		if (!normalfont)
-			normalfont = gtkut_font_load(prefs_common.normalfont);
-		if (normalfont)
-			normal_style->font = normalfont;
+		font_desc = pango_font_description_from_string(NORMAL_FONT);
+		if (font_desc) {
+			if (normal_style->font_desc)
+				pango_font_description_free
+					(normal_style->font_desc);
+			normal_style->font_desc = font_desc;
+		}
 		normal_color_style = gtk_style_copy(normal_style);
 		normal_color_style->fg[GTK_STATE_NORMAL] = folderview->color_new;
 
@@ -513,11 +516,17 @@ void folderview_init(FolderView *folderview)
 	}
 
 	if (!bold_style) {
+		PangoFontDescription *font_desc;
 		bold_style = gtk_style_copy(gtk_widget_get_style(ctree));
-		if (!boldfont)
-			boldfont = gtkut_font_load(prefs_common.boldfont);
-		if (boldfont)
-			bold_style->font = boldfont;
+		font_desc = pango_font_description_from_string(BOLD_FONT);
+		if (font_desc) {
+			if (bold_style->font_desc)
+				pango_font_description_free
+					(bold_style->font_desc);
+			bold_style->font_desc = font_desc;
+		}
+		pango_font_description_set_weight
+			(bold_style->font_desc, PANGO_WEIGHT_BOLD);
 		bold_color_style = gtk_style_copy(bold_style);
 		bold_color_style->fg[GTK_STATE_NORMAL] = folderview->color_new;
 
@@ -765,13 +774,13 @@ static GtkWidget *label_window_create(const gchar *str)
 	GtkWidget *window;
 	GtkWidget *label;
 
-	window = gtk_window_new(GTK_WINDOW_DIALOG);
-	gtk_widget_set_usize(window, 380, 60);
+	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+	gtk_widget_set_size_request(window, 380, 60);
 	gtk_container_set_border_width(GTK_CONTAINER(window), 8);
 	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
 	gtk_window_set_title(GTK_WINDOW(window), str);
 	gtk_window_set_modal(GTK_WINDOW(window), TRUE);
-	gtk_window_set_policy(GTK_WINDOW(window), FALSE, FALSE, FALSE);
+	gtk_window_set_resizable(GTK_WINDOW(window), FALSE);
 	manage_window_set_transient(GTK_WINDOW(window));
 
 	label = gtk_label_new(str);
@@ -1429,8 +1438,8 @@ static void folderview_append_folder(FolderView *folderview, Folder *folder)
 
 /* callback functions */
 
-static void folderview_button_pressed(GtkWidget *ctree, GdkEventButton *event,
-				      FolderView *folderview)
+static gboolean folderview_button_pressed(GtkWidget *ctree, GdkEventButton *event,
+					  FolderView *folderview)
 {
 	GtkCList *clist = GTK_CLIST(ctree);
 	gint prev_row = -1, row = -1, column = -1;
@@ -1440,23 +1449,11 @@ static void folderview_button_pressed(GtkWidget *ctree, GdkEventButton *event,
 	GtkItemFactory *fpopup_factory;
 	GtkWidget *popup;
 
-	if (!event) return;
+	if (!event) return FALSE;
 
 	if (event->button == 1) {
 		folderview->open_folder = TRUE;
-
-	        if (event->type == GDK_2BUTTON_PRESS) {
-			if (clist->selection) {
-				GtkCTreeNode *node;
-
-				node = GTK_CTREE_NODE(clist->selection->data);
-				if (node)
-					gtk_ctree_toggle_expansion(
-						GTK_CTREE(ctree),
-						node);
-			}
-		}
-		return;
+		return FALSE;
 	}
 
 	if (event->button == 2 || event->button == 3) {
@@ -1472,7 +1469,7 @@ static void folderview_button_pressed(GtkWidget *ctree, GdkEventButton *event,
 
 		if (!gtk_clist_get_selection_info(clist, event->x, event->y,
 						  &row, &column))
-			return;
+			return FALSE;
 		if (prev_row != row) {
 			gtk_clist_unselect_all(clist);
 			if (event->button == 2)
@@ -1485,11 +1482,11 @@ static void folderview_button_pressed(GtkWidget *ctree, GdkEventButton *event,
 		}
 	}
 
-	if (event->button != 3) return;
+	if (event->button != 3) return FALSE;
 
 	item = gtk_clist_get_row_data(clist, row);
-	g_return_if_fail(item != NULL);
-	g_return_if_fail(item->folder != NULL);
+	g_return_val_if_fail(item != NULL, FALSE);
+	g_return_val_if_fail(item->folder != NULL, FALSE);
 	folder = item->folder;
 
 	fpopup = g_hash_table_lookup(folderview_popups, folder->klass->idstr);
@@ -1527,12 +1524,14 @@ static void folderview_button_pressed(GtkWidget *ctree, GdkEventButton *event,
 	popup = gtk_item_factory_get_widget(fpopup_factory, fpopup->path);
 	gtk_menu_popup(GTK_MENU(popup), NULL, NULL, NULL, NULL,
 		       event->button, event->time);
+
+	return FALSE;
 }
 
-static void folderview_button_released(GtkWidget *ctree, GdkEventButton *event,
-				       FolderView *folderview)
+static gboolean folderview_button_released(GtkWidget *ctree, GdkEventButton *event,
+					   FolderView *folderview)
 {
-	if (!event) return;
+	if (!event) return FALSE;
 
 	if (event->button == 1 && folderview->open_folder == FALSE &&
 	    folderview->opened != NULL) {
@@ -1540,12 +1539,17 @@ static void folderview_button_released(GtkWidget *ctree, GdkEventButton *event,
 					  folderview->opened);
 		gtk_ctree_select(GTK_CTREE(ctree), folderview->opened);
 	}
+
+	return FALSE;
 }
 
-static void folderview_key_pressed(GtkWidget *widget, GdkEventKey *event,
-				   FolderView *folderview)
+static gboolean folderview_key_pressed(GtkWidget *widget, GdkEventKey *event,
+				       FolderView *folderview)
 {
-	if (!event) return;
+	if (!event) return FALSE;
+
+	if (quicksearch_has_focus(folderview->summaryview->quicksearch))
+		return FALSE;
 
 	switch (event->keyval) {
 	case GDK_Return:
@@ -1568,6 +1572,8 @@ static void folderview_key_pressed(GtkWidget *widget, GdkEventKey *event,
 	default:
 		break;
 	}
+
+	return FALSE;
 }
 
 static void folderview_selected(GtkCTree *ctree, GtkCTreeNode *row,
@@ -1803,8 +1809,6 @@ static void folderview_empty_trash_cb(FolderView *folderview, guint action,
 	
 	for (cur = mlist ; cur != NULL ; cur = cur->next) {
 		MsgInfo * msginfo = (MsgInfo *) cur->data;
-		if (MSG_IS_LOCKED(msginfo->flags))
-			continue;
 		/* is it partially received? (partial_recv isn't cached) */
 		if (msginfo->total_size != 0 && 
 		    msginfo->size != (off_t)msginfo->total_size)
@@ -1874,22 +1878,6 @@ void folderview_move_folder(FolderView *folderview, FolderItem *from_folder,
 
 	src_node = gtk_ctree_find_by_row_data(GTK_CTREE(folderview->ctree), NULL, from_folder);
 	from_parent = folder_item_parent(from_folder);
-	
-	if (prefs_common.warn_dnd) {
-		buf = g_strdup_printf(_("Do you really want to move folder `%s' to a "
-					"sub-folder of `%s' ?"), from_folder->name,
-					to_folder->name);
-		status = alertpanel_message_with_disable(_("Move folder"), buf, 
-				_("Yes"), _("No"), NULL, ALERT_QUESTION);
-		g_free(buf);
-
-		if (status != G_ALERTDEFAULT
-		 && status != (G_ALERTDEFAULT | G_ALERTDISABLE))
-			return;
-		if (status & G_ALERTDISABLE)
-			prefs_common.warn_dnd = FALSE;
-	}
-
 	buf = g_strdup_printf(_("Moving %s to %s..."), from_folder->name, to_folder->name);
 	STATUSBAR_PUSH(folderview->mainwin, buf);
 	g_free(buf);
@@ -1947,7 +1935,7 @@ static gint folderview_clist_compare(GtkCList *clist,
 	if (!item2->name)
 		return -1;
 
-	return g_strcasecmp(item1->name, item2->name);
+	return g_utf8_collate(item1->name, item2->name);
 }
 
 static void folderview_processing_cb(FolderView *folderview, guint action,
@@ -2109,11 +2097,11 @@ static gboolean folderview_drag_motion_cb(GtkWidget      *widget,
 	FolderItem *item, *src_item = NULL;
 	GtkCTreeNode *node = NULL;
 	gboolean acceptable = FALSE;
-	gint height = folderview->ctree->allocation.height;
-	gint total_height = folderview->ctree->requisition.height;
 	GtkAdjustment *pos = gtk_scrolled_window_get_vadjustment(
 				GTK_SCROLLED_WINDOW(folderview->scrolledwin));
-	gfloat vpos = pos->value;
+	int height = (int)pos->page_size;
+	int total_height = (int)pos->upper;
+	int vpos = (int) pos->value;
 
 	if (gtk_clist_get_selection_info
 		(GTK_CLIST(widget), x - 24, y - 24, &row, &column)) {
@@ -2122,7 +2110,7 @@ static gboolean folderview_drag_motion_cb(GtkWidget      *widget,
 		if (y > height - 24 && height + vpos < total_height)
 			gtk_adjustment_set_value(pos, (vpos+5 > height ? height : vpos+5));
 
-		if (y < 24 && y > 0)
+		if (y < 48 && y > 0)
 			gtk_adjustment_set_value(pos, (vpos-5 < 0 ? 0 : vpos-5));
 
 		node = gtk_ctree_node_nth(GTK_CTREE(widget), row);
@@ -2152,13 +2140,13 @@ static gboolean folderview_drag_motion_cb(GtkWidget      *widget,
 		drag_state_start(folderview, node, item);
 	
 	if (acceptable) {
-		gtk_signal_handler_block_by_func
-			(GTK_OBJECT(widget),
-			 GTK_SIGNAL_FUNC(folderview_selected), folderview);
+		g_signal_handlers_block_by_func
+			(G_OBJECT(widget),
+			 G_CALLBACK(folderview_selected), folderview);
 		gtk_ctree_select(GTK_CTREE(widget), node);
-		gtk_signal_handler_unblock_by_func
-			(GTK_OBJECT(widget),
-			 GTK_SIGNAL_FUNC(folderview_selected), folderview);
+		g_signal_handlers_unblock_by_func
+			(G_OBJECT(widget),
+			 G_CALLBACK(folderview_selected), folderview);
 		gdk_drag_status(context, 
 					(context->actions == GDK_ACTION_COPY ?
 					GDK_ACTION_COPY : GDK_ACTION_MOVE) , time);
