@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2004 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2005 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,6 +32,7 @@
 #include <gtk/gtkeditable.h>
 #include <gtk/gtkctree.h>
 #include <gtk/gtkcombo.h>
+#include <gtk/gtktextview.h>
 #include <gtk/gtkitemfactory.h>
 #include <stdlib.h>
 #if HAVE_WCHAR_H
@@ -40,8 +41,6 @@
 
 typedef struct _ComboButton	ComboButton;
 
-#include "gtkstext.h"
-
 struct _ComboButton
 {
 	GtkWidget *arrow;
@@ -88,13 +87,11 @@ struct _ComboButton
 	(GTK_CTREE_ROW(node)->row.data)
 
 #define GTKUT_CTREE_REFRESH(clist) \
-	GTK_CLIST_CLASS(GTK_OBJECT(clist)->klass)->refresh(clist)
+	GTK_CLIST_GET_CLASS(clist)->refresh(clist)
 
-gint gtkut_get_font_width		(GdkFont	*font);
-gint gtkut_get_font_height		(GdkFont	*font);
-
-GdkFont *gtkut_font_load		(const gchar	*fontset_name);
-GdkFont *gtkut_font_load_from_fontset	(const gchar	*fontset_name);
+gboolean gtkut_get_font_size		(GtkWidget	*widget,
+					 gint		*width,
+					 gint		*height);
 
 void gtkut_convert_int_to_gdk_color	(gint		 rgbvalue,
 					 GdkColor	*color);
@@ -107,6 +104,14 @@ void gtkut_button_set_create		(GtkWidget	**bbox,
 					 GtkWidget	**button3,
 					 const gchar	 *label3);
 
+void gtkut_stock_button_set_create	(GtkWidget	**bbox,
+					 GtkWidget	**button1,
+					 const gchar	 *label1,
+					 GtkWidget	**button2,
+					 const gchar	 *label2,
+					 GtkWidget	**button3,
+					 const gchar	 *label3);
+
 ComboButton *gtkut_combo_button_create	(GtkWidget		*button,
 					 GtkItemFactoryEntry	*entries,
 					 gint			 n_entries,
@@ -146,13 +151,29 @@ void gtkut_editable_disable_im		(GtkEditable	*editable);
 void gtkut_container_remove		(GtkContainer	*container,
 					 GtkWidget	*widget);
 
+gboolean gtkut_text_buffer_match_string	(GtkTextBuffer		*buffer,
+					 const GtkTextIter	*iter,
+					 gunichar		*wcs,
+					 gint			 len,
+					 gboolean		 case_sens);
+gboolean gtkut_text_buffer_find		(GtkTextBuffer		*buffer,
+					 const GtkTextIter	*iter,
+					 const gchar		*str,
+					 gboolean		 case_sens,
+					 GtkTextIter		*match_pos);
+gboolean gtkut_text_buffer_find_backward(GtkTextBuffer		*buffer,
+					 const GtkTextIter	*iter,
+					 const gchar		*str,
+					 gboolean		 case_sens,
+					 GtkTextIter		*match_pos);
+
+gchar *gtkut_text_view_get_selection	(GtkTextView	*textview);
 
 void gtkut_window_popup			(GtkWidget	*window);
 
 void gtkut_widget_get_uposition		(GtkWidget	*widget,
 					 gint		*px,
 					 gint		*py);
-void gtkut_widget_disable_theme_engine	(GtkWidget	*widget);
 void gtkut_widget_wait_for_draw		(GtkWidget	*widget);
 void gtkut_widget_init			(void);
 
@@ -160,10 +181,11 @@ void gtkut_widget_set_app_icon		(GtkWidget	*widget);
 void gtkut_widget_set_composer_icon	(GtkWidget	*widget);
 
 GtkWidget *gtkut_account_menu_new	(GList			*ac_list,
-				  	 GtkSignalFunc	 	 callback,
+				  	 GCallback	 	 callback,
 					 gpointer		 data);
 
 void gtkut_set_widget_bgcolor_rgb	(GtkWidget 	*widget,
 					 guint 		 rgbvalue);
+
 GtkWidget *gtkut_get_focused_child	(GtkContainer 	*parent);
 #endif /* __GTKUTILS_H__ */
