@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2001 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2005 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,15 +32,38 @@ struct _ProgressDialog
 	GtkWidget *cancel_btn;
 	GtkWidget *progressbar;
 	GtkWidget *clist;
+	GtkWidget *list_view;
 };
 
 ProgressDialog *progress_dialog_create	(void);
 void progress_dialog_set_label		(ProgressDialog	*progress,
 					 gchar		*str);
-void progress_dialog_set_value		(ProgressDialog	*progress,
-					 gfloat		 value);
-void progress_dialog_set_percentage	(ProgressDialog	*progress,
+void progress_dialog_get_fraction	(ProgressDialog	*progress);
+void progress_dialog_set_fraction	(ProgressDialog	*progress,
 					 gfloat		 percentage);
 void progress_dialog_destroy		(ProgressDialog	*progress);
 
+
+/*
+ * Use these functions to access the dialog list
+ */
+
+gint progress_dialog_list_set_account	(ProgressDialog *progress,
+					 gint		 row,
+					 const gchar	*account_name);
+gint progress_dialog_list_set_image	(ProgressDialog *progress,
+					 gint		 row,
+					 GdkPixbuf	*image);
+gint progress_dialog_list_set_status	(ProgressDialog *progress,
+					 gint		 row,
+					 const gchar    *status);
+gint progress_dialog_list_set		(ProgressDialog *progress,
+					 gint		 row,
+					 GdkPixbuf	*image,
+					 const gchar	*account_name,
+					 const gchar    *status);
+
+gboolean progress_dialog_list_select_row(ProgressDialog *progress,
+					 gint		 row);
+
 #endif /* __PROGRESS_H__ */
