@@ -53,6 +53,7 @@
 #include "jpilot.h"
 #include "codeconv.h"
 #include "adbookbase.h"
+#include "utils.h"
 
 #define JPILOT_DBHOME_DIR   ".jpilot"
 #define JPILOT_DBHOME_FILE  "AddressDB.pdb"
@@ -1099,8 +1100,10 @@ static void jpilot_parse_label( JPilotFile *pilotFile, gchar *labelEntry, ItemPe
 			email = addritem_create_item_email();
 			addritem_email_set_address( email, node->data );
 			if (convert_charcode) {
-				conv_sjistoeuc(convertBuff, JPILOT_LEN_LABEL, buffer);
-				addritem_email_set_remarks(email, convertBuff);
+				gchar *convertBuff;
+				convertBuff = conv_codeset_strdup( labelEntry, CS_SHIFT_JIS, CS_INTERNAL );
+				addritem_email_set_remarks( email, convertBuff );
+				g_free( convertBuff );
 			}
 			else {
 				addritem_email_set_remarks(email, buffer);
@@ -1180,8 +1183,8 @@ static void jpilot_load_address(
 
 		if( convert_charcode ) {
 			gchar *nameConv;
-			nameConv = g_strdup( fullName );
-			conv_sjistoeuc( fullName, FULLNAME_BUFSIZE, nameConv );
+			nameConv = conv_codeset_strdup( fullName, CS_SHIFT_JIS, CS_INTERNAL );
+			strncpy2( fullName, nameConv, FULLNAME_BUFSIZE );
 			g_free( nameConv );
 		}
 
@@ -1218,7 +1221,6 @@ static void jpilot_load_address(
 		/* Add entry for each custom label */
 		node = pilotFile->labelInd;
 		while( node ) {
-			gchar convertBuff[JPILOT_LEN_LABEL];
 			gint ind;
 
 			ind = GPOINTER_TO_INT( node->data );
@@ -1348,11 +1350,13 @@ static gboolean jpilot_setup_labels( JPilotFile *pilotFile ) {
 				gchar convertBuff[ JPILOT_LEN_LABEL ];
 
 				if( convert_charcode ) {
-					conv_sjistoeuc( convertBuff, JPILOT_LEN_LABEL, labelName );
+					labelName = conv_codeset_strdup( labelName, CS_SHIFT_JIS, CS_INTERNAL );
+					strncpy2( convertBuff, labelName, JPILOT_LEN_LABEL );
+					g_free( labelName );
 					labelName = convertBuff;
 				}
 
-				if( g_strcasecmp( labelName, lbl ) == 0 ) {
+				if( g_utf8_collate( labelName, lbl ) == 0 ) {
 					ind = i;
 					break;
 				}
@@ -1381,14 +1385,15 @@ GList *jpilot_load_label( JPilotFile *pilotFile, GList *labelList ) {
 		struct AddressAppInfo *ai = & pilotFile->addrInfo;
 		for( i = 0; i < JPILOT_NUM_LABELS; i++ ) {
 			gchar *labelName = ai->labels[i];
-			gchar convertBuff[JPILOT_LEN_LABEL];
 
 			if( labelName ) {
 				if( convert_charcode ) {
-					conv_sjistoeuc( convertBuff, JPILOT_LEN_LABEL, labelName );
-					labelName = convertBuff;
+					labelName = conv_codeset_strdup( labelName, CS_SHIFT_JIS, CS_INTERNAL );
 				}
-				labelList = g_list_append( labelList, g_strdup( labelName ) );
+				else {
+					labelName = g_strdup( labelName );
+				}
+				labelList = g_list_append( labelList, labelName );
 			}
 			else {
 				labelList = g_list_append(
@@ -1461,7 +1466,6 @@ GList *jpilot_load_phone_label( JPilotFile *pilotFile, GList *labelList ) {
  */
 GList *jpilot_load_custom_label( JPilotFile *pilotFile, GList *labelList ) {
 	gint i;
-	char convertBuff[JPILOT_LEN_LABEL];
 
 	g_return_val_if_fail( pilotFile != NULL, NULL );
 
@@ -1474,10 +1478,12 @@ GList *jpilot_load_custom_label( JPilotFile *pilotFile, GList *labelList ) {
 				g_strchug( labelName );
 				if( *labelName != '\0' ) {
 					if( convert_charcode ) {
-						conv_sjistoeuc( convertBuff, JPILOT_LEN_LABEL, labelName );
-						labelName = convertBuff;
+						labelName = conv_codeset_strdup( labelName, CS_SHIFT_JIS, CS_INTERNAL );
 					}
-					labelList = g_list_append( labelList, g_strdup( labelName ) );
+					else {
+						labelName = g_strdup( labelName );
+					}
+					labelList = g_list_append( labelList, labelName );
 				}
 			}
 		}
@@ -1527,9 +1533,10 @@ static void jpilot_build_category_list( JPilotFile *pilotFile ) {
 		ItemFolder *folder = addritem_create_item_folder();
 
 		if( convert_charcode ) {
-			gchar catName[ JPILOT_LEN_CATEG ];
-			conv_sjistoeuc( catName, JPILOT_LEN_CATEG, cat->name[i] );
+			gchar *catName;
+			catName = conv_codeset_strdup( cat->name[i], CS_SHIFT_JIS, CS_INTERNAL );
 			addritem_folder_set_name( folder, catName );
+			g_free( catName );
 		}
 		else {
 			addritem_folder_set_name( folder, cat->name[i] );
@@ -1654,11 +1661,11 @@ gint jpilot_read_data( JPilotFile *pilotFile ) {
 
 	cur_locale = conv_get_current_locale();
 
-	if( g_strncasecmp( cur_locale, "ja", 2 ) == 0 ) {
+	if( g_ascii_strncasecmp( cur_locale, "ja", 2 ) == 0 ) {
 		name_order = FAMILY_FIRST;
 	}
 
-	if( conv_get_current_charset() == C_EUC_JP ) {
+	if( conv_get_locale_charset() == C_EUC_JP ) {
 		convert_charcode = TRUE;
 	}
 
@@ -1757,7 +1764,7 @@ gboolean jpilot_validate( JPilotFile *pilotFile ) {
  *         no home. Filename should be <code>g_free()</code> when done.
  */
 gchar *jpilot_find_pilotdb( void ) {
-	gchar *homedir;
+	const gchar *homedir;
 	gchar str[ WORK_BUFLEN ];
 	gint len;
 	FILE *fp;
@@ -1825,7 +1832,7 @@ gboolean jpilot_test_custom_label( JPilotFile *pilotFile, const gchar *labelName
 	if( labelName ) {
 		node = pilotFile->customLabels;
 		while( node ) {
-			if( g_strcasecmp( labelName, ( gchar * ) node->data ) == 0 ) {
+			if( g_utf8_collate( labelName, ( gchar * ) node->data ) == 0 ) {
 				retVal = TRUE;
 				break;
 			}
