@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2004 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2005 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkrc.h>
 
@@ -36,7 +37,7 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <signal.h>
-
+#include "wizard.h"
 #ifdef HAVE_STARTUP_NOTIFICATION
 # define SN_API_NOT_YET_FROZEN
 # include <libsn/sn-launchee.h>
@@ -44,7 +45,6 @@
 #endif
 
 #include "sylpheed.h"
-#include "intl.h"
 #include "main.h"
 #include "mainwindow.h"
 #include "folderview.h"
@@ -221,7 +221,6 @@ void sylpheed_gtk_idle(void)
 
 int main(int argc, char *argv[])
 {
-
 	gchar *userrc;
 	MainWindow *mainwin;
 	FolderView *folderview;
@@ -278,22 +277,26 @@ int main(int argc, char *argv[])
 #endif
 
 	/* parse gtkrc files */
-	userrc = g_strconcat(get_home_dir(), G_DIR_SEPARATOR_S, ".gtkrc",
+	userrc = g_strconcat(get_home_dir(), G_DIR_SEPARATOR_S, ".gtkrc-2.0",
 			     NULL);
 	gtk_rc_parse(userrc);
 	g_free(userrc);
 	userrc = g_strconcat(get_home_dir(), G_DIR_SEPARATOR_S, ".gtk",
-			     G_DIR_SEPARATOR_S, "gtkrc", NULL);
+			     G_DIR_SEPARATOR_S, "gtkrc-2.0", NULL);
 	gtk_rc_parse(userrc);
 	g_free(userrc);
-	userrc = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, "gtkrc", NULL);
+	userrc = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, "gtkrc-2.0", NULL);
 	gtk_rc_parse(userrc);
 	g_free(userrc);
 
 	userrc = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, MENU_RC, NULL);
-	gtk_item_factory_parse_rc(userrc);
+	gtk_accel_map_load (userrc);
 	g_free(userrc);
 
+	gtk_settings_set_long_property(gtk_settings_get_default(), 
+				       "gtk-can-change-accels",
+				       (glong)TRUE, "XProperty");
+
 	CHDIR_RETURN_VAL_IF_FAIL(get_home_dir(), 1);
 
 	MAKE_DIR_IF_NOT_EXIST(RC_DIR);
@@ -333,6 +336,7 @@ int main(int argc, char *argv[])
 	mh_gtk_init();
 	imap_gtk_init();
 	news_gtk_init();
+		
 	mainwin = main_window_create
 		(prefs_common.sep_folder | prefs_common.sep_msg << 1);
 	folderview = mainwin->folderview;
@@ -347,12 +351,17 @@ int main(int argc, char *argv[])
 	account_read_config_all();
 
 	if (folder_read_list() < 0) {
-		setup(mainwin);
+		if (!run_wizard(mainwin, TRUE))
+			exit(1);
 		folder_write_list();
 	}
+
 	if (!account_get_list()) {
-		account_edit_open();
-		account_add();
+		if (!run_wizard(mainwin, FALSE))
+			exit(1);
+		account_read_config_all();
+		if(!account_get_list())
+			exit_sylpheed(mainwin);
 	}
 
 	account_set_missing_folder();
@@ -399,7 +408,7 @@ int main(int argc, char *argv[])
 
 	prefs_toolbar_init();
 
-	plugin_load_all("GTK");
+	plugin_load_all("GTK2");
 	
 	static_mainwindow = mainwin;
 
@@ -427,7 +436,7 @@ int main(int argc, char *argv[])
 	}
 	if (cmd.send)
 		send_queue();
-
+	
 	gtk_main();
 
 	exit_sylpheed(mainwin);
@@ -473,11 +482,10 @@ static void exit_sylpheed(MainWindow *mainwin)
 	addressbook_export_to_file();
 
 	filename = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, MENU_RC, NULL);
-	gtk_item_factory_dump_rc(filename, NULL, TRUE);
+	gtk_accel_map_save(filename);
 	g_free(filename);
 
 	/* delete temporary files */
-	remove_all_files(get_tmp_dir());
 	remove_all_files(get_mime_tmp_dir());
 
 	close_log_file();
@@ -490,7 +498,7 @@ static void exit_sylpheed(MainWindow *mainwin)
 
 	main_window_destroy(mainwin);
 	
-	plugin_unload_all("GTK");
+	plugin_unload_all("GTK2");
 
 	prefs_toolbar_done();
 
@@ -582,26 +590,27 @@ static void parse_cmd_opt(int argc, char *argv[])
 		} else if (!strncmp(argv[i], "--offline", 9)) {
 			cmd.online_mode = ONLINE_MODE_OFFLINE;
 		} else if (!strncmp(argv[i], "--help", 6)) {
-			g_print(_("Usage: %s [OPTION]...\n"),
-				g_basename(argv[0]));
+			gchar *base = g_path_get_basename(argv[0]);
+			g_print(_("Usage: %s [OPTION]...\n"), base);
 
-			puts(_("  --compose [address]    open composition window"));
-			puts(_("  --attach file1 [file2]...\n"
-			       "                         open composition window with specified files\n"
-			       "                         attached"));
-			puts(_("  --receive              receive new messages"));
-			puts(_("  --receive-all          receive new messages of all accounts"));
-			puts(_("  --send                 send all queued messages"));
- 			puts(_("  --status [folder]...   show the total number of messages"));
- 			puts(_("  --status-full [folder]...\n"
+			g_print("%s\n", _("  --compose [address]    open composition window"));
+			g_print("%s\n", _("  --attach file1 [file2]...\n"
+			          "                         open composition window with specified files\n"
+			          "                         attached"));
+			g_print("%s\n", _("  --receive              receive new messages"));
+			g_print("%s\n", _("  --receive-all          receive new messages of all accounts"));
+			g_print("%s\n", _("  --send                 send all queued messages"));
+ 			g_print("%s\n", _("  --status [folder]...   show the total number of messages"));
+ 			g_print("%s\n", _("  --status-full [folder]...\n"
  			       "                         show the status of each folder"));
-			puts(_("  --online               switch to online mode"));
-			puts(_("  --offline              switch to offline mode"));
-			puts(_("  --debug                debug mode"));
-			puts(_("  --help                 display this help and exit"));
-			puts(_("  --version              output version information and exit"));
-			puts(_("  --config-dir           output configuration directory"));
+			g_print("%s\n", _("  --online               switch to online mode"));
+			g_print("%s\n", _("  --offline              switch to offline mode"));
+			g_print("%s\n", _("  --debug                debug mode"));
+			g_print("%s\n", _("  --help                 display this help and exit"));
+			g_print("%s\n", _("  --version              output version information and exit"));
+			g_print("%s\n", _("  --config-dir           output configuration directory"));
 
+			g_free(base);
 			exit(1);
 		} else if (!strncmp(argv[i], "--crash", 7)) {
 			cmd.crash = TRUE;
@@ -644,6 +653,7 @@ static void initial_processing(FolderItem *item, gpointer data)
 	debug_print("%s\n", buf);
 	g_free(buf);
 
+	
         if (item->prefs->enable_processing)
                 folder_item_apply_processing(item);
 
@@ -653,12 +663,15 @@ static void initial_processing(FolderItem *item, gpointer data)
 
 static void draft_all_messages(void)
 {
-	GList *compose_list = NULL;
+	GList *compose_list = compose_get_compose_list();
 	GList *elem = NULL;
 	
-	while ((compose_list = compose_get_compose_list()) != NULL) {
-		Compose *c = (Compose*)compose_list->data;
-		compose_draft(c);
+	if (compose_list) {
+		for (elem = compose_list; elem != NULL && elem->data != NULL; 
+		     elem = elem->next) {
+			Compose *c = (Compose*)elem->data;
+			compose_draft(c);
+		}
 	}	
 }
 
@@ -716,7 +729,8 @@ void app_will_exit(GtkWidget *widget, gpointer data)
 	if (prefs_common.warn_queued_on_exit && get_queued_message_num() > 0) {
 		if (alertpanel(_("Queued messages"),
 			       _("Some unsent messages are queued. Exit now?"),
-			       _("OK"), _("Cancel"), NULL) != G_ALERTDEFAULT)
+			       GTK_STOCK_OK, GTK_STOCK_CANCEL, NULL)
+		    != G_ALERTDEFAULT)
 			return;
 		manage_window_focus_in(mainwin->window, NULL, NULL);
 	}
