@@ -20,6 +20,7 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkwindow.h>
@@ -42,7 +43,6 @@
 #include <gtk/gtktooltips.h>
 #include <string.h>
 
-#include "intl.h"
 #include "main.h"
 #include "mainwindow.h"
 #include "folderview.h"
@@ -107,7 +107,7 @@ static void main_window_set_widgets		(MainWindow	*mainwin,
 						 SeparateType	 type);
 
 #if 0
-static void toolbar_account_button_pressed	(GtkWidget	*widget,
+static gboolean toolbar_account_button_pressed	(GtkWidget	*widget,
 						 GdkEventButton	*event,
 						 gpointer	 data);
 #endif
@@ -119,7 +119,7 @@ static void toolbar_child_detached		(GtkWidget	*widget,
 						 GtkWidget	*child,
 						 gpointer	 data);
 
-static void ac_label_button_pressed		(GtkWidget	*widget,
+static gboolean ac_label_button_pressed		(GtkWidget	*widget,
 						 GdkEventButton	*event,
 						 gpointer	 data);
 static void ac_menu_popup_closed		(GtkMenuShell	*menu_shell,
@@ -447,7 +447,7 @@ static GtkItemFactoryEntry mainwin_entries[] =
 	{N_("/_File/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_File/_Import mbox file..."),	NULL, import_mbox_cb, 0, NULL},
 	{N_("/_File/_Export to mbox file..."),	NULL, export_mbox_cb, 0, NULL},
-	{N_("/_File/Exp_ort selected to mbox file..."),	
+	{N_("/_File/_Export selected to mbox file..."),	
 						NULL, export_list_mbox_cb, 0, NULL},
 	{N_("/_File/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_File/Empty all _Trash folders"),	"<shift>D", empty_trash_cb, 0, NULL},
@@ -556,17 +556,14 @@ static GtkItemFactoryEntry mainwin_entries[] =
 	{N_("/_View/_Code set/7bit ascii (US-ASC_II)"),
 	 CODESET_ACTION(C_US_ASCII)},
 
-#if HAVE_ICONV
 	{N_("/_View/_Code set/Unicode (_UTF-8)"),
 	 CODESET_ACTION(C_UTF_8)},
 	CODESET_SEPARATOR,
-#endif
 	{N_("/_View/_Code set/Western European (ISO-8859-_1)"),
 	 CODESET_ACTION(C_ISO_8859_1)},
 	{N_("/_View/_Code set/Western European (ISO-8859-15)"),
 	 CODESET_ACTION(C_ISO_8859_15)},
 	CODESET_SEPARATOR,
-#if HAVE_ICONV
 	{N_("/_View/_Code set/Central European (ISO-8859-_2)"),
 	 CODESET_ACTION(C_ISO_8859_2)},
 	CODESET_SEPARATOR,
@@ -590,18 +587,14 @@ static GtkItemFactoryEntry mainwin_entries[] =
 	{N_("/_View/_Code set/Cyrillic (Windows-1251)"),
 	 CODESET_ACTION(C_WINDOWS_1251)},
 	CODESET_SEPARATOR,
-#endif
 	{N_("/_View/_Code set/Japanese (ISO-2022-_JP)"),
 	 CODESET_ACTION(C_ISO_2022_JP)},
-#if HAVE_ICONV
 	{N_("/_View/_Code set/Japanese (ISO-2022-JP-2)"),
 	 CODESET_ACTION(C_ISO_2022_JP_2)},
-#endif
 	{N_("/_View/_Code set/Japanese (_EUC-JP)"),
 	 CODESET_ACTION(C_EUC_JP)},
 	{N_("/_View/_Code set/Japanese (_Shift__JIS)"),
 	 CODESET_ACTION(C_SHIFT_JIS)},
-#if HAVE_ICONV
 	CODESET_SEPARATOR,
 	{N_("/_View/_Code set/Simplified Chinese (_GB2312)"),
 	 CODESET_ACTION(C_GB2312)},
@@ -621,7 +614,6 @@ static GtkItemFactoryEntry mainwin_entries[] =
 	 CODESET_ACTION(C_TIS_620)},
 	{N_("/_View/_Code set/Thai (Windows-874)"),
 	 CODESET_ACTION(C_WINDOWS_874)},
-#endif
 
 #undef CODESET_SEPARATOR
 #undef CODESET_ACTION
@@ -645,7 +637,7 @@ static GtkItemFactoryEntry mainwin_entries[] =
 	{N_("/_View/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_View/Open in new _window"),	"<control><alt>N", open_msg_cb, 0, NULL},
 	{N_("/_View/Mess_age source"),		"<control>U", view_source_cb, 0, NULL},
-	{N_("/_View/Show all headers"),		"<control>H", show_all_header_cb, 0, "<ToggleItem>"},
+	{N_("/_View/Show all _headers"),	"<control>H", show_all_header_cb, 0, "<ToggleItem>"},
 	{N_("/_View/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_View/_Update summary"),		"<control><alt>U", update_summary_cb,  0, NULL},
 
@@ -712,19 +704,19 @@ static GtkItemFactoryEntry mainwin_entries[] =
 						NULL, create_filter_cb, FILTER_BY_TO, NULL},
 	{N_("/_Tools/_Create filter rule/by _Subject"),
 						NULL, create_filter_cb, FILTER_BY_SUBJECT, NULL},
-	{N_("/_Tools/C_reate processing rule"),	NULL, NULL, 0, "<Branch>"},
-	{N_("/_Tools/C_reate processing rule/_Automatically"),
+	{N_("/_Tools/_Create processing rule"),	NULL, NULL, 0, "<Branch>"},
+	{N_("/_Tools/_Create processing rule/_Automatically"),
 						NULL, create_processing_cb, FILTER_BY_AUTO, NULL},
-	{N_("/_Tools/C_reate processing rule/by _From"),
+	{N_("/_Tools/_Create processing rule/by _From"),
 						NULL, create_processing_cb, FILTER_BY_FROM, NULL},
-	{N_("/_Tools/C_reate processing rule/by _To"),
+	{N_("/_Tools/_Create processing rule/by _To"),
 						NULL, create_processing_cb, FILTER_BY_TO, NULL},
-	{N_("/_Tools/C_reate processing rule/by _Subject"),
+	{N_("/_Tools/_Create processing rule/by _Subject"),
 						NULL, create_processing_cb, FILTER_BY_SUBJECT, NULL},
 	{N_("/_Tools/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Tools/Actio_ns"),		NULL, NULL, 0, "<Branch>"},
 	{N_("/_Tools/---"),			NULL, NULL, 0, "<Separator>"},
-	{N_("/_Tools/Ch_eck for new messages in all folders"),
+	{N_("/_Tools/_Check for new messages in all folders"),
 						NULL, update_folderview_cb, 0, NULL},
 	{N_("/_Tools/Delete du_plicated messages"),
 						NULL, NULL, 0, "<Branch>"},
@@ -736,7 +728,7 @@ static GtkItemFactoryEntry mainwin_entries[] =
 	{N_("/_Tools/E_xecute"),		"X", execute_summary_cb, 0, NULL},
 #ifdef USE_OPENSSL
 	{N_("/_Tools/---"),			NULL, NULL, 0, "<Separator>"},
-	{N_("/_Tools/SSL cer_tificates..."),	
+	{N_("/_Tools/SSL certi_ficates..."),	
 						NULL, ssl_manager_open_cb, 0, NULL},
 #endif
 	{N_("/_Tools/---"),			NULL, NULL, 0, "<Separator>"},
@@ -752,24 +744,24 @@ static GtkItemFactoryEntry mainwin_entries[] =
 	{N_("/_Configuration/_Edit accounts..."),
 						NULL, account_edit_open, 0, NULL},
 	{N_("/_Configuration/---"),		NULL, NULL, 0, "<Separator>"},
-	{N_("/_Configuration/P_references..."),
+	{N_("/_Configuration/_Preferences..."),
 						NULL, prefs_open_cb, 0, NULL},
-	{N_("/_Configuration/Pre-pr_ocessing..."),
+	{N_("/_Configuration/Pre-processing..."),
 						NULL, prefs_pre_processing_open_cb, 0, NULL},
-	{N_("/_Configuration/Post-pro_cessing..."),
+	{N_("/_Configuration/Post-processing..."),
 						NULL, prefs_post_processing_open_cb, 0, NULL},
 	{N_("/_Configuration/_Filtering..."),
 						NULL, prefs_filtering_open_cb, 0, NULL},
 	{N_("/_Configuration/_Templates..."),	NULL, prefs_template_open_cb, 0, NULL},
 	{N_("/_Configuration/_Actions..."),	NULL, prefs_actions_open_cb, 0, NULL},
-	{N_("/_Configuration/Plu_gins..."),  	NULL, plugins_open_cb, 0, NULL},
+	{N_("/_Configuration/Plugins..."),  	NULL, plugins_open_cb, 0, NULL},
 
 	{N_("/_Help"),				NULL, NULL, 0, "<Branch>"},
 	{N_("/_Help/_Manual (Local)"),		NULL, manual_open_cb, MANUAL_MANUAL_LOCAL, NULL},
-	{N_("/_Help/Ma_nual (Sylpheed Doc Homepage)"),
+	{N_("/_Help/_Manual (Sylpheed Doc Homepage)"),
 						NULL, manual_open_cb, MANUAL_MANUAL_SYLDOC, NULL},
 	{N_("/_Help/_FAQ (Local)"),		NULL, manual_open_cb, MANUAL_FAQ_LOCAL, NULL},
-	{N_("/_Help/FA_Q (Sylpheed Doc Homepage)"),
+	{N_("/_Help/_FAQ (Sylpheed Doc Homepage)"),
 						NULL, manual_open_cb, MANUAL_FAQ_SYLDOC, NULL},
 	{N_("/_Help/_Claws FAQ (Claws Documentation)"),
 						NULL, manual_open_cb, MANUAL_FAQ_CLAWS, NULL},
@@ -777,6 +769,24 @@ static GtkItemFactoryEntry mainwin_entries[] =
 	{N_("/_Help/_About"),			NULL, about_show, 0, NULL}
 };
 
+static gboolean main_window_accel_activate (GtkAccelGroup *accelgroup,
+                                            GObject *arg1,
+                                            guint value,
+                                            GdkModifierType mod,
+                                            gpointer user_data) 
+{
+	MainWindow *mainwin = (MainWindow *)user_data;
+
+	if (mainwin->summaryview &&
+	    mainwin->summaryview->quicksearch &&
+	    quicksearch_has_focus(mainwin->summaryview->quicksearch) &&
+	    (mod == 0 || mod == GDK_SHIFT_MASK)) {
+		quicksearch_pass_key(mainwin->summaryview->quicksearch, value, mod);
+		return TRUE;
+	}
+	return FALSE;
+}
+
 MainWindow *main_window_create(SeparateType type)
 {
 	MainWindow *mainwin;
@@ -820,8 +830,7 @@ MainWindow *main_window_create(SeparateType type)
 	/* main window */
 	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_title(GTK_WINDOW(window), PROG_VERSION);
-	gtk_window_set_policy(GTK_WINDOW(window), TRUE, TRUE, FALSE);
-	gtk_window_set_wmclass(GTK_WINDOW(window), "main_window", "Sylpheed");
+	gtk_window_set_resizable(GTK_WINDOW(window), TRUE);
 
 	if (!geometry.min_height) {
 		geometry.min_width = 320;
@@ -830,14 +839,14 @@ MainWindow *main_window_create(SeparateType type)
 	gtk_window_set_geometry_hints(GTK_WINDOW(window), NULL, &geometry,
 				      GDK_HINT_MIN_SIZE);
 
-	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
-			   GTK_SIGNAL_FUNC(main_window_close_cb), mainwin);
+	g_signal_connect(G_OBJECT(window), "delete_event",
+			 G_CALLBACK(main_window_close_cb), mainwin);
 	MANAGE_WINDOW_SIGNALS_CONNECT(window);
-	gtk_signal_connect(GTK_OBJECT(window), "focus_in_event",
-			   GTK_SIGNAL_FUNC(mainwindow_focus_in_event),
-			   mainwin);
-	gtk_signal_connect(GTK_OBJECT(window), "key_press_event",
-				GTK_SIGNAL_FUNC(mainwindow_key_pressed), mainwin);
+	g_signal_connect(G_OBJECT(window), "focus_in_event",
+			 G_CALLBACK(mainwindow_focus_in_event),
+			 mainwin);
+	g_signal_connect(G_OBJECT(window), "key_press_event",
+			 G_CALLBACK(mainwindow_key_pressed), mainwin);
 
 	gtk_widget_realize(window);
 	gtk_widget_add_events(window, GDK_KEY_PRESS_MASK|GDK_KEY_RELEASE_MASK);
@@ -863,10 +872,10 @@ MainWindow *main_window_create(SeparateType type)
 	handlebox = gtk_handle_box_new();
 	gtk_widget_show(handlebox);
 	gtk_box_pack_start(GTK_BOX(vbox), handlebox, FALSE, FALSE, 0);
-	gtk_signal_connect(GTK_OBJECT(handlebox), "child_attached",
-			   GTK_SIGNAL_FUNC(toolbar_child_attached), mainwin);
-	gtk_signal_connect(GTK_OBJECT(handlebox), "child_detached",
-			   GTK_SIGNAL_FUNC(toolbar_child_detached), mainwin);
+	g_signal_connect(G_OBJECT(handlebox), "child_attached",
+			 G_CALLBACK(toolbar_child_attached), mainwin);
+	g_signal_connect(G_OBJECT(handlebox), "child_detached",
+			 G_CALLBACK(toolbar_child_detached), mainwin);
 
 	/* link window to mainwin->window to avoid gdk warnings */
 	mainwin->window       = window;
@@ -889,7 +898,7 @@ MainWindow *main_window_create(SeparateType type)
 	gtk_box_pack_start(GTK_BOX(hbox_stat), statusbar, TRUE, TRUE, 0);
 
 	progressbar = gtk_progress_bar_new();
-	gtk_widget_set_usize(progressbar, 120, 1);
+	gtk_widget_set_size_request(progressbar, 120, 1);
 	gtk_box_pack_start(GTK_BOX(hbox_stat), progressbar, FALSE, FALSE, 0);
 
 	online_pixmap = stock_pixmap_widget(hbox_stat, STOCK_PIXMAP_ONLINE);
@@ -905,11 +914,11 @@ MainWindow *main_window_create(SeparateType type)
 			     NULL);
 	gtk_container_add (GTK_CONTAINER(online_switch), online_pixmap);
 	gtk_button_set_relief (GTK_BUTTON(online_switch), GTK_RELIEF_NONE);
-	gtk_signal_connect (GTK_OBJECT(online_switch), "clicked", (GtkSignalFunc)online_switch_clicked, mainwin);
+	g_signal_connect (G_OBJECT(online_switch), "clicked", G_CALLBACK(online_switch_clicked), mainwin);
 	gtk_box_pack_start (GTK_BOX(hbox_stat), online_switch, FALSE, FALSE, 0);
 	gtk_container_add (GTK_CONTAINER(offline_switch), offline_pixmap);
 	gtk_button_set_relief (GTK_BUTTON(offline_switch), GTK_RELIEF_NONE);
-	gtk_signal_connect (GTK_OBJECT(offline_switch), "clicked", (GtkSignalFunc)online_switch_clicked, mainwin);
+	g_signal_connect (G_OBJECT(offline_switch), "clicked", G_CALLBACK(online_switch_clicked), mainwin);
 	gtk_box_pack_start (GTK_BOX(hbox_stat), offline_switch, FALSE, FALSE, 0);
 	
 	statuslabel = gtk_label_new("");
@@ -921,10 +930,10 @@ MainWindow *main_window_create(SeparateType type)
 			     ac_button, _("Select account"), NULL);
 	gtk_button_set_relief(GTK_BUTTON(ac_button), GTK_RELIEF_NONE);
 	GTK_WIDGET_UNSET_FLAGS(ac_button, GTK_CAN_FOCUS);
-	gtk_widget_set_usize(ac_button, -1, 1);
+	gtk_widget_set_size_request(ac_button, -1, 0);
 	gtk_box_pack_end(GTK_BOX(hbox_stat), ac_button, FALSE, FALSE, 0);
-	gtk_signal_connect(GTK_OBJECT(ac_button), "button_press_event",
-			   GTK_SIGNAL_FUNC(ac_label_button_pressed), mainwin);
+	g_signal_connect(G_OBJECT(ac_button), "button_press_event",
+			 G_CALLBACK(ac_label_button_pressed), mainwin);
 
 	ac_label = gtk_label_new("");
 	gtk_container_add(GTK_CONTAINER(ac_button), ac_label);
@@ -1011,9 +1020,9 @@ MainWindow *main_window_create(SeparateType type)
 	
 	main_window_set_widgets(mainwin, type);
 
-	gtk_signal_connect(GTK_OBJECT(window), "size_allocate",
-			   GTK_SIGNAL_FUNC(main_window_size_allocate_cb),
-			   mainwin);
+	g_signal_connect(G_OBJECT(window), "size_allocate",
+			 G_CALLBACK(main_window_size_allocate_cb),
+			 mainwin);
 
 	/* set menu items */
 	menuitem = gtk_item_factory_get_item
@@ -1048,8 +1057,8 @@ MainWindow *main_window_create(SeparateType type)
 	/* set account selection menu */
 	ac_menu = gtk_item_factory_get_widget
 		(ifactory, "/Configuration/Change current account");
-	gtk_signal_connect(GTK_OBJECT(ac_menu), "selection_done",
-			   GTK_SIGNAL_FUNC(ac_menu_popup_closed), mainwin);
+	g_signal_connect(G_OBJECT(ac_menu), "selection_done",
+			 G_CALLBACK(ac_menu_popup_closed), mainwin);
 	mainwin->ac_menu = ac_menu;
 
 	toolbar_main_set_sensitive(mainwin);
@@ -1058,10 +1067,14 @@ MainWindow *main_window_create(SeparateType type)
 	main_window_update_actions_menu(mainwin);
 
 	/* attach accel groups to main window */
-#define	ADD_MENU_ACCEL_GROUP_TO_WINDOW(menu,win)	\
-	gtk_window_add_accel_group			\
-		(GTK_WINDOW(win), 			\
-		 gtk_item_factory_from_widget(menu)->accel_group)		 
+#define	ADD_MENU_ACCEL_GROUP_TO_WINDOW(menu,win)			\
+	gtk_window_add_accel_group					\
+		(GTK_WINDOW(win), 					\
+		 gtk_item_factory_from_widget(menu)->accel_group); 	\
+	g_signal_connect(G_OBJECT(gtk_item_factory_from_widget(menu)->accel_group), \
+			"accel_activate", 				\
+		       	G_CALLBACK(main_window_accel_activate), mainwin);
+			 
 	
 	ADD_MENU_ACCEL_GROUP_TO_WINDOW(summaryview->popupmenu, mainwin->window);
 	
@@ -1110,7 +1123,7 @@ void main_window_destroy(MainWindow *mainwin)
 {
 	/* TODO : destroy other component */
 	messageview_destroy(mainwin->messageview);
-	mainwin->messageview = NULL;
+	mainwin->messageview = NULL;	
 }
 
 void main_window_update_actions_menu(MainWindow *mainwin)
@@ -1258,9 +1271,9 @@ static void main_window_set_account_selector_menu(MainWindow *mainwin,
 			 ? ac_prefs->account_name : _("Untitled"));
 		gtk_widget_show(menuitem);
 		gtk_menu_append(GTK_MENU(mainwin->ac_menu), menuitem);
-		gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
-				   GTK_SIGNAL_FUNC(account_selector_menu_cb),
-				   ac_prefs);
+		g_signal_connect(G_OBJECT(menuitem), "activate",
+				 G_CALLBACK(account_selector_menu_cb),
+				 ac_prefs);
 	}
 }
 
@@ -1299,9 +1312,9 @@ static void main_window_set_account_receive_menu(MainWindow *mainwin,
 			 : _("Untitled"));
 		gtk_widget_show(menuitem);
 		gtk_menu_append(GTK_MENU(menu), menuitem);
-		gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
-				   GTK_SIGNAL_FUNC(account_receive_menu_cb),
-				   ac_prefs);
+		g_signal_connect(G_OBJECT(menuitem), "activate",
+				 G_CALLBACK(account_receive_menu_cb),
+				 ac_prefs);
 	}
 }
 
@@ -1518,7 +1531,7 @@ void main_window_get_size(MainWindow *mainwin)
 		prefs_common.msgview_height = allocation->height;
 	}
 
-	debug_print("summaryview size: %d x %d\n",
+/*	debug_print("summaryview size: %d x %d\n",
 		    prefs_common.summaryview_width,
 		    prefs_common.summaryview_height);
 	debug_print("folderview size: %d x %d\n",
@@ -1526,7 +1539,7 @@ void main_window_get_size(MainWindow *mainwin)
 		    prefs_common.folderview_height);
 	debug_print("messageview size: %d x %d\n",
 		    prefs_common.msgview_width,
-		    prefs_common.msgview_height);
+		    prefs_common.msgview_height); */
 }
 
 void main_window_get_position(MainWindow *mainwin)
@@ -1561,15 +1574,13 @@ void main_window_get_position(MainWindow *mainwin)
 
 void main_window_progress_on(MainWindow *mainwin)
 {
-	gtk_progress_set_show_text(GTK_PROGRESS(mainwin->progressbar), TRUE);
-	gtk_progress_set_format_string(GTK_PROGRESS(mainwin->progressbar), "");
+	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(mainwin->progressbar), "");
 }
 
 void main_window_progress_off(MainWindow *mainwin)
 {
-	gtk_progress_set_show_text(GTK_PROGRESS(mainwin->progressbar), FALSE);
-	gtk_progress_bar_update(GTK_PROGRESS_BAR(mainwin->progressbar), 0.0);
-	gtk_progress_set_format_string(GTK_PROGRESS(mainwin->progressbar), "");
+	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(mainwin->progressbar), 0.0);
+	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(mainwin->progressbar), "");
 }
 
 void main_window_progress_set(MainWindow *mainwin, gint cur, gint total)
@@ -1577,8 +1588,8 @@ void main_window_progress_set(MainWindow *mainwin, gint cur, gint total)
 	gchar buf[32];
 
 	g_snprintf(buf, sizeof(buf), "%d / %d", cur, total);
-	gtk_progress_set_format_string(GTK_PROGRESS(mainwin->progressbar), buf);
-	gtk_progress_bar_update(GTK_PROGRESS_BAR(mainwin->progressbar),
+	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(mainwin->progressbar), buf);
+	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(mainwin->progressbar),
 				(cur == 0 && total == 0) ? 0 :
 				(gfloat)cur / (gfloat)total);
 }
@@ -1600,7 +1611,8 @@ void main_window_empty_trash(MainWindow *mainwin, gboolean confirm)
 	if (confirm) {
 		if (alertpanel(_("Empty trash"),
 			       _("Empty all messages in trash?"),
-			       _("Yes"), _("No"), NULL) != G_ALERTDEFAULT)
+			       GTK_STOCK_YES, GTK_STOCK_NO, NULL)
+		    != G_ALERTDEFAULT)
 			return;
 		manage_window_focus_in(mainwin->window, NULL, NULL);
 	}
@@ -1629,8 +1641,8 @@ void main_window_add_mailbox(MainWindow *mainwin)
 		return;
 	}
 	folder = folder_new(folder_get_class_from_string("mh"), 
-			    !strcmp(path, "Mail") ? _("Mailbox") : g_basename(path),
-			    path);
+			    !strcmp(path, "Mail") ? _("Mailbox") : 
+			    g_path_get_basename(path), path);
 	g_free(path);
 
 	if (folder->klass->create_tree(folder) < 0) {
@@ -1975,16 +1987,13 @@ static void main_window_set_widgets(MainWindow *mainwin, SeparateType type)
 		folderwin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 		gtk_window_set_title(GTK_WINDOW(folderwin),
 				     _("Sylpheed - Folder View"));
-		gtk_window_set_wmclass(GTK_WINDOW(folderwin),
-				       "folder_view", "Sylpheed");
-		gtk_window_set_policy(GTK_WINDOW(folderwin),
-				      TRUE, TRUE, FALSE);
-		gtk_widget_set_uposition(folderwin, prefs_common.folderwin_x,
-					 prefs_common.folderwin_y);
+		gtk_window_set_resizable(GTK_WINDOW(folderwin), TRUE);
+		gtk_window_move(GTK_WINDOW(folderwin), prefs_common.folderwin_x,
+				prefs_common.folderwin_y);
 		gtk_container_set_border_width(GTK_CONTAINER(folderwin),
 					       BORDER_WIDTH);
-		gtk_signal_connect(GTK_OBJECT(folderwin), "delete_event",
-				   GTK_SIGNAL_FUNC(folder_window_close_cb),
+		g_signal_connect(G_OBJECT(folderwin), "delete_event",
+				 G_CALLBACK(folder_window_close_cb),
 				   mainwin);
 		gtk_container_add(GTK_CONTAINER(folderwin),
 				  GTK_WIDGET_PTR(mainwin->folderview));
@@ -1997,17 +2006,18 @@ static void main_window_set_widgets(MainWindow *mainwin, SeparateType type)
 		messagewin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 		gtk_window_set_title(GTK_WINDOW(messagewin),
 				     _("Sylpheed - Message View"));
-		gtk_window_set_wmclass(GTK_WINDOW(messagewin),
-				       "message_view", "Sylpheed");
-		gtk_window_set_policy(GTK_WINDOW(messagewin),
-				      TRUE, TRUE, FALSE);
-		gtk_widget_set_uposition(messagewin, prefs_common.main_msgwin_x,
-					 prefs_common.main_msgwin_y);
+		gtk_window_set_resizable(GTK_WINDOW(messagewin), TRUE);
+		gtk_window_move(GTK_WINDOW(messagewin), 
+				prefs_common.main_msgwin_x,
+				prefs_common.main_msgwin_y);
+		gtk_widget_set_size_request(messagewin, 
+					    prefs_common.msgwin_width,
+					    prefs_common.msgwin_height);
 		gtk_container_set_border_width(GTK_CONTAINER(messagewin),
 					       BORDER_WIDTH);
-		gtk_signal_connect(GTK_OBJECT(messagewin), "delete_event",
-				   GTK_SIGNAL_FUNC(message_window_close_cb),
-				   mainwin);
+		g_signal_connect(G_OBJECT(messagewin), "delete_event",
+				 G_CALLBACK(message_window_close_cb),
+				 mainwin);
 		if (messageview_is_visible(mainwin->messageview))
 			gtk_widget_show(messagewin);
 	} else {
@@ -2015,15 +2025,15 @@ static void main_window_set_widgets(MainWindow *mainwin, SeparateType type)
 		mainwin->messageview->statusbar_cid = mainwin->messageview_cid;
 	}
 
-	gtk_widget_set_usize(GTK_WIDGET_PTR(mainwin->folderview),
-			     prefs_common.folderview_width,
-			     prefs_common.folderview_height);
-	gtk_widget_set_usize(GTK_WIDGET_PTR(mainwin->summaryview),
-			     prefs_common.summaryview_width,
-			     prefs_common.summaryview_height);
-	gtk_widget_set_usize(GTK_WIDGET_PTR(mainwin->messageview),
-			     prefs_common.msgview_width,
-			     prefs_common.msgview_height);
+	gtk_widget_set_size_request(GTK_WIDGET_PTR(mainwin->folderview),
+				    prefs_common.folderview_width,
+				    prefs_common.folderview_height);
+	gtk_widget_set_size_request(GTK_WIDGET_PTR(mainwin->summaryview),
+				    prefs_common.summaryview_width,
+				    prefs_common.summaryview_height);
+	gtk_widget_set_size_request(GTK_WIDGET_PTR(mainwin->messageview),
+				    prefs_common.msgview_width,
+				    prefs_common.msgview_height);
 
 	switch (type) {
 	case SEPARATE_NONE:
@@ -2128,9 +2138,9 @@ static void main_window_set_widgets(MainWindow *mainwin, SeparateType type)
 		break;
 	}
 
-	gtk_widget_set_uposition(mainwin->window,
-				 prefs_common.mainwin_x,
-				 prefs_common.mainwin_y);
+	gtk_window_move(GTK_WINDOW(mainwin->window),
+			prefs_common.mainwin_x,
+			prefs_common.mainwin_y);
 
 	gtk_widget_queue_resize(vbox_body);
 	gtk_widget_queue_resize(mainwin->vbox);
@@ -2172,15 +2182,15 @@ static void main_window_set_widgets(MainWindow *mainwin, SeparateType type)
 				       ((type & SEPARATE_MESSAGE) != 0));
 
 	if (folderwin) {
-		gtk_signal_connect
-			(GTK_OBJECT(folderwin), "size_allocate",
-			 GTK_SIGNAL_FUNC(folder_window_size_allocate_cb),
+		g_signal_connect
+			(G_OBJECT(folderwin), "size_allocate",
+			 G_CALLBACK(folder_window_size_allocate_cb),
 			 mainwin);
 	}
 	if (messagewin) {
-		gtk_signal_connect
-			(GTK_OBJECT(messagewin), "size_allocate",
-			 GTK_SIGNAL_FUNC(message_window_size_allocate_cb),
+		g_signal_connect
+			(G_OBJECT(messagewin), "size_allocate",
+			 G_CALLBACK(message_window_size_allocate_cb),
 			 mainwin);
 	}
 
@@ -2206,22 +2216,24 @@ void main_window_destroy_all(void)
 }
 
 #if 0
-static void toolbar_account_button_pressed(GtkWidget *widget,
-					   GdkEventButton *event,
-					   gpointer data)
+static gboolean toolbar_account_button_pressed(GtkWidget *widget,
+					       GdkEventButton *event,
+					       gpointer data)
 {
 	MainWindow *mainwin = (MainWindow *)data;
 
-	if (!event) return;
-	if (event->button != 3) return;
+	if (!event) return FALSE;
+	if (event->button != 3) return FALSE;
 
 	gtk_button_set_relief(GTK_BUTTON(widget), GTK_RELIEF_NORMAL);
-	gtk_object_set_data(GTK_OBJECT(mainwin->ac_menu), "menu_button",
-			    widget);
+	g_object_set_data(G_OBJECT(mainwin->ac_menu), "menu_button",
+			  widget);
 
 	gtk_menu_popup(GTK_MENU(mainwin->ac_menu), NULL, NULL,
 		       menu_button_position, widget,
 		       event->button, event->time);
+
+	return FALSE;
 }
 #endif
 
@@ -2237,20 +2249,22 @@ static void toolbar_child_detached(GtkWidget *widget, GtkWidget *child,
 	gtk_widget_set_usize(child, -1, -1);
 }
 
-static void ac_label_button_pressed(GtkWidget *widget, GdkEventButton *event,
+static gboolean ac_label_button_pressed(GtkWidget *widget, GdkEventButton *event,
 				    gpointer data)
 {
 	MainWindow *mainwin = (MainWindow *)data;
 
-	if (!event) return;
+	if (!event) return FALSE;
 
 	gtk_button_set_relief(GTK_BUTTON(widget), GTK_RELIEF_NORMAL);
-	gtk_object_set_data(GTK_OBJECT(mainwin->ac_menu), "menu_button",
-			    widget);
+	g_object_set_data(G_OBJECT(mainwin->ac_menu), "menu_button",
+			  widget);
 
 	gtk_menu_popup(GTK_MENU(mainwin->ac_menu), NULL, NULL,
 		       menu_button_position, widget,
 		       event->button, event->time);
+
+	return TRUE;
 }
 
 static void ac_menu_popup_closed(GtkMenuShell *menu_shell, gpointer data)
@@ -2258,10 +2272,10 @@ static void ac_menu_popup_closed(GtkMenuShell *menu_shell, gpointer data)
 	MainWindow *mainwin = (MainWindow *)data;
 	GtkWidget *button;
 
-	button = gtk_object_get_data(GTK_OBJECT(menu_shell), "menu_button");
+	button = g_object_get_data(G_OBJECT(menu_shell), "menu_button");
 	if (!button) return;
 	gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
-	gtk_object_remove_data(GTK_OBJECT(mainwin->ac_menu), "menu_button");
+	g_object_set_data(G_OBJECT(mainwin->ac_menu), "menu_button", NULL);
 	manage_window_focus_in(mainwin->window, NULL, NULL);
 }
 
@@ -2389,7 +2403,8 @@ static void app_exit_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
 {
 	if (prefs_common.confirm_on_exit) {
 		if (alertpanel(_("Exit"), _("Exit this program?"),
-			       _("OK"), _("Cancel"), NULL) != G_ALERTDEFAULT)
+			       GTK_STOCK_OK, GTK_STOCK_CANCEL, NULL)
+		    != G_ALERTDEFAULT)
 			return;
 		manage_window_focus_in(mainwin->window, NULL, NULL);
 	}
@@ -2673,7 +2688,7 @@ static void hide_read_messages (MainWindow *mainwin, guint action,
 				GtkWidget *widget)
 {
 	if (!mainwin->summaryview->folder_item
-	    || gtk_object_get_data(GTK_OBJECT(widget), "dont_toggle"))
+	    || g_object_get_data(G_OBJECT(widget), "dont_toggle"))
 		return;
 	summary_toggle_show_read_messages(mainwin->summaryview);
 }
@@ -2724,6 +2739,7 @@ static void sort_summary_cb(MainWindow *mainwin, guint action,
 		summary_sort(mainwin->summaryview, (FolderSortKey)action,
 			     GTK_CHECK_MENU_ITEM(menuitem)->active
 			     ? SORT_ASCENDING : SORT_DESCENDING);
+		item->sort_key = action;
 	}
 }
 
@@ -2787,9 +2803,7 @@ static void delete_duplicated_all_cb(MainWindow *mainwin, guint action,
 	struct DelDupsData data = {0, 0};
 
 	folder_func_to_all_folders(deldup_all, &data);
-	alertpanel_notice(ngettext("Deleted %d duplicate message in %d folders.\n",
-				   "Deleted %d duplicate messages in %d folders.\n",
-				   data.dups),
+	alertpanel_notice(_("Deleted %d duplicate message(s) in %d folders.\n"),
 			  data.dups, data.folders);
 }
 
@@ -3059,19 +3073,23 @@ static gboolean mainwindow_focus_in_event(GtkWidget *widget, GdkEventFocus *focu
 #define BREAK_ON_MODIFIER_KEY() \
 	if ((event->state & (GDK_MOD1_MASK|GDK_CONTROL_MASK)) != 0) break
 
-void mainwindow_key_pressed (GtkWidget *widget, GdkEventKey *event,
+gboolean mainwindow_key_pressed (GtkWidget *widget, GdkEventKey *event,
 				    gpointer data)
 {
 	MainWindow *mainwin = (MainWindow*) data;
 	
-	if (!mainwin || !event) return;
-		
+	if (!mainwin || !event) 
+		return FALSE;
+
+	if (quicksearch_has_focus(mainwin->summaryview->quicksearch))
+		return FALSE;
+
 	switch (event->keyval) {
 	case GDK_Q:             /* Quit */
 		BREAK_ON_MODIFIER_KEY();
 
 		app_exit_cb(mainwin, 0, NULL);
-		return;
+		return FALSE;
 	case GDK_space:
 		if (mainwin->folderview && mainwin->summaryview
 		    && !mainwin->summaryview->displayed) {
@@ -3083,6 +3101,7 @@ void mainwindow_key_pressed (GtkWidget *widget, GdkEventKey *event,
 	default:
 		break;
 	}
+	return FALSE;
 }
 
 #undef BREAK_ON_MODIFIER_KEY
@@ -3128,10 +3147,12 @@ gboolean mainwindow_progressindicator_hook(gpointer source, gpointer userdata)
 	switch (data->cmd) {
 	case PROGRESS_COMMAND_START:
 	case PROGRESS_COMMAND_STOP:
-		gtk_progress_set_percentage(GTK_PROGRESS(mainwin->progressbar), 0.0);
+		gtk_progress_bar_set_fraction
+			(GTK_PROGRESS_BAR(mainwin->progressbar), 0.0);
 		break;
 	case PROGRESS_COMMAND_SET_PERCENTAGE:
-		gtk_progress_set_percentage(GTK_PROGRESS(mainwin->progressbar), data->value);
+		gtk_progress_bar_set_fraction
+			(GTK_PROGRESS_BAR(mainwin->progressbar), data->value);
 		break;		
 	}
 	while (gtk_events_pending()) gtk_main_iteration ();
@@ -3142,3 +3163,4 @@ gboolean mainwindow_progressindicator_hook(gpointer source, gpointer userdata)
 /*
 * End of Source.
 */
+
