@@ -257,8 +257,8 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 
-#include "intl.h"
 #include "utils.h"
 #include "filtering.h"
 #include "matcher.h"
@@ -328,7 +328,7 @@ FilteringProp *matcher_parser_get_filtering(gchar *str)
 
 static gboolean check_quote_symetry(gchar *str)
 {
-	gchar *walk;
+	const gchar *walk;
 	int ret = 0;
 	
 	if (str == NULL)
