@@ -29,6 +29,10 @@ typedef enum {
 	LOCK_FLOCK
 } LockType;
 
+
+#warning FIXME_GTK2
+/*  file name should be locale encode. */
+
 gint proc_mbox		(FolderItem	*dest,
 			 const gchar	*mbox,
 			 gboolean	 apply_filter);
