@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2004 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtkvbox.h>
 #include <gtk/gtkcontainer.h>
@@ -32,7 +33,6 @@
 #include <ctype.h>
 #include <string.h>
 
-#include "intl.h"
 #include "main.h"
 #include "messageview.h"
 #include "message_search.h"
@@ -69,7 +69,7 @@ static void messageview_destroy_cb	(GtkWidget	*widget,
 					 MessageView	*messageview);
 static void messageview_size_allocate_cb(GtkWidget	*widget,
 					 GtkAllocation	*allocation);
-static void key_pressed			(GtkWidget	*widget,
+static gboolean key_pressed		(GtkWidget	*widget,
 					 GdkEventKey	*event,
 					 MessageView	*messageview);
 
@@ -178,17 +178,14 @@ static GtkItemFactoryEntry msgview_entries[] =
 	{N_("/_View/_Code set/7bit ascii (US-ASC_II)"),
 	 CODESET_ACTION(C_US_ASCII)},
 
-#if HAVE_ICONV
 	{N_("/_View/_Code set/Unicode (_UTF-8)"),
 	 CODESET_ACTION(C_UTF_8)},
 	CODESET_SEPARATOR,
-#endif
 	{N_("/_View/_Code set/Western European (ISO-8859-_1)"),
 	 CODESET_ACTION(C_ISO_8859_1)},
 	{N_("/_View/_Code set/Western European (ISO-8859-15)"),
 	 CODESET_ACTION(C_ISO_8859_15)},
 	CODESET_SEPARATOR,
-#if HAVE_ICONV
 	{N_("/_View/_Code set/Central European (ISO-8859-_2)"),
 	 CODESET_ACTION(C_ISO_8859_2)},
 	CODESET_SEPARATOR,
@@ -212,18 +209,14 @@ static GtkItemFactoryEntry msgview_entries[] =
 	{N_("/_View/_Code set/Cyrillic (Windows-1251)"),
 	 CODESET_ACTION(C_CP1251)},
 	CODESET_SEPARATOR,
-#endif
 	{N_("/_View/_Code set/Japanese (ISO-2022-_JP)"),
 	 CODESET_ACTION(C_ISO_2022_JP)},
-#if HAVE_ICONV
 	{N_("/_View/_Code set/Japanese (ISO-2022-JP-2)"),
 	 CODESET_ACTION(C_ISO_2022_JP_2)},
-#endif
 	{N_("/_View/_Code set/Japanese (_EUC-JP)"),
 	 CODESET_ACTION(C_EUC_JP)},
 	{N_("/_View/_Code set/Japanese (_Shift__JIS)"),
 	 CODESET_ACTION(C_SHIFT_JIS)},
-#if HAVE_ICONV
 	CODESET_SEPARATOR,
 	{N_("/_View/_Code set/Simplified Chinese (_GB2312)"),
 	 CODESET_ACTION(C_GB2312)},
@@ -243,7 +236,6 @@ static GtkItemFactoryEntry msgview_entries[] =
 	 CODESET_ACTION(C_TIS_620)},
 	{N_("/_View/_Code set/Thai (Windows-874)"),
 	 CODESET_ACTION(C_WINDOWS_874)},
-#endif
 
 #undef CODESET_SEPARATOR
 #undef CODESET_ACTION
@@ -426,32 +418,22 @@ MessageView *messageview_create_with_new_window(MainWindow *mainwin)
 {
 	MessageView *msgview;
 	GtkWidget *window;
-	static GdkGeometry geometry;
 
 	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_title(GTK_WINDOW(window), _("Sylpheed - Message View"));
-	gtk_window_set_wmclass(GTK_WINDOW(window), "message_view", "Sylpheed");
-	gtk_window_set_policy(GTK_WINDOW(window), TRUE, TRUE, FALSE);
+	gtk_window_set_resizable(GTK_WINDOW(window), TRUE);
+	gtk_widget_set_size_request(window, prefs_common.msgwin_width,
+				    prefs_common.msgwin_height);
 
-	if (!geometry.min_height) {
-		geometry.min_width = 320;
-		geometry.min_height = 200;
-	}
-	gtk_window_set_geometry_hints(GTK_WINDOW(window), NULL, &geometry,
-				      GDK_HINT_MIN_SIZE);
-
-	gtk_widget_set_usize(window, prefs_common.msgwin_width,
-			     prefs_common.msgwin_height);
-	
 	msgview = messageview_create(mainwin);
 
-	gtk_signal_connect(GTK_OBJECT(window), "size_allocate",
-			   GTK_SIGNAL_FUNC(messageview_size_allocate_cb),
-			   msgview);
-	gtk_signal_connect(GTK_OBJECT(window), "destroy",
-			   GTK_SIGNAL_FUNC(messageview_destroy_cb), msgview);
-	gtk_signal_connect(GTK_OBJECT(window), "key_press_event",
-			   GTK_SIGNAL_FUNC(key_pressed), msgview);
+	g_signal_connect(G_OBJECT(window), "size_allocate",
+			 G_CALLBACK(messageview_size_allocate_cb),
+			 msgview);
+	g_signal_connect(G_OBJECT(window), "destroy",
+			 G_CALLBACK(messageview_destroy_cb), msgview);
+	g_signal_connect(G_OBJECT(window), "key_press_event",
+			 G_CALLBACK(key_pressed), msgview);
 
 	messageview_add_toolbar(msgview, window);
 
@@ -676,7 +658,7 @@ static gint disposition_notification_send(MsgInfo *msginfo)
 		unlink(tmp);
 		return -1;
 	}
-	
+		
 	if (prefs_common.work_offline)
 		if (alertpanel(_("Offline warning"), 
 			       _("You're working offline. Override?"),
@@ -760,6 +742,7 @@ gint messageview_show(MessageView *messageview, MsgInfo *msginfo,
 	textview_set_all_headers(messageview->mimeview->textview, all_headers);
 
 	mimeview_show_message(messageview->mimeview, mimeinfo, file);
+	messageview_set_position(messageview, 0);
 
 	if (messageview->msginfo->partial_recv)
 		partial_recv_show(messageview->noticeview, 
@@ -924,17 +907,41 @@ void messageview_copy_clipboard(MessageView *messageview)
 	TextView *text;
 
 	text = messageview_get_current_textview(messageview);
-	if (text)
-		gtk_editable_copy_clipboard(GTK_EDITABLE(text->text));
+	if (text) {
+		GtkTextView *textview = GTK_TEXT_VIEW(text->text);
+		GtkTextBuffer *buffer = gtk_text_view_get_buffer(textview);
+		GtkClipboard *clipboard
+			= gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
+
+		gtk_text_buffer_copy_clipboard(buffer, clipboard);
+	}
 }
 
 void messageview_select_all(MessageView *messageview)
 {
 	TextView *text;
+	GtkTextView *edit = NULL;
+	GtkTextBuffer *textbuf;
+	gint body_pos = 0;
+	GtkTextIter start, end;
+	
+	g_return_if_fail(messageview != NULL);
 
 	text = messageview_get_current_textview(messageview);
-	if (text)
-		gtk_editable_select_region(GTK_EDITABLE(text->text), 0, -1);
+	g_return_if_fail(text != NULL);
+
+	edit = GTK_TEXT_VIEW(text->text);
+	g_return_if_fail(edit != NULL);
+
+	textbuf = gtk_text_view_get_buffer(edit);
+	g_return_if_fail(textbuf != NULL);
+
+	gtk_text_buffer_get_bounds(textbuf, &start, &end);
+	
+	gtk_text_buffer_move_mark_by_name(textbuf, 
+		"selection_bound", &start);
+	gtk_text_buffer_move_mark_by_name(textbuf, 
+		"insert", &end);
 }
 
 void messageview_set_position(MessageView *messageview, gint pos)
@@ -979,7 +986,7 @@ void messageview_save_as(MessageView *messageview)
 {
 	gchar *filename = NULL;
 	MsgInfo *msginfo;
-	gchar *src, *dest;
+	gchar *src, *dest, *tmp;
 
 	if (!messageview->msginfo) return;
 	msginfo = messageview->msginfo;
@@ -988,22 +995,24 @@ void messageview_save_as(MessageView *messageview)
 		Xstrdup_a(filename, msginfo->subject, return);
 		subst_for_filename(filename);
 	}
-	dest = filesel_select_file(_("Save as"), filename);
+	dest = filesel_select_file_save(_("Save as"), filename);
 	if (!dest) return;
 	if (is_file_exist(dest)) {
 		AlertValue aval;
 
 		aval = alertpanel(_("Overwrite"),
 				  _("Overwrite existing file?"),
-				  _("OK"), _("Cancel"), NULL);
+				  GTK_STOCK_OK, GTK_STOCK_CANCEL, NULL);
 		if (G_ALERTDEFAULT != aval) return;
 	}
 
 	src = procmsg_get_message_file(msginfo);
 	if (copy_file(src, dest, TRUE) < 0) {
-		alertpanel_error(_("Can't save the file `%s'."),
-				 g_basename(dest));
+		tmp =  g_path_get_basename(dest);
+		alertpanel_error(_("Can't save the file `%s'."), tmp);
+		g_free(tmp);
 	}
+	g_free(dest);
 	g_free(src);
 }
 
@@ -1021,11 +1030,12 @@ static void messageview_size_allocate_cb(GtkWidget *widget,
 	prefs_common.msgwin_height = allocation->height;
 }
 
-static void key_pressed(GtkWidget *widget, GdkEventKey *event,
+static gboolean key_pressed(GtkWidget *widget, GdkEventKey *event,
 			MessageView *messageview)
 {
 	if (event && event->keyval == GDK_Escape && messageview->window)
 		gtk_widget_destroy(messageview->window);
+	return FALSE;
 }
 
 void messageview_toggle_view_real(MessageView *messageview)
@@ -1069,7 +1079,7 @@ static void return_receipt_show(NoticeView *noticeview, MsgInfo *msginfo)
 	noticeview_set_text(noticeview, _("This message asks for a return receipt."));
 	noticeview_set_button_text(noticeview, _("Send receipt"));
 	noticeview_set_button_press_callback(noticeview,
-					     GTK_SIGNAL_FUNC(return_receipt_send_clicked),
+					     G_CALLBACK(return_receipt_send_clicked),
 					     (gpointer) msginfo);
 	noticeview_show(noticeview);
 }
@@ -1153,11 +1163,11 @@ static void partial_recv_show(NoticeView *noticeview, MsgInfo *msginfo)
 	g_free(text);
 	noticeview_set_button_text(noticeview, button1);
 	noticeview_set_button_press_callback(noticeview,
-		     GTK_SIGNAL_FUNC(button1_cb), (gpointer) msginfo);
+		     G_CALLBACK(button1_cb), (gpointer) msginfo);
 
 	noticeview_set_2ndbutton_text(noticeview, button2);
 	noticeview_set_2ndbutton_press_callback(noticeview,
-		     GTK_SIGNAL_FUNC(button2_cb), (gpointer) msginfo);
+		     G_CALLBACK(button2_cb), (gpointer) msginfo);
 
 	noticeview_show(noticeview);
 }
@@ -1188,7 +1198,7 @@ static void partial_recv_unmark_clicked(NoticeView *noticeview,
 
 static void select_account_cb(GtkWidget *w, gpointer data)
 {
-	*(gint*)data = GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(w)));
+	*(gint*)data = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(w), MENU_VAL_ID));
 }
 	
 static PrefsAccount *select_account_from_list(GList *ac_list)
@@ -1201,7 +1211,9 @@ static PrefsAccount *select_account_from_list(GList *ac_list)
 	g_return_val_if_fail(ac_list->data != NULL, NULL);
 	
 	optmenu = gtk_option_menu_new();
-	menu = gtkut_account_menu_new(ac_list, select_account_cb, &account_id);
+	menu = gtkut_account_menu_new(ac_list, 
+			G_CALLBACK(select_account_cb), 
+			&account_id);
 	if (!menu)
 		return NULL;
 	gtk_option_menu_set_menu(GTK_OPTION_MENU(optmenu), menu);
@@ -1231,7 +1243,8 @@ gchar *messageview_get_selection(MessageView *msgview)
 {
 	TextView *textview;
 	gchar *text = NULL;
-	GtkEditable *edit = NULL;
+	GtkTextView *edit = NULL;
+	GtkTextBuffer *textbuf;
 	gint body_pos = 0;
 	
 	g_return_val_if_fail(msgview != NULL, NULL);
@@ -1239,17 +1252,22 @@ gchar *messageview_get_selection(MessageView *msgview)
 	textview = messageview_get_current_textview(msgview);
 	g_return_val_if_fail(textview != NULL, NULL);
 
-	edit = GTK_EDITABLE(textview->text);
+	edit = GTK_TEXT_VIEW(textview->text);
 	g_return_val_if_fail(edit != NULL, NULL);
 	body_pos = textview->body_pos;
 
-	if (edit->has_selection)
-		text = gtkut_editable_get_selection(edit);
-	else if (msgview->filtered) 
-		text = gtk_editable_get_chars (edit, body_pos, -1);
-	else
+	textbuf = gtk_text_view_get_buffer(edit);
+
+	if (gtk_text_buffer_get_selection_bounds(textbuf, NULL, NULL))
+		return gtkut_text_view_get_selection(edit);
+	else if (msgview->filtered) {
+		GtkTextIter start_iter, end_iter;
+		gtk_text_buffer_get_iter_at_offset(textbuf, &start_iter, body_pos);
+		gtk_text_buffer_get_end_iter(textbuf, &end_iter);
+		gtk_text_buffer_get_text(textbuf, &start_iter, &end_iter, FALSE);
+	} else
 		text = NULL;
-	
+
 	return text;
 }
 
