@@ -24,6 +24,7 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <dirent.h>
 #include <sys/stat.h>
 #include <unistd.h>
@@ -36,12 +37,12 @@
 #  include <sys/time.h>
 #endif
 
-#include "intl.h"
 #include "folder.h"
 #include "mh.h"
 #include "procmsg.h"
 #include "procheader.h"
 #include "utils.h"
+#include "codeconv.h"
 
 static void	mh_folder_init		(Folder		*folder,
 					 const gchar	*name,
@@ -97,6 +98,8 @@ static gchar   *mh_get_new_msg_filename		(FolderItem	*dest);
 static MsgInfo *mh_parse_msg			(const gchar	*file,
 						 FolderItem	*item);
 static void	mh_remove_missing_folder_items	(Folder		*folder);
+static gchar	*mh_filename_from_utf8		(const gchar	*path);
+static gchar	*mh_filename_to_utf8		(const gchar	*path);
 static void	mh_scan_tree_recursive		(FolderItem	*item);
 
 static gboolean mh_rename_folder_func		(GNode		*node,
@@ -182,8 +185,7 @@ gboolean mh_scan_required(Folder *folder, FolderItem *item)
 		return FALSE;
 	}
 
-	if ((s.st_mtime > item->mtime) &&
-		(s.st_mtime - 3600 != item->mtime)) {
+	if (s.st_mtime > item->mtime) {
 		debug_print("MH scan required, folder updated: %s (%ld > %ld)\n",
 			    path,
 			    s.st_mtime,
@@ -287,7 +289,7 @@ static gchar *mh_fetch_msg(Folder *folder, FolderItem *item, gint num)
 
 	path = folder_item_get_path(item);
 	file = g_strconcat(path, G_DIR_SEPARATOR_S, itos(num), NULL);
-	g_free(path);
+
 	if (!is_file_exist(file)) {
 		g_free(file);
 		return NULL;
@@ -516,9 +518,7 @@ static gboolean mh_is_msg_changed(Folder *folder, FolderItem *item,
 
 	if (stat(itos(msginfo->msgnum), &s) < 0 ||
 	    msginfo->size  != s.st_size ||
-		(msginfo->mtime - s.st_mtime != 0) &&
-		(msginfo->mtime - s.st_mtime != 3600) &&
-		(msginfo->mtime - s.st_mtime != -3600))
+	    msginfo->mtime != s.st_mtime)
 		return TRUE;
 
 	return FALSE;
@@ -619,7 +619,7 @@ static gchar *mh_item_get_path(Folder *folder, FolderItem *item)
 static FolderItem *mh_create_folder(Folder *folder, FolderItem *parent,
 				    const gchar *name)
 {
-	gchar *path;
+	gchar *path, *real_name;
 	gchar *fullpath;
 	FolderItem *new_item;
 	gchar *mh_sequences_filename;
@@ -634,7 +634,9 @@ static FolderItem *mh_create_folder(Folder *folder, FolderItem *parent,
 		if (make_dir_hier(path) != 0)
 			return NULL;
 		
-	fullpath = g_strconcat(path, G_DIR_SEPARATOR_S, name, NULL);
+	real_name = mh_filename_from_utf8(name);
+	fullpath = g_strconcat(path, G_DIR_SEPARATOR_S, real_name, NULL);
+	g_free(real_name);
 	g_free(path);
 
 	if (make_dir(fullpath) < 0) {
@@ -669,9 +671,10 @@ static FolderItem *mh_create_folder(Folder *folder, FolderItem *parent,
 static gint mh_rename_folder(Folder *folder, FolderItem *item,
 			     const gchar *name)
 {
+ 	gchar *real_name;
 	gchar *oldpath;
 	gchar *dirname;
-	gchar *newpath;
+	gchar *newpath, *utf8newpath;
 	gchar *paths[2];
 
 	g_return_val_if_fail(folder != NULL, -1);
@@ -683,9 +686,10 @@ static gint mh_rename_folder(Folder *folder, FolderItem *item,
 	if (!is_dir_exist(oldpath))
 		make_dir_hier(oldpath);
 
-	dirname = g_dirname(oldpath);
-	newpath = g_strconcat(dirname, G_DIR_SEPARATOR_S, name, NULL);
-	g_free(dirname);
+	dirname = g_path_get_dirname(oldpath);
+	real_name = mh_filename_from_utf8(name);
+	newpath = g_strconcat(dirname, G_DIR_SEPARATOR_S, real_name, NULL);
+	g_free(real_name);
 
 	if (rename(oldpath, newpath) < 0) {
 		FILE_OP_ERROR(oldpath, "rename");
@@ -698,17 +702,18 @@ static gint mh_rename_folder(Folder *folder, FolderItem *item,
 	g_free(newpath);
 
 	if (strchr(item->path, G_DIR_SEPARATOR) != NULL) {
-		dirname = g_dirname(item->path);
-		newpath = g_strconcat(dirname, G_DIR_SEPARATOR_S, name, NULL);
+		dirname = g_path_get_dirname(item->path);
+		utf8newpath = g_strconcat(dirname, G_DIR_SEPARATOR_S,
+					  name, NULL);
 		g_free(dirname);
 	} else
-		newpath = g_strdup(name);
+		utf8newpath = g_strdup(name);
 
 	g_free(item->name);
 	item->name = g_strdup(name);
 
 	paths[0] = g_strdup(item->path);
-	paths[1] = newpath;
+	paths[1] = utf8newpath;
 	g_node_traverse(item->node, G_PRE_ORDER, G_TRAVERSE_ALL, -1,
 			mh_rename_folder_func, paths);
 
@@ -827,7 +832,7 @@ static void mh_scan_tree_recursive(FolderItem *item)
 	DIR *dp;
 	struct dirent *d;
 	struct stat s;
-	gchar *entry;
+ 	gchar *real_path, *entry, *utf8entry, *utf8name;
 	gint n_msg = 0;
 
 	g_return_if_fail(item != NULL);
@@ -835,11 +840,13 @@ static void mh_scan_tree_recursive(FolderItem *item)
 
 	folder = item->folder;
 
-	dp = opendir(item->path ? item->path : ".");
+	real_path = item->path ? mh_filename_from_utf8(item->path) : g_strdup(".");
+	dp = opendir(real_path);
 	if (!dp) {
-		FILE_OP_ERROR(item->path ? item->path : ".", "opendir");
+		FILE_OP_ERROR(real_path, "opendir");
 		return;
 	}
+	g_free(real_path);
 
 	debug_print("scanning %s ...\n",
 		    item->path ? item->path
@@ -850,11 +857,13 @@ static void mh_scan_tree_recursive(FolderItem *item)
 	while ((d = readdir(dp)) != NULL) {
 		if (d->d_name[0] == '.') continue;
 
+		utf8name = mh_filename_to_utf8(d->d_name);
 		if (item->path)
-			entry = g_strconcat(item->path, G_DIR_SEPARATOR_S,
-					    d->d_name, NULL);
+			utf8entry = g_strconcat(item->path, G_DIR_SEPARATOR_S,
+						utf8name, NULL);
 		else
-			entry = g_strdup(d->d_name);
+			utf8entry = g_strdup(utf8name);
+		entry = mh_filename_from_utf8(utf8entry);
 
 		if (
 #ifdef HAVE_DIRENT_D_TYPE
@@ -872,6 +881,8 @@ static void mh_scan_tree_recursive(FolderItem *item)
 #if 0
 			if (mh_is_maildir(entry)) {
 				g_free(entry);
+				g_free(utf8entry);
+				g_free(utf8name);
 				continue;
 			}
 #endif
@@ -886,7 +897,7 @@ static void mh_scan_tree_recursive(FolderItem *item)
 			}
 			if (!new_item) {
 				debug_print("new folder '%s' found.\n", entry);
-				new_item = folder_item_new(folder, d->d_name, entry);
+				new_item = folder_item_new(folder, utf8name, utf8entry);
 				folder_item_append(item, new_item);
 			}
 
@@ -918,6 +929,8 @@ static void mh_scan_tree_recursive(FolderItem *item)
 		} else if (to_number(d->d_name) != -1) n_msg++;
 
 		g_free(entry);
+		g_free(utf8entry);
+		g_free(utf8name);
 	}
 
 	closedir(dp);
@@ -969,3 +982,37 @@ static gboolean mh_rename_folder_func(GNode *node, gpointer data)
 
 	return FALSE;
 }
+
+#warning FIXME_GTK2 /* should we use g_filename_from_utf8()? */
+static gchar *mh_filename_from_utf8(const gchar *path)
+{
+	const gchar *src_codeset = CS_UTF_8;
+	const gchar *dest_codeset = conv_get_locale_charset_str();
+	gchar *real_path;
+
+	real_path = conv_codeset_strdup(path, src_codeset, dest_codeset);
+	if (!real_path) {
+		g_warning("mh_filename_from_utf8: faild to convert character set\n");
+		/* FIXME: show dialog? */
+		real_path = g_strdup(path);
+	}
+
+	return real_path;
+}
+
+#warning FIXME_GTK2 /* should we use g_filename_to_utf8()? */
+static gchar *mh_filename_to_utf8(const gchar *path)
+{
+	const gchar *src_codeset = conv_get_locale_charset_str();
+	const gchar *dest_codeset = CS_UTF_8;
+	gchar *utf8path;
+
+	utf8path = conv_codeset_strdup(path, src_codeset, dest_codeset);
+	if (!utf8path) {
+		g_warning("mh_filename_to_utf8: faild to convert character set\n");
+		/* FIXME: show dialog? */
+		utf8path = g_strdup(path);
+	}
+
+	return utf8path;
+}
