@@ -24,10 +24,10 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 
 #include <gtk/gtk.h>
 
-#include "intl.h"
 #include "utils.h"
 #include "folder.h"
 #include "folderview.h"
@@ -164,7 +164,7 @@ static void delete_folder_cb(FolderView *folderview, guint action,
 	name = trim_string(item->name, 32);
 	AUTORELEASE_STR(name, {g_free(name); return;});
 	message = g_strdup_printf
-		(_("All folders and messages under `%s' will be deleted.\n"
+		(_("All folder(s) and message(s) under `%s' will be deleted.\n"
 		   "Do you really want to delete?"), name);
 	avalue = alertpanel(_("Delete folder"), message,
 			    _("Yes"), _("+No"), NULL);
@@ -206,6 +206,7 @@ static void rename_folder_cb(FolderView *folderview, guint action,
 	gchar *old_path;
 	gchar *old_id;
 	gchar *new_id;
+	gchar *base;
 
 	item = folderview_get_selected_item(folderview);
 	g_return_if_fail(item != NULL);
@@ -214,10 +215,11 @@ static void rename_folder_cb(FolderView *folderview, guint action,
 
 	name = trim_string(item->name, 32);
 	message = g_strdup_printf(_("Input new name for `%s':"), name);
-	new_folder = input_dialog(_("Rename folder"), message,
-				  g_basename(item->path));
+	base = g_path_get_basename(item->path);
+	new_folder = input_dialog(_("Rename folder"), message, base);
 	g_free(message);
 	g_free(name);
+	g_free(base);
 	if (!new_folder) return;
 	AUTORELEASE_STR(new_folder, {g_free(new_folder); return;});
 
