@@ -24,6 +24,7 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtknotebook.h>
 #include <gtk/gtkscrolledwindow.h>
@@ -47,7 +48,6 @@
 #include <fnmatch.h>
 #endif
 
-#include "intl.h"
 #include "main.h"
 #include "mimeview.h"
 #include "textview.h"
@@ -141,7 +141,7 @@ static gboolean icon_list_select_by_number(MimeView	*mimeview,
 					   gint		 number);
 static void mime_toggle_button_cb 	(GtkWidget 	*button,
 					 MimeView 	*mimeview);
-static void part_button_pressed		(MimeView 	*mimeview, 
+static gboolean part_button_pressed	(MimeView 	*mimeview, 
 					 GdkEventButton *event, 
 					 MimeInfo 	*partinfo);
 static void icon_scroll_size_allocate_cb(GtkWidget 	*widget, 
@@ -201,7 +201,7 @@ MimeView *mimeview_create(MainWindow *mainwin)
 	gtk_widget_show(scrolledwin);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwin),
 				       GTK_POLICY_AUTOMATIC,
-				       GTK_POLICY_ALWAYS);
+				       GTK_POLICY_AUTOMATIC);
 
 	ctree = gtk_sctree_new_with_titles(N_MIMEVIEW_COLS, 0, titles);
 	gtk_widget_show(ctree);
@@ -216,16 +216,16 @@ MimeView *mimeview_create(MainWindow *mainwin)
 				       GTK_CAN_FOCUS);
 	gtk_container_add(GTK_CONTAINER(scrolledwin), ctree);
 
-	gtk_signal_connect(GTK_OBJECT(ctree), "tree_select_row",
-			   GTK_SIGNAL_FUNC(mimeview_selected), mimeview);
-	gtk_signal_connect(GTK_OBJECT(ctree), "button_release_event",
-			   GTK_SIGNAL_FUNC(mimeview_button_pressed), mimeview);
-	gtk_signal_connect(GTK_OBJECT(ctree), "key_press_event",
-			   GTK_SIGNAL_FUNC(mimeview_key_pressed), mimeview);
-	gtk_signal_connect(GTK_OBJECT (ctree),"start_drag",
-			   GTK_SIGNAL_FUNC (mimeview_start_drag), mimeview);
-	gtk_signal_connect(GTK_OBJECT(ctree), "drag_data_get",
-			   GTK_SIGNAL_FUNC(mimeview_drag_data_get), mimeview);
+	g_signal_connect(G_OBJECT(ctree), "tree_select_row",
+			 G_CALLBACK(mimeview_selected), mimeview);
+	g_signal_connect(G_OBJECT(ctree), "button_release_event",
+			 G_CALLBACK(mimeview_button_pressed), mimeview);
+	g_signal_connect(G_OBJECT(ctree), "key_press_event",
+			 G_CALLBACK(mimeview_key_pressed), mimeview);
+	g_signal_connect(G_OBJECT (ctree),"start_drag",
+			 G_CALLBACK (mimeview_start_drag), mimeview);
+	g_signal_connect(G_OBJECT(ctree), "drag_data_get",
+			 G_CALLBACK(mimeview_drag_data_get), mimeview);
 
 	mime_notebook = gtk_notebook_new();
         gtk_widget_show(mime_notebook);
@@ -246,16 +246,16 @@ MimeView *mimeview_create(MainWindow *mainwin)
 	arrow = gtk_arrow_new(GTK_ARROW_LEFT, GTK_SHADOW_NONE);
 	gtk_widget_show(arrow);
 	gtk_container_add(GTK_CONTAINER(mime_toggle), arrow);
-	gtk_signal_connect(GTK_OBJECT(mime_toggle), "toggled", 
-			   GTK_SIGNAL_FUNC(mime_toggle_button_cb), mimeview);
+	g_signal_connect(G_OBJECT(mime_toggle), "toggled", 
+			 G_CALLBACK(mime_toggle_button_cb), mimeview);
 
 	icon_mainbox = gtk_vbox_new(FALSE, 0);
 	gtk_widget_show(icon_mainbox);
 	gtk_box_pack_start(GTK_BOX(icon_mainbox), mime_toggle, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(icon_mainbox), icon_scroll, TRUE, TRUE, 3);
 	gtk_box_pack_end(GTK_BOX(icon_mainbox), scrollbutton, FALSE, FALSE, 0);
-	gtk_signal_connect(GTK_OBJECT(icon_mainbox), "size_allocate", 
-			GTK_SIGNAL_FUNC(icon_scroll_size_allocate_cb), mimeview);
+	g_signal_connect(G_OBJECT(icon_mainbox), "size_allocate", 
+			 G_CALLBACK(icon_scroll_size_allocate_cb), mimeview);
 	
 	ctree_mainbox = gtk_hbox_new(FALSE, 0);	
 	gtk_box_pack_start(GTK_BOX(ctree_mainbox), scrolledwin, TRUE, TRUE, 0);
@@ -336,14 +336,14 @@ void mimeview_show_message(MimeView *mimeview, MimeInfo *mimeinfo,
 
 	mimeview->file = g_strdup(file);
 
-	gtk_signal_handler_block_by_func(GTK_OBJECT(ctree), mimeview_selected,
-					 mimeview);
+	g_signal_handlers_block_by_func(G_OBJECT(ctree), mimeview_selected,
+					mimeview);
 
 	mimeview_set_multipart_tree(mimeview, mimeinfo, NULL);
 	icon_list_create(mimeview, mimeinfo);
 
-	gtk_signal_handler_unblock_by_func(GTK_OBJECT(ctree),
-					   mimeview_selected, mimeview);
+	g_signal_handlers_unblock_by_func(G_OBJECT(ctree),
+					  mimeview_selected, mimeview);
 
 	node = GTK_CTREE_NODE(GTK_CLIST(ctree)->row_list);
 	if (node) {
@@ -532,7 +532,7 @@ static MimeViewer *get_viewer_for_mimeinfo(MimeView *mimeview, MimeInfo *partinf
 	MimeViewer *viewer = NULL;
 
 	if ((partinfo->type == MIMETYPE_APPLICATION) &&
-            (!g_strcasecmp(partinfo->subtype, "octet-stream"))) {
+            (!g_ascii_strcasecmp(partinfo->subtype, "octet-stream"))) {
 		const gchar *filename;
 
 		filename = procmime_mimeinfo_get_parameter(partinfo, "filename");
@@ -584,12 +584,12 @@ static void mimeview_change_view_type(MimeView *mimeview, MimeViewType type)
 
 	switch (type) {
 	case MIMEVIEW_TEXT:
-		gtk_notebook_set_page(GTK_NOTEBOOK(mimeview->mime_notebook),
+		gtk_notebook_set_current_page(GTK_NOTEBOOK(mimeview->mime_notebook),
 			gtk_notebook_page_num(GTK_NOTEBOOK(mimeview->mime_notebook), 
 			GTK_WIDGET_PTR(textview)));
 		break;
 	case MIMEVIEW_VIEWER:
-		gtk_notebook_set_page(GTK_NOTEBOOK(mimeview->mime_notebook),
+		gtk_notebook_set_current_page(GTK_NOTEBOOK(mimeview->mime_notebook),
 			gtk_notebook_page_num(GTK_NOTEBOOK(mimeview->mime_notebook), 
 			GTK_WIDGET(mimeview->mimeviewer->get_widget(mimeview->mimeviewer))));
 		break;
@@ -629,7 +629,7 @@ static void display_full_info_cb(GtkWidget *widget, gpointer user_data);
 static void update_signature_noticeview(MimeView *mimeview, MimeInfo *mimeinfo)
 {
 	gchar *text = NULL, *button_text = NULL;
-	GtkSignalFunc func = NULL;
+	void  *func = NULL;
 	StockPixmap icon = STOCK_PIXMAP_PRIVACY_SIGNED;
 
 	g_return_if_fail(mimeview != NULL);
@@ -669,7 +669,7 @@ static void update_signature_noticeview(MimeView *mimeview, MimeInfo *mimeinfo)
 	noticeview_set_button_text(mimeview->siginfoview, button_text);
 	noticeview_set_button_press_callback(
 		mimeview->siginfoview,
-		func,
+		G_CALLBACK(func),
 		(gpointer) mimeview);
 	noticeview_set_icon(mimeview->siginfoview, icon);
 }
@@ -794,14 +794,14 @@ static gint mimeview_button_pressed(GtkWidget *widget, GdkEventButton *event,
 	}
 	part_button_pressed(mimeview, event, mimeview_get_selected_part(mimeview));
 
-	return TRUE;
+	return FALSE;
 }
 
 /* from gdkevents.c */
 #define DOUBLE_CLICK_TIME 250
 
-static void part_button_pressed(MimeView *mimeview, GdkEventButton *event, 
-				MimeInfo *partinfo)
+static gboolean part_button_pressed(MimeView *mimeview, GdkEventButton *event, 
+				    MimeInfo *partinfo)
 {
 	static MimeInfo *lastinfo;
 	static guint32 lasttime;
@@ -810,6 +810,7 @@ static void part_button_pressed(MimeView *mimeview, GdkEventButton *event,
 	    (event->button == 1 && (event->time - lasttime) < DOUBLE_CLICK_TIME && lastinfo == partinfo)) {
 		/* call external program for image, audio or html */
 		mimeview_launch(mimeview);
+		return TRUE;
 	} else if (event->button == 3) {
 		if (partinfo && (partinfo->type == MIMETYPE_TEXT ||
 				 partinfo->type == MIMETYPE_MESSAGE ||
@@ -822,23 +823,25 @@ static void part_button_pressed(MimeView *mimeview, GdkEventButton *event,
 					   "/Display as text", TRUE);
 		if (partinfo &&
 		    partinfo->type == MIMETYPE_APPLICATION &&
-		    !g_strcasecmp(partinfo->subtype, "octet-stream"))
+		    !g_ascii_strcasecmp(partinfo->subtype, "octet-stream"))
 			menu_set_sensitive(mimeview->popupfactory,
 					   "/Open", FALSE);
 		else
 			menu_set_sensitive(mimeview->popupfactory,
 					   "/Open", TRUE);
 
-		gtk_object_set_data(GTK_OBJECT(mimeview->popupmenu),
-				    "pop_partinfo", partinfo);
+		g_object_set_data(G_OBJECT(mimeview->popupmenu),
+				  "pop_partinfo", partinfo);
 				    
 		gtk_menu_popup(GTK_MENU(mimeview->popupmenu),
 			       NULL, NULL, NULL, NULL,
 			       event->button, event->time);
+		return TRUE;
 	}
 
 	lastinfo = partinfo;
 	lasttime = event->time;
+	return FALSE;
 }
 
 
@@ -851,11 +854,8 @@ void mimeview_pass_key_press_event(MimeView *mimeview, GdkEventKey *event)
 	if ((event->state & (GDK_MOD1_MASK|GDK_CONTROL_MASK)) != 0) break
 
 #define KEY_PRESS_EVENT_STOP() \
-	if (gtk_signal_n_emissions_by_name \
-		(GTK_OBJECT(ctree), "key_press_event") > 0) { \
-		gtk_signal_emit_stop_by_name(GTK_OBJECT(ctree), \
-					     "key_press_event"); \
-	}
+        g_signal_stop_emission_by_name(G_OBJECT(ctree), \
+                                       "key_press_event");
 
 static gint mimeview_key_pressed(GtkWidget *widget, GdkEventKey *event,
 				 MimeView *mimeview)
@@ -867,6 +867,11 @@ static gint mimeview_key_pressed(GtkWidget *widget, GdkEventKey *event,
 	if (!event) return FALSE;
 	if (!mimeview->opened) return FALSE;
 
+	summaryview = mimeview->messageview->mainwin->summaryview;
+	
+	if (summaryview && quicksearch_has_focus(summaryview->quicksearch))
+		return FALSE;
+		
 	switch (event->keyval) {
 	case GDK_space:
 		if (textview_scroll_page(mimeview->textview, FALSE))
@@ -891,8 +896,7 @@ static gint mimeview_key_pressed(GtkWidget *widget, GdkEventKey *event,
 		BREAK_ON_MODIFIER_KEY();
 		if (!GTK_CTREE_NODE_NEXT(mimeview->opened)) break;
 		KEY_PRESS_EVENT_STOP();
-
-		gtk_signal_emit_by_name(GTK_OBJECT(ctree), "scroll_vertical",
+		g_signal_emit_by_name(G_OBJECT(ctree), "scroll_vertical",
 					GTK_SCROLL_STEP_FORWARD, 0.0);
 		return TRUE;
 	case GDK_p:
@@ -900,8 +904,7 @@ static gint mimeview_key_pressed(GtkWidget *widget, GdkEventKey *event,
 		BREAK_ON_MODIFIER_KEY();
 		if (!GTK_CTREE_NODE_PREV(mimeview->opened)) break;
 		KEY_PRESS_EVENT_STOP();
-
-		gtk_signal_emit_by_name(GTK_OBJECT(ctree), "scroll_vertical",
+		g_signal_emit_by_name(G_OBJECT(ctree), "scroll_vertical",
 					GTK_SCROLL_STEP_BACKWARD, 0.0);
 		return TRUE;
 	case GDK_y:
@@ -929,7 +932,7 @@ static gint mimeview_key_pressed(GtkWidget *widget, GdkEventKey *event,
 	}
 
 	if (!mimeview->messageview->mainwin) return FALSE;
-	summaryview = mimeview->messageview->mainwin->summaryview;
+
 	summary_pass_key_press_event(summaryview, event);
 	return TRUE;
 }
@@ -941,7 +944,7 @@ static void mimeview_drag_data_get(GtkWidget	    *widget,
 				   guint	     time,
 				   MimeView	    *mimeview)
 {
-	gchar *filename, *uriname;
+	gchar *filename, *uriname, *tmp;
 	MimeInfo *partinfo;
 
 	if (!mimeview->opened) return;
@@ -950,12 +953,16 @@ static void mimeview_drag_data_get(GtkWidget	    *widget,
 	partinfo = mimeview_get_selected_part(mimeview);
 	if (!partinfo) return;
 
-	filename = g_basename(get_part_name(partinfo));
+	filename = g_path_get_basename(get_part_name(partinfo));
 	if (*filename == '\0') return;
 
+	tmp = filename;
+	
 	filename = g_strconcat(get_mime_tmp_dir(), G_DIR_SEPARATOR_S,
 			       filename, NULL);
 
+	g_free(tmp);
+	
 	if (procmime_get_part(filename, partinfo) < 0)
 		alertpanel_error
 			(_("Can't save the part of multipart message."));
@@ -985,6 +992,12 @@ gchar *mimeview_get_filename_for_part(MimeInfo *partinfo,
 	if (!filename || !*filename)
 		filename = g_strdup_printf("noname.%d", number);
 
+	if (!g_utf8_validate(filename, -1, NULL)) {
+		gchar *tmp = conv_filename_to_utf8(filename);
+		g_free(filename);
+		filename = tmp;
+	}
+	
 	subst_for_filename(filename);
 
 	fullname = g_strconcat
@@ -1005,7 +1018,7 @@ static gboolean mimeview_write_part(const gchar *filename,
 {
 	gchar *dir;
 	
-	dir= g_dirname(filename);
+	dir= g_path_get_dirname(filename);
 	if (!is_dir_exist(dir))
 		make_dir_hier(dir);
 	g_free(dir);
@@ -1016,8 +1029,8 @@ static gboolean mimeview_write_part(const gchar *filename,
 		
 		res = g_strdup_printf(_("Overwrite existing file '%s'?"),
 				      filename);
-		aval = alertpanel(_("Overwrite"), res, _("OK"), 
-				  _("Cancel"), NULL);
+		aval = alertpanel(_("Overwrite"), res, GTK_STOCK_OK, 
+				  GTK_STOCK_CANCEL, NULL);
 		g_free(res);					  
 		if (G_ALERTDEFAULT != aval) return FALSE;
 	}
@@ -1051,7 +1064,7 @@ static void mimeview_save_all(MimeView *mimeview)
 		startdir = g_strconcat(prefs_common.attach_save_dir,
 				       G_DIR_SEPARATOR_S, NULL);
 
-	dirname = filesel_select_file(_("Select destination folder"), startdir);
+	dirname = filesel_select_file_open_folder(_("Select destination folder"), startdir);
 	if (!dirname) {
 		if (startdir) g_free(startdir);
 		return;
@@ -1075,7 +1088,7 @@ static void mimeview_save_all(MimeView *mimeview)
 		if (partinfo && partinfo->type == MIMETYPE_TEXT)
 			partinfo = procmime_mimeinfo_next(partinfo);
 	}
-
+		
 	while (partinfo != NULL) {
 		if (partinfo->type != MIMETYPE_MESSAGE &&
 		    partinfo->type != MIMETYPE_MULTIPART &&
@@ -1114,21 +1127,27 @@ static void mimeview_save_as(MimeView *mimeview)
 
 	partinfo = mimeview_get_selected_part(mimeview);
 	if (!partinfo) { 
-		partinfo = (MimeInfo *) gtk_object_get_data
-			(GTK_OBJECT(mimeview->popupmenu),
+		partinfo = (MimeInfo *) g_object_get_data
+			 (G_OBJECT(mimeview->popupmenu),
 			 "pop_partinfo");
-		gtk_object_set_data(GTK_OBJECT(mimeview->popupmenu),
-				    "pop_partinfo", NULL);
+		g_object_set_data(G_OBJECT(mimeview->popupmenu),
+				  "pop_partinfo", NULL);
 	}			 
 	g_return_if_fail(partinfo != NULL);
 	
 	if (get_part_name(partinfo) == NULL) {
 		return;
 	}
-
 	partname = g_strdup(get_part_name(partinfo));
-	subst_for_filename(partname);
+	
+	if (!g_utf8_validate(partname, -1, NULL)) {
+		gchar *tmp = conv_filename_to_utf8(partname);
+		g_free(partname);
+		partname = tmp;
+	}
 
+	subst_for_filename(partname);
+	
 	if (prefs_common.attach_save_dir)
 		filepath = g_strconcat(prefs_common.attach_save_dir,
 				       G_DIR_SEPARATOR_S, partname, NULL);
@@ -1137,7 +1156,7 @@ static void mimeview_save_as(MimeView *mimeview)
 
 	g_free(partname);
 
-	filename = filesel_select_file(_("Save as"), filepath);
+	filename = filesel_select_file_save(_("Save as"), filepath);
 	if (!filename) {
 		g_free(filepath);
 		return;
@@ -1145,7 +1164,7 @@ static void mimeview_save_as(MimeView *mimeview)
 
 	mimeview_write_part(filename, partinfo);
 
-	filedir = g_dirname(filename);
+	filedir = g_path_get_dirname(filename);
 	if (filedir && strcmp(filedir, ".")) {
 		if (prefs_common.attach_save_dir)
 			g_free(prefs_common.attach_save_dir);
@@ -1164,11 +1183,11 @@ static void mimeview_display_as_text(MimeView *mimeview)
 
 	partinfo = mimeview_get_selected_part(mimeview);
 	if (!partinfo)  {
-		partinfo = (MimeInfo *) gtk_object_get_data
-			(GTK_OBJECT(mimeview->popupmenu),
+		partinfo = (MimeInfo *) g_object_get_data
+			(G_OBJECT(mimeview->popupmenu),
 			 "pop_partinfo");
-		gtk_object_set_data(GTK_OBJECT(mimeview->popupmenu),
-				    "pop_partinfo", NULL);
+		g_object_set_data(G_OBJECT(mimeview->popupmenu),
+				  "pop_partinfo", NULL);
 	
 	}			 
 	g_return_if_fail(partinfo != NULL);
@@ -1185,11 +1204,11 @@ static void mimeview_launch(MimeView *mimeview)
 
 	partinfo = mimeview_get_selected_part(mimeview);
 	if (!partinfo) { 
-		partinfo = (MimeInfo *) gtk_object_get_data
-			(GTK_OBJECT(mimeview->popupmenu),
+		partinfo = (MimeInfo *) g_object_get_data
+			(G_OBJECT(mimeview->popupmenu),
 			 "pop_partinfo");
-		gtk_object_set_data(GTK_OBJECT(mimeview->popupmenu),
-				    "pop_partinfo", NULL);
+		g_object_set_data(G_OBJECT(mimeview->popupmenu),
+				  "pop_partinfo", NULL);
 	}			 
 	g_return_if_fail(partinfo != NULL);
 
@@ -1215,11 +1234,11 @@ static void mimeview_open_with(MimeView *mimeview)
 
 	partinfo = mimeview_get_selected_part(mimeview);
 	if (!partinfo) { 
-		partinfo = (MimeInfo *) gtk_object_get_data
-			(GTK_OBJECT(mimeview->popupmenu),
+		partinfo = (MimeInfo *) g_object_get_data
+			(G_OBJECT(mimeview->popupmenu),
 			 "pop_partinfo");
-		gtk_object_set_data(GTK_OBJECT(mimeview->popupmenu),
-				    "pop_partinfo", NULL);
+		g_object_set_data(G_OBJECT(mimeview->popupmenu),
+				  "pop_partinfo", NULL);
 	}			 
 	g_return_if_fail(partinfo != NULL);
 
@@ -1271,7 +1290,7 @@ static void mimeview_view_file(const gchar *filename, MimeInfo *partinfo,
 		cmd = cmdline;
 		def_cmd = NULL;
 	} else if (MIMETYPE_APPLICATION == partinfo->type &&
-		   !g_strcasecmp(partinfo->subtype, "octet-stream")) {
+		   !g_ascii_strcasecmp(partinfo->subtype, "octet-stream")) {
 		return;
 	} else if (MIMETYPE_IMAGE == partinfo->type) {
 		cmd = prefs_common.mime_image_viewer;
@@ -1346,11 +1365,11 @@ static gboolean icon_clicked_cb (GtkWidget *button, GdkEventButton *event, MimeV
 	gint      num;
 	MimeInfo *partinfo;
 
-	num      = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(button), "icon_number"));
-	partinfo = gtk_object_get_data(GTK_OBJECT(button), "partinfo");
+	num      = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(button), "icon_number"));
+	partinfo = g_object_get_data(G_OBJECT(button), "partinfo");
 
 	icon_selected(mimeview, num, partinfo);
- 	gtk_widget_grab_focus(button);
+	gtk_widget_grab_focus(button);
 	if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button))) {
 		toggle_icon(GTK_TOGGLE_BUTTON(button), mimeview);
 		if (event->button == 2 || event->button == 3)
@@ -1373,11 +1392,9 @@ static void icon_selected (MimeView *mimeview, gint num, MimeInfo *partinfo)
 
 #undef  KEY_PRESS_EVENT_STOP
 #define KEY_PRESS_EVENT_STOP() \
-	if (gtk_signal_n_emissions_by_name \
-		(GTK_OBJECT(button), "key_press_event") > 0) { \
-		gtk_signal_emit_stop_by_name(GTK_OBJECT(button), \
-					     "key_press_event"); \
-	}
+        g_signal_stop_emission_by_name(G_OBJECT(button), \
+                                       "key_press_event");
+
 static gint icon_key_pressed(GtkWidget *button, GdkEventKey *event,
 			     MimeView *mimeview)
 {
@@ -1386,8 +1403,8 @@ static gint icon_key_pressed(GtkWidget *button, GdkEventKey *event,
 	SummaryView  *summaryview;
 	TextView     *textview;
 
-	num      = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(button), "icon_number"));
-	partinfo = gtk_object_get_data(GTK_OBJECT(button), "partinfo");
+	num      = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(button), "icon_number"));
+	partinfo = g_object_get_data(G_OBJECT(button), "partinfo");
 	
 	if (!event) return FALSE;
 
@@ -1477,7 +1494,7 @@ static void toggle_icon(GtkToggleButton *button, MimeView *mimeview)
 {
 	GList *child;
 	
-	child = gtk_container_children(GTK_CONTAINER(mimeview->icon_vbox));
+	child = gtk_container_get_children(GTK_CONTAINER(mimeview->icon_vbox));
 	for (; child != NULL; child = g_list_next(child)) {
 		if (GTK_IS_TOGGLE_BUTTON(child->data) && 
 		    GTK_TOGGLE_BUTTON(child->data) != button &&
@@ -1507,17 +1524,17 @@ static void icon_list_append_icon (MimeView *mimeview, MimeInfo *mimeinfo)
 	mimeview->icon_count++;
 	button = gtk_toggle_button_new();
 	gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
-	gtk_object_set_data(GTK_OBJECT(button), "icon_number", 
-		GINT_TO_POINTER(mimeview->icon_count));
-	gtk_object_set_data(GTK_OBJECT(button), "partinfo", 
-		mimeinfo);
+	g_object_set_data(G_OBJECT(button), "icon_number", 
+			  GINT_TO_POINTER(mimeview->icon_count));
+	g_object_set_data(G_OBJECT(button), "partinfo", 
+		          mimeinfo);
 	
 	switch (mimeinfo->type) {
 		
 	case MIMETYPE_TEXT:
-		if (mimeinfo->subtype && !g_strcasecmp(mimeinfo->subtype, "html"))
+		if (mimeinfo->subtype && !g_ascii_strcasecmp(mimeinfo->subtype, "html"))
 			stockp = STOCK_PIXMAP_MIME_TEXT_HTML;
-		else if  (mimeinfo->subtype && !g_strcasecmp(mimeinfo->subtype, "enriched"))
+		else if  (mimeinfo->subtype && !g_ascii_strcasecmp(mimeinfo->subtype, "enriched"))
 			stockp = STOCK_PIXMAP_MIME_TEXT_ENRICHED;
 		else
 			stockp = STOCK_PIXMAP_MIME_TEXT_PLAIN;
@@ -1596,9 +1613,16 @@ static void icon_list_append_icon (MimeView *mimeview, MimeInfo *mimeinfo)
 			to_human_readable(mimeinfo->length), NULL);
 	g_free(content_type);
 	if (desc && *desc) {
-		tiptmp = g_strjoin("\n", desc, tip, NULL);
+		gchar *tmp = NULL;
+		if (!g_utf8_validate(desc, -1, NULL)) {
+			tmp = conv_filename_to_utf8(desc);
+		} else {
+			tmp = g_strdup(desc);
+		}
+		tiptmp = g_strjoin("\n", tmp, tip, NULL);
 		g_free(tip);
 		tip = tiptmp;
+		g_free(tmp);
 	}
 	if (sigshort && *sigshort) {
 		tiptmp = g_strjoin("\n", tip, sigshort, NULL);
@@ -1610,10 +1634,10 @@ static void icon_list_append_icon (MimeView *mimeview, MimeInfo *mimeinfo)
 	gtk_tooltips_set_tip(mimeview->tooltips, button, tip, NULL);
 	g_free(tip);
 	gtk_widget_show_all(button);
-	gtk_signal_connect(GTK_OBJECT(button), "button_release_event", 
-			   GTK_SIGNAL_FUNC(icon_clicked_cb), mimeview);
-	gtk_signal_connect(GTK_OBJECT(button), "key_press_event", 
-			   GTK_SIGNAL_FUNC(icon_key_pressed), mimeview);
+	g_signal_connect(G_OBJECT(button), "button_release_event", 
+			 G_CALLBACK(icon_clicked_cb), mimeview);
+	g_signal_connect(G_OBJECT(button), "key_press_event", 
+			 G_CALLBACK(icon_key_pressed), mimeview);
 	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
 
 }
@@ -1630,8 +1654,7 @@ static void icon_list_clear (MimeView *mimeview)
 	}
 	mimeview->icon_count = 0;
 	adj  = gtk_layout_get_vadjustment(GTK_LAYOUT(mimeview->icon_scroll));
-	adj->value = adj->lower;
-	gtk_signal_emit_by_name(GTK_OBJECT (adj), "value_changed");
+	gtk_adjustment_set_value(adj, adj->lower);
 }
 
 static void icon_list_toggle_by_mime_info(MimeView	*mimeview,
@@ -1642,8 +1665,8 @@ static void icon_list_toggle_by_mime_info(MimeView	*mimeview,
 	child = gtk_container_children(GTK_CONTAINER(mimeview->icon_vbox));
 	for (; child != NULL; child = g_list_next(child)) {
 		if (GTK_IS_TOGGLE_BUTTON(child->data) &&  
-		    gtk_object_get_data(GTK_OBJECT(child->data),
-					"partinfo") == (gpointer)mimeinfo) {
+		    g_object_get_data(G_OBJECT(child->data),
+			 	      "partinfo") == (gpointer)mimeinfo) {
 			toggle_icon(GTK_TOGGLE_BUTTON(child->data), mimeview);
 			gtk_toggle_button_set_active
 				(GTK_TOGGLE_BUTTON(child->data), TRUE);
@@ -1665,11 +1688,11 @@ static gboolean icon_list_select_by_number(MimeView	*mimeview,
 	child = gtk_container_children(GTK_CONTAINER(mimeview->icon_vbox));
 	for (; child != NULL; child = g_list_next(child)) {
 		if (GTK_IS_TOGGLE_BUTTON(child->data) &&  
-		    GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(child->data),
+		    GPOINTER_TO_INT(g_object_get_data(G_OBJECT(child->data),
 					"icon_number")) == number) {
 			icon_selected(mimeview, number,
-				      (MimeInfo*)gtk_object_get_data(GTK_OBJECT(child->data),
-								     "partinfo"));
+				      (MimeInfo*)g_object_get_data(G_OBJECT(child->data),
+								   "partinfo"));
 			toggle_icon(GTK_TOGGLE_BUTTON(child->data), mimeview);
 			gtk_toggle_button_set_active
 				(GTK_TOGGLE_BUTTON(child->data), TRUE);
@@ -1682,7 +1705,7 @@ static gboolean icon_list_select_by_number(MimeView	*mimeview,
 }
 
 static void icon_scroll_size_allocate_cb(GtkWidget *widget, 
-					GtkAllocation *size, MimeView *mimeview)
+					 GtkAllocation *size, MimeView *mimeview)
 {
 	GtkAllocation *mainbox_size;
 	GtkAllocation *vbox_size;
@@ -1694,11 +1717,7 @@ static void icon_scroll_size_allocate_cb(GtkWidget *widget,
 	mainbox_size = &mimeview->icon_mainbox->allocation;
 	vbox_size = &mimeview->icon_vbox->allocation;
 	layout_size = &mimeview->icon_scroll->allocation;
-	
-	/* centralise the vbox */
-	gtk_layout_move(GTK_LAYOUT(mimeview->icon_scroll), mimeview->icon_vbox, 
-			(mainbox_size->width - vbox_size->width)/2, 0);
-	
+		
 	gtk_layout_set_size(GTK_LAYOUT(mimeview->icon_scroll), 
 			    GTK_LAYOUT(mimeview->icon_scroll)->width, 
 			    MAX(vbox_size->height, layout_size->height));
@@ -1708,21 +1727,23 @@ static void icon_scroll_size_allocate_cb(GtkWidget *widget,
 static void icon_list_create(MimeView *mimeview, MimeInfo *mimeinfo)
 {
 	GtkRequisition size;
-	gint           width;
+
 	g_return_if_fail(mimeinfo != NULL);
 
 	while (mimeinfo != NULL) {
 		if (mimeinfo->type != MIMETYPE_MULTIPART)
 			icon_list_append_icon(mimeview, mimeinfo);
 		if (mimeinfo->node->children != NULL)
-			icon_list_create(mimeview, (MimeInfo *) mimeinfo->node->children->data);
-		mimeinfo = mimeinfo->node->next != NULL ? (MimeInfo *) mimeinfo->node->next->data : NULL;
+			icon_list_create(mimeview, 
+				(MimeInfo *) mimeinfo->node->children->data);
+		mimeinfo = mimeinfo->node->next != NULL 
+			 ? (MimeInfo *) mimeinfo->node->next->data 
+			 : NULL;
 	}
 	gtk_widget_size_request(mimeview->icon_vbox, &size);
-	width = size.width + 4;
-	if (width > mimeview->icon_mainbox->requisition.width) {
-		gtk_widget_set_usize(mimeview->icon_mainbox, 
-				     width, -1);
+	if (size.width > mimeview->icon_mainbox->requisition.width) {
+		gtk_widget_set_size_request(mimeview->icon_mainbox, 
+					    size.width, -1);
 	}
 
 }
