@@ -20,10 +20,10 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 
 #include <time.h>
 
-#include "intl.h"
 #include "msgcache.h"
 #include "utils.h"
 #include "procmsg.h"
@@ -136,7 +136,7 @@ void msgcache_update_msg(MsgCache *cache, MsgInfo *msginfo)
 	g_return_if_fail(msginfo != NULL);
 
 	oldmsginfo = g_hash_table_lookup(cache->msgnum_table, &msginfo->msgnum);
-	if (oldmsginfo && oldmsginfo->msgid)
+	if(oldmsginfo && oldmsginfo->msgid) 
 		g_hash_table_remove(cache->msgid_table, oldmsginfo->msgid);
 	if (oldmsginfo) {
 		g_hash_table_remove(cache->msgnum_table, &oldmsginfo->msgnum);
@@ -413,7 +413,7 @@ MsgCache *msgcache_read_cache(FolderItem *item, const gchar *cache_file)
 
 	if (msgcache_read_cache_data_str(fp, &srccharset, NULL) < 0)
 		return NULL;
-	dstcharset = conv_get_current_charset_str();
+	dstcharset = CS_UTF_8;
 	if (srccharset == NULL || dstcharset == NULL) {
 		conv = NULL;
 	} else if (strcmp(srccharset, dstcharset) == 0) {
@@ -484,7 +484,7 @@ MsgCache *msgcache_read_cache(FolderItem *item, const gchar *cache_file)
 		g_free(conv);
 	}
 
-	if (error) {
+	if(error) {
 		msgcache_destroy(cache);
 		return NULL;
 	}
@@ -585,7 +585,7 @@ gint msgcache_write(const gchar *cache_file, const gchar *mark_file, MsgCache *c
 	if (write_fps.cache_fp == NULL)
 		return -1;
 
-	WRITE_CACHE_DATA(conv_get_current_charset_str(), write_fps.cache_fp);
+	WRITE_CACHE_DATA(CS_UTF_8, write_fps.cache_fp);
 
 	write_fps.mark_fp = msgcache_open_data_file(mark_file, MARK_VERSION,
 		DATA_WRITE, NULL, 0);
