@@ -24,6 +24,7 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
@@ -31,7 +32,6 @@
 #include <unistd.h>
 #include <time.h>
 
-#include "intl.h"
 #include "news.h"
 #include "nntp.h"
 #include "socket.h"
@@ -40,6 +40,7 @@
 #include "procheader.h"
 #include "folder.h"
 #include "session.h"
+#include "statusbar.h"
 #include "codeconv.h"
 #include "utils.h"
 #include "prefs_common.h"
@@ -363,7 +364,7 @@ static void news_group_info_free(NewsGroupInfo *ginfo)
 static gint news_group_info_compare(NewsGroupInfo *ginfo1,
 				    NewsGroupInfo *ginfo2)
 {
-	return g_strcasecmp(ginfo1->name, ginfo2->name);
+	return g_ascii_strcasecmp(ginfo1->name, ginfo2->name);
 }
 
 GSList *news_get_group_list(Folder *folder)
@@ -577,7 +578,7 @@ static gint news_select_group(NNTPSession *session, const gchar *group,
 	gint num_, first_, last_;
 
 	if (!num || !first || !last) {
-		if (session->group && g_strcasecmp(session->group, group) == 0)
+		if (session->group && g_ascii_strcasecmp(session->group, group) == 0)
 			return NN_SUCCESS;
 		num = &num_;
 		first = &first_;
@@ -607,7 +608,6 @@ static gint news_select_group(NNTPSession *session, const gchar *group,
 static MsgInfo *news_parse_xover(const gchar *xover_str)
 {
 	MsgInfo *msginfo;
-	gchar buf[NNTPBUFSIZE];
 	gchar *subject, *sender, *size, *line, *date, *msgid, *ref, *tmp;
 	gchar *p;
 	gint num, size_int, line_int;
@@ -647,12 +647,10 @@ static MsgInfo *news_parse_xover(const gchar *xover_str)
 	msginfo->date = g_strdup(date);
 	msginfo->date_t = procheader_date_parse(NULL, date, 0);
 
-	conv_unmime_header(buf, sizeof(buf), sender, NULL);
-	msginfo->from = g_strdup(buf);
-	msginfo->fromname = procheader_get_fromname(buf);
+        msginfo->from = conv_unmime_header(sender, NULL);
+	msginfo->fromname = procheader_get_fromname(msginfo->from);
 
-	conv_unmime_header(buf, sizeof(buf), subject, NULL);
-	msginfo->subject = g_strdup(buf);
+        msginfo->subject = conv_unmime_header(subject, NULL);
 
 	extract_parenthesis(msgid, '<', '>');
 	remove_space(msgid);
