@@ -24,10 +24,10 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 
 #include <gtk/gtk.h>
 
-#include "intl.h"
 #include "utils.h"
 #include "folder.h"
 #include "folderview.h"
@@ -131,7 +131,7 @@ static void subscribe_newsgroup_cb(FolderView *folderview, guint action, GtkWidg
 
 		item = FOLDER_ITEM(gnode->data);
 		if (g_slist_find_custom(new_subscr, item->path,
-					(GCompareFunc)g_strcasecmp) != NULL) {
+					(GCompareFunc)g_ascii_strcasecmp) != NULL) {
 			gnode = next;
 			continue;
 		}
