@@ -27,6 +27,7 @@
 #include <sys/wait.h>
 
 #include <glib.h>
+#include <glib/gi18n.h>
 
 #if HAVE_LOCALE_H
 #  include <locale.h>
@@ -41,7 +42,6 @@
 #include "folder.h"
 #include "prefs.h"
 #include "prefs_gtk.h"
-#include "intl.h"
 
 #include "libspamc.h"
 #include "spamassassin.h"
@@ -278,6 +278,8 @@ void spamassassin_set_message_callback(MessageCallback callback)
 
 gint plugin_init(gchar **error)
 {
+	gchar *rcpath;
+
 	if ((sylpheed_get_version() > VERSION_NUMERIC)) {
 		*error = g_strdup("Your sylpheed version is newer than the version the plugin was built with");
 		return -1;
@@ -296,13 +298,14 @@ gint plugin_init(gchar **error)
 
 	username = g_get_user_name();
 	if (username == NULL) {
-		hooks_unregister_hook(MAIL_FILTERING_HOOKLIST, hook_id);
 		*error = g_strdup("Failed to get username");
 		return -1;
 	}
 
 	prefs_set_default(param);
-	prefs_read_config(param, "SpamAssassin", COMMON_RC);
+	rcpath = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, COMMON_RC, NULL);
+	prefs_read_config(param, "SpamAssassin", rcpath, NULL);
+	g_free(rcpath);
 
 	debug_print("Spamassassin plugin loaded\n");
 
