@@ -17,9 +17,14 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
+
 #include <stdio.h>
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
 #include "common/sylpheed.h"
@@ -30,7 +35,6 @@
 #include "folder.h"
 #include "mainwindow.h"
 #include "gtkutils.h"
-#include "intl.h"
 #include "menu.h"
 #include "toolbar.h"
 #include "prefs_common.h"
@@ -75,11 +79,12 @@ typedef enum
 	TRAYICON_NOTHING,
 } TrayIconType;
 
-static void trayicon_get_cb	    ( gpointer data, guint action, GtkWidget *widget );
-static void trayicon_get_all_cb	    ( gpointer data, guint action, GtkWidget *widget );
-static void trayicon_compose_cb	    ( gpointer data, guint action, GtkWidget *widget );
-static void trayicon_addressbook_cb ( gpointer data, guint action, GtkWidget *widget );
-static void trayicon_exit_cb	    ( gpointer data, guint action, GtkWidget *widget );
+static void trayicon_get_cb	    (gpointer data, guint action, GtkWidget *widget);
+static void trayicon_get_all_cb	    (gpointer data, guint action, GtkWidget *widget);
+static void trayicon_compose_cb	    (gpointer data, guint action, GtkWidget *widget);
+static void trayicon_addressbook_cb (gpointer data, guint action, GtkWidget *widget);
+static void trayicon_exit_cb	    (gpointer data, guint action, GtkWidget *widget);
+static void resize_cb		    (GtkWidget *widget, GtkRequisition *req, gpointer user_data);
 
 static GtkItemFactoryEntry trayicon_popup_menu_entries[] =
 {
@@ -96,6 +101,7 @@ static void set_trayicon_pixmap(TrayIconType icontype)
 {
 	GdkPixmap *pixmap = NULL;
 	GdkBitmap *bitmap = NULL;
+	static GdkPixmap *last_pixmap = NULL;
 
 	switch(icontype) {
 	case TRAYICON_NEW:
@@ -120,8 +126,13 @@ static void set_trayicon_pixmap(TrayIconType icontype)
 		break;
 	}
 
-	gtk_pixmap_set(GTK_PIXMAP(image), pixmap, bitmap);
+	if (pixmap == last_pixmap)
+		return;
+
+	gtk_image_set_from_pixmap(GTK_IMAGE(image), pixmap, bitmap);
 	gtk_widget_shape_combine_mask(GTK_WIDGET(trayicon), bitmap, GTK_WIDGET(image)->allocation.x, GTK_WIDGET(image)->allocation.y);
+
+	last_pixmap = pixmap;
 }
 
 static void update(void)
@@ -155,6 +166,12 @@ static gboolean folder_item_update_hook(gpointer source, gpointer data)
 	return FALSE;
 }
 
+static void resize_cb(GtkWidget *widget, GtkRequisition *req,
+		      gpointer user_data)
+{
+	update();
+}
+
 static gboolean click_cb(GtkWidget * widget,
 		         GdkEventButton * event, gpointer user_data)
 {
@@ -183,11 +200,6 @@ static gboolean click_cb(GtkWidget * widget,
 	return TRUE;
 }
 
-static void resize_cb(GtkWidget *widget, GtkAllocation *allocation)
-{
-	update();
-}
-
 static void create_trayicon(void);
 
 static void destroy_cb(GtkWidget *widget, gpointer *data)
@@ -199,9 +211,11 @@ static void destroy_cb(GtkWidget *widget, gpointer *data)
 
 static void create_trayicon()
 {
- 	gint n_entries = 0;
- 	GtkItemFactory *traymenu_factory;
+	gint n_entries = 0;
+	GtkItemFactory *traymenu_factory;
+#if 0
 	GtkPacker *packer;
+#endif
 
         trayicon = egg_tray_icon_new("Sylpheed-Claws");
 	gtk_widget_realize(GTK_WIDGET(trayicon));
@@ -218,20 +232,16 @@ static void create_trayicon()
         gtk_container_set_border_width(GTK_CONTAINER(eventbox), 0);
         gtk_container_add(GTK_CONTAINER(trayicon), GTK_WIDGET(eventbox));
 
-	packer = GTK_PACKER(gtk_packer_new());
-        gtk_container_add(GTK_CONTAINER(eventbox), GTK_WIDGET(packer));
-        gtk_container_set_border_width(GTK_CONTAINER(packer), 0);
-
-        image = gtk_pixmap_new(nomail_pixmap, nomail_bitmap);
-        gtk_packer_add_defaults(GTK_PACKER(packer), GTK_WIDGET(image), GTK_SIDE_TOP, GTK_ANCHOR_CENTER, GTK_PACK_EXPAND);
+        image = gtk_image_new_from_pixmap(nomail_pixmap, nomail_bitmap);
+        gtk_container_add(GTK_CONTAINER(eventbox), image);
 
 	destroy_signal_id =
-	gtk_signal_connect(GTK_OBJECT(trayicon), "destroy",
-                     GTK_SIGNAL_FUNC(destroy_cb), NULL);
-	gtk_signal_connect(GTK_OBJECT(trayicon), "size_allocate",
-		    GTK_SIGNAL_FUNC(resize_cb), NULL);
-	gtk_signal_connect(GTK_OBJECT(eventbox), "button-press-event",
-		    GTK_SIGNAL_FUNC(click_cb), NULL);
+	g_signal_connect(G_OBJECT(trayicon), "destroy",
+                     	 G_CALLBACK(destroy_cb), NULL);
+	g_signal_connect(GTK_OBJECT(trayicon), "size-request",
+		    	 G_CALLBACK(resize_cb), NULL);
+	g_signal_connect(G_OBJECT(eventbox), "button-press-event",
+		    	 G_CALLBACK(click_cb), NULL);
 
         tooltips = gtk_tooltips_new();
         gtk_tooltips_set_delay(tooltips, 1000);
@@ -274,7 +284,7 @@ int plugin_init(gchar **error)
 
 void plugin_done(void)
 {
-	gtk_signal_disconnect(GTK_OBJECT(trayicon), destroy_signal_id);
+	g_signal_handler_disconnect(G_OBJECT(trayicon), destroy_signal_id);
 
 	gtk_widget_destroy(GTK_WIDGET(trayicon));
 	hooks_unregister_hook(FOLDER_ITEM_UPDATE_HOOKLIST, hook_id);
@@ -300,7 +310,7 @@ const gchar *plugin_desc(void)
 
 const gchar *plugin_type(void)
 {
-	return "GTK";
+	return "GTK2";
 }
 
 /* popup menu callbacks */
