@@ -22,6 +22,7 @@
 #endif
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdarg.h>
@@ -30,7 +31,6 @@
 #include <time.h>
 #include <errno.h>
 
-#include "intl.h"
 #include "pop.h"
 #include "md5.h"
 #include "prefs_account.h"
@@ -488,7 +488,7 @@ static void pop3_gen_send(Pop3Session *session, const gchar *format, ...)
 	g_vsnprintf(buf, sizeof(buf) - 2, format, args);
 	va_end(args);
 
-	if (!g_strncasecmp(buf, "PASS ", 5))
+	if (!g_ascii_strncasecmp(buf, "PASS ", 5))
 		log_print("POP3> PASS ********\n");
 	else
 		log_print("POP3> %s\n", buf);
@@ -563,18 +563,13 @@ void pop3_get_uidl_table(PrefsAccount *ac_prefs, Pop3Session *session)
 	time_t recv_time;
 	time_t now;
 	gint partial_recv;
-	gchar *sanitized_uid = g_strdup(ac_prefs->userid);
-	
-	subst_for_filename(sanitized_uid);
 	
 	table = g_hash_table_new(g_str_hash, g_str_equal);
 	partial_recv_table = g_hash_table_new(g_str_hash, g_str_equal);
 
 	path = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S,
 			   "uidl", G_DIR_SEPARATOR_S, ac_prefs->recv_server,
-			   "-", sanitized_uid, NULL);
-			   
-	g_free(sanitized_uid);
+			   "-", ac_prefs->userid, NULL);
 	if ((fp = fopen(path, "rb")) == NULL) {
 		if (ENOENT != errno) FILE_OP_ERROR(path, "fopen");
 		g_free(path);
@@ -638,20 +633,13 @@ gint pop3_write_uidl_list(Pop3Session *session)
 	FILE *fp;
 	Pop3MsgInfo *msg;
 	gint n;
-	gchar *sanitized_uid = g_strdup(session->ac_prefs->userid);
-	
-	subst_for_filename(sanitized_uid);
-	
 
 	if (!session->uidl_is_valid) return 0;
 
 	path = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S,
 			   "uidl", G_DIR_SEPARATOR_S,
 			   session->ac_prefs->recv_server,
-			   "-", sanitized_uid, NULL);
-	
-	g_free(sanitized_uid);
-
+			   "-", session->ac_prefs->userid, NULL);
 	if ((fp = fopen(path, "wb")) == NULL) {
 		FILE_OP_ERROR(path, "fopen");
 		g_free(path);
