@@ -163,8 +163,6 @@ struct _PrefsAccount
 
 	gboolean set_sent_folder;
 	gchar *sent_folder;
-	gboolean set_queue_folder;
-	gchar *queue_folder;
 	gboolean set_draft_folder;
 	gchar *draft_folder;
 	gboolean set_trash_folder;
@@ -176,7 +174,6 @@ struct _PrefsAccount
 	gint account_id;
 
 	Folder *folder;
-
 	GHashTable *privacy_prefs;
 };
 
