@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2003 Hiroyuki Yamamoto & The Sylpheed Claws Team
+ * Copyright (C) 1999-2005 Hiroyuki Yamamoto & The Sylpheed Claws Team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include <stdio.h>
@@ -31,7 +32,6 @@
 #include <string.h>
 #include <errno.h>
 
-#include "intl.h"
 #include "prefs_gtk.h"
 #include "inc.h"
 #include "utils.h"
@@ -43,7 +43,17 @@
 #include "prefs_actions.h"
 #include "action.h"
 #include "description_window.h"
-#include "gtkutils.h"
+
+enum {
+	PREFS_ACTIONS_STRING,	/*!< string pointer managed by list store, 
+				 *   and never touched or retrieved by 
+				 *   us */ 
+	PREFS_ACTIONS_DATA,	/*!< pointer to string that is not managed by 
+				 *   the list store, and which is retrieved
+				 *   and touched by us */
+	PREFS_ACTIONS_VALID,	/*!< contains a valid action, otherwise "(New)" */
+	N_PREFS_ACTIONS_COLUMNS
+};
 
 static struct Actions
 {
@@ -54,15 +64,13 @@ static struct Actions
 	GtkWidget *name_entry;
 	GtkWidget *cmd_entry;
 
-	GtkWidget *actions_clist;
+	GtkWidget *actions_list_view;
 } actions;
 
-static int modified = FALSE;
-
 /* widget creating functions */
 static void prefs_actions_create	(MainWindow *mainwin);
 static void prefs_actions_set_dialog	(void);
-static gint prefs_actions_clist_set_row	(gint row);
+static gint prefs_actions_clist_set_row	(GtkTreeIter *row);
 
 /* callback functions */
 static void prefs_actions_help_cb	(GtkWidget	*w,
@@ -77,25 +85,32 @@ static void prefs_actions_up		(GtkWidget	*w,
 					 gpointer	 data);
 static void prefs_actions_down		(GtkWidget	*w,
 					 gpointer	 data);
-static void prefs_actions_select	(GtkCList	*clist,
-					 gint		 row,
-					 gint		 column,
-					 GdkEvent	*event);
-static void prefs_actions_row_move	(GtkCList	*clist,
-					 gint		 source_row,
-					 gint		 dest_row);
 static gint prefs_actions_deleted	(GtkWidget	*widget,
 					 GdkEventAny	*event,
 					 gpointer	*data);
-static void prefs_actions_key_pressed	(GtkWidget	*widget,
-					 GdkEventKey	*event,
-					 gpointer	 data);
+static gboolean prefs_actions_key_pressed(GtkWidget	*widget,
+					  GdkEventKey	*event,
+					  gpointer	 data);
 static void prefs_actions_cancel	(GtkWidget	*w,
 					 gpointer	 data);
 static void prefs_actions_ok		(GtkWidget	*w,
 					 gpointer	 data);
 
 
+static GtkListStore* prefs_actions_create_data_store	(void);
+
+static void prefs_actions_list_view_insert_action	(GtkWidget *list_view,
+							 GtkTreeIter *row_iter,
+							 gchar *action,
+							 gboolean is_valid);
+static GtkWidget *prefs_actions_list_view_create	(void);
+static void prefs_actions_create_list_view_columns	(GtkWidget *list_view);
+static gboolean prefs_actions_selected			(GtkTreeSelection *selector,
+							 GtkTreeModel *model, 
+							 GtkTreePath *path,
+							 gboolean currently_selected,
+							 gpointer data);
+
 void prefs_actions_open(MainWindow *mainwin)
 {
 	inc_lock();
@@ -137,7 +152,7 @@ static void prefs_actions_create(MainWindow *mainwin)
 
 	GtkWidget *cond_hbox;
 	GtkWidget *cond_scrolledwin;
-	GtkWidget *cond_clist;
+	GtkWidget *cond_list_view;
 
 	GtkWidget *help_button;
 
@@ -145,38 +160,37 @@ static void prefs_actions_create(MainWindow *mainwin)
 	GtkWidget *up_btn;
 	GtkWidget *down_btn;
 
-	gchar *title[1];
-
 	debug_print("Creating actions configuration window...\n");
 
-	window = gtk_window_new (GTK_WINDOW_DIALOG);
+	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 
 	gtk_container_set_border_width(GTK_CONTAINER (window), 8);
-	gtk_window_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
+	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
 	gtk_window_set_modal(GTK_WINDOW(window), TRUE);
-	gtk_window_set_policy(GTK_WINDOW(window), FALSE, TRUE, TRUE);
+	gtk_window_set_resizable(GTK_WINDOW(window), TRUE);
 	gtk_window_set_default_size(GTK_WINDOW(window), 400, -1);
 
 	vbox = gtk_vbox_new(FALSE, 6);
 	gtk_widget_show(vbox);
 	gtk_container_add(GTK_CONTAINER(window), vbox);
 
-	gtkut_button_set_create(&confirm_area, &ok_btn, _("OK"),
-				&cancel_btn, _("Cancel"), NULL, NULL);
+	gtkut_stock_button_set_create(&confirm_area, &ok_btn, GTK_STOCK_OK,
+				      &cancel_btn, GTK_STOCK_CANCEL,
+				      NULL, NULL);
 	gtk_widget_show(confirm_area);
 	gtk_box_pack_end(GTK_BOX(vbox), confirm_area, FALSE, FALSE, 0);
 	gtk_widget_grab_default(ok_btn);
 
 	gtk_window_set_title(GTK_WINDOW(window), _("Actions configuration"));
-	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
-			   GTK_SIGNAL_FUNC(prefs_actions_deleted), NULL);
-	gtk_signal_connect(GTK_OBJECT(window), "key_press_event",
-			   GTK_SIGNAL_FUNC(prefs_actions_key_pressed), NULL);
+	g_signal_connect(G_OBJECT(window), "delete_event",
+			 G_CALLBACK(prefs_actions_deleted), NULL);
+	g_signal_connect(G_OBJECT(window), "key_press_event",
+			 G_CALLBACK(prefs_actions_key_pressed), NULL);
 	MANAGE_WINDOW_SIGNALS_CONNECT(window);
-	gtk_signal_connect(GTK_OBJECT(ok_btn), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_actions_ok), mainwin);
-	gtk_signal_connect(GTK_OBJECT(cancel_btn), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_actions_cancel), NULL);
+	g_signal_connect(G_OBJECT(ok_btn), "clicked",
+			 G_CALLBACK(prefs_actions_ok), mainwin);
+	g_signal_connect(G_OBJECT(cancel_btn), "clicked",
+			 G_CALLBACK(prefs_actions_cancel), NULL);
 
 	vbox1 = gtk_vbox_new(FALSE, 8);
 	gtk_widget_show(vbox1);
@@ -215,36 +229,36 @@ static void prefs_actions_create(MainWindow *mainwin)
 	arrow = gtk_arrow_new(GTK_ARROW_DOWN, GTK_SHADOW_OUT);
 	gtk_widget_show(arrow);
 	gtk_box_pack_start(GTK_BOX(reg_hbox), arrow, FALSE, FALSE, 0);
-	gtk_widget_set_usize(arrow, -1, 16);
+	gtk_widget_set_size_request(arrow, -1, 16);
 
 	btn_hbox = gtk_hbox_new(TRUE, 4);
 	gtk_widget_show(btn_hbox);
 	gtk_box_pack_start(GTK_BOX(reg_hbox), btn_hbox, FALSE, FALSE, 0);
 
-	reg_btn = gtk_button_new_with_label(_("Add"));
+	reg_btn = gtk_button_new_from_stock(GTK_STOCK_ADD);
 	gtk_widget_show(reg_btn);
 	gtk_box_pack_start(GTK_BOX(btn_hbox), reg_btn, FALSE, TRUE, 0);
-	gtk_signal_connect(GTK_OBJECT(reg_btn), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_actions_register_cb), NULL);
+	g_signal_connect(G_OBJECT(reg_btn), "clicked",
+			 G_CALLBACK(prefs_actions_register_cb), NULL);
 
 	subst_btn = gtk_button_new_with_label(_(" Replace "));
 	gtk_widget_show(subst_btn);
 	gtk_box_pack_start(GTK_BOX(btn_hbox), subst_btn, FALSE, TRUE, 0);
-	gtk_signal_connect(GTK_OBJECT(subst_btn), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_actions_substitute_cb),
-			   NULL);
+	g_signal_connect(G_OBJECT(subst_btn), "clicked",
+			 G_CALLBACK(prefs_actions_substitute_cb),
+			 NULL);
 
-	del_btn = gtk_button_new_with_label(_("Delete"));
+	del_btn = gtk_button_new_from_stock(GTK_STOCK_REMOVE);
 	gtk_widget_show(del_btn);
 	gtk_box_pack_start(GTK_BOX(btn_hbox), del_btn, FALSE, TRUE, 0);
-	gtk_signal_connect(GTK_OBJECT(del_btn), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_actions_delete_cb), NULL);
+	g_signal_connect(G_OBJECT(del_btn), "clicked",
+			 G_CALLBACK(prefs_actions_delete_cb), NULL);
 
 	help_button = gtk_button_new_with_label(_(" Syntax help "));
 	gtk_widget_show(help_button);
 	gtk_box_pack_end(GTK_BOX(reg_hbox), help_button, FALSE, FALSE, 0);
-	gtk_signal_connect(GTK_OBJECT(help_button), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_actions_help_cb), NULL);
+	g_signal_connect(G_OBJECT(help_button), "clicked",
+			 G_CALLBACK(prefs_actions_help_cb), NULL);
 
 	cond_hbox = gtk_hbox_new(FALSE, 8);
 	gtk_widget_show(cond_hbox);
@@ -252,43 +266,32 @@ static void prefs_actions_create(MainWindow *mainwin)
 
 	cond_scrolledwin = gtk_scrolled_window_new(NULL, NULL);
 	gtk_widget_show(cond_scrolledwin);
-	gtk_widget_set_usize(cond_scrolledwin, -1, 150);
+	gtk_widget_set_size_request(cond_scrolledwin, -1, 150);
 	gtk_box_pack_start(GTK_BOX(cond_hbox), cond_scrolledwin,
 			   TRUE, TRUE, 0);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (cond_scrolledwin),
 				       GTK_POLICY_AUTOMATIC,
 				       GTK_POLICY_AUTOMATIC);
 
-	title[0] = _("Current actions");
-	cond_clist = gtk_clist_new_with_titles(1, title);
-	gtk_widget_show(cond_clist);
-	gtk_container_add(GTK_CONTAINER (cond_scrolledwin), cond_clist);
-	gtk_clist_set_column_width(GTK_CLIST (cond_clist), 0, 80);
-	gtk_clist_set_selection_mode(GTK_CLIST (cond_clist),
-				     GTK_SELECTION_BROWSE);
-	GTK_WIDGET_UNSET_FLAGS(GTK_CLIST(cond_clist)->column[0].button,
-			       GTK_CAN_FOCUS);
-	gtk_signal_connect(GTK_OBJECT(cond_clist), "select_row",
-			   GTK_SIGNAL_FUNC(prefs_actions_select), NULL);
-	gtk_signal_connect_after(GTK_OBJECT(cond_clist), "row_move",
-				 GTK_SIGNAL_FUNC(prefs_actions_row_move),
-				 NULL);
+	cond_list_view = prefs_actions_list_view_create();				       
+	gtk_widget_show(cond_list_view);
+	gtk_container_add(GTK_CONTAINER (cond_scrolledwin), cond_list_view);
 
 	btn_vbox = gtk_vbox_new(FALSE, 8);
 	gtk_widget_show(btn_vbox);
 	gtk_box_pack_start(GTK_BOX(cond_hbox), btn_vbox, FALSE, FALSE, 0);
 
-	up_btn = gtk_button_new_with_label(_("Up"));
+	up_btn = gtk_button_new_from_stock(GTK_STOCK_GO_UP);
 	gtk_widget_show(up_btn);
 	gtk_box_pack_start(GTK_BOX(btn_vbox), up_btn, FALSE, FALSE, 0);
-	gtk_signal_connect(GTK_OBJECT(up_btn), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_actions_up), NULL);
+	g_signal_connect(G_OBJECT(up_btn), "clicked",
+			 G_CALLBACK(prefs_actions_up), NULL);
 
-	down_btn = gtk_button_new_with_label(_("Down"));
+	down_btn = gtk_button_new_from_stock(GTK_STOCK_GO_DOWN);
 	gtk_widget_show(down_btn);
 	gtk_box_pack_start(GTK_BOX(btn_vbox), down_btn, FALSE, FALSE, 0);
-	gtk_signal_connect(GTK_OBJECT(down_btn), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_actions_down), NULL);
+	g_signal_connect(G_OBJECT(down_btn), "clicked",
+			 G_CALLBACK(prefs_actions_down), NULL);
 
 	gtk_widget_show(window);
 
@@ -298,7 +301,7 @@ static void prefs_actions_create(MainWindow *mainwin)
 	actions.name_entry = name_entry;
 	actions.cmd_entry  = cmd_entry;
 
-	actions.actions_clist = cond_clist;
+	actions.actions_list_view = cond_list_view;
 }
 
 
@@ -327,13 +330,25 @@ void prefs_actions_read_config(void)
 	}
 
 	while (fgets(buf, sizeof(buf), fp) != NULL) {
-		g_strchomp(buf);
-		act = strstr(buf, ": ");
+		const gchar *src_codeset = conv_get_locale_charset_str();
+		const gchar *dest_codeset = CS_UTF_8;
+		gchar *tmp;
+
+		tmp = conv_codeset_strdup(buf, src_codeset, dest_codeset);
+		if (!tmp) {
+			g_warning("Faild to convert character set of action configuration\n");
+			tmp = g_strdup(buf);
+		}
+
+		g_strchomp(tmp);
+		act = strstr(tmp, ": ");
 		if (act && act[2] && 
 		    action_get_type(&act[2]) != ACTION_ERROR)
 			prefs_common.actions_list =
 				g_slist_append(prefs_common.actions_list,
-					       g_strdup(buf));
+					       tmp);
+		else
+			g_free(tmp);
 	}
 	fclose(fp);
 }
@@ -354,7 +369,17 @@ void prefs_actions_write_config(void)
 	}
 
 	for (cur = prefs_common.actions_list; cur != NULL; cur = cur->next) {
-		gchar *act = (gchar *)cur->data;
+		gchar *tmp = (gchar *)cur->data;
+		const gchar *src_codeset = CS_UTF_8;
+		const gchar *dest_codeset = conv_get_locale_charset_str();
+		gchar *act;
+
+		act = conv_codeset_strdup(tmp, src_codeset, dest_codeset);
+		if (!act) {
+			g_warning("Faild to convert character set of action configuration\n");
+			act = g_strdup(act);
+		}
+
 		if (fputs(act, pfile->fp) == EOF ||
 		    fputc('\n', pfile->fp) == EOF) {
 			FILE_OP_ERROR(rcpath, "fputs || fputc");
@@ -362,6 +387,7 @@ void prefs_actions_write_config(void)
 			g_free(rcpath);
 			return;
 		}
+		g_free(act);
 	}
 	
 	g_free(rcpath);
@@ -374,57 +400,79 @@ void prefs_actions_write_config(void)
 
 static void prefs_actions_set_dialog(void)
 {
-	GtkCList *clist = GTK_CLIST(actions.actions_clist);
+	GtkListStore *store;
 	GSList *cur;
-	gchar *action_str[1];
-	gint row;
+	GtkTreeSelection *selection;
+	GtkTreeIter iter;
 
-	gtk_clist_freeze(clist);
-	gtk_clist_clear(clist);
+	store = GTK_LIST_STORE(gtk_tree_view_get_model
+				(GTK_TREE_VIEW(actions.actions_list_view)));
+	gtk_list_store_clear(store);
 
-	action_str[0] = _("(New)");
-	row = gtk_clist_append(clist, action_str);
-	gtk_clist_set_row_data(clist, row, NULL);
+	prefs_actions_list_view_insert_action(actions.actions_list_view,
+					      NULL, _("New"), FALSE);
 
 	for (cur = prefs_common.actions_list; cur != NULL; cur = cur->next) {
-		gchar *action[1];
-
-		action[0] = (gchar *)cur->data;
-		row = gtk_clist_append(clist, action);
-		gtk_clist_set_row_data(clist, row, action[0]);
+		gchar *action = (gchar *) cur->data;
+		
+		prefs_actions_list_view_insert_action(actions.actions_list_view,
+						      NULL, action, TRUE);
 	}
 
-	gtk_clist_thaw(clist);
+	/* select first entry */
+	selection = gtk_tree_view_get_selection
+		(GTK_TREE_VIEW(actions.actions_list_view));
+	if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store),
+					  &iter))
+		gtk_tree_selection_select_iter(selection, &iter);
 }
 
 static void prefs_actions_set_list(void)
 {
-	gint row = 1;
 	gchar *action;
-
+	GtkTreeIter iter;
+	GtkListStore *store;
+	
 	g_slist_free(prefs_common.actions_list);
 	prefs_common.actions_list = NULL;
 
-	while ((action = (gchar *)gtk_clist_get_row_data
-		(GTK_CLIST(actions.actions_clist), row)) != NULL) {
-		prefs_common.actions_list =
-			g_slist_append(prefs_common.actions_list, action);
-		row++;
+	store = GTK_LIST_STORE(gtk_tree_view_get_model
+				(GTK_TREE_VIEW(actions.actions_list_view)));
+
+	if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter)) {
+		do {
+			gchar *action;
+			gboolean is_valid;
+
+			gtk_tree_model_get(GTK_TREE_MODEL(store), &iter,
+					   PREFS_ACTIONS_DATA, &action,
+					   PREFS_ACTIONS_VALID, &is_valid,
+					   -1);
+			
+			if (is_valid) 
+				prefs_common.actions_list = 
+					g_slist_append(prefs_common.actions_list,
+						       action);
+
+		} while (gtk_tree_model_iter_next(GTK_TREE_MODEL(store),
+						  &iter));
 	}
 }
 
 #define GET_ENTRY(entry) \
 	entry_text = gtk_entry_get_text(GTK_ENTRY(entry))
 
-static gint prefs_actions_clist_set_row(gint row)
+static gint prefs_actions_clist_set_row(GtkTreeIter *row)
 {
-	GtkCList *clist = GTK_CLIST(actions.actions_clist);
-	gchar *entry_text;
+	const gchar *entry_text;
 	gint len;
 	gchar action[PREFSBUFSIZE];
-	gchar *buf[1];
+	gchar *new_action;
+	GtkListStore *store;
 
-	g_return_val_if_fail(row != 0, -1);
+	store = GTK_LIST_STORE(gtk_tree_view_get_model
+				(GTK_TREE_VIEW(actions.actions_list_view)));
+	
 
 	GET_ENTRY(actions.name_entry);
 	if (entry_text[0] == '\0') {
@@ -469,21 +517,10 @@ static gint prefs_actions_clist_set_row(gint row)
 
 	strcat(action, entry_text);
 
-	buf[0] = action;
-	if (row < 0)
-		row = gtk_clist_append(clist, buf);
-	else {
-		gchar *old_action;
-		gtk_clist_set_text(clist, row, 0, action);
-		old_action = (gchar *) gtk_clist_get_row_data(clist, row);
-		if (old_action)
-			g_free(old_action);
-	}
-
-	buf[0] = g_strdup(action);
-
-	gtk_clist_set_row_data(clist, row, buf[0]);
-
+	new_action = g_strdup(action);	
+	prefs_actions_list_view_insert_action(actions.actions_list_view,
+	                                      row, new_action, TRUE);
+						
 	prefs_actions_set_list();
 
 	return 0;
@@ -493,112 +530,134 @@ static gint prefs_actions_clist_set_row(gint row)
 
 static void prefs_actions_register_cb(GtkWidget *w, gpointer data)
 {
-	prefs_actions_clist_set_row(-1);
-	modified = FALSE;
+	prefs_actions_clist_set_row(NULL);
 }
 
 static void prefs_actions_substitute_cb(GtkWidget *w, gpointer data)
 {
-	GtkCList *clist = GTK_CLIST(actions.actions_clist);
-	gchar *action;
-	gint row;
+	GtkTreeIter isel, inew;
+	GtkTreePath *path_sel, *path_new;
+	GtkTreeSelection *selection = gtk_tree_view_get_selection
+					(GTK_TREE_VIEW(actions.actions_list_view));
+	GtkTreeModel *model;					
 
-	if (!clist->selection) return;
+	if (!gtk_tree_selection_get_selected(selection, &model, &isel))
+		return;
+	if (!gtk_tree_model_get_iter_first(model, &inew))
+		return;
 
-	row = GPOINTER_TO_INT(clist->selection->data);
-	if (row == 0) return;
+	path_sel = gtk_tree_model_get_path(model, &isel);
+	path_new = gtk_tree_model_get_path(model, &inew);
 
-	action = gtk_clist_get_row_data(clist, row);
-	if (!action) return;
+	if (path_sel && path_new 
+	&&  gtk_tree_path_compare(path_sel, path_new) != 0)
+		prefs_actions_clist_set_row(&isel);
 
-	prefs_actions_clist_set_row(row);
-	modified = FALSE;
+	gtk_tree_path_free(path_sel);
+	gtk_tree_path_free(path_new);
 }
 
 static void prefs_actions_delete_cb(GtkWidget *w, gpointer data)
 {
-	GtkCList *clist = GTK_CLIST(actions.actions_clist);
+	GtkTreeIter sel;
+	GtkTreeModel *model;
 	gchar *action;
-	gint row;
 
-	if (!clist->selection) return;
-	row = GPOINTER_TO_INT(clist->selection->data);
-	if (row == 0) return;
+	if (!gtk_tree_selection_get_selected(gtk_tree_view_get_selection
+				(GTK_TREE_VIEW(actions.actions_list_view)),
+				&model, &sel))
+		return;				
 
 	if (alertpanel(_("Delete action"),
 		       _("Do you really want to delete this action?"),
-		       _("Yes"), _("No"), NULL) != G_ALERTDEFAULT)
+		       GTK_STOCK_YES, GTK_STOCK_NO, NULL) != G_ALERTDEFAULT)
 		return;
 
-	action = gtk_clist_get_row_data(clist, row);
-	g_free(action);
-	gtk_clist_remove(clist, row);
+	/* XXX: Here's the reason why we need to store the original 
+	 * pointer: we search the slist for it. */
+	gtk_tree_model_get(model, &sel,
+			   PREFS_ACTIONS_DATA, &action,
+			   -1);
+	gtk_list_store_remove(GTK_LIST_STORE(model), &sel);
+
 	prefs_common.actions_list = g_slist_remove(prefs_common.actions_list,
 						   action);
 }
 
 static void prefs_actions_up(GtkWidget *w, gpointer data)
 {
-	GtkCList *clist = GTK_CLIST(actions.actions_clist);
-	gint row;
+	GtkTreePath *prev, *sel, *try;
+	GtkTreeIter isel;
+	GtkListStore *store;
+	GtkTreeIter iprev;
+	
+	if (!gtk_tree_selection_get_selected
+		(gtk_tree_view_get_selection
+			(GTK_TREE_VIEW(actions.actions_list_view)),
+		 (GtkTreeModel **) &store,	
+		 &isel))
+		return;
 
-	if (!clist->selection) return;
+	sel = gtk_tree_model_get_path(GTK_TREE_MODEL(store), &isel);
+	if (!sel)
+		return;
+	
+	/* no move if we're at row 0 or 1, looks phony, but other
+	 * solutions are more convoluted... */
+	try = gtk_tree_path_copy(sel);
+	if (!gtk_tree_path_prev(try) || !gtk_tree_path_prev(try)) {
+		gtk_tree_path_free(try);
+		gtk_tree_path_free(sel);
+		return;
+	}
+	gtk_tree_path_free(try);
 
-	row = GPOINTER_TO_INT(clist->selection->data);
-	if (row > 1)
-		gtk_clist_row_move(clist, row, row - 1);
+	prev = gtk_tree_path_copy(sel);		
+	if (!gtk_tree_path_prev(prev)) {
+		gtk_tree_path_free(prev);
+		gtk_tree_path_free(sel);
+		return;
+	}
+
+	gtk_tree_model_get_iter(GTK_TREE_MODEL(store),
+				&iprev, prev);
+	gtk_tree_path_free(sel);
+	gtk_tree_path_free(prev);
+
+	gtk_list_store_swap(store, &iprev, &isel);
+	prefs_actions_set_list();
 }
 
 static void prefs_actions_down(GtkWidget *w, gpointer data)
 {
-	GtkCList *clist = GTK_CLIST(actions.actions_clist);
-	gint row;
+	GtkListStore *store;
+	GtkTreeIter next, sel;
+	GtkTreePath *try;
+	
+	if (!gtk_tree_selection_get_selected
+		(gtk_tree_view_get_selection
+			(GTK_TREE_VIEW(actions.actions_list_view)),
+		 (GtkTreeModel **) &store,
+		 &sel))
+		return;
 
-	if (!clist->selection) return;
+	try = gtk_tree_model_get_path(GTK_TREE_MODEL(store), &sel);
+	if (!try) 
+		return;
 
-	row = GPOINTER_TO_INT(clist->selection->data);
-	if (row > 0 && row < clist->rows - 1)
-		gtk_clist_row_move(clist, row, row + 1);
-}
-
-#define ENTRY_SET_TEXT(entry, str) \
-	gtk_entry_set_text(GTK_ENTRY(entry), str ? str : "")
-
-static void prefs_actions_select(GtkCList *clist, gint row, gint column,
-				 GdkEvent *event)
-{
-	gchar *action;
-	gchar *cmd;
-	gchar buf[PREFSBUFSIZE];
-	action = gtk_clist_get_row_data(clist, row);
-
-	if (!action) {
-		ENTRY_SET_TEXT(actions.name_entry, "");
-		ENTRY_SET_TEXT(actions.cmd_entry, "");
+	/* no move when we're at row 0 */
+	if (!gtk_tree_path_prev(try)) {
+		gtk_tree_path_free(try);
 		return;
 	}
+	gtk_tree_path_free(try);
 
-	strncpy(buf, action, PREFSBUFSIZE - 1);
-	buf[PREFSBUFSIZE - 1] = 0x00;
-	cmd = strstr(buf, ": ");
-
-	if (cmd && cmd[2])
-		ENTRY_SET_TEXT(actions.cmd_entry, &cmd[2]);
-	else
+	next = sel;
+	if (!gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &next)) 
 		return;
 
-	*cmd = 0x00;
-	ENTRY_SET_TEXT(actions.name_entry, buf);
-}
-
-static void prefs_actions_row_move(GtkCList *clist,
-				   gint source_row, gint dest_row)
-{
+	gtk_list_store_swap(store, &next, &sel);
 	prefs_actions_set_list();
-	if (gtk_clist_row_is_visible(clist, dest_row) != GTK_VISIBILITY_FULL) {
-		gtk_clist_moveto(clist, dest_row, -1,
-				 source_row < dest_row ? 1.0 : 0.0, 0.0);
-	}
 }
 
 static gint prefs_actions_deleted(GtkWidget *widget, GdkEventAny *event,
@@ -608,29 +667,16 @@ static gint prefs_actions_deleted(GtkWidget *widget, GdkEventAny *event,
 	return TRUE;
 }
 
-static void prefs_actions_key_pressed(GtkWidget *widget, GdkEventKey *event,
-				      gpointer data)
+static gboolean prefs_actions_key_pressed(GtkWidget *widget, GdkEventKey *event,
+					  gpointer data)
 {
 	if (event && event->keyval == GDK_Escape)
 		prefs_actions_cancel(widget, data);
-	else {
-		GtkWidget *focused = gtkut_get_focused_child(
-					GTK_CONTAINER(widget));
-		if (focused && GTK_IS_EDITABLE(focused)) {
-			modified = TRUE;
-		}
-	}
-	return;
+	return FALSE;
 }
 
 static void prefs_actions_cancel(GtkWidget *w, gpointer data)
 {
-	if (modified && alertpanel(_("Entry not saved"),
-				 _("The entry was not saved. Close anyway?"),
-				 _("Yes"), _("No"), NULL) != G_ALERTDEFAULT) {
-		return;
-	}
-	modified = FALSE;
 	prefs_actions_read_config();
 	gtk_widget_hide(actions.window);
 	inc_unlock();
@@ -644,12 +690,6 @@ static void prefs_actions_ok(GtkWidget *widget, gpointer data)
 	MessageView *msgview;
 	Compose *compose;
 
-	if (modified && alertpanel(_("Entry not saved"),
-				 _("The entry was not saved. Close anyway?"),
-				 _("Yes"), _("No"), NULL) != G_ALERTDEFAULT) {
-		return;
-	}
-	modified = FALSE;
 	prefs_actions_write_config();
 
 	/* Update mainwindow actions menu */
@@ -715,3 +755,129 @@ static void prefs_actions_help_cb(GtkWidget *w, gpointer data)
 {
 	description_window_create(&actions_desc_win);
 }
+
+static GtkListStore* prefs_actions_create_data_store(void)
+{
+	return gtk_list_store_new(N_PREFS_ACTIONS_COLUMNS,
+				  G_TYPE_STRING,	
+				  G_TYPE_POINTER,
+				  G_TYPE_BOOLEAN,
+				  -1);
+}
+
+static void prefs_actions_list_view_insert_action(GtkWidget *list_view,
+						  GtkTreeIter *row_iter,
+						  gchar *action,
+						  gboolean is_valid) 
+{
+	GtkTreeIter iter;
+	GtkListStore *list_store = GTK_LIST_STORE(gtk_tree_view_get_model
+					(GTK_TREE_VIEW(list_view)));
+
+	if (row_iter == NULL) {
+		/* append new */
+		gtk_list_store_append(list_store, &iter);
+		gtk_list_store_set(list_store, &iter,
+				   PREFS_ACTIONS_STRING, action,
+				   PREFS_ACTIONS_DATA, action,
+				   PREFS_ACTIONS_VALID,  is_valid,
+				   -1);
+	} else {
+		/* change existing */
+		gchar *old_action;
+
+		gtk_tree_model_get(GTK_TREE_MODEL(list_store), row_iter,
+				   PREFS_ACTIONS_DATA, &old_action,
+				   -1);
+		
+		g_free(old_action);				
+		gtk_list_store_set(list_store, row_iter,
+				   PREFS_ACTIONS_STRING, action,
+				   PREFS_ACTIONS_DATA, action,
+				   -1);
+	}
+}
+
+static GtkWidget *prefs_actions_list_view_create(void)
+{
+	GtkTreeView *list_view;
+	GtkTreeSelection *selector;
+	GtkTreeModel *model;
+
+	model = GTK_TREE_MODEL(prefs_actions_create_data_store());
+	list_view = GTK_TREE_VIEW(gtk_tree_view_new_with_model(model));
+	g_object_unref(model);	
+	
+	gtk_tree_view_set_rules_hint(list_view, TRUE);
+	
+	selector = gtk_tree_view_get_selection(list_view);
+	gtk_tree_selection_set_mode(selector, GTK_SELECTION_BROWSE);
+	gtk_tree_selection_set_select_function(selector, prefs_actions_selected,
+					       NULL, NULL);
+
+	/* create the columns */
+	prefs_actions_create_list_view_columns(GTK_WIDGET(list_view));
+
+	return GTK_WIDGET(list_view);
+}
+
+static void prefs_actions_create_list_view_columns(GtkWidget *list_view)
+{
+	GtkTreeViewColumn *column;
+	GtkCellRenderer *renderer;
+
+	renderer = gtk_cell_renderer_text_new();
+	column = gtk_tree_view_column_new_with_attributes
+		(_("Current actions"),
+		 renderer,
+		 "text", PREFS_ACTIONS_STRING,
+		 NULL);
+	gtk_tree_view_append_column(GTK_TREE_VIEW(list_view), column);		
+}
+
+#define ENTRY_SET_TEXT(entry, str) \
+	gtk_entry_set_text(GTK_ENTRY(entry), str ? str : "")
+
+static gboolean prefs_actions_selected(GtkTreeSelection *selector,
+				       GtkTreeModel *model, 
+				       GtkTreePath *path,
+				       gboolean currently_selected,
+				       gpointer data)
+{
+	gchar *action;
+	gchar *cmd;
+	gchar buf[PREFSBUFSIZE];
+	GtkTreeIter iter;
+	gboolean is_valid;
+
+	if (currently_selected)
+		return TRUE;
+
+	if (!gtk_tree_model_get_iter(model, &iter, path))
+		return TRUE;
+
+	gtk_tree_model_get(model, &iter, 
+			   PREFS_ACTIONS_VALID,  &is_valid,
+			   PREFS_ACTIONS_DATA, &action,
+			   -1);
+	if (!is_valid) {
+		ENTRY_SET_TEXT(actions.name_entry, "");
+		ENTRY_SET_TEXT(actions.cmd_entry, "");
+		return TRUE;
+	}
+	
+	strncpy(buf, action, PREFSBUFSIZE - 1);
+	buf[PREFSBUFSIZE - 1] = 0x00;
+	cmd = strstr(buf, ": ");
+
+	if (cmd && cmd[2])
+		ENTRY_SET_TEXT(actions.cmd_entry, &cmd[2]);
+	else
+		return;
+
+	*cmd = 0x00;
+	ENTRY_SET_TEXT(actions.name_entry, buf);
+
+	return TRUE;
+}
+
