@@ -25,8 +25,10 @@
 #  include "config.h"
 #endif
 
-#include "intl.h"
 #include "defs.h"
+
+#include <glib.h>
+#include <glib/gi18n.h>
 #include "folder.h"
 #include "prefs_folder_item.h"
 #include "folderview.h"
@@ -158,27 +160,28 @@ void prefs_folder_item_general_create_widget_func(PrefsPage * page_,
 	page->item	   = item;
 
 	/* Table */
-	table = gtk_table_new(5, 3, FALSE);
-	gtk_table_set_row_spacings(GTK_TABLE(table), -1);
+	table = gtk_table_new(5, 2, FALSE);
+	gtk_table_set_row_spacings(GTK_TABLE(table), 4);
+	gtk_table_set_col_spacings(GTK_TABLE(table), 4);
 	rowcount = 0;
 
 	/* Apply to subfolders */
 	label = gtk_label_new(_("Apply to\nsubfolders"));
 	gtk_misc_set_alignment(GTK_MISC(label), 0.5, 0.5);
-	gtk_table_attach_defaults(GTK_TABLE(table), label, 2, 3,
-				  rowcount, rowcount + 1);
+	gtk_table_attach(GTK_TABLE(table), label, 2, 3,
+			 rowcount, rowcount + 1, GTK_SHRINK, GTK_SHRINK, 0, 0);
 	rowcount++;
 
 	/* Simplify Subject */
 	checkbtn_simplify_subject = gtk_check_button_new_with_label(_("Simplify Subject RegExp: "));
 	gtk_table_attach(GTK_TABLE(table), checkbtn_simplify_subject, 0, 1, 
-			 rowcount, rowcount + 1, GTK_SHRINK | GTK_FILL, GTK_SHRINK, 0, 0);
+			 rowcount, rowcount + 1, GTK_SHRINK | GTK_FILL, GTK_FILL, 0, 0);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbtn_simplify_subject), 
 				     item->prefs->enable_simplify_subject);
 
 	entry_simplify_subject = gtk_entry_new();
-	gtk_table_attach_defaults(GTK_TABLE(table), entry_simplify_subject, 1, 2, 
-				  rowcount, rowcount + 1);
+	gtk_table_attach(GTK_TABLE(table), entry_simplify_subject, 1, 2, 
+			 rowcount, rowcount + 1, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
 	SET_TOGGLE_SENSITIVITY(checkbtn_simplify_subject, entry_simplify_subject);
 	gtk_entry_set_text(GTK_ENTRY(entry_simplify_subject), 
 			   SAFE_STRING(item->prefs->simplify_subject_regexp));
@@ -192,14 +195,14 @@ void prefs_folder_item_general_create_widget_func(PrefsPage * page_,
 	/* Folder chmod */
 	checkbtn_folder_chmod = gtk_check_button_new_with_label(_("Folder chmod: "));
 	gtk_table_attach(GTK_TABLE(table), checkbtn_folder_chmod, 0, 1, 
-			 rowcount, rowcount + 1, GTK_SHRINK | GTK_FILL, GTK_SHRINK, 0, 0);
+			 rowcount, rowcount + 1, GTK_SHRINK | GTK_FILL, GTK_FILL, 0, 0);
 
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbtn_folder_chmod), 
 				     item->prefs->enable_folder_chmod);
 
 	entry_folder_chmod = gtk_entry_new();
-	gtk_table_attach_defaults(GTK_TABLE(table), entry_folder_chmod, 1, 2, 
-				  rowcount, rowcount + 1);
+	gtk_table_attach(GTK_TABLE(table), entry_folder_chmod, 1, 2, 
+			 rowcount, rowcount + 1, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
 	SET_TOGGLE_SENSITIVITY(checkbtn_folder_chmod, entry_folder_chmod);
 	if (item->prefs->folder_chmod) {
 		gchar *buf;
@@ -218,22 +221,22 @@ void prefs_folder_item_general_create_widget_func(PrefsPage * page_,
 	/* Folder color */
 	folder_color = gtk_label_new(_("Folder color: "));
 	gtk_misc_set_alignment(GTK_MISC(folder_color), 0, 0.5);
-	gtk_table_attach_defaults(GTK_TABLE(table), folder_color, 0, 1, 
-			 rowcount, rowcount + 1);
+	gtk_table_attach(GTK_TABLE(table), folder_color, 0, 1, 
+			 rowcount, rowcount + 1, GTK_SHRINK | GTK_FILL, GTK_FILL, 0, 0);
 
 	hbox = gtk_hbox_new(FALSE, 0);
-	gtk_table_attach_defaults(GTK_TABLE(table), hbox, 1, 2, 
-				  rowcount, rowcount + 1);
+	gtk_table_attach(GTK_TABLE(table), hbox, 1, 2, 
+			 rowcount, rowcount + 1, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
 
 	folder_color_btn = gtk_button_new_with_label("");
-	gtk_widget_set_usize(folder_color_btn, 36, 26);
+	gtk_widget_set_size_request(folder_color_btn, 36, 26);
   	gtk_box_pack_start (GTK_BOX(hbox), folder_color_btn, FALSE, FALSE, 0);
 
 	page->folder_color = item->prefs->color;
 
-	gtk_signal_connect(GTK_OBJECT(folder_color_btn), "clicked",
-			   GTK_SIGNAL_FUNC(folder_color_set_dialog),
-			   page);
+	g_signal_connect(G_OBJECT(folder_color_btn), "clicked",
+			 G_CALLBACK(folder_color_set_dialog),
+			 page);
 
 	gtkut_set_widget_bgcolor_rgb(folder_color_btn, item->prefs->color);
 
@@ -246,7 +249,7 @@ void prefs_folder_item_general_create_widget_func(PrefsPage * page_,
 	/* Enable processing at startup */
 	checkbtn_enable_processing = gtk_check_button_new_with_label(_("Process at startup"));
 	gtk_table_attach(GTK_TABLE(table), checkbtn_enable_processing, 0, 2, 
-			 rowcount, rowcount + 1, GTK_SHRINK | GTK_FILL, GTK_FILL, 0, 0);
+			 rowcount, rowcount + 1, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
 
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbtn_enable_processing), 
 				     item->prefs->enable_processing);
@@ -260,7 +263,7 @@ void prefs_folder_item_general_create_widget_func(PrefsPage * page_,
 	/* Check folder for new mail */
 	checkbtn_newmailcheck = gtk_check_button_new_with_label(_("Scan for new mail"));
 	gtk_table_attach(GTK_TABLE(table), checkbtn_newmailcheck, 0, 2,
-					 rowcount, rowcount+1, GTK_SHRINK | GTK_FILL, GTK_FILL, 0, 0);
+			 rowcount, rowcount+1, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
 	
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbtn_newmailcheck),
 								 item->prefs->newmailcheck);
@@ -433,15 +436,16 @@ void prefs_folder_item_compose_create_widget_func(PrefsPage * page_,
 #else
 # define TABLEHEIGHT 5
 #endif
-	table = gtk_table_new(TABLEHEIGHT, 3, FALSE);
-	gtk_table_set_row_spacings(GTK_TABLE(table), -1);
+	table = gtk_table_new(TABLEHEIGHT, 2, FALSE);
+	gtk_table_set_row_spacings(GTK_TABLE(table), 4);
+	gtk_table_set_col_spacings(GTK_TABLE(table), 4);
 	rowcount = 0;
 
 	/* Apply to subfolders */
 	label = gtk_label_new(_("Apply to\nsubfolders"));
 	gtk_misc_set_alignment(GTK_MISC(label), 0.5, 0.5);
-	gtk_table_attach_defaults(GTK_TABLE(table), label, 2, 3,
-				  rowcount, rowcount + 1);
+	gtk_table_attach(GTK_TABLE(table), label, 2, 3,
+			 rowcount, rowcount + 1, GTK_SHRINK, GTK_SHRINK, 0, 0);
 	rowcount++;
 
 	/* Request Return Receipt */
@@ -462,7 +466,6 @@ void prefs_folder_item_compose_create_widget_func(PrefsPage * page_,
 	/* Save Copy to Folder */
 	checkbtn_save_copy_to_folder = gtk_check_button_new_with_label
 		(_("Save copy of outgoing messages to this folder instead of Sent"));
-	gtk_widget_show(checkbtn_save_copy_to_folder);
 	gtk_table_attach(GTK_TABLE(table), checkbtn_save_copy_to_folder, 0, 2, 
 			 rowcount, rowcount + 1, GTK_SHRINK | GTK_FILL, GTK_FILL, 0, 0);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbtn_save_copy_to_folder),
@@ -482,7 +485,8 @@ void prefs_folder_item_compose_create_widget_func(PrefsPage * page_,
 				     item->prefs->enable_default_to);
 
 	entry_default_to = gtk_entry_new();
-	gtk_table_attach_defaults(GTK_TABLE(table), entry_default_to, 1, 2, rowcount, rowcount + 1);
+	gtk_table_attach(GTK_TABLE(table), entry_default_to, 1, 2,
+			 rowcount, rowcount + 1, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 0, 0);
 	SET_TOGGLE_SENSITIVITY(checkbtn_default_to, entry_default_to);
 	gtk_entry_set_text(GTK_ENTRY(entry_default_to), SAFE_STRING(item->prefs->default_to));
 	address_completion_register_entry(GTK_ENTRY(entry_default_to));
@@ -495,14 +499,14 @@ void prefs_folder_item_compose_create_widget_func(PrefsPage * page_,
 
 	/* Default address to reply to */
 	checkbtn_default_reply_to = gtk_check_button_new_with_label(_("Send replies to: "));
-	gtk_widget_show(checkbtn_default_reply_to);
 	gtk_table_attach(GTK_TABLE(table), checkbtn_default_reply_to, 0, 1, 
 			 rowcount, rowcount + 1, GTK_SHRINK | GTK_FILL, GTK_SHRINK, 0, 0);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbtn_default_reply_to), 
 				     item->prefs->enable_default_reply_to);
 
 	entry_default_reply_to = gtk_entry_new();
-	gtk_table_attach_defaults(GTK_TABLE(table), entry_default_reply_to, 1, 2, rowcount, rowcount + 1);
+	gtk_table_attach(GTK_TABLE(table), entry_default_reply_to, 1, 2,
+			 rowcount, rowcount + 1, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 0, 0);
 	SET_TOGGLE_SENSITIVITY(checkbtn_default_reply_to, entry_default_reply_to);
 	gtk_entry_set_text(GTK_ENTRY(entry_default_reply_to), SAFE_STRING(item->prefs->default_reply_to));
 	address_completion_register_entry(GTK_ENTRY(entry_default_reply_to));
@@ -521,8 +525,8 @@ void prefs_folder_item_compose_create_widget_func(PrefsPage * page_,
 				     item->prefs->enable_default_account);
 
  	optmenu_default_account = gtk_option_menu_new ();
-	gtk_table_attach_defaults(GTK_TABLE(table), optmenu_default_account, 1, 2, 
-				  rowcount, rowcount + 1);
+	gtk_table_attach(GTK_TABLE(table), optmenu_default_account, 1, 2, 
+			 rowcount, rowcount + 1, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 0, 0);
  	optmenu_default_account_menu = gtk_menu_new ();
 
 	account_list = account_get_list();
@@ -565,8 +569,8 @@ void prefs_folder_item_compose_create_widget_func(PrefsPage * page_,
 	    			     item->prefs->enable_default_dictionary);
 
 	optmenu_default_dictionary = gtk_option_menu_new();
-	gtk_table_attach_defaults(GTK_TABLE(table), optmenu_default_dictionary, 1, 2,
-	    			rowcount, rowcount + 1);
+	gtk_table_attach(GTK_TABLE(table), optmenu_default_dictionary, 1, 2,
+	    		 rowcount, rowcount + 1, GTK_EXPAND | GTK_FILL, GTK_SHRINK, 0, 0);
 
 	gtk_option_menu_set_menu(GTK_OPTION_MENU(optmenu_default_dictionary), 
 				 gtkaspell_dictionary_option_menu_new(
@@ -680,7 +684,7 @@ static void compose_save_folder_prefs(FolderItem *folder, FolderItemComposePage
 			gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(page->checkbtn_enable_default_account));
 		menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(page->optmenu_default_account));
 		menuitem = gtk_menu_get_active(GTK_MENU(menu));
-		prefs->default_account = GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(menuitem)));
+		prefs->default_account = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(menuitem), MENU_VAL_ID));
 	}
 
 #if USE_ASPELL
