@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2002 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2005 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,6 +21,8 @@
 #  include "config.h"
 #endif
 
+#include <glib.h>
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -28,7 +30,6 @@
 #include <unistd.h>
 #include <errno.h>
 
-#include "intl.h"
 #include "main.h"
 #include "prefs.h"
 #include "prefs_gtk.h"
@@ -36,6 +37,7 @@
 #include "gtkutils.h"
 #include "passcrypt.h"
 #include "base64.h"
+#include "codeconv.h"
 
 #define CL(x)	(((gulong) (x) >> (gulong) 8) & 0xFFUL)
 #define RGB_FROM_GDK_COLOR(c) \
@@ -49,11 +51,10 @@ typedef enum
 } DummyEnum;
 
 void prefs_read_config(PrefParam *param, const gchar *label,
-		       const gchar *rcfile)
+		       const gchar *rcfile, const gchar *encoding)
 {
 	FILE *fp;
 	gchar buf[PREFSBUFSIZE];
-	gchar *rcpath;
 	gchar *block_label;
 
 	g_return_if_fail(param != NULL);
@@ -64,13 +65,10 @@ void prefs_read_config(PrefParam *param, const gchar *label,
 
 	prefs_set_default(param);
 
-	rcpath = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, rcfile, NULL);
-	if ((fp = fopen(rcpath, "rb")) == NULL) {
-		if (ENOENT != errno) FILE_OP_ERROR(rcpath, "fopen");
-		g_free(rcpath);
+	if ((fp = fopen(rcfile, "rb")) == NULL) {
+		if (ENOENT != errno) FILE_OP_ERROR(rcfile, "fopen");
 		return;
 	}
-	g_free(rcpath);
 
 	block_label = g_strdup_printf("[%s]", label);
 
@@ -78,7 +76,18 @@ void prefs_read_config(PrefParam *param, const gchar *label,
 	while (fgets(buf, sizeof(buf), fp) != NULL) {
 		gint val;
 
-		val = strncmp(buf, block_label, strlen(block_label));
+		if (encoding) {
+			gchar *conv_str;
+
+			conv_str = conv_codeset_strdup
+				(buf, encoding, CS_INTERNAL);
+			if (!conv_str)
+				conv_str = g_strdup(buf);
+			val = strncmp
+				(conv_str, block_label, strlen(block_label));
+			g_free(conv_str);
+		} else
+			val = strncmp(buf, block_label, strlen(block_label));
 		if (val == 0) {
 			debug_print("Found %s\n", block_label);
 			break;
@@ -91,7 +100,17 @@ void prefs_read_config(PrefParam *param, const gchar *label,
 		/* reached next block */
 		if (buf[0] == '[') break;
 
-		prefs_config_parse_one_line(param, buf);
+		if (encoding) {
+			gchar *conv_str;
+
+			conv_str = conv_codeset_strdup
+				(buf, encoding, CS_INTERNAL);
+			if (!conv_str)
+				conv_str = g_strdup(buf);
+			prefs_config_parse_one_line(param, conv_str);
+			g_free(conv_str);
+		} else
+			prefs_config_parse_one_line(param, buf);
 	}
 
 	debug_print("Finished reading configuration.\n");
@@ -107,7 +126,7 @@ void prefs_config_parse_one_line(PrefParam *param, const gchar *buf)
 
 	for (i = 0; param[i].name != NULL; i++) {
 		name_len = strlen(param[i].name);
-		if (g_strncasecmp(buf, param[i].name, name_len))
+		if (g_ascii_strncasecmp(buf, param[i].name, name_len))
 			continue;
 		if (buf[name_len] != '=')
 			continue;
@@ -116,10 +135,22 @@ void prefs_config_parse_one_line(PrefParam *param, const gchar *buf)
 
 		switch (param[i].type) {
 		case P_STRING:
+		{
+			gchar *tmp;
+
+			tmp = *value ?
+				conv_codeset_strdup(value,
+						    conv_get_locale_charset_str(),
+						    CS_UTF_8)
+				: g_strdup("");
+			if (!tmp) {
+				g_warning("failed to convert character set.");
+				tmp = g_strdup(value);
+			}
 			g_free(*((gchar **)param[i].data));
-			*((gchar **)param[i].data) =
-				*value ? g_strdup(value) : NULL;
+			*((gchar **)param[i].data) = tmp;
 			break;
+		}
 		case P_INT:
 			*((gint *)param[i].data) =
 				(gint)atoi(value);
@@ -256,10 +287,23 @@ gint prefs_write_param(PrefParam *param, FILE *fp)
 	for (i = 0; param[i].name != NULL; i++) {
 		switch (param[i].type) {
 		case P_STRING:
+		{
+			gchar *tmp = NULL;
+
+			if (*((gchar **)param[i].data)) {
+				tmp = conv_codeset_strdup(*((gchar **)param[i].data),
+							  CS_UTF_8,
+							  conv_get_locale_charset_str());
+				if (!tmp)
+					tmp = g_strdup(*((gchar **)param[i].data));
+			}
+
 			g_snprintf(buf, sizeof(buf), "%s=%s\n", param[i].name,
-				   *((gchar **)param[i].data) ?
-				   *((gchar **)param[i].data) : "");
+				   tmp ? tmp : "");
+
+			g_free(tmp);
 			break;
+		}
 		case P_INT:
 			g_snprintf(buf, sizeof(buf), "%s=%d\n", param[i].name,
 				   *((gint *)param[i].data));
@@ -332,10 +376,22 @@ void prefs_set_default(PrefParam *param)
 		case P_PASSWORD:
 			g_free(*((gchar **)param[i].data));
 			if (param[i].defval != NULL) {
-				if (!g_strncasecmp(param[i].defval, "ENV_", 4))
-					*((gchar **)param[i].data) =
-						g_strdup(g_getenv(param[i].defval + 4));
-				else if (param[i].defval[0] == '~')
+				if (!strncasecmp(param[i].defval, "ENV_", 4)) {
+					const gchar *envstr;
+					gchar *tmp;
+
+					envstr = g_getenv(param[i].defval + 4);
+					tmp = envstr && *envstr ?
+						conv_codeset_strdup(envstr,
+								    conv_get_locale_charset_str(),
+								    CS_UTF_8)
+						: g_strdup("");
+					if (!tmp) {
+						g_warning("faild to convert character set.");
+						tmp = g_strdup(envstr);
+					}
+					*((gchar **)param[i].data) = tmp;
+				} else if (param[i].defval[0] == '~')
 					*((gchar **)param[i].data) =
 						g_strconcat(get_home_dir(),
 							    param[i].defval + 1,
@@ -357,7 +413,7 @@ void prefs_set_default(PrefParam *param)
 			break;
 		case P_BOOL:
 			if (param[i].defval != NULL) {
-				if (!g_strcasecmp(param[i].defval, "TRUE"))
+				if (!g_ascii_strcasecmp(param[i].defval, "TRUE"))
 					*((gboolean *)param[i].data) = TRUE;
 				else
 					*((gboolean *)param[i].data) =
@@ -428,11 +484,11 @@ void prefs_dialog_create(PrefsDialog *dialog)
 
 	g_return_if_fail(dialog != NULL);
 
-	window = gtk_window_new (GTK_WINDOW_DIALOG);
+	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	gtk_container_set_border_width (GTK_CONTAINER (window), 8);
-	gtk_window_position (GTK_WINDOW(window), GTK_WIN_POS_CENTER);
+	gtk_window_set_position (GTK_WINDOW(window), GTK_WIN_POS_CENTER);
 	gtk_window_set_modal (GTK_WINDOW (window), TRUE);
-	gtk_window_set_policy (GTK_WINDOW(window), FALSE, TRUE, FALSE);
+	gtk_window_set_resizable(GTK_WINDOW(window), TRUE);
 
 	vbox = gtk_vbox_new (FALSE, 6);
 	gtk_widget_show(vbox);
@@ -446,11 +502,11 @@ void prefs_dialog_create(PrefsDialog *dialog)
 	gtk_notebook_set_scrollable (GTK_NOTEBOOK (notebook), TRUE);
 	
 	gtk_notebook_popup_enable (GTK_NOTEBOOK (notebook));
-	
-	gtkut_button_set_create(&confirm_area,
-				&ok_btn,	_("OK"),
-				&cancel_btn,	_("Cancel"),
-				&apply_btn,	_("Apply"));
+
+	gtkut_stock_button_set_create(&confirm_area,
+				      &ok_btn, GTK_STOCK_OK,
+				      &cancel_btn, GTK_STOCK_CANCEL,
+				      &apply_btn, GTK_STOCK_APPLY);
 	gtk_widget_show(confirm_area);
 	gtk_box_pack_end (GTK_BOX(vbox), confirm_area, FALSE, FALSE, 0);
 	gtk_widget_grab_default(ok_btn);
@@ -519,7 +575,7 @@ void prefs_set_dialog_to_default(PrefParam *param)
 		case P_STRING:
 		case P_PASSWORD:
 			if (tmpparam.defval) {
-				if (!g_strncasecmp(tmpparam.defval, "ENV_", 4)) {
+				if (!g_ascii_strncasecmp(tmpparam.defval, "ENV_", 4)) {
 					str_data = g_strdup(g_getenv(param[i].defval + 4));
 					tmpparam.data = &str_data;
 					break;
@@ -550,7 +606,7 @@ void prefs_set_dialog_to_default(PrefParam *param)
 			break;
 		case P_BOOL:
 			if (tmpparam.defval) {
-				if (!g_strcasecmp(tmpparam.defval, "TRUE"))
+				if (!g_ascii_strcasecmp(tmpparam.defval, "TRUE"))
 					bool_data = TRUE;
 				else
 					bool_data = atoi(tmpparam.defval)
@@ -578,7 +634,8 @@ void prefs_set_dialog_to_default(PrefParam *param)
 
 void prefs_set_data_from_entry(PrefParam *pparam)
 {
-	gchar **str, *entry_str;
+	gchar **str;
+	const gchar *entry_str;
 
 	g_return_if_fail(*pparam->widget != NULL);
 
@@ -633,7 +690,7 @@ void prefs_set_entry(PrefParam *pparam)
 void prefs_set_data_from_text(PrefParam *pparam)
 {
 	gchar **str;
-	gchar *text, *tp;
+	gchar *text = NULL, *tp = NULL;
 	gchar *tmp, *tmpp;
 
 	g_return_if_fail(*pparam->widget != NULL);
@@ -643,8 +700,20 @@ void prefs_set_data_from_text(PrefParam *pparam)
 	case P_PASSWORD:
 		str = (gchar **)pparam->data;
 		g_free(*str);
-		tp = text = gtk_editable_get_chars
-			(GTK_EDITABLE(*pparam->widget), 0, -1);
+		if (GTK_IS_EDITABLE(*pparam->widget)) {   /* need? */
+			tp = text = gtk_editable_get_chars
+					(GTK_EDITABLE(*pparam->widget), 0, -1);
+		} else if (GTK_IS_TEXT_VIEW(*pparam->widget)) {
+			GtkTextView *textview = GTK_TEXT_VIEW(*pparam->widget);
+			GtkTextBuffer *buffer = gtk_text_view_get_buffer(textview);
+			GtkTextIter start, end;
+			gtk_text_buffer_get_start_iter(buffer, &start);
+			gtk_text_buffer_get_iter_at_offset(buffer, &end, -1);
+			tp = text = gtk_text_buffer_get_text(buffer, &start, &end, FALSE);
+		}
+
+		g_return_if_fail (tp && text);
+
 		if (text[0] == '\0') {
 			*str = NULL;
 			g_free(text);
@@ -675,7 +744,9 @@ void prefs_set_text(PrefParam *pparam)
 {
 	gchar *buf, *sp, *bufp;
 	gchar **str;
-	GtkText *text;
+	GtkTextView *text;
+	GtkTextBuffer *buffer;
+	GtkTextIter iter;
 
 	g_return_if_fail(*pparam->widget != NULL);
 
@@ -700,14 +771,14 @@ void prefs_set_text(PrefParam *pparam)
 		} else
 			buf = "";
 
-		text = GTK_TEXT(*pparam->widget);
-		gtk_text_set_point(text, 0);
-		gtk_text_forward_delete(text, gtk_text_get_length(text));
-		gtk_text_set_point(text, 0);
-		gtk_text_insert(text, NULL, NULL, NULL, buf, -1);
+		text = GTK_TEXT_VIEW(*pparam->widget);
+		buffer = gtk_text_view_get_buffer(text);
+		gtk_text_buffer_set_text(buffer, "", -1);
+		gtk_text_buffer_get_start_iter(buffer, &iter);
+		gtk_text_buffer_insert(buffer, &iter, buf, -1);
 		break;
 	default:
-		g_warning("Invalid PrefType for GtkText widget: %d\n",
+		g_warning("Invalid PrefType for GtkTextView widget: %d\n",
 			  pparam->type);
 	}
 }
