@@ -1,7 +1,7 @@
 /*
  * Sylpheed templates subsystem 
  * Copyright (C) 2001 Alexander Barinov
- * Copyright (C) 2001-2002 Hiroyuki Yamamoto
+ * Copyright (C) 2001-2005 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,13 +21,13 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include <string.h>
 #include <dirent.h>
 #include <sys/stat.h>
 
-#include "intl.h"
 #include "template.h"
 #include "main.h"
 #include "inc.h"
@@ -39,10 +39,17 @@
 #include "addr_compl.h"
 #include "quote_fmt.h"
 
+enum {
+	TEMPL_TEXT,
+	TEMPL_DATA,
+	TEMPL_AUTO_DATA,	/*!< auto pointer */
+	N_TEMPL_COLUMNS
+};
+
 static struct Templates {
 	GtkWidget *window;
 	GtkWidget *ok_btn;
-	GtkWidget *clist_tmpls;
+	GtkWidget *list_view;
 	GtkWidget *entry_name;
 	GtkWidget *entry_subject;
 	GtkWidget *entry_to;
@@ -51,8 +58,6 @@ static struct Templates {
 	GtkWidget *text_value;
 } templates;
 
-static int modified = FALSE;
-
 /* widget creating functions */
 static void prefs_template_window_create	(void);
 static void prefs_template_window_setup		(void);
@@ -64,19 +69,28 @@ static GSList *prefs_template_get_list		(void);
 static gint prefs_template_deleted_cb		(GtkWidget	*widget,
 						 GdkEventAny	*event,
 						 gpointer	 data);
-static void prefs_template_key_pressed_cb	(GtkWidget	*widget,
+static gboolean prefs_template_key_pressed_cb	(GtkWidget	*widget,
 						 GdkEventKey	*event,
 						 gpointer	 data);
 static void prefs_template_cancel_cb		(void);
 static void prefs_template_ok_cb		(void);
-static void prefs_template_select_cb		(GtkCList	*clist,
-						 gint		 row,
-						 gint		 column,
-						 GdkEvent	*event);
 static void prefs_template_register_cb		(void);
 static void prefs_template_substitute_cb	(void);
 static void prefs_template_delete_cb		(void);
 
+static GtkListStore* prefs_template_create_data_store	(void);
+static void prefs_template_list_view_insert_template	(GtkWidget *list_view,
+							 GtkTreeIter *row_iter,
+							 const gchar *template,
+							 Template *data);
+static GtkWidget *prefs_template_list_view_create	(void);
+static void prefs_template_create_list_view_columns	(GtkWidget *list_view);
+static gboolean prefs_template_selected			(GtkTreeSelection *selector,
+							 GtkTreeModel *model, 
+							 GtkTreePath *path,
+							 gboolean currently_selected,
+							 gpointer data);
+
 /* Called from mainwindow.c */
 void prefs_template_open(void)
 {
@@ -128,18 +142,16 @@ static void prefs_template_window_create(void)
 	GtkWidget           *del_btn;
 	GtkWidget         *desc_btn;
 	GtkWidget       *scroll1;
-	GtkWidget         *clist_tmpls;
+	GtkWidget         *list_view;
 	GtkWidget       *confirm_area;
 	GtkWidget         *ok_btn;
 	GtkWidget         *cancel_btn;
 
-	gchar *title[1];
-
 	/* main window */
-	window = gtk_window_new(GTK_WINDOW_DIALOG);
-	gtk_window_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
+	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
 	gtk_window_set_modal(GTK_WINDOW(window), TRUE);
-	gtk_window_set_policy(GTK_WINDOW(window), FALSE, TRUE, FALSE);
+	gtk_window_set_resizable(GTK_WINDOW(window), TRUE);
 	gtk_window_set_default_size(GTK_WINDOW(window), 400, -1);
 
 	/* vpaned to separate template settings from templates list */
@@ -185,15 +197,15 @@ static void prefs_template_window_create(void)
 	gtk_widget_show(scroll2);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll2),
 				       GTK_POLICY_NEVER,
-				       GTK_POLICY_ALWAYS);
+				       GTK_POLICY_AUTOMATIC);
 	gtk_box_pack_start(GTK_BOX(vbox1), scroll2, TRUE, TRUE, 0);
 
-	text_value = gtk_text_new(NULL, NULL);
+	text_value = gtk_text_view_new();
 	gtk_widget_show(text_value);
-	gtk_widget_set_usize(text_value, -1, 120);
+	gtk_widget_set_size_request(text_value, -1, 120);
 	gtk_container_add(GTK_CONTAINER(scroll2), text_value);
-	gtk_text_set_editable(GTK_TEXT(text_value), TRUE);
-	gtk_text_set_word_wrap(GTK_TEXT(text_value), TRUE);
+	gtk_text_view_set_editable(GTK_TEXT_VIEW(text_value), TRUE);
+	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(text_value), GTK_WRAP_WORD);
 
 	/* vbox for buttons and templates list */
 	vbox2 = gtk_vbox_new(FALSE, 6);
@@ -209,36 +221,36 @@ static void prefs_template_window_create(void)
 	arrow1 = gtk_arrow_new(GTK_ARROW_DOWN, GTK_SHADOW_OUT);
 	gtk_widget_show(arrow1);
 	gtk_box_pack_start(GTK_BOX(hbox2), arrow1, FALSE, FALSE, 0);
-	gtk_widget_set_usize(arrow1, -1, 16);
+	gtk_widget_set_size_request(arrow1, -1, 16);
 
 	hbox3 = gtk_hbox_new(TRUE, 4);
 	gtk_widget_show(hbox3);
 	gtk_box_pack_start(GTK_BOX(hbox2), hbox3, FALSE, FALSE, 0);
 
-	reg_btn = gtk_button_new_with_label(_("Add"));
+	reg_btn = gtk_button_new_from_stock(GTK_STOCK_ADD);
 	gtk_widget_show(reg_btn);
 	gtk_box_pack_start(GTK_BOX(hbox3), reg_btn, FALSE, TRUE, 0);
-	gtk_signal_connect(GTK_OBJECT (reg_btn), "clicked",
-			   GTK_SIGNAL_FUNC (prefs_template_register_cb), NULL);
+	g_signal_connect(G_OBJECT (reg_btn), "clicked",
+			 G_CALLBACK (prefs_template_register_cb), NULL);
 
 	subst_btn = gtk_button_new_with_label(_("  Replace  "));
 	gtk_widget_show(subst_btn);
 	gtk_box_pack_start(GTK_BOX(hbox3), subst_btn, FALSE, TRUE, 0);
-	gtk_signal_connect(GTK_OBJECT(subst_btn), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_template_substitute_cb),
-			   NULL);
+	g_signal_connect(G_OBJECT(subst_btn), "clicked",
+			 G_CALLBACK(prefs_template_substitute_cb),
+			 NULL);
 
-	del_btn = gtk_button_new_with_label(_("Delete"));
+	del_btn = gtk_button_new_from_stock(GTK_STOCK_REMOVE);
 	gtk_widget_show(del_btn);
 	gtk_box_pack_start(GTK_BOX(hbox3), del_btn, FALSE, TRUE, 0);
-	gtk_signal_connect(GTK_OBJECT(del_btn), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_template_delete_cb), NULL);
+	g_signal_connect(G_OBJECT(del_btn), "clicked",
+			 G_CALLBACK(prefs_template_delete_cb), NULL);
 
 	desc_btn = gtk_button_new_with_label(_(" Symbols "));
 	gtk_widget_show(desc_btn);
 	gtk_box_pack_end(GTK_BOX(hbox2), desc_btn, FALSE, FALSE, 0);
-	gtk_signal_connect(GTK_OBJECT(desc_btn), "clicked",
-			   GTK_SIGNAL_FUNC(quote_fmt_quote_description), NULL);
+	g_signal_connect(G_OBJECT(desc_btn), "clicked",
+			 G_CALLBACK(quote_fmt_quote_description), NULL);
 
 	/* templates list */
 	scroll1 = gtk_scrolled_window_new(NULL, NULL);
@@ -247,44 +259,36 @@ static void prefs_template_window_create(void)
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll1),
 				       GTK_POLICY_AUTOMATIC,
 				       GTK_POLICY_AUTOMATIC);
-
-	title[0] = _("Current templates");
-	clist_tmpls = gtk_clist_new_with_titles(1, title);
-	gtk_widget_show(clist_tmpls);
-	gtk_widget_set_usize(scroll1, -1, 140);
-	gtk_container_add(GTK_CONTAINER(scroll1), clist_tmpls);
-	gtk_clist_set_column_width(GTK_CLIST(clist_tmpls), 0, 80);
-	gtk_clist_set_selection_mode(GTK_CLIST(clist_tmpls),
-				     GTK_SELECTION_BROWSE);
-	GTK_WIDGET_UNSET_FLAGS(GTK_CLIST(clist_tmpls)->column[0].button,
-			       GTK_CAN_FOCUS);
-	gtk_signal_connect(GTK_OBJECT (clist_tmpls), "select_row",
-			   GTK_SIGNAL_FUNC (prefs_template_select_cb), NULL);
+				       
+	list_view = prefs_template_list_view_create();
+	gtk_widget_show(list_view);
+	gtk_widget_set_size_request(scroll1, -1, 140);
+	gtk_container_add(GTK_CONTAINER(scroll1), list_view);
 
 	/* ok | cancel */
-	gtkut_button_set_create(&confirm_area, &ok_btn, _("OK"),
-				&cancel_btn, _("Cancel"), NULL, NULL);
+	gtkut_stock_button_set_create(&confirm_area, &ok_btn, GTK_STOCK_OK,
+				&cancel_btn, GTK_STOCK_CANCEL, NULL, NULL);
 	gtk_widget_show(confirm_area);
 	gtk_box_pack_end(GTK_BOX(vbox2), confirm_area, FALSE, FALSE, 0);
 	gtk_widget_grab_default(ok_btn);
 
 	gtk_window_set_title(GTK_WINDOW(window), _("Template configuration"));
 
-	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
-			   GTK_SIGNAL_FUNC(prefs_template_deleted_cb), NULL);
-	gtk_signal_connect(GTK_OBJECT(window), "key_press_event",
-			   GTK_SIGNAL_FUNC(prefs_template_key_pressed_cb), NULL);
+	g_signal_connect(G_OBJECT(window), "delete_event",
+			 G_CALLBACK(prefs_template_deleted_cb), NULL);
+	g_signal_connect(G_OBJECT(window), "key_press_event",
+			 G_CALLBACK(prefs_template_key_pressed_cb), NULL);
 	MANAGE_WINDOW_SIGNALS_CONNECT(window);
-	gtk_signal_connect(GTK_OBJECT(ok_btn), "clicked",
-			   GTK_SIGNAL_FUNC(prefs_template_ok_cb), NULL);
-	gtk_signal_connect(GTK_OBJECT(cancel_btn), "clicked",
-			    GTK_SIGNAL_FUNC(prefs_template_cancel_cb), NULL);
+	g_signal_connect(G_OBJECT(ok_btn), "clicked",
+			 G_CALLBACK(prefs_template_ok_cb), NULL);
+	g_signal_connect(G_OBJECT(cancel_btn), "clicked",
+			 G_CALLBACK(prefs_template_cancel_cb), NULL);
 
 	address_completion_start(window);
 
 	templates.window = window;
 	templates.ok_btn = ok_btn;
-	templates.clist_tmpls = clist_tmpls;
+	templates.list_view = list_view;
 	templates.entry_name = entry_name;
 	templates.entry_subject = entry_subject;
 	templates.entry_to = entry_to;
@@ -295,49 +299,44 @@ static void prefs_template_window_create(void)
 
 static void prefs_template_window_setup(void)
 {
-	GtkCList *clist = GTK_CLIST(templates.clist_tmpls);
 	GSList *tmpl_list;
 	GSList *cur;
 	gchar *title[1];
 	gint row;
 	Template *tmpl;
+	GtkListStore *store;
+
+	store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW
+				(templates.list_view)));
 
 	manage_window_set_transient(GTK_WINDOW(templates.window));
 	gtk_widget_grab_focus(templates.ok_btn);
 
-	gtk_clist_freeze(clist);
-	gtk_clist_clear(clist);
-
-	title[0] = _("(New)");
-	row = gtk_clist_append(clist, title);
-	gtk_clist_set_row_data(clist, row, NULL);
+	gtk_list_store_clear(store);
 
+	prefs_template_list_view_insert_template(templates.list_view,
+						 NULL, _("(New)"),
+						 NULL);
+	
 	tmpl_list = template_read_config();
 
 	for (cur = tmpl_list; cur != NULL; cur = cur->next) {
 		tmpl = (Template *)cur->data;
-		title[0] = tmpl->name;
-		row = gtk_clist_append(clist, title);
-		gtk_clist_set_row_data(clist, row, tmpl);
+		prefs_template_list_view_insert_template(templates.list_view,
+							 NULL, tmpl->name, 
+							 tmpl);
 	}
 
 	g_slist_free(tmpl_list);
-
-	gtk_clist_thaw(clist);
 }
 
 static void prefs_template_clear(void)
 {
-	Template *tmpl;
-	gint row = 1;
+	GtkListStore *store;
 
-	while ((tmpl = gtk_clist_get_row_data
-		(GTK_CLIST(templates.clist_tmpls), row)) != NULL) {
-		template_free(tmpl);
-		row++;
-	}
-
-	gtk_clist_clear(GTK_CLIST(templates.clist_tmpls));
+	store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW
+				(templates.list_view)));
+	gtk_list_store_clear(store);
 }
 
 static gint prefs_template_deleted_cb(GtkWidget *widget, GdkEventAny *event,
@@ -347,106 +346,85 @@ static gint prefs_template_deleted_cb(GtkWidget *widget, GdkEventAny *event,
 	return TRUE;
 }
 
-static void prefs_template_key_pressed_cb(GtkWidget *widget,
-					  GdkEventKey *event, gpointer data)
+static gboolean prefs_template_key_pressed_cb(GtkWidget *widget,
+					      GdkEventKey *event, gpointer data)
 {
 	if (event && event->keyval == GDK_Escape)
 		prefs_template_cancel_cb();
-	else {
-		GtkWidget *focused = gtkut_get_focused_child(
-					GTK_CONTAINER(widget));
-		if (focused && GTK_IS_EDITABLE(focused)) {
-			modified = TRUE;
-		}
-	}
-	return;
+	return FALSE;
 }
 
 static void prefs_template_ok_cb(void)
 {
 	GSList *tmpl_list;
 
-	if (modified && alertpanel(_("Entry not saved"),
-				 _("The entry was not saved. Close anyway?"),
-				 _("Yes"), _("No"), NULL) != G_ALERTDEFAULT) {
-		return;
-	}
-	modified = FALSE;
 	tmpl_list = prefs_template_get_list();
 	template_set_config(tmpl_list);
 	compose_reflect_prefs_all();
-	gtk_clist_clear(GTK_CLIST(templates.clist_tmpls));
+	prefs_template_clear();
 	gtk_widget_hide(templates.window);
 	inc_unlock();
 }
 
 static void prefs_template_cancel_cb(void)
 {
-	if (modified && alertpanel(_("Entry not saved"),
-				 _("The entry was not saved. Close anyway?"),
-				 _("Yes"), _("No"), NULL) != G_ALERTDEFAULT) {
-		return;
-	}
-	modified = FALSE;
 	prefs_template_clear();
 	gtk_widget_hide(templates.window);
 	inc_unlock();
 }
 
-static void prefs_template_select_cb(GtkCList *clist, gint row, gint column,
-				     GdkEvent *event)
-{
-	Template *tmpl;
-	Template tmpl_def;
-
-	tmpl_def.name = _("Template");
-	tmpl_def.subject = "";
-	tmpl_def.to = "";
-	tmpl_def.cc = "";
-	tmpl_def.bcc = "";	
-	tmpl_def.value = "";
-
-	if (!(tmpl = gtk_clist_get_row_data(clist, row)))
-		tmpl = &tmpl_def;
-
-	gtk_entry_set_text(GTK_ENTRY(templates.entry_name), tmpl->name);
-	gtk_entry_set_text(GTK_ENTRY(templates.entry_to),
-			   tmpl->to ? tmpl->to : "");
-	gtk_entry_set_text(GTK_ENTRY(templates.entry_cc),
-			   tmpl->cc ? tmpl->cc : "");
-	gtk_entry_set_text(GTK_ENTRY(templates.entry_bcc),
-			   tmpl->bcc ? tmpl->bcc : "");			
-	gtk_entry_set_text(GTK_ENTRY(templates.entry_subject),
-			   tmpl->subject ? tmpl->subject : "");
-	
-	gtk_text_freeze(GTK_TEXT(templates.text_value));
-	gtk_text_set_point(GTK_TEXT(templates.text_value), 0);
-	gtk_text_forward_delete
-		(GTK_TEXT(templates.text_value), 
-		 gtk_text_get_length(GTK_TEXT(templates.text_value)));
-	gtk_text_insert(GTK_TEXT(templates.text_value), NULL, NULL, NULL,
-	                tmpl->value, -1);
-	gtk_text_thaw(GTK_TEXT(templates.text_value));
-}
-
+/*!
+ *\brief	Request list for storage. New list is owned
+ *		by template.c...
+ */
 static GSList *prefs_template_get_list(void)
 {
-	gint row = 1;
 	GSList *tmpl_list = NULL;
 	Template *tmpl;
+	GtkTreeModel *model;
+	GtkTreeIter iter;
 
-	while ((tmpl = gtk_clist_get_row_data
-		(GTK_CLIST(templates.clist_tmpls), row)) != NULL) {
-		tmpl_list = g_slist_append(tmpl_list, tmpl);
-		row++;
-	}
+	model = gtk_tree_view_get_model(GTK_TREE_VIEW(templates.list_view));
+	if (!gtk_tree_model_get_iter_first(model, &iter))
+		return NULL;
+
+	do {
+		gtk_tree_model_get(model, &iter,
+				   TEMPL_DATA, &tmpl,
+				   -1);
+		
+		if (tmpl) {
+			Template *ntmpl;
+			
+			ntmpl = g_new(Template, 1);
+			ntmpl->name    = tmpl->name && *(tmpl->name) 
+					 ? g_strdup(tmpl->name) 
+					 : NULL;
+			ntmpl->subject = tmpl->subject && *(tmpl->subject) 
+					 ? g_strdup(tmpl->subject) 
+					 : NULL;
+			ntmpl->to      = tmpl->to && *(tmpl->to)
+					 ? g_strdup(tmpl->to)
+					 : NULL;
+			ntmpl->cc      = tmpl->cc && *(tmpl->cc)
+					 ? g_strdup(tmpl->cc)
+					 : NULL;
+			ntmpl->bcc     = tmpl->bcc && *(tmpl->bcc)
+					 ? g_strdup(tmpl->bcc)
+					 : NULL;	
+			ntmpl->value   = tmpl->value && *(tmpl->value)
+			                 ? g_strdup(tmpl->value)
+					 : NULL;
+			tmpl_list = g_slist_append(tmpl_list, ntmpl);
+		}			
+	
+	} while (gtk_tree_model_iter_next(model, &iter)); 
 
 	return tmpl_list;
 }
 
-static gint prefs_template_clist_set_row(gint row)
+static void prefs_template_list_view_set_row(GtkTreeIter *row)
 {
-	GtkCList *clist = GTK_CLIST(templates.clist_tmpls);
 	Template *tmpl;
 	Template *tmp_tmpl;
 	gchar *name;
@@ -455,12 +433,16 @@ static gint prefs_template_clist_set_row(gint row)
 	gchar *cc;
 	gchar *bcc;	
 	gchar *value;
-	gchar *title[1];
+	GtkTextBuffer *buffer;
+	GtkTextIter start, end;
+	GtkTreeModel *model;
 
-	g_return_val_if_fail(row != 0, -1);
+	model = gtk_tree_view_get_model(GTK_TREE_VIEW(templates.list_view));
 
-	value = gtk_editable_get_chars(GTK_EDITABLE(templates.text_value),
-				       0, -1);
+	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(templates.text_value));
+	gtk_text_buffer_get_start_iter(buffer, &start);
+	gtk_text_buffer_get_iter_at_offset(buffer, &end, -1);
+	value = gtk_text_buffer_get_text(buffer, &start, &end, FALSE);
 
 	if (value && *value != '\0') {
 		gchar *parsed_buf;
@@ -474,7 +456,7 @@ static gint prefs_template_clist_set_row(gint row)
 		if (!parsed_buf) {
 			alertpanel_error(_("Template format error."));
 			g_free(value);
-			return -1;
+			return;
 		}
 	}
 
@@ -514,59 +496,189 @@ static gint prefs_template_clist_set_row(gint row)
 	tmpl->bcc = bcc;	
 	tmpl->value = value;
 
-	title[0] = name;
-
-	if (row < 0) {
-		row = gtk_clist_append(clist, title);
-	} else {
-		gtk_clist_set_text(clist, row, 0, name);
-		tmp_tmpl = gtk_clist_get_row_data(clist, row);
-		if (tmp_tmpl)
-			template_free(tmp_tmpl);
-	}
-
-	gtk_clist_set_row_data(clist, row, tmpl);
-	return row;
+	prefs_template_list_view_insert_template(templates.list_view,
+						 row, tmpl->name, tmpl);
 }
 
 static void prefs_template_register_cb(void)
 {
-	prefs_template_clist_set_row(-1);
+	prefs_template_list_view_set_row(NULL);
 }
 
 static void prefs_template_substitute_cb(void)
 {
-	GtkCList *clist = GTK_CLIST(templates.clist_tmpls);
 	Template *tmpl;
-	gint row;
+	GtkTreeIter row;
+	GtkTreeSelection *selection;
+	GtkTreeModel *model;
 
-	if (!clist->selection) return;
+	selection = gtk_tree_view_get_selection
+			(GTK_TREE_VIEW(templates.list_view));
+	
+	if (!gtk_tree_selection_get_selected(selection, &model, &row))
+		return;
 
-	row = GPOINTER_TO_INT(clist->selection->data);
-	if (row == 0) return;
+	gtk_tree_model_get(model, &row, 
+			   TEMPL_DATA, &tmpl,
+			   -1);
 
-	tmpl = gtk_clist_get_row_data(clist, row);
 	if (!tmpl) return;
 
-	prefs_template_clist_set_row(row);
+	prefs_template_list_view_set_row(&row);
 }
 
 static void prefs_template_delete_cb(void)
 {
-	GtkCList *clist = GTK_CLIST(templates.clist_tmpls);
 	Template *tmpl;
-	gint row;
+	GtkTreeIter row;
+	GtkTreeSelection *selection;
+	GtkTreeModel *model;
 
-	if (!clist->selection) return;
-	row = GPOINTER_TO_INT(clist->selection->data);
-	if (row == 0) return;
+	selection = gtk_tree_view_get_selection
+			(GTK_TREE_VIEW(templates.list_view));
+	
+	if (!gtk_tree_selection_get_selected(selection, &model, &row))
+		return;
+
+	gtk_tree_model_get(model, &row, 
+			   TEMPL_DATA, &tmpl,
+			   -1);
+
+	if (!tmpl) 
+		return;
 
 	if (alertpanel(_("Delete template"),
 		       _("Do you really want to delete this template?"),
-		       _("Yes"), _("No"), NULL) != G_ALERTDEFAULT)
+		       GTK_STOCK_YES, GTK_STOCK_NO, NULL) != G_ALERTDEFAULT)
 		return;
 
-	tmpl = gtk_clist_get_row_data(clist, row);
-	template_free(tmpl);
-	gtk_clist_remove(clist, row);
+	gtk_list_store_remove(GTK_LIST_STORE(model), &row);		
 }
+
+static GtkListStore* prefs_template_create_data_store(void)
+{
+	return gtk_list_store_new(N_TEMPL_COLUMNS,
+				  G_TYPE_STRING,	
+				  G_TYPE_POINTER,
+				  G_TYPE_AUTO_POINTER,
+				  -1);
+}
+
+static void prefs_template_list_view_insert_template(GtkWidget *list_view,
+						     GtkTreeIter *row_iter,
+						     const gchar *template,
+						     Template *data)
+{
+	GtkTreeIter iter;
+	GtkListStore *list_store = GTK_LIST_STORE(gtk_tree_view_get_model
+					(GTK_TREE_VIEW(list_view)));
+	GAuto *auto_data;					
+
+	if (row_iter == NULL) {
+		/* append new */
+		gtk_list_store_append(list_store, &iter);
+	} else {
+		iter = *row_iter;
+	}
+
+	auto_data = g_auto_pointer_new_with_free(data, (GFreeFunc) template_free);  
+
+	/* if replacing data in an existing row, the auto pointer takes care
+	 * of destroying the Template data */
+	gtk_list_store_set(list_store, &iter,
+			   TEMPL_TEXT, template,
+			   TEMPL_DATA, data,
+			   TEMPL_AUTO_DATA, auto_data,
+			   -1);
+
+	g_auto_pointer_free(auto_data);			   
+}
+
+static GtkWidget *prefs_template_list_view_create(void)
+{
+	GtkTreeView *list_view;
+	GtkTreeSelection *selector;
+	GtkTreeModel *model;
+
+	model = GTK_TREE_MODEL(prefs_template_create_data_store());
+	list_view = GTK_TREE_VIEW(gtk_tree_view_new_with_model(model));
+	g_object_unref(model);	
+	
+	gtk_tree_view_set_rules_hint(list_view, TRUE);
+	
+	selector = gtk_tree_view_get_selection(list_view);
+	gtk_tree_selection_set_mode(selector, GTK_SELECTION_BROWSE);
+	gtk_tree_selection_set_select_function(selector, prefs_template_selected,
+					       NULL, NULL);
+
+	/* create the columns */
+	prefs_template_create_list_view_columns(GTK_WIDGET(list_view));
+
+	return GTK_WIDGET(list_view);
+}
+
+static void prefs_template_create_list_view_columns(GtkWidget *list_view)
+{
+	GtkTreeViewColumn *column;
+	GtkCellRenderer *renderer;
+
+	renderer = gtk_cell_renderer_text_new();
+	column = gtk_tree_view_column_new_with_attributes
+			(_("Current templates"),
+			 renderer,
+			 "text", TEMPL_TEXT,
+			 NULL);
+	gtk_tree_view_append_column(GTK_TREE_VIEW(list_view), column);		
+}
+
+static gboolean prefs_template_selected(GtkTreeSelection *selector,
+				        GtkTreeModel *model, 
+				        GtkTreePath *path,
+				        gboolean currently_selected,
+				        gpointer data)
+{
+	Template *tmpl;
+	Template tmpl_def;
+	GtkTextBuffer *buffer;
+	GtkTextIter iter;
+	GtkTreeIter titer;
+
+	if (currently_selected)
+		return TRUE;
+
+	if (!gtk_tree_model_get_iter(model, &titer, path))
+		return TRUE;
+
+	tmpl_def.name = _("Template");
+	tmpl_def.subject = "";
+	tmpl_def.to = "";
+	tmpl_def.cc = "";
+	tmpl_def.bcc = "";	
+	tmpl_def.value = "";
+
+	gtk_tree_model_get(model, &titer,
+			   TEMPL_DATA, &tmpl,
+			   -1);
+
+	if (!tmpl) 
+		tmpl = &tmpl_def;
+
+	gtk_entry_set_text(GTK_ENTRY(templates.entry_name), tmpl->name);
+	gtk_entry_set_text(GTK_ENTRY(templates.entry_to),
+			   tmpl->to ? tmpl->to : "");
+	gtk_entry_set_text(GTK_ENTRY(templates.entry_cc),
+			   tmpl->cc ? tmpl->cc : "");
+	gtk_entry_set_text(GTK_ENTRY(templates.entry_bcc),
+			   tmpl->bcc ? tmpl->bcc : "");			
+	gtk_entry_set_text(GTK_ENTRY(templates.entry_subject),
+			   tmpl->subject ? tmpl->subject : "");
+	
+	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(templates.text_value));
+	gtk_text_buffer_set_text(buffer, "", -1);
+	gtk_text_buffer_get_start_iter(buffer, &iter);
+	gtk_text_buffer_insert(buffer, &iter, tmpl->value, -1);
+
+	return TRUE;
+}
+
+
