@@ -22,13 +22,13 @@
 #endif
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 #include <time.h>
 #include <sys/stat.h>
 
-#include "intl.h"
 #include "procheader.h"
 #include "procmsg.h"
 #include "codeconv.h"
@@ -37,18 +37,18 @@
 
 #define BUFFSIZE	8192
 
-typedef char *(*getlinefunc)(char *, int, void *);
-typedef int (*peekcharfunc)(void *);
-typedef int (*getcharfunc)(void *);
-typedef gint (*get_one_field_func)(gchar *, gint, void *, HeaderEntry[]);
+typedef char *(*getlinefunc) (char *, size_t, void *);
+typedef int (*peekcharfunc) (void *);
+typedef int (*getcharfunc) (void *);
+typedef gint (*get_one_field_func) (gchar *, size_t, void *, HeaderEntry[]);
 
-static gint string_get_one_field(gchar *buf, gint len, char **str,
+static gint string_get_one_field(gchar *buf, size_t len, char **str,
 				 HeaderEntry hentry[]);
 
-static char *string_getline(char *buf, int len, char **str);
+static char *string_getline(char *buf, size_t len, char **str);
 static int string_peekchar(char **str);
 static int file_peekchar(FILE *fp);
-static gint generic_get_one_field(gchar *buf, gint len, void *data,
+static gint generic_get_one_field(gchar *buf, size_t len, void *data,
 				  HeaderEntry hentry[],
 				  getlinefunc getline, 
 				  peekcharfunc peekchar,
@@ -57,7 +57,7 @@ static MsgInfo *parse_stream(void *data, gboolean isstring, MsgFlags flags,
 			     gboolean full, gboolean decrypted);
 
 
-gint procheader_get_one_field(gchar *buf, gint len, FILE *fp,
+gint procheader_get_one_field(gchar *buf, size_t len, FILE *fp,
 			      HeaderEntry hentry[])
 {
 	return generic_get_one_field(buf, len, fp, hentry,
@@ -65,7 +65,7 @@ gint procheader_get_one_field(gchar *buf, gint len, FILE *fp,
 				     TRUE);
 }
 
-static gint string_get_one_field(gchar *buf, gint len, char **str,
+static gint string_get_one_field(gchar *buf, size_t len, char **str,
 				 HeaderEntry hentry[])
 {
 	return generic_get_one_field(buf, len, str, hentry,
@@ -74,7 +74,7 @@ static gint string_get_one_field(gchar *buf, gint len, char **str,
 				     TRUE);
 }
 
-static char *string_getline(char *buf, int len, char **str)
+static char *string_getline(char *buf, size_t len, char **str)
 {
 	if (!**str)
 		return NULL;
@@ -97,7 +97,7 @@ static int file_peekchar(FILE *fp)
 	return ungetc(getc(fp), fp);
 }
 
-static gint generic_get_one_field(gchar *buf, gint len, void *data,
+static gint generic_get_one_field(gchar *buf, size_t len, void *data,
 			  HeaderEntry *hentry,
 			  getlinefunc getline, peekcharfunc peekchar,
 			  gboolean unfold)
@@ -118,7 +118,7 @@ static gint generic_get_one_field(gchar *buf, gint len, void *data,
 
 			for (hp = hentry, hnum = 0; hp->name != NULL;
 			     hp++, hnum++) {
-				if (!g_strncasecmp(hp->name, buf,
+				if (!g_ascii_strncasecmp(hp->name, buf,
 						 strlen(hp->name)))
 					break;
 			}
@@ -157,7 +157,7 @@ static gint generic_get_one_field(gchar *buf, gint len, void *data,
 	return hnum;
 }
 
-gint procheader_get_one_field_asis(gchar *buf, gint len, FILE *fp)
+gint procheader_get_one_field_asis(gchar *buf, size_t len, FILE *fp)
 {
 	return generic_get_one_field(buf, len, fp, NULL,
 				     (getlinefunc)fgets, 
@@ -400,7 +400,7 @@ gboolean procheader_headername_equal(char * hdr1, char * hdr2)
 	if (len1 != len2)
 		return 0;
 
-	return (g_strncasecmp(hdr1, hdr2, len1) == 0);
+	return (g_ascii_strncasecmp(hdr1, hdr2, len1) == 0);
 }
 
 /*
@@ -412,7 +412,6 @@ gboolean procheader_headername_equal(char * hdr1, char * hdr2)
 
 Header * procheader_parse_header(gchar * buf)
 {
-	gchar tmp[BUFFSIZE];
 	gchar *p = buf;
 	Header * header;
 
@@ -425,12 +424,7 @@ Header * procheader_parse_header(gchar * buf)
 			header->name = g_strndup(buf, p - buf + 1);
 			p++;
 			while (*p == ' ' || *p == '\t') p++;
-                        *tmp = 0;
-			conv_unmime_header(tmp, sizeof(tmp), p, NULL);
-                        if (*tmp)
-				header->body = g_strdup(tmp);
-                        else                                
-				header->body = g_strdup(p);
+                        header->body = conv_unmime_header(p, NULL);
 			return header;
 		}
 	}
@@ -583,9 +577,9 @@ static MsgInfo *parse_stream(void *data, gboolean isstring, MsgFlags flags,
 			     gboolean full, gboolean decrypted)
 {
 	MsgInfo *msginfo;
-	gchar buf[BUFFSIZE], tmp[BUFFSIZE];
+	gchar buf[BUFFSIZE];
 	gchar *reference = NULL;
-	gchar *p;
+	gchar *p, *tmp;
 	gchar *hp;
 	HeaderEntry *hentry;
 	gint hnum;
@@ -623,12 +617,11 @@ static MsgInfo *parse_stream(void *data, gboolean isstring, MsgFlags flags,
 			break;
 		case H_FROM:
 			if (msginfo->from) break;
-			conv_unmime_header(tmp, sizeof(tmp), hp, NULL);
-			msginfo->from = g_strdup(tmp);
-			msginfo->fromname = procheader_get_fromname(tmp);
+                        msginfo->from = conv_unmime_header(hp, NULL);
+			msginfo->fromname = procheader_get_fromname(msginfo->from);
 			break;
 		case H_TO:
-			conv_unmime_header(tmp, sizeof(tmp), hp, NULL);
+                        tmp = conv_unmime_header(hp, NULL);
 			if (msginfo->to) {
 				p = msginfo->to;
 				msginfo->to =
@@ -636,9 +629,10 @@ static MsgInfo *parse_stream(void *data, gboolean isstring, MsgFlags flags,
 				g_free(p);
 			} else
 				msginfo->to = g_strdup(tmp);
+                        g_free(tmp);                                
 			break;
 		case H_CC:
-			conv_unmime_header(tmp, sizeof(tmp), hp, NULL);
+                        tmp = conv_unmime_header(hp, NULL);
 			if (msginfo->cc) {
 				p = msginfo->cc;
 				msginfo->cc =
@@ -646,6 +640,7 @@ static MsgInfo *parse_stream(void *data, gboolean isstring, MsgFlags flags,
 				g_free(p);
 			} else
 				msginfo->cc = g_strdup(tmp);
+                        g_free(tmp);                                
 			break;
 		case H_NEWSGROUPS:
 			if (msginfo->newsgroups) {
@@ -658,8 +653,7 @@ static MsgInfo *parse_stream(void *data, gboolean isstring, MsgFlags flags,
 			break;
 		case H_SUBJECT:
 			if (msginfo->subject) break;
-			conv_unmime_header(tmp, sizeof(tmp), hp, NULL);
-			msginfo->subject = g_strdup(tmp);
+                        msginfo->subject = conv_unmime_header(hp, NULL);
 			break;
 		case H_MSG_ID:
 			if (msginfo->msgid) break;
@@ -683,7 +677,7 @@ static MsgInfo *parse_stream(void *data, gboolean isstring, MsgFlags flags,
 			}
 			break;
 		case H_CONTENT_TYPE:
-			if (!g_strncasecmp(hp, "multipart/", 10))
+			if (!g_ascii_strncasecmp(hp, "multipart/", 10))
 				MSG_SET_TMP_FLAGS(msginfo->flags, MSG_MULTIPART);
 			break;
 #ifdef ALLOW_HEADER_HINT			
@@ -916,7 +910,7 @@ time_t procheader_date_parse(gchar *dest, const gchar *src, gint len)
 
 	month[3] = '\0';
 	for (p = monthstr; *p != '\0'; p += 3) {
-		if (!g_strncasecmp(p, month, 3)) {
+		if (!g_ascii_strncasecmp(p, month, 3)) {
 			dmonth = (gint)(p - monthstr) / 3 + 1;
 			break;
 		}
@@ -949,6 +943,8 @@ void procheader_date_get_localtime(gchar *dest, gint len, const time_t timer)
 {
 	struct tm *lt;
 	gchar *default_format = "%y/%m/%d(%a) %H:%M";
+	gchar *str;
+	const gchar *src_codeset, *dest_codeset;
 
 	lt = localtime(&timer);
 
@@ -956,6 +952,15 @@ void procheader_date_get_localtime(gchar *dest, gint len, const time_t timer)
 		strftime(dest, len, prefs_common.date_format, lt);
 	else
 		strftime(dest, len, default_format, lt);
+
+	src_codeset = conv_get_locale_charset_str();
+	dest_codeset = CS_UTF_8;
+	str = conv_codeset_strdup(dest, src_codeset, dest_codeset);
+	if (str) {
+		g_snprintf(dest, len, "%s", str);
+		strncpy2(dest, str, len);
+		g_free(str);
+	}
 }
 
 /* Added by Mel Hadasht on 27 Aug 2001 */
