@@ -25,6 +25,7 @@
 
 #include <stdio.h>
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <stdio.h>
 #include <string.h>
 #include <locale.h>
@@ -33,7 +34,6 @@
 #include <sys/stat.h>
 #include <unistd.h>
 
-#include "intl.h"
 #include "procmime.h"
 #include "procheader.h"
 #include "base64.h"
@@ -136,7 +136,8 @@ void procmime_mimeinfo_free_all(MimeInfo *mimeinfo)
 {
 	GNode *node;
 
-	g_return_if_fail(mimeinfo);
+	if (!mimeinfo)
+		return;
 
 	node = mimeinfo->node;
 	g_node_traverse(node, G_IN_ORDER, G_TRAVERSE_ALL, -1, free_func, NULL);
@@ -274,7 +275,6 @@ gboolean procmime_decode_content(MimeInfo *mimeinfo)
 	gchar buf[BUFFSIZE];
 	gint readend;
 	gchar *tmpfilename;
-	gchar *mimetmpdir;
 	FILE *outfp, *infp;
 	struct stat statbuf;
 	gboolean tmp_file = FALSE;
@@ -296,8 +296,7 @@ gboolean procmime_decode_content(MimeInfo *mimeinfo)
 	}
 	fseek(infp, mimeinfo->offset, SEEK_SET);
 
-	mimetmpdir = get_mime_tmp_dir();
-	outfp = get_tmpfile_in_dir(mimetmpdir, &tmpfilename);
+	outfp = get_tmpfile_in_dir(get_mime_tmp_dir(), &tmpfilename);
 	if (!outfp) {
 		perror("tmpfile");
 		return FALSE;
@@ -333,12 +332,8 @@ gboolean procmime_decode_content(MimeInfo *mimeinfo)
 		while ((ftell(infp) < readend) && (fgets(buf, sizeof(buf), infp) != NULL)) {
 			len = base64_decoder_decode(decoder, buf, outbuf);
 			if (len < 0) {
-				g_warning("Bad BASE64 content.\n");
-				fwrite(_("[Error decoding BASE64]\n"),
-					sizeof(gchar),
-					strlen(_("[Error decoding BASE64]\n")),
-					tmpfp);
-				continue;
+				g_warning("Bad BASE64 content\n");
+				break;
 			}
 			fwrite(outbuf, sizeof(gchar), len, tmpfp);
 		}
@@ -402,7 +397,6 @@ gboolean procmime_encode_content(MimeInfo *mimeinfo, EncodingType encoding)
 	FILE *infp, *outfp;
 	gint len;
 	gchar *tmpfilename;
-	gchar *mimetmpdir;
 	struct stat statbuf;
 
 	if (mimeinfo->encoding_type != ENC_UNKNOWN &&
@@ -412,8 +406,7 @@ gboolean procmime_encode_content(MimeInfo *mimeinfo, EncodingType encoding)
 		if(!procmime_decode_content(mimeinfo))
 			return FALSE;
 
-	mimetmpdir = get_mime_tmp_dir();
-	outfp = get_tmpfile_in_dir(mimetmpdir, &tmpfilename);
+	outfp = get_tmpfile_in_dir(get_mime_tmp_dir(), &tmpfilename);
 	if (!outfp) {
 		perror("tmpfile");
 		return FALSE;
@@ -436,8 +429,8 @@ gboolean procmime_encode_content(MimeInfo *mimeinfo, EncodingType encoding)
 		gchar *tmp_file = NULL;
 
 		if (mimeinfo->type == MIMETYPE_TEXT ||
-		    mimeinfo->type == MIMETYPE_MESSAGE) {
-		    	if (mimeinfo->content == MIMECONTENT_FILE) {
+		     mimeinfo->type == MIMETYPE_MESSAGE) {
+		     	if (mimeinfo->content == MIMECONTENT_FILE) {
 				tmp_file = get_tmp_file();
 				if (canonicalize_file(mimeinfo->data.filename, tmp_file) < 0) {
 					g_free(tmp_file);
@@ -454,7 +447,7 @@ gboolean procmime_encode_content(MimeInfo *mimeinfo, EncodingType encoding)
 			} else {
 				gchar *out = canonicalize_str(mimeinfo->data.mem);
 				fclose(infp);
-				tmp_fp = infp = str_open_as_stream(out);
+				infp = str_open_as_stream(out);
 				g_free(out);
 				if (infp == NULL)
 					return FALSE;
@@ -478,7 +471,7 @@ gboolean procmime_encode_content(MimeInfo *mimeinfo, EncodingType encoding)
 			fclose(tmp_fp);
 			unlink(tmp_file);
 			g_free(tmp_file);
-		} 
+		}
 	} else if (encoding == ENC_QUOTED_PRINTABLE) {
 		gchar inbuf[BUFFSIZE], outbuf[BUFFSIZE * 4];
 
@@ -715,7 +708,7 @@ FILE *procmime_get_text_content(MimeInfo *mimeinfo)
 		struct ContentRenderer * cr;
 
 		cr = cur->data;
-		if (g_strcasecmp(cr->content_type, content_type) == 0) {
+		if (g_ascii_strcasecmp(cr->content_type, content_type) == 0) {
 			renderer = cr;
 			break;
 		}
@@ -742,7 +735,9 @@ FILE *procmime_get_text_content(MimeInfo *mimeinfo)
 		}
 		
 		dup2(oldout, 1);
-	} else if (mimeinfo->type == MIMETYPE_TEXT && !g_strcasecmp(mimeinfo->subtype, "html")) {
+#warning FIXME_GTK2 HTML/RTF not yet utf8
+/* CodeConverter seems to have no effect here */
+	} else if (mimeinfo->type == MIMETYPE_TEXT && !g_ascii_strcasecmp(mimeinfo->subtype, "html")) {
 		HTMLParser *parser;
 		CodeConverter *conv;
 
@@ -753,7 +748,7 @@ FILE *procmime_get_text_content(MimeInfo *mimeinfo)
 		}
 		html_parser_destroy(parser);
 		conv_code_converter_destroy(conv);
-	} else if (mimeinfo->type == MIMETYPE_TEXT && !g_strcasecmp(mimeinfo->subtype, "enriched")) {
+	} else if (mimeinfo->type == MIMETYPE_TEXT && !g_ascii_strcasecmp(mimeinfo->subtype, "enriched")) {
 		ERTFParser *parser;
 		CodeConverter *conv;
 
@@ -766,7 +761,7 @@ FILE *procmime_get_text_content(MimeInfo *mimeinfo)
 		conv_code_converter_destroy(conv);
 	} else if (mimeinfo->type == MIMETYPE_TEXT) {
 		while (fgets(buf, sizeof(buf), tmpfp) != NULL) {
-			str = conv_codeset_strdup(buf, src_codeset, NULL);
+			str = conv_codeset_strdup(buf, src_codeset, CS_UTF_8);
 			if (str) {
 				fputs(str, outfp);
 				g_free(str);
@@ -885,8 +880,8 @@ gchar *procmime_get_tmp_file_name(MimeInfo *mimeinfo)
 
 	g_snprintf(f_prefix, sizeof(f_prefix), "%08x.", id++);
 
-	if ((mimeinfo->type == MIMETYPE_TEXT) && !g_strcasecmp(mimeinfo->subtype, "html"))
-		base = "mimetmp.html";
+	if ((mimeinfo->type == MIMETYPE_TEXT) && !g_ascii_strcasecmp(mimeinfo->subtype, "html"))
+		base = g_strdup("mimetmp.html");
 	else {
 		const gchar *basetmp;
 
@@ -895,15 +890,17 @@ gchar *procmime_get_tmp_file_name(MimeInfo *mimeinfo)
 			basetmp = procmime_mimeinfo_get_parameter(mimeinfo, "name");
 		if (basetmp == NULL)
 			basetmp = "mimetmp";
-		base = g_basename(basetmp);
-		if (*base == '\0') base = "mimetmp";
-		Xstrdup_a(base, base, return NULL);
+		basetmp = g_path_get_basename(basetmp);
+		if (*basetmp == '\0') basetmp = g_strdup("mimetmp");
+		base = conv_filename_from_utf8(basetmp);
 		subst_for_shellsafe_filename(base);
 	}
 
 	filename = g_strconcat(get_mime_tmp_dir(), G_DIR_SEPARATOR_S,
 			       f_prefix, base, NULL);
 
+	g_free(base);
+	
 	return filename;
 }
 
@@ -915,14 +912,16 @@ gchar *procmime_get_mime_type(const gchar *filename)
 	MimeType *mime_type;
 	const gchar *p;
 	gchar *ext;
+	gchar *base;
 
 	if (!mime_type_table) {
 		mime_type_table = procmime_get_mime_type_table();
 		if (!mime_type_table) return NULL;
 	}
 
-	filename = g_basename(filename);
-	p = strrchr(filename, '.');
+	base = g_path_get_basename(filename);
+	p = strrchr(base, '.');
+	g_free(base);
 	if (!p) return NULL;
 
 	Xstrdup_a(ext, p + 1, return NULL);
@@ -957,7 +956,7 @@ static gint procmime_str_equal(gconstpointer gptr1, gconstpointer gptr2)
 	const char *str1 = gptr1;
 	const char *str2 = gptr2;
 
-	return !g_strcasecmp(str1, str2);
+	return !g_utf8_collate(str1, str2);
 }
 
 static GHashTable *procmime_get_mime_type_table(void)
@@ -1058,14 +1057,14 @@ EncodingType procmime_get_encoding_for_charset(const gchar *charset)
 {
 	if (!charset)
 		return ENC_8BIT;
-	else if (!g_strncasecmp(charset, "ISO-2022-", 9) ||
-		 !g_strcasecmp(charset, "US-ASCII"))
+	else if (!g_ascii_strncasecmp(charset, "ISO-2022-", 9) ||
+		 !g_ascii_strcasecmp(charset, "US-ASCII"))
 		return ENC_7BIT;
-	else if (!g_strcasecmp(charset, "ISO-8859-5") ||
-		 !g_strncasecmp(charset, "KOI8-", 5) ||
-		 !g_strcasecmp(charset, "Windows-1251"))
+	else if (!g_ascii_strcasecmp(charset, "ISO-8859-5") ||
+		 !g_ascii_strncasecmp(charset, "KOI8-", 5) ||
+		 !g_ascii_strcasecmp(charset, "Windows-1251"))
 		return ENC_8BIT;
-	else if (!g_strncasecmp(charset, "ISO-8859-", 9))
+	else if (!g_ascii_strncasecmp(charset, "ISO-8859-", 9))
 		return ENC_QUOTED_PRINTABLE;
 	else
 		return ENC_8BIT;
@@ -1180,7 +1179,7 @@ MimeMediaType procmime_get_media_type(const gchar *str)
 	struct TypeTable *typetablearray;
 
 	for (typetablearray = mime_type_table; typetablearray->str != NULL; typetablearray++)
-		if (g_strncasecmp(str, typetablearray->str, strlen(typetablearray->str)) == 0)
+		if (g_ascii_strncasecmp(str, typetablearray->str, strlen(typetablearray->str)) == 0)
 			return typetablearray->type;
 
 	return MIMETYPE_UNKNOWN;
@@ -1201,7 +1200,7 @@ gchar *procmime_get_content_type_str(MimeMediaType type,
 	return g_strdup_printf("%s/%s", type_str, subtype);
 }
 
-int procmime_parse_mimepart(MimeInfo *parent,
+void procmime_parse_mimepart(MimeInfo *parent,
 			     gchar *content_type,
 			     gchar *content_encoding,
 			     gchar *content_description,
@@ -1227,8 +1226,8 @@ void procmime_parse_message_rfc822(MimeInfo *mimeinfo)
 				{NULL,		   NULL, FALSE}};
 	guint content_start, i;
 	FILE *fp;
-	gchar buf[BUFFSIZE];
 	gint mime_major, mime_minor;
+        gchar *tmp;
 
 	procmime_decode_content(mimeinfo);
 
@@ -1239,15 +1238,21 @@ void procmime_parse_message_rfc822(MimeInfo *mimeinfo)
 	}
 	fseek(fp, mimeinfo->offset, SEEK_SET);
 	procheader_get_header_fields(fp, hentry);
-	if (hentry[0].body != NULL)
-		conv_unmime_header(buf, sizeof(buf), hentry[0].body,
-				   NULL);
-	if (hentry[2].body != NULL)
-		conv_unmime_header(buf, sizeof(buf), hentry[2].body,
-				   NULL);
-	if (hentry[4].body != NULL)
-		conv_unmime_header(buf, sizeof(buf), hentry[4].body,
-				   NULL);
+	if (hentry[0].body != NULL) {
+                tmp = conv_unmime_header(hentry[0].body, NULL);
+                g_free(hentry[0].body);
+                hentry[0].body = tmp;
+        }                
+	if (hentry[2].body != NULL) {
+                tmp = conv_unmime_header(hentry[2].body, NULL);
+                g_free(hentry[2].body);
+                hentry[2].body = tmp;
+        }                
+	if (hentry[4].body != NULL) {
+                tmp = conv_unmime_header(hentry[4].body, NULL);
+                g_free(hentry[4].body);
+                hentry[4].body = tmp;
+        }                
 	content_start = ftell(fp);
 	fclose(fp);
 
@@ -1292,12 +1297,11 @@ void procmime_parse_multipart(MimeInfo *mimeinfo)
 				{"Content-Disposition:",
 				                   NULL, TRUE},
 				{NULL,		   NULL, FALSE}};
-	gchar *p;
+	gchar *p, *tmp;
 	gchar *boundary;
 	gint boundary_len = 0, lastoffset = -1, i;
 	gchar buf[BUFFSIZE];
 	FILE *fp;
-	int result = 0;
 
 	boundary = g_hash_table_lookup(mimeinfo->typeparameters, "boundary");
 	if (!boundary)
@@ -1312,13 +1316,13 @@ void procmime_parse_multipart(MimeInfo *mimeinfo)
 		return;
 	}
 	fseek(fp, mimeinfo->offset, SEEK_SET);
-	while ((p = fgets(buf, sizeof(buf), fp)) != NULL && result == 0) {
+	while ((p = fgets(buf, sizeof(buf), fp)) != NULL) {
 		if (ftell(fp) > (mimeinfo->offset + mimeinfo->length))
 			break;
 
 		if (IS_BOUNDARY(buf, boundary, boundary_len)) {
 			if (lastoffset != -1) {
-				result = procmime_parse_mimepart(mimeinfo,
+				procmime_parse_mimepart(mimeinfo,
 				                        hentry[0].body, hentry[1].body,
 							hentry[2].body, hentry[3].body, 
 							hentry[4].body, 
@@ -1335,15 +1339,21 @@ void procmime_parse_multipart(MimeInfo *mimeinfo)
 				hentry[i].body = NULL;
 			}
 			procheader_get_header_fields(fp, hentry);
-			if (hentry[0].body != NULL)
-				conv_unmime_header(buf, sizeof(buf), hentry[0].body,
-				NULL);
-			if (hentry[2].body != NULL)
-				conv_unmime_header(buf, sizeof(buf), hentry[2].body,
-				NULL);
-			if (hentry[4].body != NULL)
-				conv_unmime_header(buf, sizeof(buf), hentry[4].body,
-				NULL);
+                        if (hentry[0].body != NULL) {
+                                tmp = conv_unmime_header(hentry[0].body, NULL);
+                                g_free(hentry[0].body);
+                                hentry[0].body = tmp;
+                        }                
+                        if (hentry[2].body != NULL) {
+                                tmp = conv_unmime_header(hentry[2].body, NULL);
+                                g_free(hentry[2].body);
+                                hentry[2].body = tmp;
+                        }                
+                        if (hentry[4].body != NULL) {
+                                tmp = conv_unmime_header(hentry[4].body, NULL);
+                                g_free(hentry[4].body);
+                                hentry[4].body = tmp;
+                        }                
 			lastoffset = ftell(fp);
 		}
 	}
@@ -1486,7 +1496,7 @@ static void parse_parameters(const gchar *parameters, GHashTable *table)
 		oldvalue[0] = '\0';
 		oldvalue++;
 
-		newvalue = conv_codeset_strdup(oldvalue, charset, conv_get_current_charset_str());
+		newvalue = conv_codeset_strdup(oldvalue, charset, CS_UTF_8);
 
 		g_hash_table_remove(table, attribute);
 		g_free(key);
@@ -1551,9 +1561,9 @@ static void procmime_parse_content_disposition(const gchar *content_disposition,
 	}	
 	g_strstrip(tmp);
 
-	if (!g_strcasecmp(tmp, "inline")) 
+	if (!g_ascii_strcasecmp(tmp, "inline")) 
 		mimeinfo->disposition = DISPOSITIONTYPE_INLINE;
-	else if (!g_strcasecmp(tmp, "attachment"))
+	else if (!g_ascii_strcasecmp(tmp, "attachment"))
 		mimeinfo->disposition = DISPOSITIONTYPE_ATTACHMENT;
 	else
 		mimeinfo->disposition = DISPOSITIONTYPE_ATTACHMENT;
@@ -1570,7 +1580,7 @@ static void procmime_parse_content_encoding(const gchar *content_encoding, MimeI
 	struct EncodingTable *enc_table;
 	
 	for (enc_table = encoding_table; enc_table->str != NULL; enc_table++) {
-		if (g_strcasecmp(enc_table->str, content_encoding) == 0) {
+		if (g_ascii_strcasecmp(enc_table->str, content_encoding) == 0) {
 			mimeinfo->encoding_type = enc_table->enc_type;
 			return;
 		}
@@ -1579,7 +1589,7 @@ static void procmime_parse_content_encoding(const gchar *content_encoding, MimeI
 	return;
 }
 
-int procmime_parse_mimepart(MimeInfo *parent,
+void procmime_parse_mimepart(MimeInfo *parent,
 			     gchar *content_type,
 			     gchar *content_encoding,
 			     gchar *content_description,
@@ -1594,17 +1604,8 @@ int procmime_parse_mimepart(MimeInfo *parent,
 	/* Create MimeInfo */
 	mimeinfo = procmime_mimeinfo_new();
 	mimeinfo->content = MIMECONTENT_FILE;
-	if (parent != NULL) {
-		if (g_node_depth(parent->node) > 32) {
-			/* 32 is an arbitrary value
-			 * this avoids DOSsing ourselves 
-			 * with enormous messages
-			 */
-			procmime_mimeinfo_free_all(mimeinfo);
-			return -1;			
-		}
+	if (parent != NULL)
 		g_node_append(parent->node, mimeinfo->node);
-	}
 	mimeinfo->data.filename = g_strdup(filename);
 	mimeinfo->offset = offset;
 	mimeinfo->length = length;
@@ -1643,7 +1644,7 @@ int procmime_parse_mimepart(MimeInfo *parent,
 	/* Call parser for mime type */
 	switch (mimeinfo->type) {
 		case MIMETYPE_MESSAGE:
-			if (g_strcasecmp(mimeinfo->subtype, "rfc822") == 0) {
+			if (g_ascii_strcasecmp(mimeinfo->subtype, "rfc822") == 0) {
 				procmime_parse_message_rfc822(mimeinfo);
 			}
 			break;
@@ -1655,8 +1656,6 @@ int procmime_parse_mimepart(MimeInfo *parent,
 		default:
 			break;
 	}
-
-	return 0;
 }
 
 static gchar *typenames[] = {
@@ -1802,23 +1801,23 @@ static void write_parameters(gpointer key, gpointer value, gpointer user_data)
 
 	switch (encas) {
 	case ENC_AS_TOKEN:
-		g_string_sprintfa(buf, "%s=%s", param, val);
+		g_string_append_printf(buf, "%s=%s", param, val);
 		break;
 
 	case ENC_AS_QUOTED_STRING:
-		g_string_sprintfa(buf, "%s=\"%s\"", param, val);
+		g_string_append_printf(buf, "%s=\"%s\"", param, val);
 		break;
 
 	case ENC_AS_EXTENDED:
-		g_string_sprintfa(buf, "%s*=%s''", param,
-			conv_get_current_charset_str());
+		g_string_append_printf(buf, "%s*=%s''", param,
+			conv_get_locale_charset_str());
 		for (valpos = val; *valpos != '\0'; valpos++) {
 			if (IS_ASCII(*valpos) && isalnum(*valpos)) {
-				g_string_sprintfa(buf, "%c", *valpos);
+				g_string_append_printf(buf, "%c", *valpos);
 			} else {
 				gchar hexstr[3] = "XX";
 				get_hex_str(hexstr, *valpos);
-				g_string_sprintfa(buf, "%%%s", hexstr);
+				g_string_append_printf(buf, "%%%s", hexstr);
 			}
 		}
 		break;
@@ -1913,12 +1912,12 @@ gint procmime_write_message_rfc822(MimeInfo *mimeinfo, FILE *fp)
 				break;
 			if (skip && (buf[0] == ' ' || buf[0] == '\t'))
 				continue;
-			if (g_strncasecmp(buf, "Mime-Version:", 13) == 0 ||
-			    g_strncasecmp(buf, "Content-Type:", 13) == 0 ||
-			    g_strncasecmp(buf, "Content-Transfer-Encoding:", 26) == 0 ||
-			    g_strncasecmp(buf, "Content-Description:", 20) == 0 ||
-			    g_strncasecmp(buf, "Content-ID:", 11) == 0 ||
-			    g_strncasecmp(buf, "Content-Disposition:", 20) == 0) {
+			if (g_ascii_strncasecmp(buf, "Mime-Version:", 13) == 0 ||
+			    g_ascii_strncasecmp(buf, "Content-Type:", 13) == 0 ||
+			    g_ascii_strncasecmp(buf, "Content-Transfer-Encoding:", 26) == 0 ||
+			    g_ascii_strncasecmp(buf, "Content-Description:", 20) == 0 ||
+			    g_ascii_strncasecmp(buf, "Content-ID:", 11) == 0 ||
+			    g_ascii_strncasecmp(buf, "Content-Disposition:", 20) == 0) {
 				skip = TRUE;
 				continue;
 			}
@@ -2036,7 +2035,7 @@ gint procmime_write_mimeinfo(MimeInfo *mimeinfo, FILE *fp)
 		/* Call writer for mime type */
 		switch (mimeinfo->type) {
 		case MIMETYPE_MESSAGE:
-			if (g_strcasecmp(mimeinfo->subtype, "rfc822") == 0)
+			if (g_ascii_strcasecmp(mimeinfo->subtype, "rfc822") == 0)
 				return procmime_write_message_rfc822(mimeinfo, fp);
 			break;
 			
