@@ -20,11 +20,11 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <ctype.h>
 
-#include "intl.h"
 #include "main.h"
 #include "utils.h"
 #include "procmsg.h"
@@ -602,19 +602,12 @@ void procmsg_get_filter_keyword(MsgInfo *msginfo, gchar **header, gchar **key,
 
 void procmsg_empty_trash(FolderItem *trash)
 {
-	FILE *fp;
-
 	if (trash && trash->total_msgs > 0) {
 		GSList *mlist = folder_item_get_msg_list(trash);
 		GSList *cur;
 		for (cur = mlist ; cur != NULL ; cur = cur->next) {
 			MsgInfo * msginfo = (MsgInfo *) cur->data;
-			if (MSG_IS_LOCKED(msginfo->flags))
-				continue;
-			if (msginfo->total_size != 0 && 
-			    msginfo->size != (off_t)msginfo->total_size)
-				partial_mark_for_delete(msginfo);
-
+			partial_mark_for_delete(msginfo);
 			procmsg_msginfo_free(msginfo);
 		}
 
@@ -1662,7 +1655,7 @@ MsgInfo *procmsg_msginfo_new_from_mimeinfo(MsgInfo *src_msginfo, MimeInfo *mimei
 	
 	
 	if (!mimeinfo || mimeinfo->type != MIMETYPE_MESSAGE ||
-	    g_strcasecmp(mimeinfo->subtype, "rfc822")) {
+	    g_ascii_strcasecmp(mimeinfo->subtype, "rfc822")) {
 		g_warning("procmsg_msginfo_new_from_mimeinfo(): unsuitable mimeinfo");
 		return NULL;
 	}
