@@ -20,7 +20,7 @@
 #include "defs.h"
 #include <glib.h>
 #include <gtk/gtkwidget.h>
-#include <gtk/gtkpixmap.h>
+#include <gtk/gtkimage.h>
 #include <string.h>
 #include <dirent.h>
 
@@ -78,6 +78,7 @@
 #include "pixmaps/paste.xpm"
 #include "pixmaps/preferences.xpm"
 #include "pixmaps/properties.xpm"
+#include "pixmaps/sylpheed_icon.xpm"
 #include "pixmaps/sylpheed_logo.xpm"
 #include "pixmaps/address_book.xpm"
 #include "pixmaps/trash.xpm"
@@ -92,7 +93,6 @@
 #include "pixmaps/notice_error.xpm"
 #include "pixmaps/notice_note.xpm"
 #include "pixmaps/quicksearch.xpm"
-#include "pixmaps/clip_gpg_signed.xpm"
 #include "pixmaps/gpg_signed.xpm"
 #include "pixmaps/drafts_close.xpm"
 #include "pixmaps/drafts_open.xpm"
@@ -127,6 +127,7 @@ struct _StockPixmapData
 	GdkBitmap *mask;
 	gchar *file;
 	gchar *icon_path;
+	GdkPixbuf *pixbuf;
 };
 
 typedef struct _OverlayData OverlayData;
@@ -160,7 +161,6 @@ static StockPixmapData pixmaps[] =
 	{check_spelling_xpm                     , NULL, NULL, "check_spelling", NULL},
 	{clip_xpm				, NULL, NULL, "clip", NULL},
 	{clipkey_xpm				, NULL, NULL, "clipkey", NULL},
-	{clip_gpg_signed_xpm			, NULL, NULL, "clip_gpg_signed", NULL},
 	{close_xpm				, NULL, NULL, "close", NULL},
 	{complete_xpm				, NULL, NULL, "complete", NULL},
 	{continue_xpm				, NULL, NULL, "continue", NULL},
@@ -250,7 +250,8 @@ static StockPixmapData pixmaps[] =
 	{privacy_emblem_failed_xpm		, NULL, NULL, "privacy_emblem_failed", NULL},	
 	{privacy_emblem_warn_xpm		, NULL, NULL, "privacy_emblem_warn", NULL},
 	{mime_message_xpm			, NULL, NULL, "mime_message", NULL},
- 	{read_xpm				, NULL, NULL, "read", NULL},
+	{sylpheed_icon_xpm			, NULL, NULL, "sylpheed_icon", NULL},
+	{read_xpm				, NULL, NULL, "read", NULL},
 	{sylpheed_logo_xpm			, NULL, NULL, "sylpheed_logo", NULL},
 };
 
@@ -264,11 +265,75 @@ GtkWidget *stock_pixmap_widget(GtkWidget *window, StockPixmap icon)
 	g_return_val_if_fail(icon >= 0 && icon < N_STOCK_PIXMAPS, NULL);
 
 	if (stock_pixmap_gdk(window, icon, &pixmap, &mask) != -1)
-		return gtk_pixmap_new(pixmap, mask);
+		return gtk_image_new_from_pixmap(pixmap, mask);
 	
 	return NULL;
 }
 
+/*!
+ *\brief	
+ */
+gint stock_pixbuf_gdk(GtkWidget *window, StockPixmap icon, GdkPixbuf **pixbuf)
+{
+	StockPixmapData *pix_d;
+
+	g_return_val_if_fail(window != NULL, -1);
+	g_return_val_if_fail(icon >= 0 && icon < N_STOCK_PIXMAPS, -1);
+
+	if (pixbuf) 
+		*pixbuf = NULL;
+
+	pix_d = &pixmaps[icon];
+
+	if (!pix_d->pixbuf || (strcmp2(pix_d->icon_path, prefs_common.pixmap_theme_path) != 0)) {
+		GdkPixbuf *pix = NULL;
+	
+		if (strcmp(prefs_common.pixmap_theme_path, DEFAULT_PIXMAP_THEME) != 0) {
+			if (is_dir_exist(prefs_common.pixmap_theme_path)) {
+				char *icon_file_name; 
+				
+				icon_file_name = g_strconcat(prefs_common.pixmap_theme_path,
+							     G_DIR_SEPARATOR_S,
+							     pix_d->file,
+							     ".xpm",
+							     NULL);
+				if (is_file_exist(icon_file_name)) {
+					GError *err = NULL;
+					pix = gdk_pixbuf_new_from_file(icon_file_name, &err);	
+					if (err) g_error_free(err);
+				}					
+				if (pix) {
+					if (pix_d->icon_path != NULL) g_free(pix_d->icon_path);
+					pix_d->icon_path = g_strdup(prefs_common.pixmap_theme_path);
+				}
+				g_free(icon_file_name);
+			} else {
+				/* even the path does not exist (deleted between two sessions), so
+				set the preferences to the internal theme */
+				prefs_common.pixmap_theme_path = g_strdup(DEFAULT_PIXMAP_THEME);
+			}
+		}
+		pix_d->pixbuf = pix;
+	}
+
+	if (!pix_d->pixbuf) {
+		pix_d->pixbuf = gdk_pixbuf_new_from_xpm_data((const gchar **) pix_d->data);
+		if (pix_d->pixbuf) {
+			if (pix_d->icon_path != NULL) g_free(pix_d->icon_path);
+			pix_d->icon_path = g_strdup(DEFAULT_PIXMAP_THEME);	
+		}
+	}
+
+	g_return_val_if_fail(pix_d->pixbuf != NULL, -1);
+
+	if (pixbuf)
+		*pixbuf = pix_d->pixbuf;
+
+	/* pixbuf should have one ref outstanding */		
+
+	return 0;
+}
+
 /* create GdkPixmap if it has not created yet */
 gint stock_pixmap_gdk(GtkWidget *window, StockPixmap icon,
 		      GdkPixmap **pixmap, GdkBitmap **mask)
@@ -509,11 +574,11 @@ static gboolean pixmap_with_overlay_expose_event_cb(GtkWidget *widget, GdkEventE
 
 static void pixmap_with_overlay_destroy_cb(GtkObject *object, OverlayData *data) 
 {
-	gdk_pixmap_unref(data->base_pixmap);
-	gdk_pixmap_unref(data->base_mask);
+	g_object_unref(data->base_pixmap);
+	g_object_unref(data->base_mask);
 	if (data->position != OVERLAY_NONE) {
-		gdk_pixmap_unref(data->overlay_pixmap);
-		gdk_pixmap_unref(data->overlay_mask);
+		g_object_unref(data->overlay_pixmap);
+		g_object_unref(data->overlay_mask);
 	}
 	g_free(data);
 }
@@ -544,9 +609,9 @@ GtkWidget *stock_pixmap_widget_with_overlay(GtkWidget *window, StockPixmap icon,
 	data = g_new0(OverlayData, 1);
 
 	stock_wid = stock_pixmap_widget(window, icon);
-	gtk_pixmap_get(GTK_PIXMAP(stock_wid), &stock_pixmap, &stock_mask);
-	gdk_pixmap_ref(stock_pixmap);
-	gdk_pixmap_ref(stock_mask);
+	gtk_image_get_pixmap(GTK_IMAGE(stock_wid), &stock_pixmap, &stock_mask);
+	g_object_ref(stock_pixmap);
+	g_object_ref(stock_mask);
 	data->base_pixmap = stock_pixmap;
 	data->base_mask   = stock_mask;
 	data->base_height = stock_wid->requisition.height;
@@ -558,9 +623,9 @@ GtkWidget *stock_pixmap_widget_with_overlay(GtkWidget *window, StockPixmap icon,
 		data->overlay_mask   = NULL;
 	} else {
 		stock_wid = stock_pixmap_widget(window, overlay);
-		gtk_pixmap_get(GTK_PIXMAP(stock_wid), &stock_pixmap, &stock_mask);
-		gdk_pixmap_ref(stock_pixmap);
-		gdk_pixmap_ref(stock_mask);
+		gtk_image_get_pixmap(GTK_IMAGE(stock_wid), &stock_pixmap, &stock_mask);
+		g_object_ref(stock_pixmap);
+		g_object_ref(stock_mask);
 		data->overlay_pixmap = stock_pixmap;
 		data->overlay_mask   = stock_mask;
 		data->overlay_height = stock_wid->requisition.height;
@@ -576,10 +641,10 @@ GtkWidget *stock_pixmap_widget_with_overlay(GtkWidget *window, StockPixmap icon,
 	widget = gtk_drawing_area_new();
 	gtk_drawing_area_size(GTK_DRAWING_AREA(widget), data->base_width + border_x * 2, 
 			      data->base_height + border_y * 2);
-	gtk_signal_connect(GTK_OBJECT(widget), "expose_event", 
-			   GTK_SIGNAL_FUNC(pixmap_with_overlay_expose_event_cb), data);
-	gtk_signal_connect(GTK_OBJECT(widget), "destroy",
-			   GTK_SIGNAL_FUNC(pixmap_with_overlay_destroy_cb), data);
+	g_signal_connect(G_OBJECT(widget), "expose_event", 
+			 G_CALLBACK(pixmap_with_overlay_expose_event_cb), data);
+	g_signal_connect(G_OBJECT(widget), "destroy",
+			 G_CALLBACK(pixmap_with_overlay_destroy_cb), data);
 	return widget;
 
 }
