@@ -35,7 +35,6 @@ typedef enum
 	STOCK_PIXMAP_CHECK_SPELLING,
 	STOCK_PIXMAP_CLIP,
 	STOCK_PIXMAP_CLIP_KEY,
-	STOCK_PIXMAP_CLIP_GPG_SIGNED,
 	STOCK_PIXMAP_CLOSE,
 	STOCK_PIXMAP_COMPLETE,
 	STOCK_PIXMAP_CONTINUE,
@@ -125,8 +124,8 @@ typedef enum
 	STOCK_PIXMAP_PRIVACY_EMBLEM_FAILED,
 	STOCK_PIXMAP_PRIVACY_EMBLEM_WARN,
 	STOCK_PIXMAP_MIME_MESSAGE,
+	STOCK_PIXMAP_SYLPHEED_ICON,
 	STOCK_PIXMAP_READ,
-	
 	STOCK_PIXMAP_SYLPHEED_LOGO,		/* last entry */
 	
 	N_STOCK_PIXMAPS
@@ -147,6 +146,8 @@ typedef enum {
 
 GtkWidget *stock_pixmap_widget	(GtkWidget	 *window,
 				 StockPixmap	  icon);
+gint stock_pixbuf_gdk		(GtkWidget *window, StockPixmap icon, 
+				 GdkPixbuf **pixbuf);
 gint stock_pixmap_gdk		(GtkWidget	 *window,
 				 StockPixmap	  icon,
 				 GdkPixmap	**pixmap,
