@@ -28,6 +28,7 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <dirent.h>
@@ -35,7 +36,6 @@
 #include <math.h>
 #include <setjmp.h>
 
-#include "intl.h"
 #include "mainwindow.h"
 #include "summaryview.h"
 #include "compose.h"
@@ -154,7 +154,7 @@ struct {
 } toolbar_text [] = {
 	{ "A_RECEIVE_ALL",   	N_("Receive Mail on all Accounts")         },
 	{ "A_RECEIVE_CUR",   	N_("Receive Mail on current Account")      },
-	{ "A_SEND_QUEUED",   	N_("Send Queued Messages")                 },
+	{ "A_SEND_QUEUED",   	N_("Send Queued Message(s)")               },
 	{ "A_COMPOSE_EMAIL", 	N_("Compose Email")                        },
 	{ "A_COMPOSE_NEWS",  	N_("Compose News")                         },
 	{ "A_REPLY_MESSAGE", 	N_("Reply to Message")                     },
@@ -232,7 +232,7 @@ gint toolbar_ret_val_from_descr(const gchar *descr)
 	gint i;
 
 	for (i = 0; i < N_ACTION_VAL; i++) {
-		if (g_strcasecmp(gettext(toolbar_text[i].descr), descr) == 0)
+		if (g_utf8_collate(gettext(toolbar_text[i].descr), descr) == 0)
 				return i;
 	}
 	
@@ -251,7 +251,7 @@ static gint toolbar_ret_val_from_text(const gchar *text)
 	gint i;
 	
 	for (i = 0; i < N_ACTION_VAL; i++) {
-		if (g_strcasecmp(toolbar_text[i].index_str, text) == 0)
+		if (g_utf8_collate(toolbar_text[i].index_str, text) == 0)
 				return i;
 	}
 	
@@ -345,11 +345,11 @@ static void toolbar_parse_item(XMLFile *file, ToolbarType source)
 		name = ((XMLAttr *)attr->data)->name;
 		value = ((XMLAttr *)attr->data)->value;
 		
-		if (g_strcasecmp(name, TOOLBAR_ICON_FILE) == 0) 
+		if (g_utf8_collate(name, TOOLBAR_ICON_FILE) == 0) 
 			item->file = g_strdup (value);
-		else if (g_strcasecmp(name, TOOLBAR_ICON_TEXT) == 0)
+		else if (g_utf8_collate(name, TOOLBAR_ICON_TEXT) == 0)
 			item->text = g_strdup (value);
-		else if (g_strcasecmp(name, TOOLBAR_ICON_ACTION) == 0)
+		else if (g_utf8_collate(name, TOOLBAR_ICON_ACTION) == 0)
 			item->index = toolbar_ret_val_from_text(value);
 
 		attr = g_list_next(attr);
@@ -528,23 +528,25 @@ void toolbar_save_config_file(ToolbarType source)
 	g_free( fileSpec );
 	if( pfile ) {
 		fp = pfile->fp;
-		fprintf(fp, "<?xml version=\"1.0\" encoding=\"%s\" ?>\n",
-			conv_get_current_charset_str());
+		fprintf(fp, "<?xml version=\"1.0\" encoding=\"%s\" ?>\n", CS_INTERNAL);
 
 		fprintf(fp, "<%s>\n", TOOLBAR_TAG_INDEX);
 
 		for (cur = toolbar_config[source].item_list; cur != NULL; cur = cur->next) {
 			ToolbarItem *toolbar_item = (ToolbarItem*) cur->data;
 			
-			if (toolbar_item->index != A_SEPARATOR) 
-				fprintf(fp, "\t<%s %s=\"%s\" %s=\"%s\" %s=\"%s\"/>\n",
+			if (toolbar_item->index != A_SEPARATOR) {
+				fprintf(fp, "\t<%s %s=\"%s\" %s=\"",
 					TOOLBAR_TAG_ITEM, 
 					TOOLBAR_ICON_FILE, toolbar_item->file,
-					TOOLBAR_ICON_TEXT, toolbar_item->text,
+					TOOLBAR_ICON_TEXT);
+				xml_file_put_escape_str(fp, toolbar_item->text);
+				fprintf(fp, "\" %s=\"%s\"/>\n",
 					TOOLBAR_ICON_ACTION, 
 					toolbar_ret_text_from_val(toolbar_item->index));
-			else 
+			} else {
 				fprintf(fp, "\t<%s/>\n", TOOLBAR_TAG_SEPARATOR); 
+			}
 		}
 
 		fprintf(fp, "</%s>\n", TOOLBAR_TAG_INDEX);	
@@ -690,7 +692,7 @@ void toolbar_action_execute(GtkWidget    *widget,
 
 				action_p = strstr(action, ": ");
 				action_p[0] = 0x00;
-				if (g_strcasecmp(act->name, action) == 0) {
+				if (g_utf8_collate(act->name, action) == 0) {
 					found = TRUE;
 					g_free(action);
 					break;
@@ -1291,17 +1293,15 @@ Toolbar *toolbar_create(ToolbarType 	 type,
 
 	toolbar_data = g_new0(Toolbar, 1); 
 
-	toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL,
-				  GTK_TOOLBAR_BOTH);
+	toolbar = gtk_toolbar_new();
 	gtk_container_add(GTK_CONTAINER(container), toolbar);
 	gtk_container_set_border_width(GTK_CONTAINER(container), 2);
-	gtk_toolbar_set_button_relief(GTK_TOOLBAR(toolbar), GTK_RELIEF_NONE);
-	gtk_toolbar_set_space_style(GTK_TOOLBAR(toolbar),
-				    GTK_TOOLBAR_SPACE_LINE);
+	gtk_toolbar_set_orientation(GTK_TOOLBAR(toolbar), GTK_ORIENTATION_HORIZONTAL);
+	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_BOTH);
 	
 	for (cur = toolbar_list; cur != NULL; cur = cur->next) {
 
-		if (g_strcasecmp(((ToolbarItem*)cur->data)->file, TOOLBAR_TAG_SEPARATOR) == 0) {
+		if (g_ascii_strcasecmp(((ToolbarItem*)cur->data)->file, TOOLBAR_TAG_SEPARATOR) == 0) {
 			gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
 			continue;
 		}
@@ -1323,7 +1323,7 @@ Toolbar *toolbar_create(ToolbarType 	 type,
 						toolbar_item->text,
 						(""),
 						(""),
-						icon_wid, toolbar_buttons_cb, 
+						icon_wid, G_CALLBACK(toolbar_buttons_cb), 
 						toolbar_item);
 		
 		switch (toolbar_item->index) {
@@ -1344,7 +1344,7 @@ Toolbar *toolbar_create(ToolbarType 	 type,
 			toolbar_data->send_btn = item; 
 			gtk_tooltips_set_tip(GTK_TOOLTIPS(toolbar_tips), 
 					     toolbar_data->send_btn,
-					   _("Send Queued Messages"), NULL);
+					   _("Send Queued Message(s)"), NULL);
 			break;
 		case A_COMPOSE_EMAIL:
 			icon_news = stock_pixmap_widget(container, STOCK_PIXMAP_NEWS_COMPOSE);
@@ -1352,7 +1352,7 @@ Toolbar *toolbar_create(ToolbarType 	 type,
 							    _("News"),
 							    (""),
 							    (""),
-							    icon_news, toolbar_buttons_cb, 
+							    icon_news, G_CALLBACK(toolbar_buttons_cb), 
 							    toolbar_item);
 			toolbar_data->compose_mail_btn = item; 
 			gtk_tooltips_set_tip(GTK_TOOLTIPS(toolbar_tips), 
@@ -1374,7 +1374,7 @@ Toolbar *toolbar_create(ToolbarType 	 type,
 					      reply_entries, n_menu_entries,
 					      "<Reply>", (gpointer)toolbar_item);
 			gtk_button_set_relief(GTK_BUTTON(reply_combo->arrow),
-			      		      GTK_TOOLBAR(toolbar)->relief);
+					      GTK_RELIEF_NONE);
 			gtk_toolbar_append_widget(GTK_TOOLBAR(toolbar),
 				  		  GTK_WIDGET_PTR(reply_combo),
 				 		  _("Reply to Message"), "Reply");
@@ -1391,7 +1391,7 @@ Toolbar *toolbar_create(ToolbarType 	 type,
 					      replysender_entries, n_menu_entries,
 					      "<ReplySender>", (gpointer)toolbar_item);
 			gtk_button_set_relief(GTK_BUTTON(replysender_combo->arrow),
-			      		      GTK_TOOLBAR(toolbar)->relief);
+					      GTK_RELIEF_NONE);
 			gtk_toolbar_append_widget(GTK_TOOLBAR(toolbar),
 				  		  GTK_WIDGET_PTR(replysender_combo),
 				 		  _("Reply to Sender"), "ReplySender");
@@ -1408,7 +1408,7 @@ Toolbar *toolbar_create(ToolbarType 	 type,
 					      replyall_entries, n_menu_entries,
 					      "<ReplyAll>", (gpointer)toolbar_item);
 			gtk_button_set_relief(GTK_BUTTON(replyall_combo->arrow),
-			      		      GTK_TOOLBAR(toolbar)->relief);
+					      GTK_RELIEF_NONE);
 			gtk_toolbar_append_widget(GTK_TOOLBAR(toolbar),
 				  		  GTK_WIDGET_PTR(replyall_combo),
 				 		  _("Reply to All"), "ReplyAll");
@@ -1425,7 +1425,7 @@ Toolbar *toolbar_create(ToolbarType 	 type,
 					      replylist_entries, n_menu_entries,
 					      "<ReplyList>", (gpointer)toolbar_item);
 			gtk_button_set_relief(GTK_BUTTON(replylist_combo->arrow),
-			      		      GTK_TOOLBAR(toolbar)->relief);
+					      GTK_RELIEF_NONE);
 			gtk_toolbar_append_widget(GTK_TOOLBAR(toolbar),
 				  		  GTK_WIDGET_PTR(replylist_combo),
 				 		  _("Reply to Mailing-list"), "ReplyList");
@@ -1442,7 +1442,7 @@ Toolbar *toolbar_create(ToolbarType 	 type,
 					      forward_entries, n_menu_entries,
 					      "<Forward>", (gpointer)toolbar_item);
 			gtk_button_set_relief(GTK_BUTTON(fwd_combo->arrow),
-			      		      GTK_TOOLBAR(toolbar)->relief);
+					      GTK_RELIEF_NONE);
 			gtk_toolbar_append_widget(GTK_TOOLBAR(toolbar),
 				  		  GTK_WIDGET_PTR(fwd_combo),
 				 		  _("Forward Message"), "Fwd");
